@@ -34,7 +34,8 @@ import           Game.LambdaHack.Common.Actor
 import qualified Game.LambdaHack.Common.Dice as Dice
 import           Game.LambdaHack.Common.Faction
 import           Game.LambdaHack.Common.Item
-import qualified Game.LambdaHack.Common.Kind as Kind
+import qualified Game.LambdaHack.Common.ItemAspect as IA
+import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Level
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.Perception
@@ -87,14 +88,15 @@ data UpdAtomic =
   | UpdDiplFaction FactionId FactionId Diplomacy Diplomacy
   | UpdTacticFaction FactionId Tactic Tactic
   | UpdAutoFaction FactionId Bool
-  | UpdRecordKill ActorId (Kind.Id ItemKind) Int
+  | UpdRecordKill ActorId (ContentId ItemKind) Int
   -- Alter map.
-  | UpdAlterTile LevelId Point (Kind.Id TileKind) (Kind.Id TileKind)
+  | UpdAlterTile LevelId Point (ContentId TileKind) (ContentId TileKind)
   | UpdAlterExplorable LevelId Int
-  | UpdSearchTile ActorId Point (Kind.Id TileKind)
-  | UpdHideTile ActorId Point (Kind.Id TileKind)
-  | UpdSpotTile LevelId [(Point, Kind.Id TileKind)]
-  | UpdLoseTile LevelId [(Point, Kind.Id TileKind)]
+  | UpdAlterGold Int
+  | UpdSearchTile ActorId Point (ContentId TileKind)
+  | UpdHideTile ActorId Point (ContentId TileKind)
+  | UpdSpotTile LevelId [(Point, ContentId TileKind)]
+  | UpdLoseTile LevelId [(Point, ContentId TileKind)]
   | UpdAlterSmell LevelId Point Time Time
   | UpdSpotSmell LevelId [(Point, Time)]
   | UpdLoseSmell LevelId [(Point, Time)]
@@ -102,14 +104,14 @@ data UpdAtomic =
   | UpdTimeItem ItemId Container ItemTimer ItemTimer
   | UpdAgeGame [LevelId]
   | UpdUnAgeGame [LevelId]
-  | UpdDiscover Container ItemId (Kind.Id ItemKind) ItemSeed
-  | UpdCover Container ItemId (Kind.Id ItemKind) ItemSeed
-  | UpdDiscoverKind Container ItemKindIx (Kind.Id ItemKind)
-  | UpdCoverKind Container ItemKindIx (Kind.Id ItemKind)
-  | UpdDiscoverSeed Container ItemId ItemSeed
-  | UpdCoverSeed Container ItemId ItemSeed
-  | UpdDiscoverServer ItemId AspectRecord
-  | UpdCoverServer ItemId AspectRecord
+  | UpdDiscover Container ItemId (ContentId ItemKind) IA.AspectRecord
+  | UpdCover Container ItemId (ContentId ItemKind) IA.AspectRecord
+  | UpdDiscoverKind Container ItemKindIx (ContentId ItemKind)
+  | UpdCoverKind Container ItemKindIx (ContentId ItemKind)
+  | UpdDiscoverAspect Container ItemId IA.AspectRecord
+  | UpdCoverAspect Container ItemId IA.AspectRecord
+  | UpdDiscoverServer ItemId IA.AspectRecord
+  | UpdCoverServer ItemId IA.AspectRecord
   | UpdPerception LevelId Perception Perception
   | UpdRestart FactionId PerLid State Challenge ClientOptions
   | UpdRestartServer State
@@ -137,6 +139,7 @@ data SfxAtomic =
   | SfxEffect FactionId ActorId IK.Effect Int64
   | SfxMsgFid FactionId SfxMsg
   | SfxSortSlots
+  | SfxCollideTile ActorId Point
   deriving (Show, Eq, Generic)
 
 instance Binary SfxAtomic
@@ -145,24 +148,29 @@ instance Binary SfxAtomic
 -- and shown to players.
 data SfxMsg =
     SfxUnexpected ReqFailure
+  | SfxExpected Text ReqFailure
   | SfxLoudUpd Bool UpdAtomic
-  | SfxLoudStrike Bool (Kind.Id ItemKind) Int
+  | SfxLoudStrike Bool (ContentId ItemKind) Int
   | SfxLoudSummon Bool (GroupName ItemKind) Dice.Dice
   | SfxFizzles
   | SfxNothingHappens
-  | SfxVoidDetection
+  | SfxVoidDetection IK.DetectKind
+  | SfxUnimpressed ActorId
   | SfxSummonLackCalm ActorId
   | SfxLevelNoMore
   | SfxLevelPushed
   | SfxBracedImmune ActorId
   | SfxEscapeImpossible
+  | SfxStasisProtects
   | SfxTransImpossible
   | SfxIdentifyNothing
   | SfxPurposeNothing CStore
   | SfxPurposeTooFew Int Int
   | SfxPurposeUnique
+  | SfxPurposeNotCommon
   | SfxColdFish
-  | SfxTimerExtended ActorId ItemId CStore
+  | SfxTimerExtended LevelId ActorId ItemId CStore
+  | SfxCollideActor LevelId ActorId ActorId
   deriving (Show, Eq, Generic)
 
 instance Binary SfxMsg
@@ -196,6 +204,7 @@ undoUpdAtomic cmd = case cmd of
   UpdAlterTile lid p fromTile toTile ->
     Just $ UpdAlterTile lid p toTile fromTile
   UpdAlterExplorable lid delta -> Just $ UpdAlterExplorable lid (-delta)
+  UpdAlterGold delta -> Just $ UpdAlterGold (-delta)
   UpdSearchTile aid p toTile -> Just $ UpdHideTile aid p toTile
   UpdHideTile aid p toTile -> Just $ UpdSearchTile aid p toTile
   UpdSpotTile lid ts -> Just $ UpdLoseTile lid ts
@@ -206,12 +215,14 @@ undoUpdAtomic cmd = case cmd of
   UpdTimeItem iid c fromIt toIt -> Just $ UpdTimeItem iid c toIt fromIt
   UpdAgeGame lids -> Just $ UpdUnAgeGame lids
   UpdUnAgeGame lids -> Just $ UpdAgeGame lids
-  UpdDiscover c iid ik seed -> Just $ UpdCover c iid ik seed
-  UpdCover c iid ik seed -> Just $ UpdDiscover c iid ik seed
+  UpdDiscover c iid ik aspectRecord -> Just $ UpdCover c iid ik aspectRecord
+  UpdCover c iid ik aspectRecord -> Just $ UpdDiscover c iid ik aspectRecord
   UpdDiscoverKind c ix ik -> Just $ UpdCoverKind c ix ik
   UpdCoverKind c ix ik -> Just $ UpdDiscoverKind c ix ik
-  UpdDiscoverSeed c iid seed -> Just $ UpdCoverSeed c iid seed
-  UpdCoverSeed c iid seed -> Just $ UpdDiscoverSeed c iid seed
+  UpdDiscoverAspect c iid aspectRecord ->
+    Just $ UpdCoverAspect c iid aspectRecord
+  UpdCoverAspect c iid aspectRecord ->
+    Just $ UpdDiscoverAspect c iid aspectRecord
   UpdDiscoverServer iid aspectRecord -> Just $ UpdCoverServer iid aspectRecord
   UpdCoverServer iid aspectRecord -> Just $ UpdDiscoverServer iid aspectRecord
   UpdPerception lid outPer inPer -> Just $ UpdPerception lid inPer outPer
@@ -237,6 +248,7 @@ undoSfxAtomic cmd = case cmd of
   SfxEffect{} -> cmd  -- not ideal?
   SfxMsgFid{} -> cmd
   SfxSortSlots -> cmd
+  SfxCollideTile{} -> cmd
 
 undoCmdAtomic :: CmdAtomic -> Maybe CmdAtomic
 undoCmdAtomic (UpdAtomic cmd) = UpdAtomic <$> undoUpdAtomic cmd
