@@ -1,3 +1,4 @@
+{-# LANGUAGE FlexibleContexts #-}
 -- | Semantics of atomic commands shared by client and server.
 --
 -- See
@@ -16,7 +17,7 @@ module Game.LambdaHack.Atomic.HandleAtomicWrite
   , updAlterSmell, updSpotSmell, updLoseSmell, updTimeItem
   , updAgeGame, updUnAgeGame, ageLevel, updDiscover, updCover
   , updDiscoverKind, discoverKind, updCoverKind
-  , updDiscoverSeed, discoverSeed, updCoverSeed
+  , updDiscoverAspect, discoverAspect, updCoverAspect
   , updDiscoverServer, updCoverServer
   , updRestart, updRestartServer, updResumeServer
 #endif
@@ -35,7 +36,8 @@ import           Game.LambdaHack.Common.Actor
 import           Game.LambdaHack.Common.ActorState
 import           Game.LambdaHack.Common.Faction
 import           Game.LambdaHack.Common.Item
-import qualified Game.LambdaHack.Common.Kind as Kind
+import qualified Game.LambdaHack.Common.ItemAspect as IA
+import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Level
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.MonadStateRead
@@ -47,6 +49,7 @@ import qualified Game.LambdaHack.Common.Tile as Tile
 import           Game.LambdaHack.Common.Time
 import           Game.LambdaHack.Common.Vector
 import           Game.LambdaHack.Content.ItemKind (ItemKind)
+import qualified Game.LambdaHack.Content.ItemKind as IK
 import           Game.LambdaHack.Content.ModeKind
 import           Game.LambdaHack.Content.TileKind (TileKind, unknownId)
 
@@ -93,6 +96,7 @@ handleUpdAtomic cmd = case cmd of
   UpdRecordKill aid ikind k -> updRecordKill aid ikind k
   UpdAlterTile lid p fromTile toTile -> updAlterTile lid p fromTile toTile
   UpdAlterExplorable lid delta -> updAlterExplorable lid delta
+  UpdAlterGold delta -> updAlterGold delta
   UpdSearchTile aid p toTile -> updSearchTile aid p toTile
   UpdHideTile{} -> undefined
   UpdSpotTile lid ts -> updSpotTile lid ts
@@ -103,12 +107,12 @@ handleUpdAtomic cmd = case cmd of
   UpdTimeItem iid c fromIt toIt -> updTimeItem iid c fromIt toIt
   UpdAgeGame lids -> updAgeGame lids
   UpdUnAgeGame lids -> updUnAgeGame lids
-  UpdDiscover c iid ik seed -> updDiscover c iid ik seed
-  UpdCover c iid ik seed -> updCover c iid ik seed
+  UpdDiscover c iid ik aspectRecord -> updDiscover c iid ik aspectRecord
+  UpdCover c iid ik aspectRecord -> updCover c iid ik aspectRecord
   UpdDiscoverKind c ix ik -> updDiscoverKind c ix ik
   UpdCoverKind c ix ik -> updCoverKind c ix ik
-  UpdDiscoverSeed c iid seed -> updDiscoverSeed c iid seed
-  UpdCoverSeed c iid seed -> updCoverSeed c iid seed
+  UpdDiscoverAspect c iid aspectRecord -> updDiscoverAspect c iid aspectRecord
+  UpdCoverAspect c iid aspectRecord -> updCoverAspect c iid aspectRecord
   UpdDiscoverServer iid aspectRecord -> updDiscoverServer iid aspectRecord
   UpdCoverServer iid aspectRecord -> updCoverServer iid aspectRecord
   UpdPerception _ outPer inPer ->
@@ -185,7 +189,7 @@ updCreateItem iid item kit@(k, _) c = assert (k > 0) $ do
   insertItemContainer iid kit c
   case c of
     CActor aid store ->
-      when (store `elem` [CEqp, COrgan]) $ addItemToActor iid item k aid
+      when (store `elem` [CEqp, COrgan]) $ addItemToActorAspect iid item k aid
     _ -> return ()
 
 -- Destroy some copies (possibly not all) of an item.
@@ -193,8 +197,11 @@ updDestroyItem :: MonadStateWrite m
                => ItemId -> Item -> ItemQuant -> Container -> m ()
 updDestroyItem iid item kit@(k, _) c = assert (k > 0) $ do
   deleteItemContainer iid kit c
-  -- Do not remove the item from @sitemD@ nor from @sitemRev@,
-  -- It's incredibly costly and not noticeable for the player.
+  -- Do not remove the item from @sitemD@ nor from @sitemRev@
+  -- nor from @DiscoveryAspect@, @ItemIxMap@, etc.
+  -- It's incredibly costly and not particularly noticeable for the player.
+  -- Moreover, copies of the item may reappear in the future
+  -- and then we save computation and the player remembers past discovery.
   -- However, assert the item is registered in @sitemD@.
   itemD <- getsState sitemD
   let !_A = assert ((case iid `EM.lookup` itemD of
@@ -203,8 +210,8 @@ updDestroyItem iid item kit@(k, _) c = assert (k > 0) $ do
                     `blame` "item already removed"
                     `swith` (iid, item, itemD)) ()
   case c of
-    CActor aid store ->
-      when (store `elem` [CEqp, COrgan]) $ addItemToActor iid item (-k) aid
+    CActor aid store -> when (store `elem` [CEqp, COrgan])
+                        $ addItemToActorAspect iid item (-k) aid
     _ -> return ()
 
 updSpotItemBag :: MonadStateWrite m
@@ -217,7 +224,7 @@ updSpotItemBag c bag ais = assert (EM.size bag > 0
     CActor aid store ->
       when (store `elem` [CEqp, COrgan]) $
         forM_ ais $ \(iid, item) ->
-                      addItemToActor iid item (fst $ bag EM.! iid) aid
+                      addItemToActorAspect iid item (fst $ bag EM.! iid) aid
     _ -> return ()
 
 updLoseItemBag :: MonadStateWrite m
@@ -236,7 +243,7 @@ updLoseItemBag c bag ais = assert (EM.size bag > 0
     CActor aid store ->
       when (store `elem` [CEqp, COrgan]) $
         forM_ ais $ \(iid, item) ->
-                      addItemToActor iid item (- (fst $ bag EM.! iid)) aid
+                      addItemToActorAspect iid item (- (fst $ bag EM.! iid)) aid
     _ -> return ()
 
 updMoveActor :: MonadStateWrite m => ActorId -> Point -> Point -> m ()
@@ -286,16 +293,16 @@ updMoveItem iid k aid s1 s2 = assert (k > 0 && s1 /= s2) $ do
       COrgan -> return ()
       _ -> do
         itemBase <- getsState $ getItemBody iid
-        addItemToActor iid itemBase (-k) aid
+        addItemToActorAspect iid itemBase (-k) aid
     COrgan -> case s2 of
       CEqp -> return ()
       _ -> do
         itemBase <- getsState $ getItemBody iid
-        addItemToActor iid itemBase (-k) aid
+        addItemToActorAspect iid itemBase (-k) aid
     _ ->
       when (s2 `elem` [CEqp, COrgan]) $ do
         itemBase <- getsState $ getItemBody iid
-        addItemToActor iid itemBase k aid
+        addItemToActorAspect iid itemBase k aid
 
 updRefillHP :: MonadStateWrite m => ActorId -> Int64 -> m ()
 updRefillHP aid nRaw =
@@ -405,7 +412,8 @@ updAutoFaction fid st =
 
 -- Record a given number (usually just 1, or -1 for undo) of actor kills
 -- for score calculation.
-updRecordKill :: MonadStateWrite m => ActorId -> Kind.Id ItemKind -> Int -> m ()
+updRecordKill :: MonadStateWrite m
+              => ActorId -> ContentId ItemKind -> Int -> m ()
 updRecordKill aid ikind k = do
   b <- getsState $ getActorBody aid
   let !_A = assert (not (bproj b) `blame` (aid, b))
@@ -426,10 +434,10 @@ updRecordKill aid ikind k = do
 -- Removing and creating embedded items when altering a tile
 -- is done separately via @UpdCreateItem@ and @UpdDestroyItem@.
 updAlterTile :: MonadStateWrite m
-             => LevelId -> Point -> Kind.Id TileKind -> Kind.Id TileKind
+             => LevelId -> Point -> ContentId TileKind -> ContentId TileKind
              -> m ()
 updAlterTile lid p fromTile toTile = assert (fromTile /= toTile) $ do
-  Kind.COps{coTileSpeedup} <- getsState scops
+  COps{coTileSpeedup} <- getsState scops
   lvl <- getLevel lid
   let t = lvl `at` p
   if t /= fromTile
@@ -445,13 +453,17 @@ updAlterTile lid p fromTile toTile = assert (fromTile /= toTile) $ do
 
 updAlterExplorable :: MonadStateWrite m => LevelId -> Int -> m ()
 updAlterExplorable lid delta = assert (delta /= 0) $
-  updateLevel lid $ \lvl -> lvl {lexplorable = lexplorable lvl + delta}
+  updateLevel lid $ \lvl -> lvl {lexpl = lexpl lvl + delta}
+
+updAlterGold :: MonadStateWrite m => Int -> m ()
+updAlterGold delta = assert (delta /= 0) $
+  modifyState $ updateGold (+ delta)
 
 -- Showing to the client the embedded items, if any, is done elsewhere.
 updSearchTile :: MonadStateWrite m
-              => ActorId -> Point -> Kind.Id TileKind -> m ()
+              => ActorId -> Point -> ContentId TileKind -> m ()
 updSearchTile aid p toTile = do
-  Kind.COps{cotile} <- getsState scops
+  COps{cotile} <- getsState scops
   b <- getsState $ getActorBody aid
   lvl <- getLevel $ blid b
   let t = lvl `at` p
@@ -466,9 +478,9 @@ updSearchTile aid p toTile = do
 -- We verify that the old tiles at the positions in question
 -- are indeed unknown.
 updSpotTile :: MonadStateWrite m
-            => LevelId -> [(Point, Kind.Id TileKind)] -> m ()
+            => LevelId -> [(Point, ContentId TileKind)] -> m ()
 updSpotTile lid ts = assert (not $ null ts) $ do
-  Kind.COps{coTileSpeedup} <- getsState scops
+  COps{coTileSpeedup} <- getsState scops
   let unk tileMap (p, _) = tileMap PointArray.! p == unknownId
       adj tileMap = assert (all (unk tileMap) ts) $ tileMap PointArray.// ts
   updateLevel lid $ updateTile adj
@@ -479,9 +491,9 @@ updSpotTile lid ts = assert (not $ null ts) $ do
 -- Stop noticing previously visible tiles. It verifies
 -- the state of the tiles before wiping them out.
 updLoseTile :: MonadStateWrite m
-            => LevelId -> [(Point, Kind.Id TileKind)] -> m ()
+            => LevelId -> [(Point, ContentId TileKind)] -> m ()
 updLoseTile lid ts = assert (not $ null ts) $ do
-  Kind.COps{coTileSpeedup} <- getsState scops
+  COps{coTileSpeedup} <- getsState scops
   let matches tileMap (p, ov) = tileMap PointArray.! p == ov
       tu = map (second (const unknownId)) ts
       adj tileMap = assert (all (matches tileMap) ts) $ tileMap PointArray.// tu
@@ -545,98 +557,87 @@ ageLevel delta lid =
   updateLevel lid $ \lvl -> lvl {ltime = timeShift (ltime lvl) delta}
 
 updDiscover :: MonadStateWrite m
-            => Container -> ItemId -> Kind.Id ItemKind -> ItemSeed -> m ()
-updDiscover _c iid ik seed = do
+            => Container -> ItemId -> ContentId ItemKind -> IA.AspectRecord
+            -> m ()
+updDiscover _c iid ik aspectRecord = do
   itemD <- getsState sitemD
+  COps{coItemSpeedup} <- getsState scops
+  let kmIsConst = IA.kmConst $ IK.getKindMean ik coItemSpeedup
+  discoKind <- getsState sdiscoKind
+  let discoverAtMostAspect = do
+        discoAspect <- getsState sdiscoAspect
+        if kmIsConst || iid `EM.member` discoAspect
+        then atomicFail "item already fully discovered"
+        else discoverAspect iid aspectRecord
   case EM.lookup iid itemD of
-    Nothing -> atomicFail "discovered item unknown"
-    Just item -> do
-      discoKind <- getsState sdiscoKind
-      case EM.lookup (jkindIx item) discoKind of
-        Just KindMean{kmConst} -> do
-          discoAspect <- getsState sdiscoAspect
-          if kmConst || iid `EM.member` discoAspect
-          then atomicFail "item already fully discovered"
-          else discoverSeed iid seed
+    Nothing -> atomicFail "discovered item unheard of"
+    Just item -> case jkind item of
+      IdentityObvious _ -> discoverAtMostAspect
+      IdentityCovered ix _ik -> case EM.lookup ix discoKind of
+        Just{} -> discoverAtMostAspect
         Nothing -> do
-          KindMean{kmConst} <- discoverKind (jkindIx item) ik
-          unless kmConst $ discoverSeed iid seed
+          discoverKind ix ik
+          unless kmIsConst $ discoverAspect iid aspectRecord
   resetActorAspect
 
-updCover :: Container -> ItemId -> Kind.Id ItemKind -> ItemSeed -> m ()
-updCover _c _iid _ik _seed = undefined
+updCover :: Container -> ItemId -> ContentId ItemKind -> IA.AspectRecord -> m ()
+updCover _c _iid _ik _aspectRecord = undefined
 
 updDiscoverKind :: MonadStateWrite m
-                => Container -> ItemKindIx -> Kind.Id ItemKind -> m ()
+                => Container -> ItemKindIx -> ContentId ItemKind -> m ()
 updDiscoverKind _c ix kmKind = do
   discoKind <- getsState sdiscoKind
   if ix `EM.member` discoKind
   then atomicFail "item kind already discovered"
   else do
-    void $ discoverKind ix kmKind
+    discoverKind ix kmKind
     resetActorAspect
 
-discoverKind :: MonadStateWrite m
-             => ItemKindIx -> Kind.Id ItemKind -> m KindMean
-discoverKind ix kmKind = do
-  Kind.COps{coitem=Kind.Ops{okind}} <- getsState scops
-  let kind = okind kmKind
-      kmMean = meanAspect kind
-      kmConst = not $ aspectsRandom kind
-      km = KindMean{..}
-      f Nothing = Just km
-      f Just{} = error $ "already discovered" `showFailure` (ix, kmKind)
+discoverKind :: MonadStateWrite m => ItemKindIx -> ContentId ItemKind -> m ()
+discoverKind ix kindId = do
+  let f Nothing = Just kindId
+      f Just{} = error $ "already discovered" `showFailure` (ix, kindId)
   modifyState $ updateDiscoKind $ \discoKind1 ->
     EM.alter f ix discoKind1
-  return km
 
-updCoverKind :: Container -> ItemKindIx -> Kind.Id ItemKind -> m ()
+updCoverKind :: Container -> ItemKindIx -> ContentId ItemKind -> m ()
 updCoverKind _c _ix _ik = undefined
 
-updDiscoverSeed :: MonadStateWrite m
-                => Container -> ItemId -> ItemSeed -> m ()
-updDiscoverSeed _c iid seed = do
+updDiscoverAspect :: MonadStateWrite m
+                  => Container -> ItemId -> IA.AspectRecord -> m ()
+updDiscoverAspect _c iid aspectRecord = do
+  COps{coItemSpeedup} <- getsState scops
   itemD <- getsState sitemD
   case EM.lookup iid itemD of
-    Nothing -> atomicFail "discovered item unknown"
+    Nothing -> atomicFail "discovered item unheard of"
     Just item -> do
-      discoKind <- getsState sdiscoKind
-      case EM.lookup (jkindIx item) discoKind of
-        Nothing -> error "discovered item kind unknown"
-        Just KindMean{kmConst} -> do
-          discoAspect <- getsState sdiscoAspect
-          if kmConst || iid `EM.member` discoAspect
-          then atomicFail "item seed already discovered"
-          else do
-            discoverSeed iid seed
-            resetActorAspect
+      -- Here the kind information is exact, hence @getItemKindIdServer@.
+      kindId <- getsState $ getItemKindIdServer item
+      discoAspect <- getsState sdiscoAspect
+      let kmIsConst = IA.kmConst $ IK.getKindMean kindId coItemSpeedup
+      if kmIsConst || iid `EM.member` discoAspect
+      then atomicFail "item aspectRecord already discovered"
+      else do
+        discoverAspect iid aspectRecord
+        resetActorAspect
 
-discoverSeed :: MonadStateWrite m => ItemId -> ItemSeed -> m ()
-discoverSeed iid seed = do
-  Kind.COps{coitem=Kind.Ops{okind}} <- getsState scops
-  item <- getsState $ getItemBody iid
-  totalDepth <- getsState stotalDepth
-  Level{ldepth} <- getLevel $ jlid item
-  discoKind <- getsState sdiscoKind
-  let KindMean{..} = fromMaybe (error "discovered item kind unknown")
-                               (EM.lookup (jkindIx item) discoKind)
-      kind = okind kmKind
-      aspects = seedToAspect seed kind ldepth totalDepth
-      f Nothing = Just aspects
-      f Just{} = error $ "already discovered" `showFailure` (iid, seed)
-  assert (not kmConst) $
-    modifyState $ updateDiscoAspect $ \discoAspect1 ->
-      EM.alter f iid discoAspect1
+discoverAspect :: MonadStateWrite m => ItemId -> IA.AspectRecord -> m ()
+discoverAspect iid aspectRecord = do
+  let f Nothing = Just aspectRecord
+      f Just{} = error $ "already discovered" `showFailure` (iid, aspectRecord)
+  -- At this point we know the item is not @kmConst@.
+  modifyState $ updateDiscoAspect $ \discoAspect1 ->
+    EM.alter f iid discoAspect1
 
-updCoverSeed :: Container -> ItemId -> ItemSeed -> m ()
-updCoverSeed _c _iid _seed = undefined
+updCoverAspect :: Container -> ItemId -> IA.AspectRecord -> m ()
+updCoverAspect _c _iid _aspectRecord = undefined
 
-updDiscoverServer :: MonadStateWrite m => ItemId -> AspectRecord -> m ()
+updDiscoverServer :: MonadStateWrite m => ItemId -> IA.AspectRecord -> m ()
 updDiscoverServer iid aspectRecord =
   modifyState $ updateDiscoAspect $ \discoAspect1 ->
     EM.insert iid aspectRecord discoAspect1
 
-updCoverServer :: MonadStateWrite m => ItemId -> AspectRecord -> m ()
+updCoverServer :: MonadStateWrite m => ItemId -> IA.AspectRecord -> m ()
 updCoverServer iid aspectRecord =
   modifyState $ updateDiscoAspect $ \discoAspect1 ->
     assert (discoAspect1 EM.! iid == aspectRecord)
