@@ -4,7 +4,7 @@ module Game.LambdaHack.Atomic.MonadStateWrite
   , putState, updateLevel, updateActor, updateFaction, moveActorMap
   , insertBagContainer, insertItemContainer, insertItemActor
   , deleteBagContainer, deleteItemContainer, deleteItemActor
-  , addAis, itemsMatch, addItemToActor, resetActorAspect
+  , addAis, itemsMatch, addItemToActorAspect, resetActorAspect
 #ifdef EXPOSE_INTERNAL
     -- * Internal operations
   , insertItemFloor, insertItemEmbed
@@ -21,17 +21,20 @@ import Game.LambdaHack.Common.Prelude
 
 import qualified Control.Exception as Ex
 import qualified Data.EnumMap.Strict as EM
+import qualified Data.EnumSet as ES
 import           Data.Key (mapWithKeyM_)
 
-import Game.LambdaHack.Common.Actor
-import Game.LambdaHack.Common.ActorState
-import Game.LambdaHack.Common.Faction
-import Game.LambdaHack.Common.Item
-import Game.LambdaHack.Common.Level
-import Game.LambdaHack.Common.Misc
-import Game.LambdaHack.Common.MonadStateRead
-import Game.LambdaHack.Common.Point
-import Game.LambdaHack.Common.State
+import           Game.LambdaHack.Common.Actor
+import           Game.LambdaHack.Common.ActorState
+import           Game.LambdaHack.Common.Faction
+import           Game.LambdaHack.Common.Item
+import qualified Game.LambdaHack.Common.ItemAspect as IA
+import           Game.LambdaHack.Common.Level
+import           Game.LambdaHack.Common.Misc
+import           Game.LambdaHack.Common.MonadStateRead
+import           Game.LambdaHack.Common.Point
+import           Game.LambdaHack.Common.State
+import qualified Game.LambdaHack.Content.ItemKind as IK
 
 -- | The monad for writing to the main game state. Atomic updates ('UpdAtomic')
 -- are given semantics in this monad.
@@ -156,22 +159,22 @@ insertItemActor iid kit aid cstore = case cstore of
 insertItemOrgan :: MonadStateWrite m
                 => ItemId -> ItemQuant -> ActorId -> m ()
 insertItemOrgan iid kit aid = do
+  itemKind <- getsState $ getIidKind iid
   let bag = EM.singleton iid kit
       upd = EM.unionWith mergeItemQuant bag
-  item <- getsState $ getItemBody iid
   updateActor aid $ \b ->
     b { borgan = upd (borgan b)
-      , bweapon = if isMelee item then bweapon b + 1 else bweapon b }
+      , bweapon = if IK.isMelee itemKind then bweapon b + 1 else bweapon b }
 
 insertItemEqp :: MonadStateWrite m
               => ItemId -> ItemQuant -> ActorId -> m ()
 insertItemEqp iid kit aid = do
+  itemKind <- getsState $ getIidKind iid
   let bag = EM.singleton iid kit
       upd = EM.unionWith mergeItemQuant bag
-  item <- getsState $ getItemBody iid
   updateActor aid $ \b ->
     b { beqp = upd (beqp b)
-      , bweapon = if isMelee item then bweapon b + 1 else bweapon b }
+      , bweapon = if IK.isMelee itemKind then bweapon b + 1 else bweapon b }
 
 insertItemInv :: MonadStateWrite m
               => ItemId -> ItemQuant -> ActorId -> m ()
@@ -248,17 +251,17 @@ deleteItemActor iid kit aid cstore = case cstore of
 
 deleteItemOrgan :: MonadStateWrite m => ItemId -> ItemQuant -> ActorId -> m ()
 deleteItemOrgan iid kit aid = do
-  item <- getsState $ getItemBody iid
+  itemKind <- getsState $ getIidKind iid
   updateActor aid $ \b ->
     b { borgan = rmFromBag kit iid (borgan b)
-      , bweapon = if isMelee item then bweapon b - 1 else bweapon b }
+      , bweapon = if IK.isMelee itemKind then bweapon b - 1 else bweapon b }
 
 deleteItemEqp :: MonadStateWrite m => ItemId -> ItemQuant -> ActorId -> m ()
 deleteItemEqp iid kit aid = do
-  item <- getsState $ getItemBody iid
+  itemKind <- getsState $ getIidKind iid
   updateActor aid $ \b ->
     b { beqp = rmFromBag kit iid (beqp b)
-      , bweapon = if isMelee item then bweapon b - 1 else bweapon b }
+      , bweapon = if IK.isMelee itemKind then bweapon b - 1 else bweapon b }
 
 deleteItemInv :: MonadStateWrite m => ItemId -> ItemQuant -> ActorId -> m ()
 deleteItemInv iid kit aid =
@@ -294,21 +297,24 @@ addAis ais = do
                 `blame` "inconsistent added items"
                 `swith` (item1, item2, ais))
                item2 -- keep the first found level
-  forM_ ais $ \(iid, item) ->
-    modifyState
-    $ updateItemIxMap (EM.insertWith (++) (jkindIx item) [iid])
-      . updateItemD (EM.insertWith h iid item)
+  forM_ ais $ \(iid, item) -> do
+    let f = case jkind item of
+          IdentityObvious _ -> id
+          IdentityCovered ix _ ->
+            updateItemIxMap $ EM.insertWith ES.union ix (ES.singleton iid)
+    modifyState $ f . updateItemD (EM.insertWith h iid item)
 
 itemsMatch :: Item -> Item -> Bool
 itemsMatch item1 item2 =
-  jkindIx item1 == jkindIx item2
+  jkind item1 == jkind item2
   -- Note that nothing else needs to be the same, since items are merged
   -- and clients have different views on dungeon items than the server.
 
-addItemToActor :: MonadStateWrite m => ItemId -> Item -> Int -> ActorId -> m ()
-addItemToActor iid itemBase k aid = do
+addItemToActorAspect :: MonadStateWrite m
+                     => ItemId -> Item -> Int -> ActorId -> m ()
+addItemToActorAspect iid itemBase k aid = do
   arItem <- getsState $ aspectRecordFromItem iid itemBase
-  let f arActor = sumAspectRecord [(arActor, 1), (arItem, k)]
+  let f arActor = IA.sumAspectRecord [(arActor, 1), (arItem, k)]
   modifyState $ updateActorAspect $ EM.adjust f aid
 
 resetActorAspect :: MonadStateWrite m => m ()
