@@ -94,7 +94,7 @@ posUpdAtomic cmd = case cmd of
   UpdDisplaceActor source target -> doubleAid source target
   UpdMoveItem _ _ _ _ CSha ->
     error $ "" `showFailure` cmd  -- shared stash is private
-  UpdMoveItem _ _ _ CSha _ ->  error $ "" `showFailure` cmd
+  UpdMoveItem _ _ _ CSha _ -> error $ "" `showFailure` cmd
   UpdMoveItem _ _ aid _ _ -> singleAid aid
   UpdRefillHP aid _ -> singleAid aid
   UpdRefillCalm aid _ -> singleAid aid
@@ -109,6 +109,7 @@ posUpdAtomic cmd = case cmd of
   UpdAlterExplorable{} -> return PosAll
     -- Can't have @PosSight@, because we'd end up with many accessible
     -- unknown tiles, but the game reporting 'all seen'.
+  UpdAlterGold{} -> return PosAll
   UpdSearchTile aid p _ -> do
     b <- getsState $ getActorBody aid
     return $! PosFidAndSight [bfid b] (blid b) [bpos b, p]
@@ -135,8 +136,8 @@ posUpdAtomic cmd = case cmd of
   UpdCover c _ _ _ -> singleContainer c
   UpdDiscoverKind c _ _ -> singleContainer c
   UpdCoverKind c _ _ -> singleContainer c
-  UpdDiscoverSeed c _ _ -> singleContainer c
-  UpdCoverSeed c _ _ -> singleContainer c
+  UpdDiscoverAspect c _ _ -> singleContainer c
+  UpdCoverAspect c _ _ -> singleContainer c
   UpdDiscoverServer{} -> return PosSer
   UpdCoverServer{} -> return PosSer
   UpdPerception{} -> return PosNone
@@ -175,6 +176,7 @@ posSfxAtomic cmd = case cmd of
   SfxEffect _ aid _ _ -> singleAid aid  -- sometimes we don't see source, OK
   SfxMsgFid fid _ -> return $! PosFid fid
   SfxSortSlots -> return PosAll
+  SfxCollideTile aid _ -> singleAid aid
 
 posProjBody :: Actor -> PosAtomic
 posProjBody body =
@@ -221,14 +223,14 @@ breakUpdAtomic cmd = case cmd of
     -- faction always sees the atomic command, so the leader doesn't
     -- need to be updated (or the actor is a projectile, hence not a leader).
     b <- getsState $ getActorBody aid
-    ais <- getsState $ getCarriedAssocs b
+    ais <- getsState $ getCarriedAssocsAndTrunk b
     return [ UpdLoseActor aid b ais
            , UpdSpotActor aid b {bpos = toP, boldpos = Just fromP} ais ]
   UpdDisplaceActor source target -> do
     sb <- getsState $ getActorBody source
-    sais <- getsState $ getCarriedAssocs sb
+    sais <- getsState $ getCarriedAssocsAndTrunk sb
     tb <- getsState $ getActorBody target
-    tais <- getsState $ getCarriedAssocs tb
+    tais <- getsState $ getCarriedAssocsAndTrunk tb
     return [ UpdLoseActor source sb sais
            , UpdSpotActor source sb { bpos = bpos tb
                                     , boldpos = Just $ bpos sb } sais
