@@ -9,12 +9,11 @@ module Game.LambdaHack.Client
   ( -- * Re-exported from "Game.LambdaHack.Client.LoopM"
     loopCli
     -- * Re-exported from "Game.LambdaHack.Client.Request"
-  , RequestAI, ReqAI(..), RequestUI, ReqUI(..)
-  , RequestAnyAbility(..), RequestTimed(..)
+  , RequestAI, ReqAI(..), RequestUI, ReqUI(..), RequestTimed(..)
     -- * Re-exported from "Game.LambdaHack.Client.Response"
   , Response (..)
     -- * Re-exported from "Game.LambdaHack.Client.ClientOptions"
-  , ClientOptions, defClientOptions, sbenchmark
+  , ClientOptions(..), defClientOptions
     -- * Re-exported from "Game.LambdaHack.Client.UI"
   , KeyKind
   , UIOptions, applyUIOptions, uCmdline, mkUIOptions
