@@ -4,7 +4,7 @@ module Game.LambdaHack.Client.AI.PickTargetM
   ( refreshTarget
 #ifdef EXPOSE_INTERNAL
     -- * Internal operations
-  , targetStrategy
+  , computeTarget
 #endif
   ) where
 
@@ -12,11 +12,11 @@ import Prelude ()
 
 import Game.LambdaHack.Common.Prelude
 
+import qualified Data.EnumMap.Lazy as LEM
 import qualified Data.EnumMap.Strict as EM
 import qualified Data.EnumSet as ES
 
 import           Game.LambdaHack.Client.AI.ConditionM
-import           Game.LambdaHack.Client.AI.Strategy
 import           Game.LambdaHack.Client.Bfs
 import           Game.LambdaHack.Client.BfsM
 import           Game.LambdaHack.Client.CommonM
@@ -28,7 +28,8 @@ import           Game.LambdaHack.Common.ActorState
 import           Game.LambdaHack.Common.Faction
 import           Game.LambdaHack.Common.Frequency
 import           Game.LambdaHack.Common.Item
-import qualified Game.LambdaHack.Common.Kind as Kind
+import qualified Game.LambdaHack.Common.ItemAspect as IA
+import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Level
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.MonadStateRead
@@ -46,9 +47,8 @@ import           Game.LambdaHack.Content.TileKind (isUknownSpace)
 -- | Verify and possibly change the target of an actor. This function both
 -- updates the target in the client state and returns the new target explicitly.
 refreshTarget :: MonadClient m => (ActorId, Actor) -> m (Maybe TgtAndPath)
--- This inline speeds up execution by 6% and increases allocation by 9%,
--- despite probably bloating executable (but it slows down execution
--- if pickAI is not inlined):
+-- This inline speeds up execution by 5% and decreases allocation by 10%,
+-- despite probably bloating executable:
 {-# INLINE refreshTarget #-}
 refreshTarget (aid, body) = do
   side <- getsClient sside
@@ -56,36 +56,34 @@ refreshTarget (aid, body) = do
                     `blame` "AI tries to move an enemy actor"
                     `swith` (aid, body, side)) ()
   let !_A = assert (isNothing (btrajectory body) && not (bproj body)
-                    `blame` "AI gets to manually move its projectiles"
+                    `blame` "AI gets to manually move its trajectory actors"
                     `swith` (aid, body, side)) ()
-  stratTarget <- targetStrategy aid
-  if nullStrategy stratTarget then do
-    -- Melee in progress and the actor can't contribute
-    -- and would slow down others if he acted.
-    modifyClient $ \cli -> cli {stargetD = EM.delete aid (stargetD cli)}
-    return Nothing
-  else do
-    -- _debugoldTgt <- getsClient $ EM.lookup aid . stargetD
-    -- Choose a target from those proposed by AI for the actor.
-    tgtMPath <- rndToAction $ frequency $ bestVariant stratTarget
-    modifyClient $ \cli ->
-      cli {stargetD = EM.insert aid tgtMPath (stargetD cli)}
-    return $ Just tgtMPath
-    -- let _debug = T.unpack
-    --       $ "\nHandleAI symbol:"    <+> tshow (bsymbol body)
-    --       <> ", aid:"               <+> tshow aid
-    --       <> ", pos:"               <+> tshow (bpos body)
-    --       <> "\nHandleAI oldTgt:"   <+> tshow _debugoldTgt
-    --       <> "\nHandleAI strTgt:"   <+> tshow stratTarget
-    --       <> "\nHandleAI target:"   <+> tshow tgtMPath
-    -- trace _debug $ return $ Just tgtMPath
+  mtarget <- computeTarget aid
+  case mtarget of
+    Nothing -> do
+      -- Melee in progress and the actor can't contribute
+      -- and would slow down others if he acted.
+      modifyClient $ \cli -> cli {stargetD = EM.delete aid (stargetD cli)}
+      return Nothing
+    Just tgtMPath -> do
+      -- _debugoldTgt <- getsClient $ EM.lookup aid . stargetD
+      -- Choose a target from those proposed by AI for the actor.
+      modifyClient $ \cli ->
+        cli {stargetD = EM.insert aid tgtMPath (stargetD cli)}
+      return mtarget
+      -- let _debug = T.unpack
+      --       $ "\nHandleAI symbol:"    <+> tshow (bsymbol body)
+      --       <> ", aid:"               <+> tshow aid
+      --       <> ", pos:"               <+> tshow (bpos body)
+      --       <> "\nHandleAI oldTgt:"   <+> tshow _debugoldTgt
+      --       <> "\nHandleAI strTgt:"   <+> tshow stratTarget
+      --       <> "\nHandleAI target:"   <+> tshow tgtMPath
+      -- trace _debug $ return $ Just tgtMPath
 
--- | AI proposes possible targets for the actor. Never empty.
-targetStrategy :: forall m. MonadClient m
-               => ActorId -> m (Strategy TgtAndPath)
-{-# INLINE targetStrategy #-}
-targetStrategy aid = do
-  Kind.COps{corule, coTileSpeedup} <- getsState scops
+computeTarget :: forall m. MonadClient m => ActorId -> m (Maybe TgtAndPath)
+{-# INLINE computeTarget #-}
+computeTarget aid = do
+  cops@COps{coTileSpeedup} <- getsState scops
   b <- getsState $ getActorBody aid
   mleader <- getsClient sleader
   scondInMelee <- getsClient scondInMelee
@@ -94,12 +92,11 @@ targetStrategy aid = do
   -- set of abilities as the leader, anyway) and set his target accordingly.
   actorAspect <- getsState sactorAspect
   let lalter = salter EM.! blid b
-      condInMelee = fromMaybe (error $ "" `showFailure` condInMelee)
-                              (scondInMelee EM.! blid b)
-      stdRuleset = Kind.stdRuleset corule
+      condInMelee = scondInMelee LEM.! blid b
+      stdRuleset = getStdRuleset cops
       nearby = rnearby stdRuleset
       ar = fromMaybe (error $ "" `showFailure` aid) (EM.lookup aid actorAspect)
-      actorMaxSk = aSkills ar
+      actorMaxSk = IA.aSkills ar
       alterSkill = EM.findWithDefault 0 AbAlter actorMaxSk
   lvl@Level{lxsize, lysize} <- getLevel $ blid b
   let stepAccesible :: AndPath -> Bool
@@ -115,7 +112,7 @@ targetStrategy aid = do
       -- This is also triggered by @UpdLeadFaction@.
       Just <$> createPath aid tapTgt
     Just tap@TgtAndPath{..} -> do
-      mvalidPos <- aidTgtToPos aid (blid b) tapTgt
+      mvalidPos <- getsState $ aidTgtToPos aid (blid b) tapTgt
       if | isNothing mvalidPos -> return Nothing  -- wrong level
          | bpos b == pathGoal tapPath ->
              return mtgtMPath  -- goal reached; stay there picking up items
@@ -138,7 +135,7 @@ targetStrategy aid = do
              NoPath -> error $ "" `showFailure` tap
     Nothing -> return Nothing  -- no target assigned yet
   fact <- getsState $ (EM.! bfid b) . sfactionD
-  allFoes <- getsState $ actorRegularAssocs (isAtWar fact) (blid b)
+  allFoes <- getsState $ foeRegularAssocs (bfid b) (blid b)
   dungeon <- getsState sdungeon
   let canMove = EM.findWithDefault 0 AbMove actorMaxSk > 0
                 || EM.findWithDefault 0 AbDisplace actorMaxSk > 0
@@ -151,9 +148,9 @@ targetStrategy aid = do
   condEnoughGear <- condEnoughGearM aid
   let condCanMelee = actorCanMelee actorAspect aid b
       condHpTooLow = hpTooLow b ar
-  friends <- getsState $ friendlyActorRegularList (bfid b) (blid b)
+  friends <- getsState $ friendRegularList (bfid b) (blid b)
   let canEscape = fcanEscape (gplayer fact)
-      canSmell = aSmell ar > 0
+      canSmell = IA.aSmell ar > 0
       meleeNearby | canEscape = nearby `div` 2
                   | otherwise = nearby
       rangedNearby = 2 * meleeNearby
@@ -169,7 +166,7 @@ targetStrategy aid = do
             -- + 2 from foe being 2 away from friend before he closed in
             -- + 1 for as a margin for ambush, given than actors exploring
             -- can't physically keep adjacent all the time
-            n | aAggression ar >= 2 = rangedNearby  -- boss never waits
+            n | IA.aAggression ar >= 2 = rangedNearby  -- boss never waits
               | condInMelee = if attacksFriends then 4 else 0
               | otherwise = meleeNearby
             nonmoving = EM.findWithDefault 0 AbMove actorMaxSkE <= 0
@@ -181,24 +178,23 @@ targetStrategy aid = do
       -- targeted, which is fine, since he is weakened by ranged, so should be
       -- meleed ASAP, even if without friends.
       targetableRanged body =
-        (not condInMelee || aAggression ar >= 2)  -- boss fires at will
+        (not condInMelee || IA.aAggression ar >= 2)  -- boss fires at will
         && chessDist (bpos body) (bpos b) < rangedNearby
         && condCanProject
       targetableEnemy (aidE, body) = do
         tMelee <- targetableMelee aidE body
         return $! targetableRanged body || tMelee
   nearbyFoes <- filterM targetableEnemy allFoes
-  explored <- getsClient sexplored
   isStairPos <- getsState $ \s lid p -> isStair lid p s
   discoBenefit <- getsClient sdiscoBenefit
+  fleeD <- getsClient sfleeD
   s <- getState
-  let lidExplored = ES.member (blid b) explored
-      desirableBagFloor bag = any (\iid ->
-        let item = getItemBody iid s
-            benPick = benPickup <$> EM.lookup iid discoBenefit
-        in desirableItem canEscape benPick item) $ EM.keys bag
+  getKind <- getsState $ flip getIidKind
+  let desirableBagFloor bag = any (\iid ->
+        let Benefit{benPickup} = discoBenefit EM.! iid
+        in desirableItem canEscape benPickup (getKind iid)) $ EM.keys bag
       desirableFloor (_, (_, bag)) = desirableBagFloor bag
-      focused = bspeed b ar < speedWalk || condHpTooLow
+      focused = gearSpeed ar < speedWalk || condHpTooLow
       couldMoveLastTurn =
         let actorSk = if mleader == Just aid then actorMaxSk else actorMinSk
         in EM.findWithDefault 0 AbMove actorSk > 0
@@ -206,7 +202,7 @@ targetStrategy aid = do
       slackTactic =
         ftactic (gplayer fact)
           `elem` [TMeleeAndRanged, TMeleeAdjacent, TBlock, TRoam, TPatrol]
-      setPath :: Target -> m (Strategy TgtAndPath)
+      setPath :: Target -> m (Maybe TgtAndPath)
       setPath tgt = do
         let take7 tap@TgtAndPath{tapTgt=TEnemy{}} =
               tap  -- @TEnemy@ needed for projecting, even by roaming actors
@@ -220,13 +216,13 @@ targetStrategy aid = do
               else tap
             take7 tap = tap
         tgtpath <- createPath aid tgt
-        return $! returN "setPath" $ take7 tgtpath
-      pickNewTarget :: m (Strategy TgtAndPath)
+        return $ Just $ take7 tgtpath
+      pickNewTarget :: m (Maybe TgtAndPath)
       pickNewTarget = do
         cfoes <- closestFoes nearbyFoes aid
         case cfoes of
           (_, (aid2, _)) : _ -> setPath $ TEnemy aid2 False
-          [] | condInMelee -> return reject  -- don't slow down fighters
+          [] | condInMelee -> return Nothing  -- don't slow down fighters
             -- this looks a bit strange, because teammates stop in their tracks
             -- all around the map (unless very close to the combatant),
             -- but the intuition is, not being able to help immediately,
@@ -259,7 +255,7 @@ targetStrategy aid = do
                                 pathList = nub tra
                                 pathGoal = last pathList
                                 pathLen = length pathList
-                            return $! returN "tgt with no exploration"
+                            return $ Just $
                               TgtAndPath
                                 { tapTgt = TVector v
                                 , tapPath = if pathLen == 0
@@ -274,15 +270,15 @@ targetStrategy aid = do
                               -- if initial altering, consider carefully below
                       then vToTgt vOld
                       else do
-                        upos <- if lidExplored
-                                then return Nothing
-                                else closestUnknown aid -- modifies sexplored
+                        upos <- closestUnknown aid
                         case upos of
                           Nothing -> do
-                            explored2 <- getsClient sexplored
-                            let allExplored2 = ES.size explored2
-                                               == EM.size dungeon
-                            if allExplored2 || nullFreq ctriggers then do
+                            modifyClient $ \cli -> cli {sexplored =
+                              ES.insert (blid b) (sexplored cli)}
+                            explored <- getsClient sexplored
+                            let allExplored =
+                                  ES.size explored == EM.size dungeon
+                            if allExplored || nullFreq ctriggers then do
                               -- All stones turned, time to win or die.
                               afoes <- closestFoes allFoes aid
                               case afoes of
@@ -318,9 +314,11 @@ targetStrategy aid = do
       tileAdj f p = any f $ vicinityUnsafe p
       followingWrong permit =
         permit && (condInMelee  -- in melee, stop following
-                   || mleader == Just aid) -- a leader, never follow
-      updateTgt :: TgtAndPath -> m (Strategy TgtAndPath)
+                   || mleader == Just aid)  -- a leader, never follow
+      updateTgt :: TgtAndPath -> m (Maybe TgtAndPath)
       updateTgt TgtAndPath{tapPath=NoPath} = pickNewTarget
+      updateTgt _ | EM.member aid fleeD = pickNewTarget
+        -- forget enemy positions to prevent attacking them again soon
       updateTgt tap@TgtAndPath{tapPath=AndPath{..},tapTgt} = case tapTgt of
         TEnemy a permit -> do
           body <- getsState $ getActorBody a
@@ -332,7 +330,7 @@ targetStrategy aid = do
                pickNewTarget
              | followingWrong permit -> pickNewTarget
              | bpos body == pathGoal ->
-               return $! returN "TEnemy" tap
+               return $ Just tap
                  -- The enemy didn't move since the target acquired.
                  -- If any walls were added that make the enemy
                  -- unreachable, AI learns that the hard way,
@@ -347,7 +345,7 @@ targetStrategy aid = do
                case mpath of
                  NoPath -> pickNewTarget  -- enemy became unreachable
                  AndPath{pathLen=0} -> pickNewTarget  -- he is his own enemy
-                 AndPath{} -> return $! returN "TEnemy" tap{tapPath=mpath}
+                 AndPath{} -> return $ Just tap{tapPath=mpath}
           -- In this case, need to retarget, to focus on foes that melee ours
           -- and not, e.g., on remembered foes or items.
         _ | condInMelee -> pickNewTarget
@@ -358,7 +356,7 @@ targetStrategy aid = do
           TEnemyPos _ permit  -- chase last position even if foe hides
             | bpos b == pos -> tellOthersNothingHere pos
             | followingWrong permit -> pickNewTarget
-            | otherwise -> return $! returN "TEnemyPos" tap
+            | otherwise -> return $ Just tap
           -- Below we check the target could not be picked again in
           -- pickNewTarget (e.g., an item got picked up by our teammate)
           -- and only in this case it is invalidated.
@@ -383,7 +381,7 @@ targetStrategy aid = do
                      -- stay there one turn (high chance to become leader)
                      -- to enable triggering; if trigger fails
                      -- (e.g, changed skills), will retarget next turn (@TAny@)
-               | otherwise -> return $! returN "TEmbed" tap
+               | otherwise -> return $ Just tap
           TItem bag -> do
             bag2 <- getsState $ getFloorBag lid pos
             if | bag /= bag2 -> pickNewTarget  -- others will notice soon enough
@@ -391,44 +389,45 @@ targetStrategy aid = do
                    setPath $ TPoint TAny lid (bpos b)
                      -- stay there one turn (high chance to become leader)
                      -- to enable pickup; if pickup fails, will retarget
-               | otherwise -> return $! returN "TItem" tap
+               | otherwise -> return $ Just tap
           TSmell ->
             let lvl2 = sdungeon s EM.! lid
             in if not canSmell
                   || let sml = EM.findWithDefault timeZero pos (lsmell lvl2)
                      in sml <= ltime lvl2
                then pickNewTarget  -- others will notice soon enough
-               else return $! returN "TSmell" tap
+               else return $ Just tap
           TUnknown ->
             let lvl2 = sdungeon s EM.! lid
                 t = lvl2 `at` pos
-            in if lidExplored
+            in if lexpl lvl2 <= lseen lvl2
                   || not (isUknownSpace t)
                   || condEnoughGear && tileAdj (isStairPos lid) pos
                        -- the unknown may be on the other side of the level
                        -- and getting there only to explore 1 tile and get back
                        -- looks silly
                then pickNewTarget  -- others will notice soon enough
-               else return $! returN "TUnknown" tap
-          TKnown ->  -- e.g., staircase or first unknown tile of an area
+               else return $ Just tap
+          TKnown -> do  -- e.g., staircase or first unknown tile of an area
+            explored <- getsClient sexplored
             let allExplored = ES.size explored == EM.size dungeon
                 lvl2 = sdungeon s EM.! lid
-            in if bpos b == pos
-                  || isStuck
-                  || alterSkill < fromEnum (lalter PointArray.! pos)
-                       -- tile was searched or altered or skill lowered
-                  || Tile.isWalkable coTileSpeedup (lvl2 `at` pos)
-                     && not allExplored  -- not patrolling explored dungeon
-                       -- tile is no longer unwalkable, so was explored
-                       -- so time to recalculate target
-               then pickNewTarget  -- others unconcerned
-               else return $! returN "TKnown" tap
+            if bpos b == pos
+               || isStuck
+               || alterSkill < fromEnum (lalter PointArray.! pos)
+                    -- tile was searched or altered or skill lowered
+               || Tile.isWalkable coTileSpeedup (lvl2 `at` pos)
+                  && not allExplored  -- not patrolling explored dungeon
+                    -- tile is no longer unwalkable, so was explored
+                    -- so time to recalculate target
+            then pickNewTarget  -- others unconcerned
+            else return $ Just tap
           TAny -> pickNewTarget  -- reset elsewhere or carried over from UI
         TVector{} -> if pathLen > 1
-                     then return $! returN "TVector" tap
+                     then return $ Just tap
                      else pickNewTarget
   if canMove
   then case oldTgtUpdatedPath of
     Nothing -> pickNewTarget
     Just tap -> updateTgt tap
-  else return $! returN "NoMove" $ TgtAndPath (TEnemy aid True) NoPath
+  else return $ Just $ TgtAndPath (TEnemy aid True) NoPath
