@@ -30,7 +30,7 @@ import           Game.LambdaHack.Common.Actor
 import           Game.LambdaHack.Common.ActorState
 import           Game.LambdaHack.Common.Faction
 import           Game.LambdaHack.Common.Item
-import qualified Game.LambdaHack.Common.Kind as Kind
+import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Level
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.MonadStateRead
@@ -80,7 +80,7 @@ updatePathFromBfs :: MonadClient m
                   => Bool -> BfsAndPath -> ActorId -> Point
                   -> m (PointArray.Array BfsDistance, AndPath)
 updatePathFromBfs canMove bfsAndPathOld aid !target = do
-  Kind.COps{coTileSpeedup} <- getsState scops
+  COps{coTileSpeedup} <- getsState scops
   let (oldBfsArr, oldBfsPath) = case bfsAndPathOld of
         BfsAndPath{bfsArr, bfsPath} -> (bfsArr, bfsPath)
         BfsInvalid -> error $ "" `showFailure` (bfsAndPathOld, aid, target)
@@ -145,7 +145,7 @@ getCachePath aid target = do
 
 createPath :: MonadClient m => ActorId -> Target -> m TgtAndPath
 createPath aid tapTgt = do
-  Kind.COps{coTileSpeedup} <- getsState scops
+  COps{coTileSpeedup} <- getsState scops
   b <- getsState $ getActorBody aid
   lvl <- getLevel $ blid b
   let stopAtUnwalkable tapPath@AndPath{..} =
@@ -164,7 +164,7 @@ createPath aid tapTgt = do
                                  , pathLen = length walkable + 1 }
             in TgtAndPath{tapTgt = newTgt, tapPath = newPath}
       stopAtUnwalkable tapPath@NoPath = TgtAndPath{..}
-  mpos <- aidTgtToPos aid (blid b) tapTgt
+  mpos <- getsState $ aidTgtToPos aid (blid b) tapTgt
   case mpos of
     Nothing -> return TgtAndPath{tapTgt, tapPath=NoPath}
     Just p -> do
@@ -230,28 +230,26 @@ closestUnknown aid = do
   bfs <- getCacheBfs aid
   let closestPoss = PointArray.minIndexesA bfs
       dist = bfs PointArray.! head closestPoss
-      !_A = assert (lexplorable lvl >= lseen lvl) ()
-  if lexplorable lvl <= lseen lvl
-       -- Some unknown may still be visible and even pathable, but we already
-       -- know from global level info that they are blocked.
-     || dist >= apartBfs
-       -- Global level info may tell us that terrain was changed and so
-       -- some new explorable tile appeared, but we don't care about those
-       -- and we know we already explored all initially seen unknown tiles
-       -- and it's enough for us (otherwise we'd need to hunt all around
-       -- the map for tiles altered by enemies).
-  then do
-    modifyClient $ \cli ->
-      cli {sexplored = ES.insert (blid body) (sexplored cli)}
-    return Nothing
-  else do
-    let unknownAround pos =
-          let vic = vicinityUnsafe pos
-              countUnknown :: Int -> Point -> Int
-              countUnknown c p = if isUknownSpace $ lvl `at` p then c + 1 else c
-          in foldl' countUnknown 0 vic
-        cmp = comparing unknownAround
-    return $ Just $ maximumBy cmp closestPoss
+      !_A = assert (lexpl lvl >= lseen lvl) ()
+  return $!
+    if lexpl lvl <= lseen lvl
+         -- Some unknown may still be visible and even pathable, but we already
+         -- know from global level info that they are blocked.
+       || dist >= apartBfs
+         -- Global level info may tell us that terrain was changed and so
+         -- some new explorable tile appeared, but we don't care about those
+         -- and we know we already explored all initially seen unknown tiles
+         -- and it's enough for us (otherwise we'd need to hunt all around
+         -- the map for tiles altered by enemies).
+    then Nothing
+    else let unknownAround pos =
+               let vic = vicinityUnsafe pos
+                   countUnknown :: Int -> Point -> Int
+                   countUnknown c p =
+                     if isUknownSpace $ lvl `at` p then c + 1 else c
+               in foldl' countUnknown 0 vic
+             cmp = comparing unknownAround
+         in Just $ maximumBy cmp closestPoss
 
 -- | Finds smells closest to the actor, except under the actor,
 -- because actors consume smell only moving over them, not standing.
@@ -278,7 +276,7 @@ embedBenefit :: MonadClient m
              -> [(Point, ItemBag)]
              -> m [(Double, (Point, ItemBag))]
 embedBenefit fleeVia aid pbags = do
-  Kind.COps{coitem=Kind.Ops{okind}, coTileSpeedup} <- getsState scops
+  COps{coTileSpeedup} <- getsState scops
   dungeon <- getsState sdungeon
   explored <- getsClient sexplored
   b <- getsState $ getActorBody aid
@@ -291,24 +289,20 @@ embedBenefit fleeVia aid pbags = do
   unexploredTrue <- unexploredDepth True (blid b)
   unexploredFalse <- unexploredDepth False (blid b)
   condEnoughGear <- condEnoughGearM aid
-  discoKind <- getsState sdiscoKind
   discoBenefit <- getsClient sdiscoBenefit
-  s <- getState
+  getKind <- getsState $ flip getIidKind
   let alterMinSkill p = Tile.alterMinSkill coTileSpeedup $ lvl `at` p
       lidExplored = ES.member (blid b) explored
       allExplored = ES.size explored == EM.size dungeon
       -- Ignoring the number of items, because only one of each @iid@
       -- is triggered at the same time, others are left to be used later on.
-      iidToEffs iid = case EM.lookup (jkindIx $ getItemBody iid s) discoKind of
-        Nothing -> []
-        Just KindMean{kmKind} -> IK.ieffects $ okind kmKind
-      isEffEscapeOrAscend IK.Ascend{} = True
-      isEffEscapeOrAscend IK.Escape{} = True
-      isEffEscapeOrAscend _ = False
+      -- Taking the kind the item hides under into consideration, because
+      -- it's a best guess only, for AI and UI.
+      iidToEffs iid = IK.ieffects $ getKind iid
       feats bag = concatMap iidToEffs $ EM.keys bag
       -- For simplicity, we assume at most one exit at each position.
       -- AI uses exit regardless of traps or treasures at the spot.
-      bens (_, bag) = case find isEffEscapeOrAscend $ feats bag of
+      bens (_, bag) = case find IK.isEffEscapeOrAscend $ feats bag of
         Just IK.Escape{} ->
           -- Escape (or guard) only after exploring, for high score, etc.
           let escapeOrGuard = fcanEscape (gplayer fact)
@@ -342,8 +336,7 @@ embedBenefit fleeVia aid pbags = do
                -- Let distance be the deciding factor and also prevent
                -- overflow on 32-bit machines.
                min 1000 $ sum
-               $ mapMaybe (\iid -> benApply <$> EM.lookup iid discoBenefit)
-                          (EM.keys bag)
+               $ map (\iid -> benApply $ discoBenefit EM.! iid) (EM.keys bag)
           else 0
       interestingHere p =
         -- For speed and to avoid greedy AI loops, filter targets.
@@ -378,8 +371,7 @@ closestTriggers fleeVia aid = do
       vicAll = concatMap vicTrigger efeat
   return $  -- keep lazy
     let mix (benefit, ppbag) dist =
-          let maxd = fromEnum (maxBound :: BfsDistance)
-                     - fromEnum apartBfs
+          let maxd = fromEnum maxBfsDistance - fromEnum apartBfs
               v = (fromIntegral maxd * 10) / (fromIntegral dist + 1)
           in (ceiling $ benefit * v, ppbag)
     in mapMaybe (\bpp@(_, (p, _)) ->
@@ -397,11 +389,11 @@ condEnoughGearM aid = do
   b <- getsState $ getActorBody aid
   fact <- getsState $ (EM.! bfid b) . sfactionD
   let followTactic = ftactic (gplayer fact) `elem` [TFollow, TFollowNoItems]
-  eqpAssocs <- getsState $ getActorAssocs aid CEqp
+  eqpAssocs <- getsState $ fullAssocs aid [CEqp]
   invAssocs <- getsState $ getActorAssocs aid CInv
   return $ not followTactic  -- keep it lazy
-           && (any (isMelee . snd) eqpAssocs
-               || length (eqpAssocs ++ invAssocs) >= 5)
+           && (any (IK.isMelee . itemKind . snd) eqpAssocs
+               || length eqpAssocs + length invAssocs >= 5)
 
 unexploredDepth :: MonadClient m => Bool -> LevelId -> m Bool
 unexploredDepth !up !lidCurrent = do
@@ -427,8 +419,7 @@ closestItems aid = do
     if EM.null lfloor then return [] else do
       bfs <- getCacheBfs aid
       let mix pbag dist =
-            let maxd = fromEnum (maxBound :: BfsDistance)
-                       - fromEnum apartBfs
+            let maxd = fromEnum maxBfsDistance - fromEnum apartBfs
                 -- Bewqre of overflowing 32-bit integers here.
                 -- Here distance is the only factor influencing frequency,
                 -- unless item not desirable, which is checked later on.
