@@ -17,7 +17,6 @@ import Game.LambdaHack.Common.Prelude
 import qualified Control.Monad.Trans.State.Strict as St
 import qualified Data.Text.IO as T
 import           System.IO (hFlush, stdout)
-import qualified System.Random as R
 
 import Game.LambdaHack.Client.ClientOptions
 import Game.LambdaHack.Client.State
@@ -48,10 +47,10 @@ debugPossiblyPrint t = do
 -- | Invoke pseudo-random computation with the generator kept in the state.
 rndToAction :: MonadClient m => Rnd a -> m a
 rndToAction r = do
-  gen <- getsClient srandom
-  let (gen1, gen2) = R.split gen
-  modifyClient $ \cli -> cli {srandom = gen1}
-  return $! St.evalState r gen2
+  gen1 <- getsClient srandom
+  let (a, gen2) = St.runState r gen1
+  modifyClient $ \cli -> cli {srandom = gen2}
+  return a
 
 -- | Invoke pseudo-random computation, don't change generator kept in state.
 rndToActionForget :: MonadClient m => Rnd a -> m a
