@@ -16,10 +16,9 @@ import Game.LambdaHack.Common.Prelude
 import qualified Game.LambdaHack.Common.Dice as Dice
 import           Game.LambdaHack.Common.Faction
 import           Game.LambdaHack.Common.Flavour
-import           Game.LambdaHack.Common.Frequency
 import           Game.LambdaHack.Common.Item
-import           Game.LambdaHack.Common.ItemStrongest
-import qualified Game.LambdaHack.Common.Kind as Kind
+import qualified Game.LambdaHack.Common.ItemAspect as IA
+import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.Time
 import           Game.LambdaHack.Content.ItemKind (ItemKind)
@@ -36,18 +35,32 @@ import           Game.LambdaHack.Content.ModeKind
 -- nothing better to do than to melee, or when the actor is stuck or idle
 -- or laying in wait or luring an enemy from a safe distance.
 -- So there is less than @averageTurnValue@ included in each benefit,
--- so in case when turn is not spent, e.g, periodic or temporary effects,
+-- so in case when turn is not spent, e.g, periodic or conditions,
 -- the difference in value is only slight.
-effectToBenefit :: Kind.COps -> Faction -> IK.Effect -> (Double, Double)
-effectToBenefit cops fact eff =
+effectToBenefit :: COps -> Faction -> Bool -> IK.Effect -> (Double, Double)
+effectToBenefit cops fact insideRecharging eff =
   let delta x = (x, x)
   in case eff of
-    IK.ELabel _ -> delta 0
-    IK.EqpSlot _ -> delta 0
     IK.Burn d -> delta $ -(min 1500 $ 15 * Dice.meanDice d)
       -- often splash damage, armor doesn't block (but HurtMelee doesn't boost)
-    IK.Explode _ -> delta 1  -- depends on explosion, but usually good,
-                             -- unless under OnSmash, but they are ignored
+    IK.Explode _ | insideRecharging ->
+      -- It's too hard to analyze, so we assume, explosion inside recharging
+      -- is never a cruel and cheap one-time trap, damaging HP of the actor
+      -- before he identifies the item and stops wearing it or meleeing with it.
+      -- So the explosion is either both focused and beneficial to self
+      -- or is not focused and so not affecting self. In either case
+      -- it can be good or bad for nearby friends and foes and, regardless,
+      -- AI chooses to equip that item, for fun and to challenge human player
+      -- with varied situations.
+      ( 1     -- equip, but not too greedily, in case it mostly harms friends
+      , -1 )  -- hit with it or throw, but beware of harming friends
+    IK.Explode "single spark" -> delta (-1)  -- hardwired; probing and flavour
+    IK.Explode _ ->
+      -- We know this explosion is not wrapped with @Recharging@ nor @OnSmash@
+      -- so we assume it's focused and very harmful and so only safe
+      -- for projecting at foes. Due to this assumption healing explosives
+      -- should be wrapped to avoid throwing them at foes.
+      delta (-100)
     IK.RefillHP p ->
       delta $ if p > 0
               then min 2000 (20 * fromIntegral p)
@@ -61,11 +74,13 @@ effectToBenefit cops fact eff =
                                else max (-9) (fromIntegral p)
     IK.Dominate -> (0, -300)  -- I obtained an actor with, say 10HP,
                               -- worth 200, and enemy lost him, another 100
-    IK.Impress -> (5, -50)  -- usually has no effect on self, hence low value
+    IK.Impress -> (10, -50)  -- can affect friends, but more often enemies,
+                             -- hence it's considered harmful when thrown
     IK.Summon grp d ->  -- contrived by not taking into account alliances
                         -- and not checking if enemies also control that group
       let ben = Dice.meanDice d * 200  -- the new actor can have, say, 10HP
-      in if grp `elem` fgroups (gplayer fact) then (ben, -ben) else (-ben, ben)
+      in if grp `elem` fgroups (gplayer fact) then (ben, -1) else (-ben, 1)
+        -- prefer applying to flinging summoning items; further, but more robust
     IK.Ascend{} -> (-99, 99)  -- note the reversed values:
                               -- only change levels sensibly, in teams,
                               -- and don't remove enemy too far, he may be
@@ -84,28 +99,28 @@ effectToBenefit cops fact eff =
                      then (1, 0)     -- blink to shoot at foes
                      else (-9, -1)  -- for self, don't derail exploration
                                     -- for foes, fight with one less at a time
-    IK.CreateItem COrgan "temporary condition" _ ->
+    IK.CreateItem COrgan "condition" _ ->
       (1, -1)  -- varied, big bunch, but try to create it anyway
     IK.CreateItem COrgan grp timer ->  -- assumed temporary
-      let turnTimer = case timer of
-            IK.TimerNone -> averageTurnValue + 1  -- copy count used instead
-            IK.TimerGameTurn n -> Dice.meanDice n
-            IK.TimerActorTurn n -> Dice.meanDice n
+      let noneResult = averageTurnValue + 1  -- copy count used instead
+          turnTimer = IK.foldTimer noneResult Dice.meanDice Dice.meanDice timer
           (total, count) = organBenefit turnTimer grp cops fact
       in delta $ total / fromIntegral count
            -- the same when created in me and in foe
            -- average over all matching grps; simplified: rarities ignored
     IK.CreateItem _ "treasure" _ -> (100, 0)  -- assumed not temporary
-    IK.CreateItem _ "useful" _ -> (70, 0)
+    IK.CreateItem _ "common item" _ -> (70, 0)
+    IK.CreateItem _ "curious item" _ -> (70, 0)
     IK.CreateItem _ "any scroll" _ -> (50, 0)
     IK.CreateItem _ "any vial" _ -> (50, 0)
     IK.CreateItem _ "potion" _ -> (50, 0)
-    IK.CreateItem _ "flask" _ -> (50, 0)
+    IK.CreateItem _ "explosive" _ -> (50, 0)
+    IK.CreateItem _ "any jewelry" _ -> (100, 0)
     IK.CreateItem _ grp _ ->  -- assumed not temporary and @grp@ tiny
       let (total, count) = recBenefit grp cops fact
       in (total / fromIntegral count, 0)
-    IK.DropItem _ _ COrgan "temporary condition" ->
-      (1, -1)  -- varied, big bunch, but try to nullify it anyway
+    IK.DropItem _ _ COrgan "condition" ->
+      delta 30  -- save for curing own bad conditions
     IK.DropItem ngroup kcopy COrgan grp ->  -- assumed temporary
       -- Simplified: we assume actor has an average number of copies
       -- (and none have yet run out, e.g., prompt curing of poisoning)
@@ -120,22 +135,19 @@ effectToBenefit cops fact eff =
                  - total / fromIntegral count
                    -- the same when dropped from me and foe
     IK.DropItem{} -> delta (-10)  -- depends a lot on what is dropped
-    IK.PolyItem -> delta 0  -- AI can't estimate item desirability vs average
-    IK.Identify -> delta 0  -- AI doesn't know how to use
-    IK.Detect radius -> (fromIntegral radius * 2, 0)
-    IK.DetectActor radius -> (fromIntegral radius, 0)
-    IK.DetectItem radius -> (fromIntegral radius, 0)
-    IK.DetectExit radius -> (fromIntegral radius, 0)
-    IK.DetectHidden radius -> (fromIntegral radius, 0)
-    IK.SendFlying _ -> (1, -10)  -- very context dependent, but it's better
-    IK.PushActor _ -> (1, -10)   -- to be the one that decides and not the one
-    IK.PullActor _ -> (1, -10)   -- that is interrupted in the middle of fleeing
+    IK.PolyItem -> (1, 0)  -- may fizzle, so AI never uses (could loop)
+    IK.Identify -> (1, 0)  -- may fizzle, so AI never uses (could loop)
+    IK.Detect IK.DetectAll radius -> (fromIntegral radius * 2, 0)
+    IK.Detect _ radius -> (fromIntegral radius, 0)
+    IK.SendFlying _ -> (1, -100)  -- very context dependent, but it's better
+    IK.PushActor _ -> (1, -100)   -- to be the one that decides whether to fly;
+    IK.PullActor _ -> (1, -100)   -- pushing others may crush them against wall
     IK.DropBestWeapon -> delta $ -50  -- often a whole turn wasted == InsertMove
     IK.ActivateInv ' ' -> delta $ -200  -- brutal and deadly
     IK.ActivateInv _ -> delta $ -50  -- depends on the items
     IK.ApplyPerfume -> delta 0  -- depends on smell sense of friends and foes
     IK.OneOf efs ->
-      let bs = map (effectToBenefit cops fact) efs
+      let bs = map (effectToBenefit cops fact insideRecharging) efs
           f (self, foe) (accSelf, accFoe) = (self + accSelf, foe + accFoe)
           (effSelf, effFoe) = foldr f (0, 0) bs
       in (effSelf / fromIntegral (length bs), effFoe / fromIntegral (length bs))
@@ -143,10 +155,8 @@ effectToBenefit cops fact eff =
       -- can be beneficial; we'd need to analyze explosions, range, etc.
     IK.Recharging _ -> delta 0  -- taken into account separately
     IK.Temporary _ -> delta 0  -- assumed for created organs only
-    IK.Unique -> delta 0
-    IK.Periodic -> delta 0  -- considered in totalUsefulness
     IK.Composite [] -> delta 0
-    IK.Composite (eff1 : _) -> effectToBenefit cops fact eff1
+    IK.Composite (eff1 : _) -> effectToBenefit cops fact insideRecharging eff1
       -- for simplicity; so in content make sure to place initial animations
       -- among normal effects, not at the start of composite effect
       -- (animations should not fail, after all), and start composite
@@ -202,10 +212,10 @@ durabilityMult = avgItemLife / avgItemDelay
 -- effects, knowing in some temporary organs, e.g., poison or regeneration,
 -- they are triggered at each item copy destruction. They are applied to self,
 -- hence we take the self component of valuation. We multiply by the count
--- of created/dropped organs, because for temporary effects it determines
+-- of created/dropped organs, because for conditions it determines
 -- how many times the effect is applied, before the last copy expires.
 --
--- The organs are not durable nor in infnite copies, so to give
+-- The temporary organs are not durable nor in infnite copies, so to give
 -- continous benefit, organ has to be recreated each @turnTimer@ turns.
 -- Creation takes a turn, so incurs @averageTurnValue@ cost.
 -- That's how the lack of durability impacts their value, not via
@@ -226,41 +236,44 @@ durabilityMult = avgItemLife / avgItemDelay
 -- are applied and we can't stop/restart them.
 --
 -- We assume, only one of timer and count mechanisms is present at once.
-organBenefit :: Double -> GroupName ItemKind -> Kind.COps -> Faction
+-- We assume no organ has effect that drops its group or creates its group;
+-- otherwise we'd loop.
+organBenefit :: Double -> GroupName ItemKind -> COps -> Faction
              -> (Double, Int)
-organBenefit turnTimer grp cops@Kind.COps{coitem=Kind.Ops{ofoldlGroup'}} fact =
+organBenefit turnTimer grp cops@COps{coitem} fact =
   let f (!sacc, !pacc) !p _ !kind =
         let paspect asp = fromIntegral p * aspectToBenefit asp
-            peffect eff = fromIntegral p * fst (effectToBenefit cops fact eff)
+            peffect eff = fromIntegral p
+                          * fst (effectToBenefit cops fact False eff)
         in ( sacc + Dice.meanDice (IK.icount kind)
                     * (sum (map paspect $ IK.iaspects kind)
-                       + sum (map peffect $ stripRecharging $ IK.ieffects kind))
+                       + sum (map peffect
+                              $ IK.stripRecharging $ IK.ieffects kind))
                   - averageTurnValue / turnTimer
            , pacc + p )
-  in ofoldlGroup' grp f (0, 0)
+  in ofoldlGroup' coitem grp f (0, 0)
 
-recBenefit :: GroupName ItemKind -> Kind.COps -> Faction -> (Double, Int)
-recBenefit grp cops@Kind.COps{coitem=Kind.Ops{ofoldlGroup'}} fact =
-  let f (!sacc, !pacc) !p _ !kind =
-        let recPickup = benPickup $
-              totalUsefulness cops fact (IK.ieffects kind)
-                                        (meanAspect kind) (fakeItem kind)
+-- We assume no item has effect that drops its group or creates its group;
+-- otherwise we'd loop.
+recBenefit :: GroupName ItemKind -> COps -> Faction -> (Double, Int)
+recBenefit grp cops@COps{coitem, coItemSpeedup} fact =
+  let f (!sacc, !pacc) !p !kindId !kind =
+        let km = IK.getKindMean kindId coItemSpeedup
+            recPickup =
+              benPickup $ totalUsefulness cops fact (fakeItem kindId kind km)
         in ( sacc + Dice.meanDice (IK.icount kind) * recPickup
            , pacc + p )
-  in ofoldlGroup' grp f (0, 0)
+  in ofoldlGroup' coitem grp f (0, 0)
 
-fakeItem :: IK.ItemKind -> Item
-fakeItem kind =
-  let jkindIx  = toEnum (-1)  -- dummy
-      jlid     = toEnum 0  -- dummy
+fakeItem :: ContentId IK.ItemKind -> IK.ItemKind -> IA.KindMean -> ItemFull
+fakeItem kindId kind km =
+  let jkind    = IdentityObvious kindId
+      jlid     = toEnum 1  -- dummy
       jfid     = Nothing  -- the default
-      jsymbol  = IK.isymbol kind
-      jname    = IK.iname kind
-      jflavour = head stdFlav  -- dummy
-      jfeature = IK.ifeature kind
-      jweight  = IK.iweight kind
-      jdamage  = fromJust $ mostFreq $ toFreq "fakeItem" $ IK.idamage kind
-  in Item{..}
+      jflavour = Flavour minBound minBound  -- dummy
+      itemBase = Item{..}
+      itemDisco = ItemDiscoMean km
+  in ItemFull itemBase kindId kind itemDisco True
 
 -- Value of aspects and effects is linked by some deep economic principles
 -- which I'm unfortunately ignorant of. E.g., average weapon hits for 5HP,
@@ -282,38 +295,39 @@ fakeItem kind =
 -- Valuation of effects, and more precisely, more the signs than absolute
 -- values, ensures that both shield and torch get picked up so that
 -- the (human) actor can nevertheless equip them in very special cases.
-aspectToBenefit :: IK.Aspect -> Double
+aspectToBenefit :: IA.Aspect -> Double
 aspectToBenefit asp =
   case asp of
-    IK.Timeout{} -> 0
-    IK.AddHurtMelee p -> Dice.meanDice p  -- offence favoured
-    IK.AddArmorMelee p -> Dice.meanDice p / 4  -- only partial protection
-    IK.AddArmorRanged p -> Dice.meanDice p / 8
-    IK.AddMaxHP p -> Dice.meanDice p
-    IK.AddMaxCalm p -> Dice.meanDice p / 5
-    IK.AddSpeed p -> Dice.meanDice p * 25
+    IA.Timeout{} -> 0
+    IA.AddHurtMelee p -> Dice.meanDice p  -- offence favoured
+    IA.AddArmorMelee p -> Dice.meanDice p / 4  -- only partial protection
+    IA.AddArmorRanged p -> Dice.meanDice p / 8
+    IA.AddMaxHP p -> Dice.meanDice p
+    IA.AddMaxCalm p -> Dice.meanDice p / 5
+    IA.AddSpeed p -> Dice.meanDice p * 25
       -- 1 speed ~ 5% melee; times 5 for no caps, escape, pillar-dancing, etc.;
       -- also, it's 1 extra turn each 20 turns, so 100/20, so 5; figures
-    IK.AddSight p -> Dice.meanDice p * 5
-    IK.AddSmell p -> Dice.meanDice p
-    IK.AddShine p -> Dice.meanDice p * 2
-    IK.AddNocto p -> Dice.meanDice p * 10  -- > sight + light; stealth, slots
-    IK.AddAggression{} -> 0
-    IK.AddAbility _ p -> Dice.meanDice p * 5
+    IA.AddSight p -> Dice.meanDice p * 5
+    IA.AddSmell p -> Dice.meanDice p
+    IA.AddShine p -> Dice.meanDice p * 2
+    IA.AddNocto p -> Dice.meanDice p * 10  -- > sight + light; stealth, slots
+    IA.AddAggression{} -> 0
+    IA.AddAbility _ p -> Dice.meanDice p * 5
 
-recordToBenefit :: AspectRecord -> [Double]
-recordToBenefit aspects = map aspectToBenefit $ aspectRecordToList aspects
+recordToBenefit :: IA.AspectRecord -> [Double]
+recordToBenefit aspects = map aspectToBenefit $ IA.aspectRecordToList aspects
 
 -- | Compute the whole 'Benefit' structure, containing various facets
 -- of AI item preference, for an item with the given effects and aspects.
 --
 -- Note: result has non-strict fields, so arguments are forced to avoid leaks.
 -- When AI looks at items (including organs) more often, force the fields.
-totalUsefulness :: Kind.COps -> Faction -> [IK.Effect] -> AspectRecord -> Item
-                -> Benefit
-totalUsefulness !cops !fact !effects !aspects !item =
-  let effPairs = map (effectToBenefit cops fact) effects
-      effDice = - damageUsefulness item
+totalUsefulness :: COps -> Faction -> ItemFull -> Benefit
+totalUsefulness !cops !fact itemFull@ItemFull{itemKind, itemSuspect} =
+  let effects = IK.ieffects itemKind
+      aspects = aspectRecordFull itemFull
+      effPairs = map (effectToBenefit cops fact False) effects
+      effDice = - IK.damageUsefulness itemKind
       f (self, foe) (accSelf, accFoe) = (self + accSelf, foe + accFoe)
       (effSelf, effFoe) = foldr f (0, 0) effPairs
       -- Timeout between 0 and 1 means item usable each turn, so we consider
@@ -322,7 +336,7 @@ totalUsefulness !cops !fact !effects !aspects !item =
       -- so a single such item may be worth half of the permanet value.
       -- Hence, we multiply item value by the proportion of the average desired
       -- delay between item uses @avgItemDelay@ and the actual timeout.
-      timeout = aTimeout aspects
+      timeout = IA.aTimeout aspects
       (chargeSelf, chargeFoe) =
         let scaleChargeBens bens
               | timeout <= 3 = bens
@@ -330,8 +344,8 @@ totalUsefulness !cops !fact !effects !aspects !item =
                   if avgItemDelay >= fromIntegral timeout
                   then eff
                   else eff * avgItemDelay / fromIntegral timeout) bens
-            (cself, cfoe) = unzip $ map (effectToBenefit cops fact)
-                                        (stripRecharging effects)
+            (cself, cfoe) = unzip $ map (effectToBenefit cops fact True)
+                                        (IK.stripRecharging effects)
         in (scaleChargeBens cself, scaleChargeBens cfoe)
       -- If the item is periodic, we add charging effects to equipment benefit,
       -- but we don't assign periodic bonus or malus, because periodic items
@@ -345,7 +359,7 @@ totalUsefulness !cops !fact !effects !aspects !item =
       -- or nearby detection is better, but infrequent periodic teleportation
       -- or harmful explosion is worse. But the rule is not strict and also
       -- dependent on gameplay context of the moment, hence no numerical value.
-      periodic = IK.Periodic `elem` effects
+      periodic = IK.Periodic `elem` IK.ifeature itemKind
       -- Durability doesn't have any numerical impact to @eqpSum,
       -- because item is never consumed by just being stored in equipment.
       -- Also no numerical impact for flinging, because we can't fling it again
@@ -359,10 +373,10 @@ totalUsefulness !cops !fact !effects !aspects !item =
       -- and save the option for using non-durable item for the future, e.g.,
       -- when both items have timeouts, starting with durable is beneficial,
       -- because it recharges while the non-durable is prepared and used.
-      durable = IK.Durable `elem` jfeature item
+      durable = IK.Durable `elem` IK.ifeature itemKind
       -- If recharging effects not periodic, we add the self part,
       -- because they are applied to self. If they are periodic we can't
-      -- effectively apply them, becasue they are never recharged,
+      -- effectively apply them, because they are never recharged,
       -- because they activate as soon as recharged.
       benApply = max 0 $  -- because optional; I don't need to apply
         (effSelf + effDice  -- hits self with dice too, when applying
@@ -373,23 +387,27 @@ totalUsefulness !cops !fact !effects !aspects !item =
         (effFoe + effDice  -- @AddHurtMelee@ already in @eqpSum@
          + if periodic then 0 else sum chargeFoe)
         / if durable then 1 else durabilityMult
+      -- Experimenting is fun, but it's better to risk foes' skin than ours,
+      -- so we only adjust flinging bonus, not apply bonus. It's also more
+      -- fun gameplay-wise when enemies throw at us rather than using items.
+      benFling = min benFlingRaw $ if itemSuspect then -10 else 0
       -- The periodic effects, if any, are activated when projectile flies,
       -- but not when it hits, so they are not added to @benFling@.
       -- However, if item is not periodic, the recharging effects
       -- are activated at projectile impact, hence their value is added.
-      benFling = min 0 $
+      benFlingRaw = min 0 $
         effFoe + benFlingDice -- nothing in @eqpSum@; normally not worn
         + if periodic then 0 else sum chargeFoe
-      benFlingDice | jdamage item == 0 = 0  -- speedup
+      benFlingDice | IK.idamage itemKind == 0 = 0  -- speedup
                    | otherwise = assert (v <= 0) v
        where
-        hurtMult = 100 + min 99 (max (-99) (aHurtMelee aspects))
+        hurtMult = 100 + min 99 (max (-99) (IA.aHurtMelee aspects))
           -- assumes no enemy armor and no block
-        dmg = Dice.meanDice (jdamage item)
+        dmg = Dice.meanDice $ IK.idamage itemKind
         rawDeltaHP = ceiling $ fromIntegral hurtMult * xD dmg / 100
         -- For simplicity, we ignore range bonus/malus and @Lobable@.
-        IK.ThrowMod{IK.throwVelocity} = strengthToThrow item
-        speed = speedFromWeight (jweight item) throwVelocity
+        IK.ThrowMod{IK.throwVelocity} = IK.getToThrow itemKind
+        speed = speedFromWeight (IK.iweight itemKind) throwVelocity
         v = - fromIntegral (modifyDamageBySpeed rawDeltaHP speed) * 10 / xD 1
           -- 1 damage valued at 10, just as in @damageUsefulness@
       -- For equipment benefit, we take into account only the self
@@ -412,14 +430,18 @@ totalUsefulness !cops !fact !effects !aspects !item =
       -- If a weapon heals enemy at impact, it won't be used for melee
       -- (but can be equipped anyway). If it harms wearer too much,
       -- won't be worn but still may be flung, etc.
-      (benInEqp, benPickup)
-        | isMelee item && benMelee < 0 && eqpSum >= -20 =
+      (benInEqp, benPickupRaw)
+        | IK.isMelee itemKind  -- probably known even if not identified
+          && (benMelee < 0 || itemSuspect)
+          && eqpSum >= -20 =
           ( True  -- equip, melee crucial, and only weapons in eqp can be used
           , if durable
             then eqpSum
                  + max benApply (- benMelee)  -- apply or melee or not
             else - benMelee)  -- melee is predominant
-        | goesIntoEqp item && eqpSum > 0 =  -- weapon or other equippable
+        | (IK.goesIntoEqp itemKind
+          || IK.isTmpCondition itemKind)  -- hack to record benefit
+          && (eqpSum > 0 || itemSuspect) =  -- weapon or other equippable
           ( True  -- equip; long time bonus usually outweighs fling or apply
           , eqpSum  -- possibly spent turn equipping, so reap the benefits
             + if durable
@@ -427,4 +449,5 @@ totalUsefulness !cops !fact !effects !aspects !item =
               else 0)  -- don't remove from equipment by using up
         | otherwise =
           (False, max benApply (- benFling))  -- apply or fling
+      benPickup = max benPickupRaw $ if itemSuspect then 10 else 0
   in Benefit{..}
