@@ -11,6 +11,7 @@ import Prelude ()
 import Game.LambdaHack.Common.Prelude
 
 import           Data.Binary
+import qualified Data.EnumMap.Lazy as LEM
 import qualified Data.EnumMap.Strict as EM
 import qualified Data.EnumSet as ES
 import qualified Data.Map.Strict as M
@@ -24,7 +25,7 @@ import           Game.LambdaHack.Common.Actor
 import           Game.LambdaHack.Common.ActorState
 import           Game.LambdaHack.Common.Faction
 import           Game.LambdaHack.Common.Item
-import qualified Game.LambdaHack.Common.Kind as Kind
+import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Level
 import           Game.LambdaHack.Common.Perception
 import           Game.LambdaHack.Common.Point
@@ -37,13 +38,16 @@ data StateClient = StateClient
   { seps          :: Int            -- ^ a parameter of the aiming digital line
   , stargetD      :: EM.EnumMap ActorId TgtAndPath
                                     -- ^ targets of our actors in the dungeon
+  , sfleeD        :: EM.EnumMap ActorId Point
+                                    -- ^ the position when fleeing requested
   , sexplored     :: ES.EnumSet LevelId
                                     -- ^ the set of fully explored levels
   , sbfsD         :: EM.EnumMap ActorId BfsAndPath
                                     -- ^ pathfinding data for our actors
   , sundo         :: [CmdAtomic]    -- ^ atomic commands performed to date
   , sdiscoBenefit :: DiscoveryBenefit
-                                    -- ^ remembered AI benefits of items
+      -- ^ remembered AI benefits of items; could be recomputed at resume,
+      --   but they are costly to generate and not too large
   , sfper         :: PerLid         -- ^ faction perception indexed by level
   , salter        :: AlterLid       -- ^ cached alter ability data for positions
   , srandom       :: R.StdGen       -- ^ current random generator
@@ -55,9 +59,9 @@ data StateClient = StateClient
   , snxtChal      :: Challenge      -- ^ next game challenge setup
   , snxtScenario  :: Int            -- ^ next game scenario number
   , smarkSuspect  :: Int            -- ^ whether to mark suspect features
-  , scondInMelee  :: EM.EnumMap LevelId (Maybe Bool)
-                                    -- ^ last in-melee condition for each level
-  , svictories    :: EM.EnumMap (Kind.Id ModeKind) (M.Map Challenge Int)
+  , scondInMelee  :: LEM.EnumMap LevelId Bool
+                                    -- ^ whether we are in melee, per level
+  , svictories    :: EM.EnumMap (ContentId ModeKind) (M.Map Challenge Int)
                                     -- ^ won games at particular difficulty lvls
   , soptions      :: ClientOptions  -- ^ client options
   }
@@ -86,6 +90,7 @@ emptyStateClient _sside =
   StateClient
     { seps = fromEnum _sside
     , stargetD = EM.empty
+    , sfleeD = EM.empty
     , sexplored = ES.empty
     , sbfsD = EM.empty
     , sundo = []
@@ -100,7 +105,7 @@ emptyStateClient _sside =
     , snxtChal = defaultChallenge
     , snxtScenario = 0
     , smarkSuspect = 1
-    , scondInMelee = EM.empty
+    , scondInMelee = LEM.empty
     , svictories = EM.empty
     , soptions = defClientOptions
     }
@@ -142,6 +147,7 @@ instance Binary StateClient where
   put StateClient{..} = do
     put seps
     put stargetD
+    put sfleeD
     put sexplored
     put sundo
     put sdiscoBenefit
@@ -154,13 +160,13 @@ instance Binary StateClient where
     put smarkSuspect
     put scondInMelee
     put svictories
-    put soptions
 #ifdef WITH_EXPENSIVE_ASSERTIONS
     put sfper
 #endif
   get = do
     seps <- get
     stargetD <- get
+    sfleeD <- get
     sexplored <- get
     sundo <- get
     sdiscoBenefit <- get
@@ -173,11 +179,11 @@ instance Binary StateClient where
     smarkSuspect <- get
     scondInMelee <- get
     svictories <- get
-    soptions <- get
     let sbfsD = EM.empty
         salter = EM.empty
         srandom = read g
         squit = False
+        soptions = defClientOptions
 #ifndef WITH_EXPENSIVE_ASSERTIONS
         sfper = EM.empty
 #else
