@@ -69,12 +69,14 @@ queryUI = do
       -- Regaining control of faction cancels --stopAfter*.
       modifyClient $ \cli ->
         cli {soptions = (soptions cli) { sstopAfterSeconds = Nothing
-                                         , sstopAfterFrames = Nothing }}
+                                       , sstopAfterFrames = Nothing }}
       return (ReqUIAutomate, Nothing)  -- stop AI
     else do
       -- As long as UI faction is under AI control, check, once per move,
       -- for benchmark game stop.
       stopAfterFrames <- getsClient $ sstopAfterFrames . soptions
+      bench <- getsClient $ sbenchmark . soptions
+      let exitCmd = if bench then ReqUIGameDropAndExit else ReqUIGameSaveAndExit
       case stopAfterFrames of
         Nothing -> do
           stopAfterSeconds <- getsClient $ sstopAfterSeconds . soptions
@@ -84,39 +86,48 @@ queryUI = do
               exit <- elapsedSessionTimeGT stopS
               if exit then do
                 tellAllClipPS
-                return (ReqUIGameExit, Nothing)  -- ask server to exit
+                return (exitCmd, Nothing)  -- ask server to exit
               else return (ReqUINop, Nothing)
         Just stopF -> do
           allNframes <- getsSession sallNframes
           gnframes <- getsSession snframes
           if allNframes + gnframes >= stopF then do
             tellAllClipPS
-            return (ReqUIGameExit, Nothing)  -- ask server to exit
+            return (exitCmd, Nothing)  -- ask server to exit
           else return (ReqUINop, Nothing)
   else do
     let mleader = gleader fact
         !_A = assert (isJust mleader) ()
     req <- humanCommand
     leader2 <- getLeaderUI
-    return (req, if mleader /= Just leader2 then Just leader2 else Nothing)
+    -- Don't send the leader switch to the server with these commands,
+    -- to avoid leader death at resume if his HP <= 0. That would violate
+    -- the principle that save and reload doesn't change game state.
+    let saveCmd cmd = case cmd of
+          ReqUIGameDropAndExit -> True
+          ReqUIGameSaveAndExit -> True
+          ReqUIGameSave -> True
+          _ -> False
+    return (req, if mleader /= Just leader2 && not (saveCmd req)
+                 then Just leader2
+                 else Nothing)
 
 -- | Let the human player issue commands until any command takes time.
 humanCommand :: forall m. MonadClientUI m => m ReqUI
 humanCommand = do
   modifySession $ \sess -> sess { slastLost = ES.empty
-                                , skeysHintMode = KeysHintAbsent }
+                                , shintMode = HintAbsent }
   let loop :: m ReqUI
       loop = do
-        report <- getsSession sreport
-        if nullReport report then do
-          -- Display keys sometimes, alternating with empty screen.
-          keysHintMode <- getsSession skeysHintMode
-          case keysHintMode of
-            KeysHintPresent -> promptMainKeys
-            KeysHintBlocked ->
-              modifySession $ \sess -> sess {skeysHintMode = KeysHintAbsent}
-            _ -> return ()
-        else modifySession $ \sess -> sess {skeysHintMode = KeysHintBlocked}
+        report <- getsSession $ newReport . shistory
+        hintMode <- getsSession shintMode
+        -- Hints are not considered non-empty reports.
+        modifySession $ \sess -> sess {sreportNull =
+          nullReport report || hintMode == HintShown}
+        case hintMode of
+          HintAbsent -> return ()
+          HintShown -> modifySession $ \sess -> sess {shintMode = HintWiped}
+          HintWiped -> modifySession $ \sess -> sess {shintMode = HintAbsent}
         slidesRaw <- reportToSlideshowKeep []
         over <- case unsnoc slidesRaw of
           Nothing -> return []
@@ -166,6 +177,6 @@ humanCommand = do
           Left Nothing -> loop
           Left (Just err) -> do
             stopPlayBack
-            promptAdd $ showFailError err
+            promptAdd1 $ showFailError err
             loop
   loop
