@@ -1,7 +1,7 @@
 -- | Screen frames and animations.
 module Game.LambdaHack.Client.UI.Animation
   ( Animation, renderAnim
-  , pushAndDelay, blinkColorActor, twirlSplash, blockHit, blockMiss
+  , pushAndDelay, blinkColorActor, twirlSplash, blockHit, blockMiss, subtleHit
   , deathBody, shortDeathBody, actorX, teleport, swapPlaces, fadeout
 #ifdef EXPOSE_INTERNAL
     -- * Internal operations
@@ -130,6 +130,16 @@ blockMiss poss = Animation $ map (mzipPairs poss)
   , (cSym Blue    '}', blank)
   ]
 
+-- | Attack that is subtle (e.g., damage dice 0).
+subtleHit :: Point -> Animation
+subtleHit pos = Animation $ map (mzipSingleton pos)
+  [ cSym BrCyan '\''
+  , cSym BrYellow '\''
+  , cSym BrYellow '^'
+  , cSym BrCyan '^'
+  , cSym BrCyan '\''
+  ]
+
 -- | Death animation for an organic body.
 deathBody :: Point -> Animation
 deathBody pos = Animation $ map (mzipSingleton pos)
@@ -230,4 +240,5 @@ fadeout out step lxsize lysize = do
         return $! concatMap fadeLine [0..ybound]
       fs | out = [3, 3 + step .. lxsize - 14]
          | otherwise = [lxsize - 14, lxsize - 14 - step .. 1]
+                       ++ [0]  -- no remnants of fadein onscreen, in case of lag
   Animation <$> mapM rollFrame fs
