@@ -24,6 +24,7 @@ import qualified Data.Text as T
 import qualified Data.Vector.Unboxed as U
 import qualified Data.Vector.Unboxed.Mutable as VM
 import           Data.Word (Word16)
+import           Game.LambdaHack.Client.UI.UIOptions
 import           GHC.Exts (inline)
 import qualified NLP.Miniutter.English as MU
 
@@ -41,11 +42,12 @@ import           Game.LambdaHack.Client.UI.SessionUI
 import           Game.LambdaHack.Common.Actor
 import           Game.LambdaHack.Common.ActorState
 import qualified Game.LambdaHack.Common.Color as Color
+import           Game.LambdaHack.Common.ContentData
+import qualified Game.LambdaHack.Common.Dice as Dice
 import           Game.LambdaHack.Common.Faction
 import           Game.LambdaHack.Common.Item
-import           Game.LambdaHack.Common.ItemStrongest
-import qualified Game.LambdaHack.Common.Kind as Kind
-import qualified Game.LambdaHack.Common.KindOps as KindOps
+import qualified Game.LambdaHack.Common.ItemAspect as IA
+import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Level
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.MonadStateRead
@@ -56,6 +58,8 @@ import           Game.LambdaHack.Common.State
 import qualified Game.LambdaHack.Common.Tile as Tile
 import           Game.LambdaHack.Common.Time
 import           Game.LambdaHack.Common.Vector
+import           Game.LambdaHack.Content.CaveKind (cname)
+import qualified Game.LambdaHack.Content.ItemKind as IK
 import qualified Game.LambdaHack.Content.ModeKind as MK
 import           Game.LambdaHack.Content.TileKind (TileKind, isUknownSpace)
 import qualified Game.LambdaHack.Content.TileKind as TK
@@ -71,7 +75,7 @@ targetDesc mtarget = do
       b <- getsState $ getActorBody aid
       bUI <- getsSession $ getActorUI aid
       ar <- getsState $ getActorAspect aid
-      let percentage = 100 * bhp b `div` xM (max 5 $ aMaxHP ar)
+      let percentage = 100 * bhp b `div` xM (max 5 $ IA.aMaxHP ar)
           chs n = "[" <> T.replicate n "*"
                       <> T.replicate (4 - n) "_" <> "]"
           stars = chs $ fromEnum $ max 0 $ min 4 $ percentage `div` 20
@@ -93,11 +97,11 @@ targetDesc mtarget = do
               [] -> return $! "exact spot" <+> tshow p
               [(iid, kit@(k, _))] -> do
                 localTime <- getsState $ getLocalTime lid
-                itemToF <- getsState itemToFull
+                itemFull <- getsState $ itemToFull iid
                 side <- getsClient sside
                 factionD <- getsState sfactionD
                 let (_, _, name, stats) =
-                      partItem side factionD CGround localTime (itemToF iid kit)
+                      partItem side factionD localTime itemFull kit
                 return $! makePhrase
                           $ if k == 1
                             then [name, stats]  -- "a sword" too wordy
@@ -109,7 +113,7 @@ targetDesc mtarget = do
       case mleader of
         Nothing -> return (Just "a relative shift", Nothing)
         Just aid -> do
-          tgtPos <- aidTgtToPos aid lidV target
+          tgtPos <- getsState $ aidTgtToPos aid lidV target
           let invalidMsg = "an invalid relative shift"
               validMsg p = "shift to" <+> tshow p
           return (Just $ maybe invalidMsg validMsg tgtPos, Nothing)
@@ -127,13 +131,13 @@ targetDescXhair = do
 
 drawFrameTerrain :: forall m. MonadClientUI m => LevelId -> m FrameForall
 drawFrameTerrain drawnLevelId = do
-  Kind.COps{coTileSpeedup, cotile=Kind.Ops{okind}} <- getsState scops
+  COps{coTileSpeedup, cotile} <- getsState scops
   StateClient{smarkSuspect} <- getClient
   Level{lxsize, ltile=PointArray.Array{avector}} <- getLevel drawnLevelId
   totVisible <- totalVisible <$> getPerFid drawnLevelId
-  let dis :: Int -> Kind.Id TileKind -> Color.AttrCharW32
+  let dis :: Int -> ContentId TileKind -> Color.AttrCharW32
       {-# INLINE dis #-}
-      dis pI tile = case okind tile of
+      dis pI tile = case okind cotile tile of
         TK.TileKind{tsymbol, tcolor, tcolor2} ->
           -- Passing @p0@ as arg in place of @pI@ is much more costly.
           let p0 :: Point
@@ -150,13 +154,13 @@ drawFrameTerrain drawnLevelId = do
                  | ES.member p0 totVisible = tcolor
                  | otherwise = tcolor2
           in Color.attrChar2ToW32 fg tsymbol
-      mapVT :: forall s. (Int -> Kind.Id TileKind -> Color.AttrCharW32)
+      mapVT :: forall s. (Int -> ContentId TileKind -> Color.AttrCharW32)
             -> FrameST s
       {-# INLINE mapVT #-}
       mapVT f v = do
         let g :: Int -> Word16 -> ST s ()
             g !pI !tile = do
-              let w = Color.attrCharW32 $ f pI (KindOps.Id tile)
+              let w = Color.attrCharW32 $ f pI (ContentId tile)
               VM.write v (pI + lxsize) w
         U.imapM_ g avector
       upd :: FrameForall
@@ -167,10 +171,10 @@ drawFrameContent :: forall m. MonadClientUI m => LevelId -> m FrameForall
 drawFrameContent drawnLevelId = do
   SessionUI{smarkSmell} <- getSession
   Level{lxsize, lsmell, ltime, lfloor} <- getLevel drawnLevelId
-  s <- getState
+  itemToF <- getsState $ flip itemToFull
   let {-# INLINE viewItemBag #-}
       viewItemBag _ floorBag = case EM.toDescList floorBag of
-        (iid, _) : _ -> viewItem $ getItemBody iid s
+        (iid, _kit) : _ -> viewItem $ itemToF iid
         [] -> error $ "lfloor not sparse" `showFailure` ()
       viewSmell :: Point -> Time -> Color.AttrCharW32
       {-# INLINE viewSmell #-}
@@ -199,7 +203,7 @@ drawFramePath :: forall m. MonadClientUI m => LevelId -> m FrameForall
 drawFramePath drawnLevelId = do
  SessionUI{saimMode} <- getSession
  if isNothing saimMode then return $! FrameForall $ \_ -> return () else do
-  Kind.COps{coTileSpeedup} <- getsState scops
+  COps{coTileSpeedup} <- getsState scops
   StateClient{seps} <- getClient
   Level{lxsize, lysize, ltile=PointArray.Array{avector}}
     <- getLevel drawnLevelId
@@ -234,7 +238,7 @@ drawFramePath drawnLevelId = do
       shiftedLine = if null shiftedBTrajectory
                     then bline
                     else shiftedBTrajectory
-      acOnPathOrLine :: Char.Char -> Point -> Kind.Id TileKind
+      acOnPathOrLine :: Char.Char -> Point -> ContentId TileKind
                      -> Color.AttrCharW32
       acOnPathOrLine !ch !p0 !tile =
         let fgOnPathOrLine =
@@ -247,7 +251,7 @@ drawFramePath drawnLevelId = do
                 (False, True)  -> Color.Green
                 (False, False) -> Color.Red
         in Color.attrChar2ToW32 fgOnPathOrLine ch
-      mapVTL :: forall s. (Point -> Kind.Id TileKind -> Color.AttrCharW32)
+      mapVTL :: forall s. (Point -> ContentId TileKind -> Color.AttrCharW32)
              -> [Point]
              -> FrameST s
       mapVTL f l v = do
@@ -255,7 +259,7 @@ drawFramePath drawnLevelId = do
             g !p0 = do
               let pI = PointArray.pindex lxsize p0
                   tile = avector U.! pI
-                  w = Color.attrCharW32 $ f p0 (KindOps.Id tile)
+                  w = Color.attrCharW32 $ f p0 (ContentId tile)
               VM.write v (pI + lxsize) w
         mapM_ g l
       upd :: FrameForall
@@ -266,12 +270,11 @@ drawFramePath drawnLevelId = do
 
 drawFrameActor :: forall m. MonadClientUI m => LevelId -> m FrameForall
 drawFrameActor drawnLevelId = do
-  SessionUI{sselected} <- getSession
+  SessionUI{sactorUI, sselected, sUIOptions} <- getSession
   Level{lxsize, lactor} <- getLevel drawnLevelId
   side <- getsClient sside
   mleader <- getsClient sleader
   s <- getState
-  sactorUI <- getsSession sactorUI
   let {-# INLINE viewActor #-}
       viewActor _ as = case as of
         aid : _ ->
@@ -288,8 +291,14 @@ drawFrameActor drawnLevelId = do
                                        then Color.HighlightWhite
                                        else Color.HighlightMagenta
                         | otherwise -> Color.HighlightNone
-          in Color.attrCharToW32
-             $ Color.AttrChar Color.Attr{fg=bcolor, bg} symbol
+              fg | bfid /= side || bproj || bhp <= 0 = bcolor
+                 | otherwise =
+                let (hpCheckWarning, calmCheckWarning) =
+                      checkWarnings sUIOptions aid s
+                in if hpCheckWarning || calmCheckWarning
+                   then Color.Red
+                   else bcolor
+         in Color.attrCharToW32 $ Color.AttrChar Color.Attr{..} symbol
         [] -> error $ "lactor not sparse" `showFailure` ()
       mapVAL :: forall a s. (Point -> a -> Color.AttrCharW32) -> [(Point, a)]
              -> FrameST s
@@ -321,7 +330,7 @@ drawFrameExtra dm drawnLevelId = do
         mtgt <- getsClient $ getTarget leader
         case mtgt of
           Nothing -> return Nothing
-          Just tgt -> aidTgtToPos leader drawnLevelId tgt
+          Just tgt -> getsState $ aidTgtToPos leader drawnLevelId tgt
   let visionMarks =
         if smarkVision
         then map (PointArray.pindex lxsize) $ ES.toList totVisible
@@ -362,6 +371,7 @@ drawFrameExtra dm drawnLevelId = do
 
 drawFrameStatus :: MonadClientUI m => LevelId -> m AttrLine
 drawFrameStatus drawnLevelId = do
+  cops <- getsState scops
   SessionUI{sselected, saimMode, swaitTimes, sitemSel} <- getSession
   mleader <- getsClient sleader
   xhairPos <- xhairToPos
@@ -370,7 +380,6 @@ drawFrameStatus drawnLevelId = do
   (mtgtDesc, mtargetHP) <-
     maybe (return (Nothing, Nothing)) targetDescLeader mleader
   (xhairDesc, mxhairHP) <- targetDescXhair
-  sexplored <- getsClient sexplored
   lvl <- getLevel drawnLevelId
   (mblid, mbpos, mbodyUI) <- case mleader of
     Just leader -> do
@@ -381,8 +390,7 @@ drawFrameStatus drawnLevelId = do
   let widthX = 80
       widthTgt = 39
       widthStats = widthX - widthTgt - 1
-      arenaStatus = drawArenaStatus (ES.member drawnLevelId sexplored) lvl
-                                    widthStats
+      arenaStatus = drawArenaStatus cops lvl widthStats
       displayPathText mp mt =
         let (plen, llen) = case (mp, mbfs, mbpos) of
               (Just target, Just bfs, Just bpos)
@@ -434,18 +442,17 @@ drawFrameStatus drawnLevelId = do
             tgtBlurb = maybe leaderName (\t ->
               "Target:" <+> trimTgtDesc n t) mtgtDesc
         case (sitemSel, mleader) of
-          (Just (fromCStore, iid), Just leader) -> do
+          (Just (iid, fromCStore, _), Just leader) -> do
             b <- getsState $ getActorBody leader
             bag <- getsState $ getBodyStoreBag b fromCStore
             case iid `EM.lookup` bag of
               Nothing -> return $! tgtBlurb
               Just kit@(k, _) -> do
                 localTime <- getsState $ getLocalTime (blid b)
-                itemToF <- getsState itemToFull
+                itemFull <- getsState $ itemToFull iid
                 factionD <- getsState sfactionD
                 let (_, _, name, stats) =
-                      partItem (bfid b) factionD
-                               fromCStore localTime (itemToF iid kit)
+                      partItem (bfid b) factionD localTime itemFull kit
                     t = makePhrase
                         $ if k == 1
                           then [name, stats]  -- "a sword" too wordy
@@ -485,40 +492,54 @@ drawBaseFrame dm drawnLevelId = do
 
 -- Comfortably accomodates 3-digit level numbers and 25-character
 -- level descriptions (currently enforced max).
-drawArenaStatus :: Bool -> Level -> Int -> AttrLine
-drawArenaStatus explored
-                Level{ldepth=AbsDepth ld, lname, lseen, lexplorable}
+drawArenaStatus :: COps -> Level -> Int -> AttrLine
+drawArenaStatus COps{cocave}
+                Level{lkind, ldepth=Dice.AbsDepth ld, lseen, lexpl}
                 width =
-  let seenN = 100 * lseen `div` max 1 lexplorable
-      seenTxt | explored || seenN >= 100 = "all"
+  let ck = okind cocave lkind
+      seenN = 100 * lseen `div` max 1 lexpl
+      seenTxt | seenN >= 100 = "all"
               | otherwise = T.justifyLeft 3 ' ' (tshow seenN <> "%")
       lvlN = T.justifyLeft 2 ' ' (tshow ld)
       seenStatus = "[" <> seenTxt <+> "seen]"
   in textToAL $ T.justifyLeft width ' '
-              $ T.take 29 (lvlN <+> T.justifyLeft 26 ' ' lname) <+> seenStatus
+              $ T.take 29 (lvlN <+> T.justifyLeft 26 ' ' (cname ck))
+                <+> seenStatus
 
-drawLeaderStatus :: MonadClient m => Int -> m AttrLine
+checkWarnings :: UIOptions -> ActorId -> State -> (Bool, Bool)
+checkWarnings UIOptions{uhpWarningPercent} leader s =
+  let b = getActorBody leader s
+      ar = getActorAspect leader s
+      isImpression iid =
+        maybe False (> 0) $ lookup "impressed" $ IK.ifreq $ getIidKind iid s
+      isImpressed = any isImpression $ EM.keys $ borgan b
+      hpCheckWarning = bhp b <= xM (uhpWarningPercent * IA.aMaxHP ar `div` 100)
+      calmCheckWarning =
+        bcalm b <= xM (uhpWarningPercent * IA.aMaxCalm ar `div` 100)
+        && isImpressed
+  in (hpCheckWarning, calmCheckWarning)
+
+drawLeaderStatus :: MonadClientUI m => Int -> m AttrLine
 drawLeaderStatus waitT = do
   let calmHeaderText = "Calm"
       hpHeaderText = "HP"
+  sUIOptions <- getsSession sUIOptions
   mleader <- getsClient sleader
   case mleader of
     Just leader -> do
+      b <- getsState $ getActorBody leader
       ar <- getsState $ getActorAspect leader
-      s <- getState
-      let showTrunc :: Show a => a -> String
-          showTrunc = (\t -> if length t > 3 then "***" else t) . show
-          (darkL, bracedL, hpDelta, calmDelta,
-           ahpS, bhpS, acalmS, bcalmS) =
-            let b@Actor{bhp, bcalm} = getActorBody leader s
-            in ( not (actorInAmbient b s)
-               , braced b, bhpDelta b, bcalmDelta b
-               , showTrunc $ aMaxHP ar, showTrunc (bhp `divUp` oneM)
-               , showTrunc $ aMaxCalm ar, showTrunc (bcalm `divUp` oneM))
+      (hpCheckWarning, calmCheckWarning)
+        <- getsState $ checkWarnings sUIOptions leader
+      bdark <- getsState $ \s -> not (actorInAmbient b s)
+      let showTrunc x = let t = show x
+                        in if length t > 3
+                           then if x > 0 then "***" else "---"
+                           else t
           -- This is a valuable feedback for the otherwise hard to observe
           -- 'wait' command.
           slashes = ["/", "|", "\\", "|"]
-          slashPick = slashes !! (max 0 (waitT - 1) `mod` length slashes)
+          slashPick = slashes !! (max 0 waitT `mod` length slashes)
           addColor c = map (Color.attrChar2ToW32 c)
           checkDelta ResDelta{..}
             | fst resCurrentTurn < 0 || fst resPreviousTurn < 0
@@ -526,54 +547,73 @@ drawLeaderStatus waitT = do
             | snd resCurrentTurn > 0 || snd resPreviousTurn > 0
               = addColor Color.BrGreen
             | otherwise = stringToAL  -- only if nothing at all noteworthy
-          calmAddAttr = checkDelta calmDelta
+          calmAddAttr = checkDelta $ bcalmDelta b
           -- We only show ambient light, because in fact client can't tell
           -- if a tile is lit, because it it's seen it may be due to ambient
           -- or dynamic light or due to infravision.
-          darkPick | darkL   = "."
+          darkPick | bdark = "."
                    | otherwise = ":"
           calmHeader = calmAddAttr $ calmHeaderText <> darkPick
-          calmText = bcalmS <> (if darkL then slashPick else "/") <> acalmS
-          bracePick | bracedL   = "}"
+          calmText = showTrunc (bcalm b `divUp` oneM)
+                     <> (if bdark || not (braced b)
+                         then slashPick
+                         else "/")
+                     <> showTrunc (max 0 $ IA.aMaxCalm ar)
+          bracePick | braced b  = "}"
                     | otherwise = ":"
-          hpAddAttr = checkDelta hpDelta
+          hpAddAttr = checkDelta $ bhpDelta b
           hpHeader = hpAddAttr $ hpHeaderText <> bracePick
-          hpText = bhpS <> (if bracedL then slashPick else "/") <> ahpS
+          hpText = showTrunc (bhp b `divUp` oneM)
+                   <> (if braced b || not bdark
+                       then slashPick
+                       else "/")
+                   <> showTrunc (max 0 $ IA.aMaxHP ar)
           justifyRight n t = replicate (n - length t) ' ' ++ t
-      return $! calmHeader <> stringToAL (justifyRight 7 calmText)
-                <+:> hpHeader <> stringToAL (justifyRight 7 hpText)
+          colorWarning w = if w then addColor Color.Red else stringToAL
+      return $! calmHeader
+                <> colorWarning calmCheckWarning (justifyRight 7 calmText)
+                <+:> hpHeader
+                <> colorWarning hpCheckWarning (justifyRight 7 hpText)
     Nothing -> return $! stringToAL (calmHeaderText ++ ":  --/--")
                          <+:> stringToAL (hpHeaderText <> ":  --/--")
 
 drawLeaderDamage :: MonadClientUI m => Int -> m AttrLine
 drawLeaderDamage width = do
   mleader <- getsClient sleader
-  let addColor = map (Color.attrChar2ToW32 Color.BrCyan)
-  stats <- case mleader of
+  (tdice, tbonus, cbonus) <- case mleader of
     Just leader -> do
-      allAssocsRaw <- getsState $ fullAssocs leader [CEqp, COrgan]
-      let allAssocs = filter (isMelee . itemBase . snd) allAssocsRaw
+      kitAssRaw <- getsState $ kitAssocs leader [CEqp, COrgan]
       actorSk <- leaderSkillsClientUI
       actorAspect <- getsState sactorAspect
-      strongest <- pickWeaponM Nothing allAssocs actorSk leader
+      let kitAssOnlyWeapons =
+            filter (IK.isMelee . itemKind . fst . snd) kitAssRaw
+      strongest <- pickWeaponM Nothing kitAssOnlyWeapons actorSk leader
       let damage = case strongest of
-            [] -> "0"
-            (_, (_, itemFull)) : _ ->
-              let tdice = show $ jdamage $ itemBase itemFull
-                  bonusRaw = aHurtMelee $ actorAspect EM.! leader
+            [] -> ("0", "", Color.White)
+            (_, (_, (itemFull, _))) : _ ->
+              let tdice = show $ IK.idamage $ itemKind itemFull
+                  bonusRaw = IA.aHurtMelee $ actorAspect EM.! leader
                   bonus = min 200 $ max (-200) bonusRaw
-                  unknownBonus = unknownMeleeBonus $ map snd allAssocs
+                  unknownBonus = unknownMeleeBonus $ map (fst . snd) kitAssRaw
                   tbonus = if bonus == 0
                            then if unknownBonus then "+?" else ""
                            else (if bonus > 0 then "+" else "")
                                 <> show bonus
                                 <> (if bonus /= bonusRaw then "$" else "")
                                 <> if unknownBonus then "%?" else "%"
-             in tdice <> tbonus
+                  tmpBonus = tmpMeleeBonus $ map snd kitAssRaw
+                  cbonus = case compare tmpBonus 0 of
+                    EQ -> Color.White
+                    GT -> Color.Green
+                    LT -> Color.Red
+             in (tdice, tbonus, cbonus)
       return $! damage
-    Nothing -> return ""
-  return $! if null stats || length stats >= width then []
-            else addColor $ stats <> " "
+    Nothing -> return ("", "", Color.White)
+  let addColorDice = map (Color.attrChar2ToW32 Color.BrCyan)
+      addColorBonus = map (Color.attrChar2ToW32 cbonus)
+  return $! if null tdice || length tdice + length tbonus >= width then []
+            else addColorDice tdice ++ addColorBonus tbonus
+                 ++ [Color.spaceAttrW32]
 
 drawSelected :: MonadClientUI m
              => LevelId -> Int -> ES.EnumSet ActorId -> m (Int, AttrLine)
