@@ -1,6 +1,6 @@
 -- | Description of effects.
 module Game.LambdaHack.Client.UI.EffectDescription
-  ( effectToSuffix
+  ( DetailLevel(..), effectToSuffix, detectToObject, detectToVerb
   , slotToSentence, slotToName, slotToDesc, slotToDecorator, statSlots
   , kindAspectToSuffix, featureToSuff, featureToSentence, affixDice
 #ifdef EXPOSE_INTERNAL
@@ -16,13 +16,18 @@ import Game.LambdaHack.Common.Prelude
 import qualified Data.Text as T
 import qualified NLP.Miniutter.English as MU
 
+import           Game.LambdaHack.Client.UI.ActorUI
 import           Game.LambdaHack.Common.Ability
 import           Game.LambdaHack.Common.Actor
 import qualified Game.LambdaHack.Common.Dice as Dice
+import           Game.LambdaHack.Common.ItemAspect (Aspect (..), EqpSlot (..))
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.Time
 import           Game.LambdaHack.Content.ItemKind
 
+data DetailLevel = DetailLow | DetailMedium | DetailHigh | DetailAll
+  deriving (Eq, Ord, Enum, Bounded)
+
 -- | Suffix to append to a basic content name if the content causes the effect.
 --
 -- We show absolute time in seconds, not @moves@, because actors can have
@@ -33,11 +38,9 @@ import           Game.LambdaHack.Content.ItemKind
 -- We show distances in @steps@, because one step, from a tile to another
 -- tile, is always 1 meter. We don't call steps @tiles@, reserving
 -- that term for the context of terrain kinds or units of area.
-effectToSuffix :: Effect -> Text
-effectToSuffix effect =
+effectToSuffix :: DetailLevel -> Effect -> Text
+effectToSuffix detailLevel effect =
   case effect of
-    ELabel _ -> ""  -- printed specially
-    EqpSlot{} -> ""  -- used in @slotToSentence@ instead
     Burn d -> wrapInParens (tshow d
                             <+> if Dice.maxDice d > 1 then "burns" else "burn")
     Explode t -> "of" <+> tshow t <+> "explosion"
@@ -51,7 +54,7 @@ effectToSuffix effect =
     Impress -> "of impression"
     Summon grp p -> makePhrase
       [ "of summoning"
-      , if p == 1 then "" else MU.Text $ tshow p
+      , if p <= 1 then "" else MU.Text $ tshow p
       , MU.Ws $ MU.Text $ tshow grp ]
     ApplyPerfume -> "of smell removal"
     Ascend True -> "of ascending"
@@ -71,47 +74,62 @@ effectToSuffix effect =
             Nothing -> tshow dice <+> "moves"
             Just p -> makePhrase [MU.CarWs p "move"]
       in "of speed surge for" <+> moves
-    Teleport dice | Dice.minDice dice <= 0 ->
-      error $ "" `showFailure` effect
     Teleport dice | Dice.maxDice dice <= 9 ->
       "of blinking" <+> wrapInParens (tshow dice)
     Teleport dice -> "of teleport" <+> wrapInParens (tshow dice)
     CreateItem COrgan grp tim ->
-      let stime = if tim == TimerNone then "" else "for" <+> tshow tim <> ":"
+      let stime = if isTimerNone tim then "" else "for" <+> tshow tim <> ":"
       in "(keep" <+> stime <+> tshow grp <> ")"
-    CreateItem _ grp _ ->
-      let object = if grp == "useful" then "" else tshow grp
-      in "of" <+> object <+> "uncovering"
+    CreateItem{} -> "of gain"
     DropItem n k store grp ->
-      let ntxt = if | n == 1 && k == 1 -> ""
-                    | n == 1 && k == maxBound -> "one kind of"
+      let ntxt = if | n == 1 && k == maxBound -> "one kind of"
                     | n == maxBound && k == maxBound -> "all kinds of"
-                    | otherwise -> "some"
-          verb = if store == COrgan then "nullify" else "drop"
-      in "of" <+> verb <+> ntxt <+> tshow grp  -- TMI: <+> ppCStore store
+                    | otherwise -> ""
+          (verb, fromStore) =
+            if store == COrgan
+            then ("nullify", "")
+            else ("drop", "from" <+> snd (ppCStore store))
+      in "of" <+> verb <+> ntxt <+> tshow grp <+> fromStore
     PolyItem -> "of repurpose on the ground"
     Identify -> "of identify"
-    Detect radius -> "of detection" <+> wrapInParens (tshow radius)
-    DetectActor radius -> "of actor detection" <+> wrapInParens (tshow radius)
-    DetectItem radius -> "of item detection" <+> wrapInParens (tshow radius)
-    DetectExit radius -> "of exit detection" <+> wrapInParens (tshow radius)
-    DetectHidden radius ->
-      "of secrets detection" <+> wrapInParens (tshow radius)
+    Detect d radius ->
+      "of" <+> detectToObject d <+> "location" <+> wrapInParens (tshow radius)
     SendFlying tmod -> "of impact" <+> tmodToSuff "" tmod
     PushActor tmod -> "of pushing" <+> tmodToSuff "" tmod
     PullActor tmod -> "of pulling" <+> tmodToSuff "" tmod
     DropBestWeapon -> "of disarming"
-    ActivateInv ' ' -> "of item pack burst"
+    ActivateInv ' ' -> "of backpack burst"
     ActivateInv symbol -> "of burst '" <> T.singleton symbol <> "'"
     OneOf l ->
       let subject = if length l <= 5 then "marvel" else "wonder"
-      in makePhrase ["of", MU.CardinalWs (length l) subject]
+          header = makePhrase ["of", MU.CardinalWs (length l) subject]
+          marvels = T.intercalate ", " $ map (effectToSuffix detailLevel) l
+      in if detailLevel >= DetailAll && marvels /= ""
+         then header <+> "[" <> marvels <> "]"
+         else header
     OnSmash _ -> ""  -- printed inside a separate section
     Recharging _ -> ""  -- printed inside Periodic or Timeout
-    Temporary _ -> ""
-    Unique -> ""  -- marked by capital letters in name
-    Periodic -> ""  -- printed specially
-    Composite effs -> T.intercalate " and then " $ map effectToSuffix effs
+    Temporary _ -> ""  -- only printed on destruction
+    Composite effs -> T.intercalate " and then "
+                    $ filter (/= "") $ map (effectToSuffix detailLevel) effs
+
+detectToObject :: DetectKind -> Text
+detectToObject d = case d of
+  DetectAll -> ""
+  DetectActor -> "actor"
+  DetectItem -> "item"
+  DetectExit -> "exit"
+  DetectHidden -> "secret"
+  DetectEmbed -> "feature"
+
+detectToVerb :: DetectKind -> Text
+detectToVerb d = case d of
+  DetectAll -> "map surrounding area"
+  DetectActor -> "spot nearby"
+  DetectItem -> "locate nearby"
+  DetectExit -> "learn nearby"
+  DetectHidden -> "uncover nearby"
+  DetectEmbed -> "notice nearby"
 
 slotToSentence :: EqpSlot -> Text
 slotToSentence es = case es of
@@ -123,7 +141,7 @@ slotToSentence es = case es of
   EqpSlotAddSpeed -> "The slughish equip it to speed up their whole life."
   EqpSlotAddSight -> "The short-sighted wear it to spot their demise sooner."
   EqpSlotLightSource -> "Explorers brave enough to highlight themselves put it in their equipment."
-  EqpSlotWeapon -> "Melee fighters consider it for their weapon combo."
+  EqpSlotWeapon -> "Melee fighters pick it for their weapon combo."
   EqpSlotMiscAbility -> "Those that don't scorn uncanny skills may equip it."
   EqpSlotAbMove -> "Those unskilled in movement equip it."
   EqpSlotAbMelee -> "Those unskilled in melee equip it."
@@ -165,9 +183,9 @@ slotToDesc eqpSlot =
       capName = "The '" <> statName <> "' stat"
   in capName <+> case eqpSlot of
     EqpSlotMiscBonus -> "represent the total power of assorted stat bonuses for the character."
-    EqpSlotAddHurtMelee -> "is a percentage of addtional damage dealt by the actor (either a character or a missile) with any weapon. The value is capped at 200%, then the armor percentage of the defender is subtracted from it and the resulting total is capped at 99%."
+    EqpSlotAddHurtMelee -> "is a percentage of additional damage dealt by the actor (either a character or a missile) with any weapon. The value is capped at 200%, then the armor percentage of the defender is subtracted from it and the resulting total is capped at 99%."
     EqpSlotAddArmorMelee -> "is a percentage of melee damage avoided by the actor. The value is capped at 200%, then the extra melee damage percentage of the attacker is subtracted from it and the resulting total is capped at 99% (always at least 1% of damage gets through). It includes 50% bonus from being braced for combat, if applicable."
-    EqpSlotAddArmorRanged ->  "is a percentage of ranged damage avoided by the actor. The value is capped at 200%, then the extra melee damage percentage of the attacker is subtracted from it and the resulting total is capped at 99% (always at least 1% of damage gets through). It includes 25% bonus from being braced for combat, if applicable."
+    EqpSlotAddArmorRanged -> "is a percentage of ranged damage avoided by the actor. The value is capped at 200%, then the extra melee damage percentage of the attacker is subtracted from it and the resulting total is capped at 99% (always at least 1% of damage gets through). It includes 25% bonus from being braced for combat, if applicable."
     EqpSlotAddMaxHP -> "is a cap on HP of the actor, except for some rare effects able to overfill HP. At any direct enemy damage (but not, e.g., incremental poisoning damage or wounds inflicted by mishandling a device) HP is cut back to the cap."
     EqpSlotAddSpeed -> "is expressed in meters per second, which corresponds to map location (1m by 1m) per two standard turns (0.5s each). Thus actor at standard speed of 2m/s moves one location per standard turn."
     EqpSlotAddSight -> "is the limit of visibility in light. The radius is measured from the middle of the map location occupied by the character to the edge of the furthest covered location."
@@ -177,13 +195,13 @@ slotToDesc eqpSlot =
     EqpSlotAbMove -> "determines whether the character can move. Actors not capable of movement can't be dominated."
     EqpSlotAbMelee -> "determines whether the character can melee. Actors that can't melee can still cause damage by flinging missiles or by ramming (being pushed) at opponents."
     EqpSlotAbDisplace -> "determines whether the character can displace adjacent actors. In some cases displacing is not possible regardless of ability: when the target is braced, dying, has no move ability or when both actors are supported by adjacent friendly units. Missiles can be displaced always, unless more than one occupies the map location."
-    EqpSlotAbAlter -> "determines which kinds of terrain can be altered or triggered by the character. Opening doors and searching suspect tiles require ability 2, stairs require 3, closing doors requires 4, some others require 5. Actors not smart enough to be capable of using stairs can't be dominated."
+    EqpSlotAbAlter -> "determines which kinds of terrain can be altered or triggered by the character. Opening doors and searching suspect tiles require ability 2, some stairs require 3, closing doors requires 4, others require 4 or 5. Actors not smart enough to be capable of using stairs can't be dominated."
     EqpSlotAbProject -> "determines which kinds of items the character can propel. Items that can be lobbed to explode at a precise location, such as flasks, require ability 3. Other items travel until they meet an obstacle and ability 1 is enough to fling them. In some cases, e.g., of too intricate or two awkward items at low Calm, throwing is not possible regardless of the ability value."
     EqpSlotAbApply -> "determines which kinds of items the character can activate. Items that assume literacy require ability 2, others can be used already at ability 1. In some cases, e.g., when the item needs recharging, has no possible effects or is too intricate for the character Calm level, applying may not be possible."
     EqpSlotAddMaxCalm -> "is a cap on Calm of the actor, except for some rare effects able to overfill Calm. At any direct enemy damage (but not, e.g., incremental poisoning damage or wounds inflicted by mishandling a device) Calm is lowered, sometimes very significantly and always at least back down to the cap."
     EqpSlotAddSmell -> "determines the maximal area smelled by the actor. The radius is measured from the middle of the map location occupied by the character to the edge of the furthest covered location."
     EqpSlotAddNocto -> "is the limit of visibility in dark. The radius is measured from the middle of the map location occupied by the character to the edge of the furthest covered location."
-    EqpSlotAddAggression -> "represents the willingness of the actor to engage in combat, especially close quarters, and conversly, to break engagement when overpowered."
+    EqpSlotAddAggression -> "represents the willingness of the actor to engage in combat, especially close quarters, and conversely, to break engagement when overpowered."
     EqpSlotAbWait -> "determines whether the character can wait, bracing for comat and potentially blocking the effects of some attacks."
     EqpSlotAbMoveItem -> "determines whether the character can pick up items and manage inventory."
 
@@ -205,7 +223,7 @@ slotToDecorator eqpSlot b t =
     EqpSlotAddArmorMelee -> "[" <> tshowBlock 50 t <> "%]"
     EqpSlotAddArmorRanged -> "{" <> tshowBlock 25 t <> "%}"
     EqpSlotAddMaxHP -> tshow $ max 0 t
-    EqpSlotAddSpeed -> showIntWith1 t <> "m/s"
+    EqpSlotAddSpeed -> showIntWith1 (max minSpeed t) <> "m/s"
     EqpSlotAddSight ->
       let tmax = max 0 t
           tcapped = min (fromEnum $ bcalm b `div` (5 * oneM)) tmax
@@ -279,30 +297,40 @@ kindAspectToSuffix aspect =
 featureToSuff :: Feature -> Text
 featureToSuff feat =
   case feat of
+    ELabel{} -> ""  -- too late
     Fragile -> wrapInChevrons "fragile"
     Lobable -> wrapInChevrons "can be lobbed"
     Durable -> wrapInChevrons "durable"
     ToThrow tmod -> wrapInChevrons $ tmodToSuff "flies" tmod
-    Identified -> ""
-    Applicable -> ""
+    HideAs{} -> ""
     Equipable -> ""
     Meleeable -> ""
     Precious -> ""
     Tactic tactics -> "overrides tactics to" <+> tshow tactics
+    Blast -> ""
+    EqpSlot{} -> ""  -- used in @slotToSentence@ instead
+    Unique -> ""  -- marked by capital letters in name
+    Periodic -> ""  -- printed specially
+    MinorEffects -> ""  -- cryptic override
 
 featureToSentence :: Feature -> Maybe Text
 featureToSentence feat =
   case feat of
+    ELabel{} -> Nothing
     Fragile -> Nothing
     Lobable -> Nothing
     Durable -> Nothing
     ToThrow{} -> Nothing
-    Identified -> Nothing
-    Applicable -> Just "It is meant to be applied."
+    HideAs{} -> Nothing
     Equipable -> Nothing
     Meleeable -> Just "It is considered for melee strikes by default."
     Precious -> Just "It seems precious."
     Tactic{}  -> Nothing
+    Blast -> Nothing
+    EqpSlot es -> Just $ slotToSentence es
+    Unique -> Nothing
+    Periodic -> Nothing
+    MinorEffects -> Nothing
 
 affixBonus :: Int -> Text
 affixBonus p = case compare p 0 of
