@@ -49,7 +49,7 @@ data FrontReq :: * -> * where
   -- | Flush frames, display a frame and ask for a keypress.
   FrontKey :: { frontKeyKeys  :: [K.KM]
               , frontKeyFrame :: FrameForall } -> FrontReq KMP
-  -- | Inspect the fkeyPressed MVar.
+  -- | Tell if a keypress is pending.
   FrontPressed :: FrontReq Bool
   -- | Discard a key in the queue, if any.
   FrontDiscard :: FrontReq ()
@@ -59,6 +59,8 @@ data FrontReq :: * -> * where
   FrontAutoYes :: Bool -> FrontReq ()
   -- | Shut the frontend down.
   FrontShutdown :: FrontReq ()
+  -- | Take screenshot.
+  FrontPrintScreen :: FrontReq ()
 
 -- | Connection channel between a frontend and a client. Frontend acts
 -- as a server, serving keys, etc., when given frames to display.
@@ -125,6 +127,7 @@ fchanFrontend soptions fs@FrontSetup{..} rf =
       -- In case the last frame display is pending:
       void $ tryTakeMVar $ fshowNow rf
       fshutdown rf
+    FrontPrintScreen -> fprintScreen rf
 
 display :: RawFrontend -> FrameForall -> IO ()
 display rf@RawFrontend{fshowNow} frontFrame = do
