@@ -4,7 +4,7 @@ module Game.LambdaHack.Client.UI.Frontend.Sdl
 #ifdef EXPOSE_INTERNAL
     -- * Internal operations
   , FontAtlas, FrontendSession(..), startupFun, shutdown, forceShutdown
-  , display, drawFrame, modTranslate, keyTranslate, colorToRGBA
+  , display, drawFrame, printScreen, modTranslate, keyTranslate, colorToRGBA
 #endif
   ) where
 
@@ -17,9 +17,14 @@ import qualified Data.Char as Char
 import qualified Data.EnumMap.Strict as EM
 import           Data.IORef
 import qualified Data.Text as T
+import           Data.Time.Clock.POSIX
+import           Data.Time.LocalTime
 import qualified Data.Vector.Unboxed as U
 import           Data.Word (Word32, Word8)
+import           Foreign.C.String (withCString)
 import           Foreign.C.Types (CInt)
+import           Foreign.Ptr (nullPtr)
+import           Foreign.Storable (peek)
 import           System.Directory
 import           System.Exit (exitSuccess)
 import           System.FilePath
@@ -27,7 +32,11 @@ import           System.FilePath
 import qualified SDL
 import qualified SDL.Font as TTF
 import           SDL.Input.Keyboard.Codes
+import qualified SDL.Internal.Types
 import qualified SDL.Raw.Basic as SDL (logSetAllPriority)
+import qualified SDL.Raw.Enum
+import qualified SDL.Raw.Types
+import qualified SDL.Raw.Video
 import qualified SDL.Vect as Vect
 
 import           Game.LambdaHack.Client.ClientOptions
@@ -35,6 +44,7 @@ import           Game.LambdaHack.Client.UI.Frame
 import           Game.LambdaHack.Client.UI.Frontend.Common
 import qualified Game.LambdaHack.Client.UI.Key as K
 import qualified Game.LambdaHack.Common.Color as Color
+import           Game.LambdaHack.Common.File
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.Point
 import qualified Game.LambdaHack.Common.PointArray as PointArray
@@ -43,16 +53,16 @@ type FontAtlas = EM.EnumMap Color.AttrCharW32 SDL.Texture
 
 -- | Session data maintained by the frontend.
 data FrontendSession = FrontendSession
-  { swindow           :: SDL.Window
-  , srenderer         :: SDL.Renderer
-  , sfont             :: TTF.Font
-  , satlas            :: IORef FontAtlas
-  , stexture          :: IORef SDL.Texture
-  , spreviousFrame    :: IORef SingleFrame
-  , sforcedShutdown   :: IORef Bool
-  , scontinueSdlLoop  :: IORef Bool
-  , sframeQueue       :: MVar SingleFrame
-  , sframeDrawn       :: MVar ()
+  { swindow          :: SDL.Window
+  , srenderer        :: SDL.Renderer
+  , sfont            :: TTF.Font
+  , satlas           :: IORef FontAtlas
+  , stexture         :: IORef SDL.Texture
+  , spreviousFrame   :: IORef SingleFrame
+  , sforcedShutdown  :: IORef Bool
+  , scontinueSdlLoop :: IORef Bool
+  , sframeQueue      :: MVar SingleFrame
+  , sframeDrawn      :: MVar ()
   }
 
 -- | The name of the frontend.
@@ -61,14 +71,8 @@ frontendName = "sdl"
 
 -- | Set up and start the main loop providing input and output.
 --
--- Apparently some SDL backends are not thread-safe
--- (<https://wiki.libsdl.org/FAQDevelopment>;
--- "this should only be run in the thread that initialized the video subsystem,
--- and for extra safety, you should consider only doing those things
--- on the main thread in any case")
--- so we stick to a single bound thread (but not to the main thread;
--- enough is enough and at least in the case of OpenGL all bound threads
--- are supposed to be as good as the main thread).
+-- Because of Windows and OS X, SDL2 needs to be on a bound thread,
+-- so we can't avoid the communication overhead of bound threads.
 startup :: ClientOptions -> IO RawFrontend
 startup soptions = startupBound $ startupFun soptions
 
@@ -97,7 +101,9 @@ startupFun soptions@ClientOptions{..} rfMVar = do
                         (toEnum $ ysize * boxSize)
       windowConfig = SDL.defaultWindow {SDL.windowInitialSize = screenV2}
       rendererConfig = SDL.RendererConfig
-        { rendererType          = SDL.AcceleratedVSyncRenderer
+        { rendererType          = if sbenchmark
+                                  then SDL.AcceleratedRenderer
+                                  else SDL.AcceleratedVSyncRenderer
         , rendererTargetTexture = True
         }
   swindow <- SDL.createWindow title windowConfig
@@ -121,7 +127,8 @@ startupFun soptions@ClientOptions{..} rfMVar = do
   sframeQueue <- newEmptyMVar
   sframeDrawn <- newEmptyMVar
   let sess = FrontendSession{..}
-  rf <- createRawFrontend (display sess) (shutdown sess)
+  rfWithoutPrintScreen <- createRawFrontend (display sess) (shutdown sess)
+  let rf = rfWithoutPrintScreen {fprintScreen = printScreen sess}
   putMVar rfMVar rf
   let pointTranslate :: forall i. (Enum i) => Vect.Point Vect.V2 i -> Point
       pointTranslate (SDL.P (SDL.V2 x y)) =
@@ -136,7 +143,7 @@ startupFun soptions@ClientOptions{..} rfMVar = do
         SDL.destroyTexture oldTexture
         writeIORef stexture newTexture
         prevFrame <- readIORef spreviousFrame
-        writeIORef spreviousFrame blankSingleFrame
+        writeIORef spreviousFrame blankSingleFrame  -- to overwrite each char
         drawFrame soptions sess prevFrame
       loopSDL :: IO ()
       loopSDL = do
@@ -146,13 +153,21 @@ startupFun soptions@ClientOptions{..} rfMVar = do
             mfr <- tryTakeMVar sframeQueue
             case mfr of
               Just fr -> do
-                -- Some SDL2 (OpenGL) backends are very thread-unsafe,
-                -- so we need to ensure we draw on the same (bound) OS thread
-                -- that initialized SDL, hence we have to poll frames.
-                drawFrame soptions sess fr
+                -- Don't present an unchanged backbuffer.
+                -- This doesn't improve FPS; probably equal frames happen
+                -- very rarely, if at all, which is actually very good.
+                prevFrame <- readIORef spreviousFrame
+                unless (prevFrame == fr) $ do
+                  -- Some SDL2 (OpenGL) backends are very thread-unsafe,
+                  -- so we need to ensure we draw on the same (bound) OS thread
+                  -- that initialized SDL, hence we have to poll frames.
+                  drawFrame soptions sess fr
+                  -- We can't print screen in @display@ due to thread-unsafety.
+                  when sprintEachScreen $ printScreen sess
                 putMVar sframeDrawn ()  -- signal that drawing ended
-              Nothing -> threadDelay 15000
-                           -- 60 polls per second, so keyboard snappy enough
+              Nothing -> threadDelay $ if sbenchmark then 150 else 15000
+                           -- 60 polls per second, so keyboard snappy enough;
+                           -- max 6000 FPS when benchmarking
           Just e -> handleEvent e
         continueSdlLoop <- readIORef scontinueSdlLoop
         if continueSdlLoop
@@ -197,8 +212,8 @@ startupFun soptions@ClientOptions{..} rfMVar = do
                 (EQ, _) -> Nothing
                 (LT, SDL.ScrollNormal) -> Just K.WheelSouth
                 (GT, SDL.ScrollNormal) -> Just K.WheelNorth
-                (LT, SDL.ScrollFlipped) -> Just K.WheelSouth
-                (GT, SDL.ScrollFlipped) -> Just K.WheelNorth
+                (LT, SDL.ScrollFlipped) -> Just K.WheelNorth
+                (GT, SDL.ScrollFlipped) -> Just K.WheelSouth
               modifier = if md == K.Shift then K.NoModifier else md
           p <- SDL.getAbsoluteMouseLocation
           maybe (return ())
@@ -322,6 +337,35 @@ drawFrame ClientOptions{..} FrontendSession{..} curFrame = do
   SDL.copy srenderer texture Nothing Nothing  -- clear the backbuffer
   SDL.present srenderer
 
+-- It can't seem to cope with SDL_PIXELFORMAT_INDEX8, so we are stuck
+-- with huge bitmaps.
+printScreen :: FrontendSession -> IO ()
+printScreen FrontendSession{..} = do
+  dataDir <- appDataDir
+  tryCreateDir dataDir
+  tryCreateDir $ dataDir </> "screenshots"
+  utcTime <- getCurrentTime
+  timezone <- getTimeZone utcTime
+  let unspace = map $ \c -> case c of  -- prevent the need for backquoting
+        ' ' -> '_'
+        ':' -> '.'
+        _ -> c
+      dateText = unspace $ take 25 $ show $ utcToLocalTime timezone utcTime
+      fileName = dataDir </> "screenshots" </> "prtscn" <> dateText <.> "bmp"
+      SDL.Internal.Types.Renderer renderer = srenderer
+  Vect.V2 sw sh <- SDL.get $ SDL.windowSize swindow
+  ptrOut <- SDL.Raw.Video.createRGBSurface 0 sw sh 32 0 0 0 0
+  surfaceOut <- peek ptrOut
+  void $ SDL.Raw.Video.renderReadPixels
+    renderer
+    nullPtr
+    SDL.Raw.Enum.SDL_PIXELFORMAT_ARGB8888
+    (SDL.Raw.Types.surfacePixels surfaceOut)
+    (sw * 4)
+  withCString fileName $ \fileNameCString ->
+    void $! SDL.Raw.Video.saveBMP ptrOut fileNameCString
+  SDL.Raw.Video.freeSurface ptrOut
+
 -- | Translates modifiers to our own encoding, ignoring Shift.
 modTranslate :: SDL.KeyModifier -> K.Modifier
 modTranslate m =
@@ -388,6 +432,7 @@ keyTranslate shiftPressed n = case n of
   KeycodePageDown   -> K.PgDn
   KeycodeInsert     -> K.Insert
   KeycodeDelete     -> K.Delete
+  KeycodePrintScreen -> K.PrintScreen
   KeycodeKPDivide   -> K.KP '/'
   KeycodeKPMultiply -> K.KP '*'
   KeycodeKPMinus    -> K.Char '-'  -- KP and normal are merged here
