@@ -6,14 +6,14 @@ module Game.LambdaHack.Client.UI.HandleHumanLocalM
   ( -- * Meta commands
     macroHuman
     -- * Local commands
-  , clearHuman, sortSlotsHuman, chooseItemHuman, chooseItemDialogMode
+  , sortSlotsHuman, chooseItemHuman, chooseItemDialogMode
   , chooseItemProjectHuman, chooseItemApplyHuman
   , psuitReq, triggerSymbols, permittedApplyClient
   , pickLeaderHuman, pickLeaderWithPointerHuman
   , memberCycleHuman, memberBackHuman
   , selectActorHuman, selectNoneHuman, selectWithPointerHuman
   , repeatHuman, recordHuman, historyHuman
-  , markVisionHuman, markSmellHuman, markSuspectHuman
+  , markVisionHuman, markSmellHuman, markSuspectHuman, printScreenHuman
     -- * Commands specific to aiming
   , cancelHuman, acceptHuman, tgtClearHuman, itemClearHuman
   , moveXhairHuman, aimTgtHuman, aimFloorHuman, aimEnemyHuman, aimItemHuman
@@ -35,7 +35,6 @@ import Game.LambdaHack.Common.Prelude
 
 import qualified Data.EnumMap.Strict as EM
 import qualified Data.EnumSet as ES
-import qualified Data.Map.Strict as M
 import           Data.Ord
 import qualified Data.Text as T
 import qualified NLP.Miniutter.English as MU
@@ -50,7 +49,7 @@ import           Game.LambdaHack.Client.UI.DrawM
 import           Game.LambdaHack.Client.UI.EffectDescription
 import           Game.LambdaHack.Client.UI.FrameM
 import           Game.LambdaHack.Client.UI.HandleHelperM
-import           Game.LambdaHack.Client.UI.HumanCmd (Trigger (..))
+import           Game.LambdaHack.Client.UI.HumanCmd
 import           Game.LambdaHack.Client.UI.InventoryM
 import           Game.LambdaHack.Client.UI.ItemDescription
 import           Game.LambdaHack.Client.UI.ItemSlot
@@ -67,8 +66,8 @@ import           Game.LambdaHack.Common.Actor
 import           Game.LambdaHack.Common.ActorState
 import           Game.LambdaHack.Common.Faction
 import           Game.LambdaHack.Common.Item
-import           Game.LambdaHack.Common.ItemStrongest
-import qualified Game.LambdaHack.Common.Kind as Kind
+import qualified Game.LambdaHack.Common.ItemAspect as IA
+import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Level
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.MonadStateRead
@@ -81,7 +80,6 @@ import           Game.LambdaHack.Common.Time
 import           Game.LambdaHack.Common.Vector
 import qualified Game.LambdaHack.Content.ItemKind as IK
 import           Game.LambdaHack.Content.ModeKind (fhasGender)
-import           Game.LambdaHack.Content.TileKind (isUknownSpace)
 
 -- * Macro
 
@@ -90,18 +88,7 @@ macroHuman kms = do
   modifySession $ \sess -> sess {slastPlay = map K.mkKM kms ++ slastPlay sess}
   UIOptions{uRunStopMsgs} <- getsSession sUIOptions
   when uRunStopMsgs $
-    promptAdd $ "Macro activated:" <+> T.pack (intercalate " " kms)
-
--- * Clear
-
--- | Clear current messages, cycle key hints mode.
-clearHuman :: MonadClientUI m => m ()
-clearHuman = do
-  keysHintMode <- getsSession skeysHintMode
-  when (keysHintMode == KeysHintPresent) historyHuman
-  modifySession $ \sess -> sess {skeysHintMode =
-    let n = fromEnum (skeysHintMode sess) + 1
-    in toEnum $ if n > fromEnum (maxBound :: KeysHintMode) then 0 else n}
+    promptAdd1 $ "Macro activated:" <+> T.pack (intercalate " " kms)
 
 -- * SortSlots
 
@@ -110,7 +97,7 @@ sortSlotsHuman = do
   leader <- getLeaderUI
   b <- getsState $ getActorBody leader
   sortSlots (bfid b) (Just b)
-  promptAdd "Items sorted by kind and stats."
+  promptAdd1 "Items sorted by kind and stats."
 
 -- * ChooseItem
 
@@ -140,7 +127,7 @@ chooseItemDialogMode c = do
               $ MU.SubjectVerbSg (subject bodyUI) (verbSha body ar)
             , MU.Text tIn
             , MU.Text t ]
-        MStore COrgan ->
+        MOrgans ->
           makePhrase
             [ MU.Capitalize $ MU.SubjectVerbSg (subject bodyUI) "feel"
             , MU.Text tIn
@@ -154,14 +141,10 @@ chooseItemDialogMode c = do
           makePhrase
             [ MU.Capitalize $ MU.SubjectVerbSg (subject bodyUI) "estimate"
             , MU.WownW (MU.Text $ bpronoun bodyUI) $ MU.Text t ]
-        MLoreItem ->
+        MLore{} ->
           makePhrase
             [ MU.Capitalize $ MU.SubjectVerbSg (subject bodyUI) "recall"
-            , MU.WownW (MU.Text $ bpronoun bodyUI) $ MU.Text t ]
-        MLoreOrgan ->
-          makePhrase
-            [ MU.Capitalize $ MU.SubjectVerbSg (subject bodyUI) "recall"
-            , MU.WownW (MU.Text $ bpronoun bodyUI) $ MU.Text t ]
+            , MU.Text t ]
         _ ->
           makePhrase
             [ MU.Capitalize $ MU.SubjectVerbSg (subject bodyUI) "see"
@@ -169,38 +152,51 @@ chooseItemDialogMode c = do
             , MU.WownW (MU.Text $ bpronoun bodyUI) $ MU.Text t ]
   ggi <- getStoreItem prompt c
   recordHistory  -- item chosen, wipe out already shown msgs
+  lidV <- viewedLevelUI
+  leader <- getLeaderUI
+  b <- getsState $ getActorBody leader
+  bUI <- getsSession $ getActorUI leader
+  itemToF <- getsState $ flip itemToFull
+  localTime <- getsState $ getLocalTime (blid b)
+  factionD <- getsState sfactionD
+  ar <- getsState $ getActorAspect leader
+  Level{lxsize, lysize} <- getLevel lidV
   case ggi of
-    (Right (iid, itemFull), (c2, _)) -> do
-      leader <- getLeaderUI
-      b <- getsState $ getActorBody leader
-      bUI <- getsSession $ getActorUI leader
-      let displayLore store prompt2 = do
-            promptAdd prompt2
-            lidV <- viewedLevelUI
-            Level{lxsize, lysize} <- getLevel lidV
-            localTime <- getsState $ getLocalTime (blid b)
-            factionD <- getsState sfactionD
-            ar <- getsState $ getActorAspect leader
-            let attrLine = itemDesc (bfid b) factionD (aHurtMelee ar)
-                                    store localTime itemFull
+    (Right (iid, itemBag, lSlots), (c2, _)) -> do
+      let lSlotsElems = EM.elems lSlots
+          lSlotsBound = length lSlotsElems - 1
+          displayLore slotIndex promptFun = do
+            let iid2 = lSlotsElems !! slotIndex
+                itemFull2 = itemToF iid2
+                kit2 = itemBag EM.! iid2
+                attrLine = itemDesc True (bfid b) factionD (IA.aHurtMelee ar)
+                                    CGround localTime itemFull2 kit2
                 ov = splitAttrLine lxsize attrLine
-            slides <-
-              overlayToSlideshow (lysize + 1) [K.spaceKM, K.escKM] (ov, [])
-            km <- getConfirms ColorFull [K.spaceKM, K.escKM] slides
-            if km == K.spaceKM
-            then chooseItemDialogMode c2
-            else failWith "never mind"
+                keys = [K.spaceKM, K.escKM]
+                       ++ [K.upKM | slotIndex /= 0]
+                       ++ [K.downKM | slotIndex /= lSlotsBound]
+            promptAdd0 $ promptFun $ itemKind itemFull2
+            slides <- overlayToSlideshow (lysize + 1) keys (ov, [])
+            km <- getConfirms ColorFull keys slides
+            case K.key km of
+              K.Space -> chooseItemDialogMode c2
+              K.Up -> displayLore (slotIndex - 1) promptFun
+              K.Down -> displayLore (slotIndex + 1) promptFun
+              K.Esc -> failWith "never mind"
+              _ -> error $ "" `showFailure` km
+          ix0 = fromJust $ findIndex (== iid) lSlotsElems
       case c2 of
-        MStore COrgan -> do
-          let symbol = jsymbol (itemBase itemFull)
-              blurb | symbol == '+' = "temporary condition"
-                    | otherwise = "organ"
-              prompt2 = makeSentence [ partActor bUI, "can't choose"
-                                     , MU.AW blurb ]
-          displayLore COrgan prompt2
         MStore fromCStore -> do
-          modifySession $ \sess -> sess {sitemSel = Just (fromCStore, iid)}
+          modifySession $ \sess ->
+            sess {sitemSel = Just (iid, fromCStore, False)}
           return $ Right c2
+        MOrgans -> do
+          let blurb itemKind
+                | IK.isTmpCondition itemKind = "condition"
+                | otherwise = "organ"
+              prompt2 itemKind = makeSentence [ partActor bUI, "can't remove"
+                                              , MU.AW $ blurb itemKind ]
+          displayLore ix0 prompt2
         MOwned -> do
           found <- getsState $ findIid leader (bfid b) iid
           let (newAid, bestStore) = case leader `lookup` found of
@@ -208,7 +204,8 @@ chooseItemDialogMode c = do
                 Nothing -> case found of
                   (aid, (_, store)) : _ -> (aid, store)
                   [] -> error $ "" `showFailure` iid
-          modifySession $ \sess -> sess {sitemSel = Just (bestStore, iid)}
+          modifySession $ \sess ->
+            sess {sitemSel = Just (iid, bestStore, False)}
           arena <- getArenaUI
           b2 <- getsState $ getActorBody newAid
           fact <- getsState $ (EM.! bfid b2) . sfactionD
@@ -216,37 +213,48 @@ chooseItemDialogMode c = do
           if | blid b2 /= arena && autoDun ->
                failSer NoChangeDunLeader
              | otherwise -> do
+               -- We switch leader only here, not in lore screens, because
+               -- lore is only about inspecting items, no activation submenu.
                void $ pickLeader True newAid
                return $ Right c2
         MStats -> error $ "" `showFailure` ggi
-        MLoreItem -> displayLore CGround
-          (makeSentence [ MU.SubjectVerbSg (partActor bUI) "remember"
-                        , "item lore" ])
-        MLoreOrgan -> displayLore COrgan
-          (makeSentence [ MU.SubjectVerbSg (partActor bUI) "remember"
-                        , "organ lore" ])
+        MLore slore -> displayLore ix0 $ const $
+          makeSentence [ MU.SubjectVerbSg (partActor bUI) "remember"
+                       , MU.Text (ppSLore slore), "lore" ]
     (Left err, (MStats, ekm)) -> case ekm of
-      Right slot -> assert (err == "stats") $ do
-        let eqpSlot = statSlots !! fromJust (elemIndex slot allZeroSlots)
-        leader <- getLeaderUI
-        b <- getsState $ getActorBody leader
-        bUI <- getsSession $ getActorUI leader
-        ar <- getsState $ getActorAspect leader
-        let valueText = slotToDecorator eqpSlot b $ prEqpSlot eqpSlot ar
-            prompt2 = makeSentence
-              [ MU.WownW (partActor bUI) (MU.Text $ slotToName eqpSlot)
-              , "is", MU.Text valueText ]
-              <+> slotToDesc eqpSlot
-        go <- displaySpaceEsc ColorFull prompt2
-        if go
-        then chooseItemDialogMode MStats
-        else failWith "never mind"
+      Right slot0 -> assert (err == "stats") $ do
+        let statListBound = length statSlots - 1
+            displayOneStat slotIndex = do
+              let slot = allSlots !! slotIndex
+                  eqpSlot = statSlots !! fromJust (elemIndex slot allSlots)
+                  valueText =
+                    slotToDecorator eqpSlot b $ IA.prEqpSlot eqpSlot ar
+                  prompt2 = makeSentence
+                    [ MU.WownW (partActor bUI) (MU.Text $ slotToName eqpSlot)
+                    , "is", MU.Text valueText ]
+                  ov0 = indentSplitAttrLine lxsize $ textToAL
+                        $ slotToDesc eqpSlot
+                  keys = [K.spaceKM, K.escKM]
+                         ++ [K.upKM | slotIndex /= 0]
+                         ++ [K.downKM | slotIndex /= statListBound]
+              promptAdd0 prompt2
+              slides <- overlayToSlideshow (lysize + 1) keys (ov0, [])
+              km <- getConfirms ColorFull keys slides
+              case K.key km of
+                K.Space -> chooseItemDialogMode MStats
+                K.Up -> displayOneStat $ slotIndex - 1
+                K.Down -> displayOneStat $ slotIndex + 1
+                K.Esc -> failWith "never mind"
+                _ -> error $ "" `showFailure` km
+            slotIndex0 = fromMaybe (error "displayOneStat: illegal slot")
+                         $ elemIndex slot0 allSlots
+        displayOneStat slotIndex0
       Left _ -> failWith "never mind"
     (Left err, _) -> failWith err
 
 -- * ChooseItemProject
 
-chooseItemProjectHuman :: forall m. MonadClientUI m => [Trigger] -> m MError
+chooseItemProjectHuman :: forall m. MonadClientUI m => [TriggerItem] -> m MError
 chooseItemProjectHuman ts = do
   leader <- getLeaderUI
   b <- getsState $ getActorBody leader
@@ -257,38 +265,56 @@ chooseItemProjectHuman ts = do
              | otherwise = delete CSha cLegalRaw
       (verb1, object1) = case ts of
         [] -> ("aim", "item")
-        tr : _ -> (verb tr, object tr)
-  mpsuitReq <- psuitReq ts
+        tr : _ -> (tiverb tr, tiobject tr)
+      triggerSyms = triggerSymbols ts
+  mpsuitReq <- psuitReq
   case mpsuitReq of
     -- If xhair aim invalid, no item is considered a (suitable) missile.
     Left err -> failMsg err
     Right psuitReqFun -> do
-      let psuit =
-            return $ SuitsSomething $ either (const False) snd . psuitReqFun
-          prompt = makePhrase ["What", object1, "to", verb1]
-          promptGeneric = "What to fling"
-      ggi <- getGroupItem psuit prompt promptGeneric cLegalRaw cLegal
-      case ggi of
-        Right ((iid, _itemFull), (MStore fromCStore, _)) -> do
-          modifySession $ \sess -> sess {sitemSel = Just (fromCStore, iid)}
-          return Nothing
-        Left err -> failMsg err
-        _ -> error $ "" `showFailure` ggi
+      itemSel <- getsSession sitemSel
+      case itemSel of
+        Just (_, _, True) -> return Nothing
+        Just (iid, fromCStore, False) -> do
+          itemFull <- getsState $ itemToFull iid
+          bag <- getsState $ getBodyStoreBag b fromCStore
+          case iid `EM.lookup` bag of
+            Just _ | either (const False) snd (psuitReqFun itemFull) ->
+              return Nothing
+            _ -> do
+              modifySession $ \sess -> sess {sitemSel = Nothing}
+              chooseItemProjectHuman ts
+        Nothing -> do
+          let psuit =
+                return $ SuitsSomething $ \itemFull _kit ->
+                  either (const False) snd (psuitReqFun itemFull)
+                  && (null triggerSyms
+                      || IK.isymbol (itemKind itemFull) `elem` triggerSyms)
+              prompt = makePhrase ["What", object1, "to", verb1]
+              promptGeneric = "What to fling"
+          ggi <- getGroupItem psuit prompt promptGeneric cLegalRaw cLegal
+          case ggi of
+            Right ((iid, _itemFull), (MStore fromCStore, _)) -> do
+              modifySession $ \sess ->
+                sess {sitemSel = Just (iid, fromCStore, False)}
+              return Nothing
+            Left err -> failMsg err
+            _ -> error $ "" `showFailure` ggi
 
 permittedProjectClient :: MonadClientUI m
-                       => [Char] -> m (ItemFull -> Either ReqFailure Bool)
-permittedProjectClient triggerSyms = do
+                       => m (ItemFull -> Either ReqFailure Bool)
+permittedProjectClient = do
   leader <- getLeaderUI
   b <- getsState $ getActorBody leader
   ar <- getsState $ getActorAspect leader
   actorSk <- leaderSkillsClientUI
   let skill = EM.findWithDefault 0 AbProject actorSk
       calmE = calmEnough b ar
-  return $ permittedProject False skill calmE triggerSyms
+  return $ permittedProject False skill calmE
 
 projectCheck :: MonadClientUI m => Point -> m (Maybe ReqFailure)
 projectCheck tpos = do
-  Kind.COps{coTileSpeedup} <- getsState scops
+  COps{coTileSpeedup} <- getsState scops
   leader <- getLeaderUI
   eps <- getsClient seps
   sb <- getsState $ getActorBody leader
@@ -373,10 +399,11 @@ posFromXhair = do
             Just reqFail -> return $ Left $ showReqFailure reqFail
     Left cause -> return $ Left cause
 
+-- | On top of @permittedProjectClient@, it also checks LOS, legality
+-- of aiming at the target, projection range.
 psuitReq :: MonadClientUI m
-         => [Trigger]
-         -> m (Either Text (ItemFull -> Either ReqFailure (Point, Bool)))
-psuitReq ts = do
+         => m (Either Text (ItemFull -> Either ReqFailure (Point, Bool)))
+psuitReq = do
   leader <- getLeaderUI
   b <- getsState $ getActorBody leader
   lidV <- viewedLevelUI
@@ -384,24 +411,24 @@ psuitReq ts = do
   then return $ Left "can't project on remote levels"
   else do
     mpos <- posFromXhair
-    p <- permittedProjectClient $ triggerSymbols ts
+    p <- permittedProjectClient
     case mpos of
       Left err -> return $ Left err
-      Right pos -> return $ Right $ \itemFull@ItemFull{itemBase} ->
+      Right pos -> return $ Right $ \itemFull ->
         case p itemFull of
           Left err -> Left err
           Right False -> Right (pos, False)
           Right True ->
-            Right (pos, totalRange itemBase >= chessDist (bpos b) pos)
+            Right (pos, IK.totalRange (itemKind itemFull)
+                        >= chessDist (bpos b) pos)
 
-triggerSymbols :: [Trigger] -> [Char]
+triggerSymbols :: [TriggerItem] -> [Char]
 triggerSymbols [] = []
-triggerSymbols (ApplyItem{symbol} : ts) = symbol : triggerSymbols ts
-triggerSymbols (_ : ts) = triggerSymbols ts
+triggerSymbols (TriggerItem{tisymbols} : ts) = tisymbols ++ triggerSymbols ts
 
 -- * ChooseItemApply
 
-chooseItemApplyHuman :: forall m. MonadClientUI m => [Trigger] -> m MError
+chooseItemApplyHuman :: forall m. MonadClientUI m => [TriggerItem] -> m MError
 chooseItemApplyHuman ts = do
   leader <- getLeaderUI
   b <- getsState $ getActorBody leader
@@ -412,24 +439,43 @@ chooseItemApplyHuman ts = do
              | otherwise = delete CSha cLegalRaw
       (verb1, object1) = case ts of
         [] -> ("apply", "item")
-        tr : _ -> (verb tr, object tr)
+        tr : _ -> (tiverb tr, tiobject tr)
+      triggerSyms = triggerSymbols ts
       prompt = makePhrase ["What", object1, "to", verb1]
       promptGeneric = "What to apply"
-      psuit :: m Suitability
-      psuit = do
-        mp <- permittedApplyClient $ triggerSymbols ts
-        return $ SuitsSomething $ either (const False) id . mp
-  ggi <- getGroupItem psuit prompt promptGeneric cLegalRaw cLegal
-  case ggi of
-    Right ((iid, _itemFull), (MStore fromCStore, _)) -> do
-      modifySession $ \sess -> sess {sitemSel = Just (fromCStore, iid)}
-      return Nothing
-    Left err -> failMsg err
-    _ -> error $ "" `showFailure` ggi
+  itemSel <- getsSession sitemSel
+  case itemSel of
+    Just (_, _, True) -> return Nothing
+    Just (iid, fromCStore, False) -> do
+      itemFull <- getsState $ itemToFull iid
+      bag <- getsState $ getBodyStoreBag b fromCStore
+      mp <- permittedApplyClient
+      case iid `EM.lookup` bag of
+        Just kit | either (const False) id (mp itemFull kit) ->
+          return Nothing
+        _ -> do
+          modifySession $ \sess -> sess {sitemSel = Nothing}
+          chooseItemApplyHuman ts
+    Nothing -> do
+      let psuit :: m Suitability
+          psuit = do
+            mp <- permittedApplyClient
+            return $ SuitsSomething $ \itemFull kit ->
+              either (const False) id (mp itemFull kit)
+              && (null triggerSyms
+                  || IK.isymbol (itemKind itemFull) `elem` triggerSyms)
+      ggi <- getGroupItem psuit prompt promptGeneric cLegalRaw cLegal
+      case ggi of
+        Right ((iid, _itemFull), (MStore fromCStore, _)) -> do
+          modifySession $ \sess ->
+            sess {sitemSel = Just (iid, fromCStore, False)}
+          return Nothing
+        Left err -> failMsg err
+        _ -> error $ "" `showFailure` ggi
 
 permittedApplyClient :: MonadClientUI m
-                     => [Char] -> m (ItemFull -> Either ReqFailure Bool)
-permittedApplyClient triggerSyms = do
+                     => m (ItemFull -> ItemQuant -> Either ReqFailure Bool)
+permittedApplyClient = do
   leader <- getLeaderUI
   b <- getsState $ getActorBody leader
   ar <- getsState $ getActorAspect leader
@@ -437,7 +483,7 @@ permittedApplyClient triggerSyms = do
   let skill = EM.findWithDefault 0 AbApply actorSk
       calmE = calmEnough b ar
   localTime <- getsState $ getLocalTime (blid b)
-  return $ permittedApply localTime skill calmE triggerSyms
+  return $ permittedApply localTime skill calmE
 
 -- * PickLeader
 
@@ -500,9 +546,9 @@ selectAid leader = do
             else ES.insert leader
   modifySession $ \sess -> sess {sselected = upd $ sselected sess}
   let subject = partActor bodyUI
-  promptAdd $ makeSentence [subject, if wasMemeber
-                                     then "deselected"
-                                     else "selected"]
+  promptAdd1 $ makeSentence [subject, if wasMemeber
+                                      then "deselected"
+                                      else "selected"]
 
 -- * SelectNone
 
@@ -519,9 +565,9 @@ selectNoneHuman = do
             else ES.difference
   modifySession $ \sess -> sess {sselected = upd (sselected sess) ours}
   let subject = "all party members on the level"
-  promptAdd $ makeSentence [subject, if wasNone
-                                     then "selected"
-                                     else "deselected"]
+  promptAdd1 $ makeSentence [subject, if wasNone
+                                      then "selected"
+                                      else "deselected"]
 
 -- * SelectWithPointer
 
@@ -567,19 +613,24 @@ maxK = 100
 
 recordHuman :: MonadClientUI m => m ()
 recordHuman = do
+  lastPlayOld <- getsSession slastPlay
   LastRecord _seqCurrent seqPrevious k <- getsSession slastRecord
   case k of
     0 -> do
       let slastRecord = LastRecord [] [] maxK
       modifySession $ \sess -> sess {slastRecord}
-      promptAdd $ "Macro will be recorded for up to"
-                  <+> tshow maxK
-                  <+> "actions. Stop recording with the same key."
+      when (null lastPlayOld) $
+        -- Don't spam if recording is a part of playing back a macro.
+        promptAdd0 $ "Macro will be recorded for up to"
+                     <+> tshow maxK
+                     <+> "actions. Stop recording with the same key."
     _ -> do
       let slastRecord = LastRecord seqPrevious [] 0
       modifySession $ \sess -> sess {slastRecord}
-      promptAdd $ "Macro recording stopped after"
-                  <+> tshow (maxK - k - 1) <+> "actions."
+      when (null lastPlayOld) $
+        -- Don't spam if recording is a part of playing back a macro.
+        promptAdd0 $ "Macro recording stopped after"
+                     <+> tshow (maxK - k - 1) <+> "actions."
 
 -- * History
 
@@ -600,42 +651,39 @@ historyHuman = do
         , MU.Text (tshow turnsLocal) <> ")" ]
       kxs = [ (Right sn, (slotPrefix sn, 0, lxsize))
             | sn <- take (length rh) intSlots ]
-  promptAdd msg
+  promptAdd0 msg
   okxs <- overlayToSlideshow (lysize + 3) [K.escKM] (rh, kxs)
   let displayAllHistory = do
-        menuIxMap <- getsSession smenuIxMap
-        let menuName = "history"
-            menuIx = fromMaybe 0 (M.lookup menuName menuIxMap)
-        (ekm, pointer) <-
-          displayChoiceScreen ColorFull True menuIx okxs [K.escKM]
-        modifySession $ \sess ->
-          sess {smenuIxMap = M.insert menuName pointer menuIxMap}
+        ekm <- displayChoiceScreen "history" ColorFull True okxs
+                                   [K.spaceKM, K.escKM]
         case ekm of
           Left km | km == K.escKM ->
-            promptAdd "Try to survive a few seconds more, if you can."
+            promptAdd0 "Try to survive a few seconds more, if you can."
+          Left km | km == K.spaceKM ->  -- click in any unused space
+            promptAdd0 "Steady on."
           Right SlotChar{..} | slotChar == 'a' ->
             displayOneReport slotPrefix
           _ -> error $ "" `showFailure` ekm
+      histBound = lengthHistory history - 1
       displayOneReport :: Int -> m ()
       displayOneReport histSlot = do
         let timeReport = case drop histSlot rh of
               [] -> error $ "" `showFailure` histSlot
               tR : _ -> tR
-            ov0 = splitReportForHistory lxsize timeReport
+            ov0 = indentSplitAttrLine lxsize timeReport
             prompt = makeSentence
               [ "the", MU.Ordinal $ histSlot + 1
               , "record of all history follows" ]
-            histBound = lengthHistory history - 1
             keys = [K.spaceKM, K.escKM] ++ [K.upKM | histSlot /= 0]
                                         ++ [K.downKM | histSlot /= histBound]
-        promptAdd prompt
+        promptAdd0 prompt
         slides <- overlayToSlideshow (lysize + 1) keys (ov0, [])
         km <- getConfirms ColorFull keys slides
         case K.key km of
           K.Space -> displayAllHistory
           K.Up -> displayOneReport $ histSlot - 1
           K.Down -> displayOneReport $ histSlot + 1
-          K.Esc -> promptAdd "Try to learn from your previous mistakes."
+          K.Esc -> promptAdd0 "Try to learn from your previous mistakes."
           _ -> error $ "" `showFailure` km
   displayAllHistory
 
@@ -657,6 +705,10 @@ markSuspectHuman = do
   invalidateBfsAll
   modifyClient cycleMarkSuspect
 
+-- * PrintScreen
+printScreenHuman :: MonadClientUI m => m ()
+printScreenHuman = printScreen
+
 -- * Cancel
 
 -- | End aiming mode, rejecting the current position.
@@ -665,7 +717,7 @@ cancelHuman = do
   saimMode <- getsSession saimMode
   when (isJust saimMode) $ do
     clearAimMode
-    promptAdd "Target not set."
+    promptAdd1 "Target not set."
 
 -- * Accept
 
@@ -690,7 +742,7 @@ endAimingMsg = do
   (mtargetMsg, _) <- targetDescLeader leader
   let targetMsg = fromJust mtargetMsg
   subject <- partAidLeader leader
-  promptAdd $
+  promptAdd1 $
     makeSentence [MU.SubjectVerbSg subject "target", MU.Text targetMsg]
 
 -- * TgtClear
@@ -718,60 +770,18 @@ doLook = do
   case saimMode of
     Nothing -> return ()
     Just aimMode -> do
-      side <- getsClient sside
       leader <- getLeaderUI
       let lidV = aimLevelId aimMode
-      lvl <- getLevel lidV
       xhairPos <- xhairToPos
       per <- getPerFid lidV
       b <- getsState $ getActorBody leader
       let p = fromMaybe (bpos b) xhairPos
-      inhabitants <- getsState $ posToAssocs p lidV
-      sactorUI <- getsSession sactorUI
-      let inhabitantsUI =
-            map (\(aid2, b2) -> (aid2, b2, sactorUI EM.! aid2)) inhabitants
-      seps <- getsClient seps
-      mnewEps <- makeLine False b p seps
-      itemToF <- getsState itemToFull
-      factionD <- getsState sfactionD
-      s <- getState
-      let aims = isJust mnewEps
-          enemyMsg = case inhabitants of
-            [] -> ""
-            (_, body) : rest ->
-              let Item{jfid} = getItemBody (btrunk body) s
-                  bfact = factionD EM.! bfid body
-                  -- Even if it's the leader, give his proper name, not 'you'.
-                  subjects = map (\(_, _, bUI) -> partActor bUI)
-                                 inhabitantsUI
-                  subject = MU.WWandW subjects
-                  verb = "be here"
-                  factDesc = case jfid of
-                    Just tfid | tfid /= bfid body ->
-                      let dominatedBy = if bfid body == side
-                                        then "us"
-                                        else gname bfact
-                          tfact = factionD EM.! tfid
-                      in "Originally of" <+> gname tfact
-                         <> ", now fighting for" <+> dominatedBy <> "."
-                    _ | bfid body == side -> ""  -- just one of us
-                    _ | bproj body -> "Launched by" <+> gname bfact <> "."
-                    _ -> "One of" <+> gname bfact <> "."
-                  idesc = case itemDisco $ itemToF (btrunk body) (1, []) of
-                    Nothing -> ""  -- no details, only show the name
-                    Just ItemDisco{itemKind} -> IK.idesc itemKind
-                  -- If many actors (projectiles), only list names.
-                  desc = if not (null rest) then "" else factDesc <+> idesc
-                  pdesc = if desc == "" then "" else "(" <> desc <> ")"
-              in makeSentence [MU.SubjectVerbSg subject verb] <+> pdesc
           canSee = ES.member p (totalVisible per)
-          vis | isUknownSpace $ lvl `at` p = "that is"
-              | not canSee = "you remember"
-              | not aims = "you are aware of"
-              | otherwise = "you see"
       -- Show general info about current position.
-      lookMsg <- lookAt True vis canSee p leader enemyMsg
-      promptAdd lookMsg
+      tileBlurb <- lookAtTile canSee p leader lidV
+      actorsBlurb <- lookAtActors p lidV
+      itemsBlurb <- lookAtItems canSee p leader
+      promptAdd1 $! tileBlurb <+> actorsBlurb <+> itemsBlurb
 
 -- * MoveXhair
 
@@ -868,7 +878,7 @@ aimEnemyHuman = do
         TPoint (TEnemyPos _ permit) _ _ -> (permit, pickUnderXhair)
         _ -> (False, pickUnderXhair)  -- the sensible default is only-foes
       gtlt = gt ++ lt
-      isEnemy b = isAtWar fact (bfid b)
+      isEnemy b = isFoe side fact (bfid b)
                   && not (bproj b)
                   && bhp b > 0
       lf = filter (isEnemy . snd) gtlt
@@ -956,7 +966,7 @@ epsIncrHuman b = do
   flashAiming
   modifySession $ \sess -> sess {saimMode}
 
---- Flash the aiming line and path.
+-- Flash the aiming line and path.
 flashAiming :: MonadClientUI m => m ()
 flashAiming = do
   lidV <- viewedLevelUI
