@@ -4,7 +4,7 @@ module Game.LambdaHack.Client.UI.HumanCmd
   ( CmdCategory(..), categoryDescription
   , CmdArea(..), areaDescription
   , CmdTriple, HumanCmd(..)
-  , Trigger(..)
+  , TriggerItem(..), TriggerTile(..)
   ) where
 
 import Prelude ()
@@ -21,7 +21,7 @@ import           Game.LambdaHack.Common.Vector
 import qualified Game.LambdaHack.Content.TileKind as TK
 
 data CmdCategory =
-    CmdMainMenu | CmdItemMenu
+    CmdMainMenu | CmdDashboard | CmdItemMenu
   | CmdMove | CmdItem | CmdAim | CmdMeta | CmdMouse
   | CmdInternal | CmdNoHelp | CmdDebug | CmdMinimal
   deriving (Show, Read, Eq, Generic)
@@ -31,15 +31,16 @@ instance NFData CmdCategory
 instance Binary CmdCategory
 
 categoryDescription :: CmdCategory -> Text
-categoryDescription CmdMainMenu = "Main Menu"
-categoryDescription CmdItemMenu = "Item Menu commands"
+categoryDescription CmdMainMenu = "Main menu"
+categoryDescription CmdDashboard = "Dashboard"
+categoryDescription CmdItemMenu = "Item menu commands"
 categoryDescription CmdMove = "Terrain exploration and alteration"
 categoryDescription CmdItem = "Remaining item-related commands"
 categoryDescription CmdAim = "Aiming"
 categoryDescription CmdMeta = "Assorted"
 categoryDescription CmdMouse = "Mouse"
 categoryDescription CmdInternal = "Internal"
-categoryDescription CmdNoHelp = "Ignored in Help"
+categoryDescription CmdNoHelp = "Ignored in help"
 categoryDescription CmdDebug = "Debug"
 categoryDescription CmdMinimal = "The minimal command set"
 
@@ -93,11 +94,11 @@ data HumanCmd =
     Macro [String]
   | ByArea [(CmdArea, HumanCmd)]  -- if outside the areas, do nothing
   | ByAimMode {exploration :: HumanCmd, aiming :: HumanCmd}
-  | ByItemMode {ts :: [Trigger], notChosen :: HumanCmd, chosen :: HumanCmd}
   | ComposeIfLocal HumanCmd HumanCmd
   | ComposeUnlessError HumanCmd HumanCmd
   | Compose2ndLocal HumanCmd HumanCmd
   | LoopOnNothing HumanCmd
+  | ExecuteIfClear HumanCmd
     -- Global.
     -- These usually take time.
   | Wait
@@ -109,13 +110,15 @@ data HumanCmd =
   | RunOnceToXhair
   | ContinueToXhair
   | MoveItem [CStore] CStore (Maybe MU.Part) Bool
-  | Project [Trigger]
-  | Apply [Trigger]
-  | AlterDir [Trigger]
-  | AlterWithPointer [Trigger]
+  | Project
+  | Apply
+  | AlterDir [TriggerTile]
+  | AlterWithPointer [TriggerTile]
   | Help
+  | Hint
   | ItemMenu
   | MainMenu
+  | Dashboard
     -- Below this line, commands do not take time.
   | GameDifficultyIncr
   | GameWolfToggle
@@ -127,12 +130,11 @@ data HumanCmd =
   | Tactic
   | Automate
     -- Local. Below this line, commands do not notify the server.
-  | Clear
   | SortSlots
   | ChooseItem ItemDialogMode
   | ChooseItemMenu ItemDialogMode
-  | ChooseItemProject [Trigger]
-  | ChooseItemApply [Trigger]
+  | ChooseItemProject [TriggerItem]
+  | ChooseItemApply [TriggerItem]
   | PickLeader Int
   | PickLeaderWithPointer
   | MemberCycle
@@ -148,6 +150,7 @@ data HumanCmd =
   | MarkSuspect
   | SettingsMenu
   | ChallengesMenu
+  | PrintScreen
     -- These are mostly related to aiming.
   | Cancel
   | Accept
@@ -175,14 +178,26 @@ instance Binary HumanCmd
 
 -- | Description of how item manipulation is triggered and communicated
 -- to the player.
-data Trigger =
-    ApplyItem {verb :: MU.Part, object :: MU.Part, symbol :: Char}
-  | AlterFeature {verb :: MU.Part, object :: MU.Part, feature :: TK.Feature}
+data TriggerItem =
+  TriggerItem {tiverb :: MU.Part, tiobject :: MU.Part, tisymbols :: [Char]}
   deriving (Show, Eq, Ord, Generic)
 
-instance Read Trigger where
-  readsPrec = error $ "parsing of Trigger not implemented" `showFailure` ()
+instance Read TriggerItem where
+  readsPrec = error $ "parsing of TriggerItem not implemented" `showFailure` ()
 
-instance NFData Trigger
+instance NFData TriggerItem
 
-instance Binary Trigger
+instance Binary TriggerItem
+
+-- | Description of how tile altering is triggered and communicated
+-- to the player.
+data TriggerTile =
+  TriggerTile  {ttverb :: MU.Part, ttobject :: MU.Part, ttfeature :: TK.Feature}
+  deriving (Show, Eq, Ord, Generic)
+
+instance Read TriggerTile where
+  readsPrec = error $ "parsing of TriggerTile not implemented" `showFailure` ()
+
+instance NFData TriggerTile
+
+instance Binary TriggerTile
