@@ -1,12 +1,8 @@
-{-# LANGUAGE DataKinds #-}
 -- | UI of inventory management.
 module Game.LambdaHack.Client.UI.InventoryM
   ( Suitability(..)
   , getFull, getGroupItem, getStoreItem
   , ppItemDialogMode, ppItemDialogModeFrom
-#ifdef EXPOSE_INTERNAL
-  , storeFromMode
-#endif
   ) where
 
 import Prelude ()
@@ -16,7 +12,6 @@ import Game.LambdaHack.Common.Prelude
 import qualified Data.Char as Char
 import           Data.Either
 import qualified Data.EnumMap.Strict as EM
-import qualified Data.Map.Strict as M
 import qualified Data.Text as T
 import           Data.Tuple (swap)
 import qualified NLP.Miniutter.English as MU
@@ -28,7 +23,6 @@ import           Game.LambdaHack.Client.UI.HandleHelperM
 import           Game.LambdaHack.Client.UI.HumanCmd
 import           Game.LambdaHack.Client.UI.ItemSlot
 import qualified Game.LambdaHack.Client.UI.Key as K
-import           Game.LambdaHack.Client.UI.KeyBindings
 import           Game.LambdaHack.Client.UI.MonadClientUI
 import           Game.LambdaHack.Client.UI.MsgM
 import           Game.LambdaHack.Client.UI.Overlay
@@ -39,6 +33,7 @@ import           Game.LambdaHack.Common.Actor
 import           Game.LambdaHack.Common.ActorState
 import           Game.LambdaHack.Common.Faction
 import           Game.LambdaHack.Common.Item
+import qualified Game.LambdaHack.Common.ItemAspect as IA
 import           Game.LambdaHack.Common.Level
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.MonadStateRead
@@ -50,10 +45,10 @@ data ItemDialogState = ISuitable | IAll
 
 ppItemDialogMode :: ItemDialogMode -> (Text, Text)
 ppItemDialogMode (MStore cstore) = ppCStore cstore
+ppItemDialogMode MOrgans = ("in", "body")
 ppItemDialogMode MOwned = ("in", "our possession")
 ppItemDialogMode MStats = ("among", "strengths")
-ppItemDialogMode MLoreItem = ("among", "item lore")
-ppItemDialogMode MLoreOrgan = ("among", "organ lore")
+ppItemDialogMode (MLore slore) = ("among", ppSLore slore <+> "lore")
 
 ppItemDialogModeIn :: ItemDialogMode -> Text
 ppItemDialogModeIn c = let (tIn, t) = ppItemDialogMode c in tIn <+> t
@@ -61,22 +56,15 @@ ppItemDialogModeIn c = let (tIn, t) = ppItemDialogMode c in tIn <+> t
 ppItemDialogModeFrom :: ItemDialogMode -> Text
 ppItemDialogModeFrom c = let (_tIn, t) = ppItemDialogMode c in "from" <+> t
 
-storeFromMode :: ItemDialogMode -> CStore
-storeFromMode c = case c of
-  MStore cstore -> cstore
-  MOwned -> CGround  -- needed to decide display mode in textAllAE
-  MStats -> CGround
-  MLoreItem -> CGround
-  MLoreOrgan -> COrgan
-
 accessModeBag :: ActorId -> State -> ItemDialogMode -> ItemBag
 accessModeBag leader s (MStore cstore) = let b = getActorBody leader s
                                          in getBodyStoreBag b cstore s
+accessModeBag leader s MOrgans = let b = getActorBody leader s
+                                 in getBodyStoreBag b COrgan s
 accessModeBag leader s MOwned = let fid = bfid $ getActorBody leader s
-                                in sharedAllOwnedFid False fid s
+                                in combinedItems fid s
 accessModeBag _ _ MStats = EM.empty
-accessModeBag _ s MLoreItem = EM.map (const (1, [])) $ sitemD s
-accessModeBag _ s MLoreOrgan = EM.map (const (1, [])) $ sitemD s
+accessModeBag _ s MLore{} = EM.map (const (1, [])) $ sitemD s
 
 -- | Let a human player choose any item from a given group.
 -- Note that this does not guarantee the chosen item belongs to the group,
@@ -99,22 +87,24 @@ getGroupItem psuit prompt promptGeneric
                  cLegalRaw cLegalAfterCalm True False
   case soc of
     Left err -> return $ Left err
-    Right ([(iid, itemFull)], cekm) -> return $ Right ((iid, itemFull), cekm)
+    Right ([(iid, (itemFull, _))], cekm) ->
+      return $ Right ((iid, itemFull), cekm)
     Right _ -> error $ "" `showFailure` soc
 
 -- | Display all items from a store and let the human player choose any
 -- or switch to any other store.
 -- Used, e.g., for viewing inventory and item descriptions.
 getStoreItem :: MonadClientUI m
-             => (Actor -> ActorUI -> AspectRecord -> ItemDialogMode -> Text)
+             => (Actor -> ActorUI -> IA.AspectRecord -> ItemDialogMode -> Text)
                                  -- ^ how to describe suitable items
              -> ItemDialogMode   -- ^ initial mode
-             -> m ( Either Text (ItemId, ItemFull)
+             -> m ( Either Text (ItemId, ItemBag, SingleItemSlots)
                   , (ItemDialogMode, Either K.KM SlotChar) )
 getStoreItem prompt cInitial = do
   let itemCs = map MStore [CInv, CGround, CEqp, CSha]
-      allCs | cInitial `elem` [MLoreItem, MLoreOrgan] = [MLoreItem, MLoreOrgan]
-            | otherwise = itemCs ++ [MOwned, MStore COrgan, MStats]
+      allCs = case cInitial of
+        MLore{} -> map MLore [minBound..maxBound]
+        _ -> itemCs ++ [MOwned, MOrgans, MStats]
       (pre, rest) = break (== cInitial) allCs
       post = dropWhile (== cInitial) rest
       remCs = post ++ pre
@@ -123,7 +113,8 @@ getStoreItem prompt cInitial = do
                  True False (cInitial : remCs)
   case soc of
     (Left err, cekm) -> return (Left err, cekm)
-    (Right [(iid, itemFull)], cekm) -> return (Right (iid, itemFull), cekm)
+    (Right ([iid], itemBag, lSlots), cekm) ->
+      return (Right (iid, itemBag, lSlots), cekm)
     (Right{}, _) -> error $ "" `showFailure` soc
 
 -- | Let the human player choose a single, preferably suitable,
@@ -131,16 +122,16 @@ getStoreItem prompt cInitial = do
 -- Start with a non-empty store.
 getFull :: MonadClientUI m
         => m Suitability    -- ^ which items to consider suitable
-        -> (Actor -> ActorUI -> AspectRecord -> ItemDialogMode -> Text)
+        -> (Actor -> ActorUI -> IA.AspectRecord -> ItemDialogMode -> Text)
                             -- ^ specific prompt for only suitable items
-        -> (Actor -> ActorUI -> AspectRecord -> ItemDialogMode -> Text)
+        -> (Actor -> ActorUI -> IA.AspectRecord -> ItemDialogMode -> Text)
                             -- ^ generic prompt
         -> [CStore]         -- ^ initial legal modes
         -> [CStore]         -- ^ legal modes with Calm taken into account
         -> Bool             -- ^ whether to ask, when the only item
                             --   in the starting mode is suitable
         -> Bool             -- ^ whether to permit multiple items as a result
-        -> m (Either Text ( [(ItemId, ItemFull)]
+        -> m (Either Text ( [(ItemId, ItemFullKit)]
                           , (ItemDialogMode, Either K.KM SlotChar) ))
 getFull psuit prompt promptGeneric cLegalRaw cLegalAfterCalm
         askWhenLone permitMulitple = do
@@ -156,7 +147,7 @@ getFull psuit prompt promptGeneric cLegalRaw cLegalAfterCalm
         return $! or bs
   mpsuit <- psuit
   let psuitFun = case mpsuit of
-        SuitsEverything -> const True
+        SuitsEverything -> \_ _ -> True
         SuitsSomething f -> f
   -- Move the first store that is non-empty for suitable items for this actor
   -- to the front, if any.
@@ -172,10 +163,10 @@ getFull psuit prompt promptGeneric cLegalRaw cLegalAfterCalm
         return $ Left $ "no items" <+> ppLegal
       else return $ Left $ showReqFailure ItemNotCalm
     haveThis@(headThisActor : _) -> do
-      itemToF <- getsState itemToFull
+      itemToF <- getsState $ flip itemToFull
       let suitsThisActor store =
             let bag = getCStoreBag store
-            in any (\(iid, kit) -> psuitFun $ itemToF iid kit) $ EM.assocs bag
+            in any (\(iid, kit) -> psuitFun (itemToF iid) kit) $ EM.assocs bag
           firstStore = fromMaybe headThisActor $ find suitsThisActor haveThis
       -- Don't display stores totally empty for all actors.
       cLegal <- filterM partyNotEmpty cLegalRaw
@@ -187,17 +178,19 @@ getFull psuit prompt promptGeneric cLegalRaw cLegalAfterCalm
       res <- getItem psuit prompt promptGeneric modeFirst modeRest
                      askWhenLone permitMulitple (map MStore cLegal)
       case res of
-        (Left x, _) -> return $ Left x
-        (Right x, cekm) -> return $ Right (x, cekm)
+        (Left t, _) -> return $ Left t
+        (Right (iids, itemBag, _lSlots), cekm) -> do
+          let f iid = (iid, (itemToF iid, itemBag EM.! iid))
+          return $ Right (map f iids, cekm)
 
 -- | Let the human player choose a single, preferably suitable,
 -- item from a list of items.
 getItem :: MonadClientUI m
         => m Suitability
                             -- ^ which items to consider suitable
-        -> (Actor -> ActorUI -> AspectRecord -> ItemDialogMode -> Text)
+        -> (Actor -> ActorUI -> IA.AspectRecord -> ItemDialogMode -> Text)
                             -- ^ specific prompt for only suitable items
-        -> (Actor -> ActorUI -> AspectRecord -> ItemDialogMode -> Text)
+        -> (Actor -> ActorUI -> IA.AspectRecord -> ItemDialogMode -> Text)
                             -- ^ generic prompt
         -> ItemDialogMode   -- ^ first mode, legal or not
         -> [ItemDialogMode] -- ^ the (rest of) legal modes
@@ -205,7 +198,7 @@ getItem :: MonadClientUI m
                             --   in the starting mode is suitable
         -> Bool             -- ^ whether to permit multiple items as a result
         -> [ItemDialogMode] -- ^ all legal modes
-        -> m ( Either Text [(ItemId, ItemFull)]
+        -> m ( Either Text ([ItemId], ItemBag, SingleItemSlots)
              , (ItemDialogMode, Either K.KM SlotChar) )
 getItem psuit prompt promptGeneric cCur cRest askWhenLone permitMulitple
         cLegal = do
@@ -215,13 +208,12 @@ getItem psuit prompt promptGeneric cCur cRest askWhenLone permitMulitple
       allAssocs = concatMap storeAssocs (cCur : cRest)
   case (cRest, allAssocs) of
     ([], [(iid, k)]) | not askWhenLone -> do
-      itemToF <- getsState itemToFull
-      ItemSlots itemSlots organSlots <- getsSession sslots
-      let isOrgan = cCur `elem` [MStore COrgan, MLoreOrgan]
-          lSlots = if isOrgan then organSlots else itemSlots
+      ItemSlots itemSlots <- getsSession sslots
+      let lSlots = itemSlots EM.! loreFromMode cCur
           slotChar = fromMaybe (error $ "" `showFailure` (iid, lSlots))
                      $ lookup iid $ map swap $ EM.assocs lSlots
-      return (Right [(iid, itemToF iid k)], (cCur, Right slotChar))
+      return ( Right ([iid], EM.singleton iid k, EM.singleton slotChar iid)
+             , (cCur, Right slotChar) )
     _ ->
       transition psuit prompt promptGeneric permitMulitple cLegal
                  0 cCur cRest ISuitable
@@ -230,30 +222,30 @@ data DefItemKey m = DefItemKey
   { defLabel  :: Either Text K.KM
   , defCond   :: Bool
   , defAction :: Either K.KM SlotChar
-                 -> m ( Either Text [(ItemId, ItemFull)]
-                      , (ItemDialogMode, Either K.KM SlotChar) )
+              -> m ( Either Text ([ItemId], ItemBag, SingleItemSlots)
+                   , (ItemDialogMode, Either K.KM SlotChar) )
   }
 
 data Suitability =
     SuitsEverything
-  | SuitsSomething (ItemFull -> Bool)
+  | SuitsSomething (ItemFull -> ItemQuant -> Bool)
 
 transition :: forall m. MonadClientUI m
            => m Suitability
-           -> (Actor -> ActorUI -> AspectRecord -> ItemDialogMode -> Text)
-           -> (Actor -> ActorUI -> AspectRecord -> ItemDialogMode -> Text)
+           -> (Actor -> ActorUI -> IA.AspectRecord -> ItemDialogMode -> Text)
+           -> (Actor -> ActorUI -> IA.AspectRecord -> ItemDialogMode -> Text)
            -> Bool
            -> [ItemDialogMode]
            -> Int
            -> ItemDialogMode
            -> [ItemDialogMode]
            -> ItemDialogState
-           -> m ( Either Text [(ItemId, ItemFull)]
+           -> m ( Either Text ([ItemId], ItemBag, SingleItemSlots)
                 , (ItemDialogMode, Either K.KM SlotChar) )
 transition psuit prompt promptGeneric permitMulitple cLegal
            numPrefix cCur cRest itemDialogState = do
   let recCall = transition psuit prompt promptGeneric permitMulitple cLegal
-  ItemSlots itemSlots organSlots <- getsSession sslots
+  ItemSlots itemSlots <- getsSession sslots
   leader <- getLeaderUI
   body <- getsState $ getActorBody leader
   bodyUI <- getsSession $ getActorUI leader
@@ -261,27 +253,26 @@ transition psuit prompt promptGeneric permitMulitple cLegal
   fact <- getsState $ (EM.! bfid body) . sfactionD
   hs <- partyAfterLeader leader
   bagAll <- getsState $ \s -> accessModeBag leader s cCur
-  itemToF <- getsState itemToFull
-  Binding{brevMap} <- getsSession sbinding
+  itemToF <- getsState $ flip itemToFull
+  organPartySet <- getsState $ partyItemSet SOrgan (bfid body) (Just body)
+  revCmd <- revCmdMap
   mpsuit <- psuit  -- when throwing, this sets eps and checks xhair validity
   psuitFun <- case mpsuit of
-    SuitsEverything -> return $ const True
+    SuitsEverything -> return $ \_ _ -> True
     SuitsSomething f -> return f
       -- When throwing, this function takes missile range into accout.
-  let getSingleResult :: ItemId -> (ItemId, ItemFull)
-      getSingleResult iid = (iid, itemToF iid (bagAll EM.! iid))
-      getResult :: Either K.KM SlotChar -> ItemId
-                -> ( Either Text [(ItemId, ItemFull)]
+  let getResult :: Either K.KM SlotChar -> [ItemId]
+                -> ( Either Text ([ItemId], ItemBag, SingleItemSlots)
                    , (ItemDialogMode, Either K.KM SlotChar) )
-      getResult ekm iid = (Right [getSingleResult iid], (cCur, ekm))
-      getMultResult :: Either K.KM SlotChar -> [ItemId]
-                    -> ( Either Text [(ItemId, ItemFull)]
-                       , (ItemDialogMode, Either K.KM SlotChar) )
-      getMultResult ekm iids = (Right $ map getSingleResult iids, (cCur, ekm))
-      filterP iid kit = psuitFun $ itemToF iid kit
+      getResult ekm iids = (Right (iids, bagAll, bagItemSlotsAll), (cCur, ekm))
+      filterP iid = psuitFun (itemToF iid)
       bagAllSuit = EM.filterWithKey filterP bagAll
-      isOrgan = cCur `elem` [MStore COrgan, MLoreOrgan]
-      lSlots = if isOrgan then organSlots else itemSlots
+      lSlots = case cCur of
+        MOrgans -> mergeItemSlots itemToF organPartySet [ itemSlots EM.! SOrgan
+                                                        , itemSlots EM.! STrunk
+                                                        , itemSlots EM.! STmp ]
+        MStats -> EM.empty
+        _ -> itemSlots EM.! loreFromMode cCur
       bagItemSlotsAll = EM.filter (`EM.member` bagAll) lSlots
       -- Predicate for slot matching the current prefix, unless the prefix
       -- is 0, in which case we display all slots, even if they require
@@ -302,13 +293,15 @@ transition psuit prompt promptGeneric permitMulitple cLegal
       multipleSlots = if itemDialogState == IAll
                       then bagItemSlotsAll
                       else suitableItemSlotsAll
-      revCmd dflt cmd = case M.lookup cmd brevMap of
-        Nothing -> dflt
-        Just (k : _) -> k
-        Just [] -> error $ "" `showFailure` brevMap
+      maySwitchLeader MOwned = False
+      maySwitchLeader MLore{} = False
+      maySwitchLeader _ = True
       keyDefs :: [(K.KM, DefItemKey m)]
       keyDefs = filter (defCond . snd) $
-        [ let km = K.mkChar '?'
+        [ let km = K.mkChar '/'
+          in (km, changeContainerDef $ Right km)
+        , (K.mkKP '/', changeContainerDef $ Left "")
+        , let km = K.mkChar '?'
           in (km, DefItemKey
            { defLabel = Right km
            , defCond = bag /= bagSuit
@@ -317,17 +310,14 @@ transition psuit prompt promptGeneric permitMulitple cLegal
                                    ISuitable -> IAll
                                    IAll -> ISuitable
            })
-        , let km = K.mkChar '/'
-          in (km, changeContainerDef $ Right km)
-        , (K.mkKP '/', changeContainerDef $ Left "")
         , let km = K.mkChar '!'
           in (km, useMultipleDef $ Right km)
         , (K.mkKP '*', useMultipleDef $ Left "")
         , let km = revCmd (K.KM K.NoModifier K.Tab) MemberCycle
           in (km, DefItemKey
            { defLabel = Right km
-           , defCond = not (cCur == MOwned
-                            || not (any (\(_, b, _) -> blid b == blid body) hs))
+           , defCond = maySwitchLeader cCur
+                       && any (\(_, b, _) -> blid b == blid body) hs
            , defAction = \_ -> do
                err <- memberCycle False
                let !_A = assert (isNothing err `blame` err) ()
@@ -337,7 +327,7 @@ transition psuit prompt promptGeneric permitMulitple cLegal
         , let km = revCmd (K.KM K.NoModifier K.BackTab) MemberBack
           in (km, DefItemKey
            { defLabel = Right km
-           , defCond = not (cCur == MOwned || autoDun || null hs)
+           , defCond = maySwitchLeader cCur && not (autoDun || null hs)
            , defAction = \_ -> do
                err <- memberBack False
                let !_A = assert (isNothing err `blame` err) ()
@@ -346,16 +336,21 @@ transition psuit prompt promptGeneric permitMulitple cLegal
            })
         , (K.KM K.NoModifier K.LeftButtonRelease, DefItemKey
            { defLabel = Left ""
-           , defCond = not (cCur == MOwned || null hs)
-           , defAction = \_ -> do
-               void pickLeaderWithPointer  -- error ignored; update anyway
-               (cCurUpd, cRestUpd) <- legalWithUpdatedLeader cCur cRest
-               recCall numPrefix cCurUpd cRestUpd itemDialogState
+           , defCond = maySwitchLeader cCur && not (null hs)
+           , defAction = \_unused -> do
+               merror <- pickLeaderWithPointer
+               case merror of
+                 Nothing -> do
+                   (cCurUpd, cRestUpd) <- legalWithUpdatedLeader cCur cRest
+                   recCall numPrefix cCurUpd cRestUpd itemDialogState
+                 Just{} ->  -- don't inspect the error, it's expected
+                   defAction (changeContainerDef $ Left "") _unused
            })
         , let km = revCmd (K.KM K.NoModifier $ K.Char '^') SortSlots
           in (km, DefItemKey
-           { defLabel = if cCur == MOwned then Right km else Left ""
-           , defCond = True
+           { defLabel = Right km
+           , defCond = cCur /= MOrgans  -- auto-sorted each time
+                       && cCur /= MStats  -- artificial slots
            , defAction = \_ -> do
                sortSlots (bfid body) (Just body)
                recCall numPrefix cCur cRest itemDialogState
@@ -369,7 +364,7 @@ transition psuit prompt promptGeneric permitMulitple cLegal
         ++ numberPrefixes
       changeContainerDef defLabel = DefItemKey
         { defLabel
-        , defCond = not $ null cRest
+        , defCond = True  -- even if single screen, just reset it
         , defAction = \_ -> do
             let calmE = calmEnough body ar
                 mcCur = filter (`elem` cLegal) [cCur]
@@ -386,7 +381,7 @@ transition psuit prompt promptGeneric permitMulitple cLegal
         , defCond = permitMulitple && not (EM.null multipleSlots)
         , defAction = \ekm ->
             let eslots = EM.elems multipleSlots
-            in return $ getMultResult ekm eslots
+            in return $ getResult ekm eslots
         }
       prefixCmdDef d =
         (K.mkChar $ Char.intToDigit d, DefItemKey
@@ -410,7 +405,7 @@ transition psuit prompt promptGeneric permitMulitple cLegal
             in case EM.lookup slot bagItemSlotsAll of
               Nothing -> error $ "unexpected slot"
                                  `showFailure` (slot, bagItemSlots)
-              Just iid -> return $! getResult (Right slot) iid
+              Just iid -> return $! getResult (Right slot) [iid]
         }
       (bagFiltered, promptChosen) =
         case itemDialogState of
@@ -434,9 +429,9 @@ transition psuit prompt promptGeneric permitMulitple cLegal
                       Right sl -> sl
                 in return (Left "stats", (MStats, Right slot))
             }
-      runDefItemKey keyDefs statsDef io slotKeys promptChosen MStats
+      runDefItemKey keyDefs statsDef io slotKeys promptChosen cCur
     _ -> do
-      io <- itemOverlay (storeFromMode cCur) (blid body) bagFiltered
+      io <- itemOverlay lSlots (blid body) bagFiltered
       let slotKeys = mapMaybe (keyOfEKM numPrefix . Right)
                      $ EM.keys bagItemSlots
       runDefItemKey keyDefs lettersDef io slotKeys promptChosen cCur
@@ -473,7 +468,7 @@ runDefItemKey :: MonadClientUI m
               -> [K.KM]
               -> Text
               -> ItemDialogMode
-              -> m ( Either Text [(ItemId, ItemFull)]
+              -> m ( Either Text ([ItemId], ItemBag, SingleItemSlots)
                    , (ItemDialogMode, Either K.KM SlotChar) )
 runDefItemKey keyDefs lettersDef okx slotKeys prompt cCur = do
   let itemKeys = slotKeys ++ map fst keyDefs
@@ -481,28 +476,12 @@ runDefItemKey keyDefs lettersDef okx slotKeys prompt cCur = do
       (keyLabelsRaw, keys) = partitionEithers $ map (defLabel . snd) keyDefs
       keyLabels = filter (not . T.null) keyLabelsRaw
       choice = T.intercalate " " $ map wrapB $ nub keyLabels
-  promptAdd $ prompt <+> choice
+  promptAdd0 $ prompt <+> choice
   lidV <- viewedLevelUI
   Level{lysize} <- getLevel lidV
   ekm <- do
     okxs <- overlayToSlideshow (lysize + 1) keys okx
-    !lastSlot <- getsSession slastSlot
-    let allOKX = concatMap snd $ slideshow okxs
-        pointer =
-          case findIndex ((== Right lastSlot) . fst) allOKX of
-            Just p | cCur `notElem` [MStats, MLoreItem, MLoreOrgan] -> p
-            _ -> case findIndex (isRight . fst) allOKX of
-              Just p -> p
-              _ -> 0
-    (okm, pointer2) <- displayChoiceScreen ColorFull False pointer okxs itemKeys
-    -- Remember item pointer, unless not a proper item container. Remember
-    -- even if not moved, in case the initial position was a default.
-    case drop pointer2 allOKX of
-      (Right slastSlot, _) : _
-        | cCur `notElem` [MStats, MLoreItem, MLoreOrgan] ->
-        modifySession $ \sess -> sess {slastSlot}
-      _ -> return ()
-    return okm
+    displayChoiceScreen (show cCur) ColorFull False okxs itemKeys
   case ekm of
     Left km -> case km `lookup` keyDefs of
       Just keyDef -> defAction keyDef ekm
