@@ -7,8 +7,8 @@ module Game.LambdaHack.Client.UI.MonadClientUI
                  )
     -- * Assorted primitives
   , clientPrintUI, mapStartY, getSession, putSession, displayFrames
-  , connFrontendFrontKey, setFrontAutoYes, frontendShutdown, chanFrontend
-  , anyKeyPressed, discardPressedKey, addPressedEsc
+  , connFrontendFrontKey, setFrontAutoYes, frontendShutdown, printScreen
+  , chanFrontend, anyKeyPressed, discardPressedKey, addPressedEsc, revCmdMap
   , getReportUI, getLeaderUI, getArenaUI, viewedLevelUI
   , leaderTgtToPos, xhairToPos, clearXhair, clearAimMode
   , scoreToSlideshow, defaultHistory
@@ -27,6 +27,7 @@ import Prelude ()
 import Game.LambdaHack.Common.Prelude
 
 import qualified Data.EnumMap.Strict as EM
+import qualified Data.Map.Strict as M
 import qualified Data.Text as T
 import qualified Data.Text.IO as T
 import           Data.Time.Clock
@@ -44,7 +45,9 @@ import           Game.LambdaHack.Client.UI.ActorUI
 import           Game.LambdaHack.Client.UI.Frame
 import           Game.LambdaHack.Client.UI.Frontend
 import qualified Game.LambdaHack.Client.UI.Frontend as Frontend
+import qualified Game.LambdaHack.Client.UI.HumanCmd as HumanCmd
 import qualified Game.LambdaHack.Client.UI.Key as K
+import           Game.LambdaHack.Client.UI.KeyBindings
 import           Game.LambdaHack.Client.UI.Msg
 import           Game.LambdaHack.Client.UI.Overlay
 import           Game.LambdaHack.Client.UI.SessionUI
@@ -55,7 +58,7 @@ import           Game.LambdaHack.Common.ActorState
 import           Game.LambdaHack.Common.Faction
 import           Game.LambdaHack.Common.File
 import qualified Game.LambdaHack.Common.HighScore as HighScore
-import qualified Game.LambdaHack.Common.Kind as Kind
+import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Level
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.MonadStateRead
@@ -129,6 +132,9 @@ setFrontAutoYes b = connFrontend $ FrontAutoYes b
 frontendShutdown :: MonadClientUI m => m ()
 frontendShutdown = connFrontend FrontShutdown
 
+printScreen :: MonadClientUI m => m ()
+printScreen = connFrontend FrontPrintScreen
+
 -- | Initialize the frontend chosen by the player via client options.
 chanFrontend :: MonadClientUI m => ClientOptions -> m ChanFrontend
 chanFrontend = liftIO . Frontend.chanFrontendIO
@@ -146,14 +152,23 @@ addPressedEsc :: MonadClientUI m => m ()
 addPressedEsc = addPressedKey K.KMP { K.kmpKeyMod = K.escKM
                                     , K.kmpPointer = originPoint }
 
+revCmdMap :: MonadClientUI m => m (K.KM -> HumanCmd.HumanCmd -> K.KM)
+revCmdMap = do
+  Binding{brevMap} <- getsSession sbinding
+  let revCmd dflt cmd = case M.lookup cmd brevMap of
+        Nothing -> dflt
+        Just (k : _) -> k
+        Just [] -> error $ "" `showFailure` brevMap
+  return revCmd
+
 getReportUI :: MonadClientUI m => m Report
 getReportUI = do
-  report <- getsSession sreport
+  report <- getsSession $ newReport . shistory
   side <- getsClient sside
   fact <- getsState $ (EM.! side) . sfactionD
   let underAI = isAIFact fact
-      promptAI = toPrompt $ stringToAL "[press ESC for Main Menu]"
-  return $! if underAI then consReportNoScrub promptAI report else report
+      promptAI = toPrompt $ stringToAL "[press ESC for main menu]"
+  return $! if underAI then consReport promptAI report else report
 
 getLeaderUI :: MonadClientUI m => m ActorId
 getLeaderUI = do
@@ -197,7 +212,7 @@ leaderTgtToPos = do
       mtgt <- getsClient $ getTarget aid
       case mtgt of
         Nothing -> return Nothing
-        Just tgt -> aidTgtToPos aid lidV tgt
+        Just tgt -> getsState $ aidTgtToPos aid lidV tgt
 
 xhairToPos :: MonadClientUI m => m (Maybe Point)
 xhairToPos = do
@@ -206,7 +221,8 @@ xhairToPos = do
   sxhair <- getsSession sxhair
   case mleader of
     Nothing -> return Nothing  -- e.g., when game start and no leader yet
-    Just aid -> aidTgtToPos aid lidV sxhair  -- e.g., xhair on another level
+    Just aid -> getsState $ aidTgtToPos aid lidV sxhair
+                  -- e.g., xhair on another level
 
 -- Reset xhair and move it to actor's position.
 clearXhair :: MonadClientUI m => m ()
@@ -239,29 +255,30 @@ scoreToSlideshow total status = do
   lidV <- viewedLevelUI
   Level{lxsize, lysize} <- getLevel lidV
   fid <- getsClient sside
-  fact <- getsState $ (EM.! fid) . sfactionD
   scoreDict <- getsState shigh
   gameModeId <- getsState sgameModeId
   gameMode <- getGameMode
   time <- getsState stime
+  dungeonTotal <- getsState sgold
   date <- liftIO getPOSIXTime
   tz <- liftIO $ getTimeZone $ posixSecondsToUTCTime date
   curChalSer <- getsClient scurChal
   factionD <- getsState sfactionD
-  let table = HighScore.getTable gameModeId scoreDict
+  let fact = factionD EM.! fid
+      table = HighScore.getTable gameModeId scoreDict
       gameModeName = mname gameMode
       chal | fhasUI $ gplayer fact = curChalSer
            | otherwise = curChalSer
                            {cdiff = difficultyInverse (cdiff curChalSer)}
-      theirVic (fi, fa) | isAtWar fact fi
+      theirVic (fi, fa) | isFoe fid fact fi
                           && not (isHorrorFact fa) = Just $ gvictims fa
                         | otherwise = Nothing
       theirVictims = EM.unionsWith (+) $ mapMaybe theirVic $ EM.assocs factionD
-      ourVic (fi, fa) | isAllied fact fi || fi == fid = Just $ gvictims fa
+      ourVic (fi, fa) | isFriend fid fact fi = Just $ gvictims fa
                       | otherwise = Nothing
       ourVictims = EM.unionsWith (+) $ mapMaybe ourVic $ EM.assocs factionD
       (worthMentioning, (ntable, pos)) =
-        HighScore.register table total time status date chal
+        HighScore.register table total dungeonTotal time status date chal
                            (T.unwords $ tail $ T.words $ gname fact)
                            ourVictims theirVictims
                            (fhiCondPoly $ gplayer fact)
@@ -278,11 +295,10 @@ defaultHistory :: MonadClientUI m => Int -> m History
 defaultHistory uHistoryMax = liftIO $ do
   utcTime <- getCurrentTime
   timezone <- getTimeZone utcTime
-  let curDate = show $ utcToLocalTime timezone utcTime
+  let curDate = take 19 $ show $ utcToLocalTime timezone utcTime
       emptyHist = emptyHistory uHistoryMax
-  return $! addReport emptyHist timeZero
-         $ singletonReport $ toMsg $ stringToAL
-         $ "Human history log started on " ++ curDate ++ "."
+      msg = toMsg $ stringToAL $ "History log started on " ++ curDate ++ "."
+  return $! fst $ addToReport emptyHist msg 0
 
 tellAllClipPS :: MonadClientUI m => m ()
 tellAllClipPS = do
@@ -384,7 +400,7 @@ partAidLeader aid = do
 -- | Try to read saved client game state from the file system.
 tryRestore :: MonadClientUI m => m (Maybe (StateClient, Maybe SessionUI))
 tryRestore = do
-  cops@Kind.COps{corule} <- getsState scops
+  cops <- getsState scops
   bench <- getsClient $ sbenchmark . soptions
   if bench then return Nothing
   else do
@@ -392,7 +408,7 @@ tryRestore = do
     prefix <- getsClient $ ssavePrefixCli . soptions
     let fileName = prefix <> Save.saveNameCli cops side
     res <- liftIO $ Save.restoreGame cops fileName
-    let stdRuleset = Kind.stdRuleset corule
+    let stdRuleset = getStdRuleset cops
         cfgUIName = rcfgUIName stdRuleset
         content = rcfgUIDefault stdRuleset
     dataDir <- liftIO appDataDir
