@@ -6,8 +6,8 @@ module Game.LambdaHack.Common.Actor
     -- * The@ Acto@r type, its components and operations on them
   , Actor(..), ResDelta(..), ActorAspect
   , deltaSerious, deltaMild, actorCanMelee
-  , bspeed, braced, actorTemplate, waitedLastTurn, actorDying
-  , actorTrunkIsBlast, hpTooLow, calmEnough, hpEnough
+  , momentarySpeed, gearSpeed, braced, actorTemplate, waitedLastTurn, actorDying
+  , hpTooLow, calmEnough, hpEnough
   , checkAdjacent, eqpOverfull, eqpFreeN
     -- * Assorted
   , ActorDict, monsterGenChance, smellTimeout
@@ -24,7 +24,9 @@ import           Data.Ratio
 import           GHC.Generics (Generic)
 
 import qualified Game.LambdaHack.Common.Ability as Ability
+import qualified Game.LambdaHack.Common.Dice as Dice
 import           Game.LambdaHack.Common.Item
+import qualified Game.LambdaHack.Common.ItemAspect as IA
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.Point
 import           Game.LambdaHack.Common.Random
@@ -63,8 +65,8 @@ data Actor = Actor
 
     -- Assorted
   , bwait       :: Bool         -- ^ is the actor waiting right now?
-  , bproj       :: Bool         -- ^ is a projectile? (shorthand only,
-                                --   this can be deduced from btrunk)
+  , bproj       :: Bool         -- ^ is a projectile? affects being able
+                                --   to fly through other projectiles, etc.
   }
   deriving (Show, Eq, Generic)
 
@@ -79,7 +81,7 @@ data ResDelta = ResDelta
 
 instance Binary ResDelta
 
-type ActorAspect = EM.EnumMap ActorId AspectRecord
+type ActorAspect = EM.EnumMap ActorId IA.AspectRecord
 
 -- | All actors on the level, indexed by actor identifier.
 type ActorDict = EM.EnumMap ActorId Actor
@@ -96,24 +98,31 @@ deltaMild ResDelta{..} = fst resCurrentTurn == minusM
 actorCanMelee :: ActorAspect -> ActorId -> Actor -> Bool
 actorCanMelee actorAspect aid b =
   let ar = actorAspect EM.! aid
-      actorMaxSk = aSkills ar
-      condUsableWeapon = bweapon b >= 0
+      actorMaxSk = IA.aSkills ar
+      condUsableWeapon = bweapon b > 0
       canMelee = EM.findWithDefault 0 Ability.AbMelee actorMaxSk > 0
   in condUsableWeapon && canMelee
 
-bspeed :: Actor -> AspectRecord -> Speed
-bspeed !b AspectRecord{aSpeed} =
+-- | Current physical speed, whether from being pushed or from organs and gear.
+momentarySpeed :: Actor -> IA.AspectRecord -> Speed
+momentarySpeed !b ar =
   case btrajectory b of
-    Nothing -> toSpeed aSpeed
+    Nothing -> gearSpeed ar
     Just (_, speed) -> speed
 
+-- | The speed from organs and gear; being pushed is ignored.
+gearSpeed :: IA.AspectRecord -> Speed
+gearSpeed IA.AspectRecord{aSpeed} =
+  toSpeed $ max minSpeed aSpeed  -- see @minimalSpeed@
+
 -- | Whether an actor is braced for combat this clip.
 braced :: Actor -> Bool
 braced = bwait
 
 actorTemplate :: ItemId -> Int64 -> Int64 -> Point -> LevelId -> FactionId
+              -> Bool
               -> Actor
-actorTemplate btrunk bhp bcalm bpos blid bfid =
+actorTemplate btrunk bhp bcalm bpos blid bfid bproj =
   let btrajectory = Nothing
       boldpos = Nothing
       borgan  = EM.empty
@@ -123,35 +132,28 @@ actorTemplate btrunk bhp bcalm bpos blid bfid =
       bwait   = False
       bhpDelta = ResDelta (0, 0) (0, 0)
       bcalmDelta = ResDelta (0, 0) (0, 0)
-      bproj = False
   in Actor{..}
 
 waitedLastTurn :: Actor -> Bool
+{-# INLINE waitedLastTurn #-}
 waitedLastTurn = bwait
 
 actorDying :: Actor -> Bool
 actorDying b = bhp b <= 0
                || bproj b && maybe True (null . fst) (btrajectory b)
 
--- This is a good enough approximation with standard content, except for gems,
--- which would be muted when hitting a wall and which would be colorful
--- when flying, both of which are OK.
-actorTrunkIsBlast :: Item -> Bool
-actorTrunkIsBlast trunk = jsymbol trunk `elem` ['`', '\'', '*']
+hpTooLow :: Actor -> IA.AspectRecord -> Bool
+hpTooLow b IA.AspectRecord{aMaxHP} =
+  5 * bhp b < xM aMaxHP && bhp b <= xM 40 || bhp b <= oneM
 
-hpTooLow :: Actor -> AspectRecord -> Bool
-hpTooLow b AspectRecord{aMaxHP} =
-  bhp b <= oneM || 5 * bhp b < xM aMaxHP && bhp b <= xM 40
-
-calmEnough :: Actor -> AspectRecord -> Bool
-calmEnough b AspectRecord{aMaxCalm} =
+calmEnough :: Actor -> IA.AspectRecord -> Bool
+calmEnough b IA.AspectRecord{aMaxCalm} =
   let calmMax = max 1 aMaxCalm
   in 2 * xM calmMax <= 3 * bcalm b && bcalm b > xM 10
 
-hpEnough :: Actor -> AspectRecord -> Bool
-hpEnough b AspectRecord{aMaxHP} =
-  let hpMax = max 1 aMaxHP
-  in xM hpMax <= 2 * bhp b && bhp b > xM 1
+hpEnough :: Actor -> IA.AspectRecord -> Bool
+hpEnough b IA.AspectRecord{aMaxHP} =
+  xM aMaxHP <= 2 * bhp b && bhp b > oneM
 
 checkAdjacent :: Actor -> Actor -> Bool
 checkAdjacent sb tb = blid sb == blid tb && adjacent (bpos sb) (bpos tb)
@@ -166,23 +168,21 @@ eqpFreeN b = let size = sum $ map fst $ EM.elems $ beqp b
              in assert (size <= 10 `blame` (b, size))
                 $ 10 - size
 
--- | Chance that a new monster is generated. Currently depends on the
--- number of monsters already present, and on the level. In the future,
--- the strength of the character and the strength of the monsters present
--- could further influence the chance, and the chance could also affect
--- which monster is generated. How many and which monsters are generated
--- will also depend on the cave kind used to build the level.
-monsterGenChance :: AbsDepth -> AbsDepth -> Int -> Int -> Rnd Bool
-monsterGenChance _ _ _ 0 = return False
-monsterGenChance (AbsDepth n) (AbsDepth totalDepth) lvlSpawned actorCoeff =
-  assert (totalDepth > 0 && n > 0)
-  -- Mimics @castDice@. On level 5/10, first 6 monsters appear fast.
-  $ let scaledDepth = n * 10 `div` totalDepth
-        -- Heroes have to endure two lvl-sized waves of spawners for each level.
-        numSpawnedCoeff = lvlSpawned `div` 2
-    in chance $ 1%fromIntegral
-                    ((actorCoeff * (numSpawnedCoeff - scaledDepth))
-                     `max` 1)  -- monsters up to level depth spawned at once
+-- | Chance that a new monster is generated. Depends on the number
+-- of monsters already present, and on the level depth and its cave kind.
+monsterGenChance :: Dice.AbsDepth -> Dice.AbsDepth -> Int -> Int -> Rnd Bool
+monsterGenChance (Dice.AbsDepth n) (Dice.AbsDepth totalDepth)
+                 lvlAlreadySpawned actorCoeff =
+  assert (totalDepth > 0 && n > 0) $
+    -- Heroes have to endure a level depth-sized wave of immediate
+    -- spawners for each level and only then the monsters start
+    -- to trickle more and more slowly, at the speed dictated
+    -- by @actorCoeff@ specified in cave kind.
+    -- On level 1/10, first 4 monsters spawn immediately, at level 5/10,
+    -- 8 spawn immediately. In general at level n, n+3 spawn at once.
+    let scaledDepth = n * 10 `div` totalDepth
+        coeff = actorCoeff * (lvlAlreadySpawned - scaledDepth - 2)
+    in chance $ 1%fromIntegral (coeff `max` 1)
 
 -- | How long until an actor's smell vanishes from a tile.
 smellTimeout :: Delta Time
