@@ -2,18 +2,22 @@
 -- | Operations on the 'Actor' type, and related, that need the 'State' type,
 -- but not our custom monad types.
 module Game.LambdaHack.Common.ActorState
-  ( fidActorNotProjAssocs, actorAssocs, actorRegularAssocs
-  , warActorRegularList, friendlyActorRegularList, fidActorRegularIds
+  ( fidActorNotProjAssocs, actorAssocs
+  , fidActorRegularAssocs, fidActorRegularIds
+  , foeRegularAssocs, foeRegularList, friendRegularAssocs, friendRegularList
   , bagAssocs, bagAssocsK, posToAidsLvl, posToAids, posToAssocs
-  , nearbyFreePoints, calculateTotal, mergeItemQuant
-  , sharedInv, sharedEqp, sharedAllOwned, sharedAllOwnedFid, findIid
-  , getActorBody, getActorAspect, getCarriedAssocs, getCarriedIidCStore
-  , getContainerBag, getFloorBag, getEmbedBag, getBodyStoreBag
+  , nearbyFreePoints, calculateTotal, itemPrice, mergeItemQuant, findIid
+  , combinedInv, combinedEqp, combinedOrgan, combinedItems, combinedFromLore
+  , getActorBody, getActorAspect, canTraverse
+  , getCarriedAssocsAndTrunk, getCarriedIidCStore, getContainerBag
+  , getFloorBag, getEmbedBag, getBodyStoreBag
   , mapActorItems_, getActorAssocs, getActorAssocsK
-  , memActor, getLocalTime, regenCalmDelta
-  , actorInAmbient, canDeAmbientList, actorSkills, dispEnemy
-  , itemToFull, fullAssocs, storeFromC, aidFromC, lidFromC, posFromC
-  , isEscape, isStair, anyFoeAdj, actorAdjacentAssocs
+  , memActor, getLocalTime, regenCalmDelta, actorInAmbient, canDeAmbientList
+  , actorSkills, dispEnemy, itemToFull, fullAssocs, kitAssocs
+  , getItemKindId, getIidKindId, getItemKind, getIidKind
+  , getItemKindIdServer, getIidKindIdServer, getItemKindServer, getIidKindServer
+  , storeFromC, aidFromC, lidFromC, posFromC
+  , isStair, anyFoeAdj, actorAdjacentAssocs
   , armorHurtBonus, inMelee
   ) where
 
@@ -22,6 +26,7 @@ import Prelude ()
 import Game.LambdaHack.Common.Prelude
 
 import qualified Data.EnumMap.Strict as EM
+import qualified Data.EnumSet as ES
 import           Data.Int (Int64)
 import           GHC.Exts (inline)
 
@@ -29,7 +34,8 @@ import qualified Game.LambdaHack.Common.Ability as Ability
 import           Game.LambdaHack.Common.Actor
 import           Game.LambdaHack.Common.Faction
 import           Game.LambdaHack.Common.Item
-import qualified Game.LambdaHack.Common.Kind as Kind
+import qualified Game.LambdaHack.Common.ItemAspect as IA
+import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Level
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.Point
@@ -37,8 +43,10 @@ import           Game.LambdaHack.Common.State
 import qualified Game.LambdaHack.Common.Tile as Tile
 import           Game.LambdaHack.Common.Time
 import           Game.LambdaHack.Common.Vector
+import qualified Game.LambdaHack.Content.ItemKind as IK
 import           Game.LambdaHack.Content.ModeKind
 import           Game.LambdaHack.Content.TileKind (TileKind)
+import qualified Game.LambdaHack.Content.TileKind as TK
 
 fidActorNotProjAssocs :: FactionId -> State -> [(ActorId, Actor)]
 fidActorNotProjAssocs fid s =
@@ -58,21 +66,34 @@ actorRegularAssocs p lid s =
   let f (_, b) = not (bproj b) && blid b == lid && p (bfid b) && bhp b > 0
   in filter f $ EM.assocs $ sactorD s
 
-warActorRegularList :: FactionId -> LevelId -> State -> [Actor]
-warActorRegularList fid lid s =
-  let fact = (EM.! fid) . sfactionD $ s
-  in map snd $ actorRegularAssocs (inline isAtWar fact) lid s
-
-friendlyActorRegularList :: FactionId -> LevelId -> State -> [Actor]
-friendlyActorRegularList fid lid s =
-  let fact = (EM.! fid) . sfactionD $ s
-      friendlyFid fid2 = fid2 == fid || inline isAllied fact fid2
-  in map snd $ actorRegularAssocs friendlyFid lid s
+fidActorRegularAssocs :: FactionId -> LevelId -> State ->  [(ActorId, Actor)]
+fidActorRegularAssocs fid lid s =
+  actorRegularAssocs (== fid) lid s
 
 fidActorRegularIds :: FactionId -> LevelId -> State -> [ActorId]
 fidActorRegularIds fid lid s =
   map fst $ actorRegularAssocs (== fid) lid s
 
+foeRegularAssocs :: FactionId -> LevelId -> State -> [(ActorId, Actor)]
+foeRegularAssocs fid lid s =
+  let fact = (EM.! fid) . sfactionD $ s
+  in actorRegularAssocs (inline isFoe fid fact) lid s
+
+foeRegularList :: FactionId -> LevelId -> State -> [Actor]
+foeRegularList fid lid s =
+  let fact = (EM.! fid) . sfactionD $ s
+  in map snd $ actorRegularAssocs (inline isFoe fid fact) lid s
+
+friendRegularAssocs :: FactionId -> LevelId -> State -> [(ActorId, Actor)]
+friendRegularAssocs fid lid s =
+  let fact = (EM.! fid) . sfactionD $ s
+  in actorRegularAssocs (inline isFriend fid fact) lid s
+
+friendRegularList :: FactionId -> LevelId -> State -> [Actor]
+friendRegularList fid lid s =
+  let fact = (EM.! fid) . sfactionD $ s
+  in map snd $ actorRegularAssocs (inline isFriend fid fact) lid s
+
 bagAssocs :: State -> ItemBag -> [(ItemId, Item)]
 bagAssocs s bag =
   let iidItem iid = (iid, getItemBody iid s)
@@ -95,12 +116,12 @@ posToAssocs pos lid s =
   let l = posToAidsLvl pos $ sdungeon s EM.! lid
   in map (\aid -> (aid, getActorBody aid s)) l
 
-nearbyFreePoints :: (Kind.Id TileKind -> Bool) -> Point -> LevelId -> State
+nearbyFreePoints :: (ContentId TileKind -> Bool) -> Point -> LevelId -> State
                  -> [Point]
 nearbyFreePoints f start lid s =
   let lvl@Level{lxsize, lysize} = sdungeon s EM.! lid
       good p = f (lvl `at` p)
-               && Tile.isWalkable (Kind.coTileSpeedup $ scops s) (lvl `at` p)
+               && Tile.isWalkable (coTileSpeedup $ scops s) (lvl `at` p)
                && null (posToAidsLvl p lvl)
       ps = nub $ start : concatMap (vicinity lxsize lysize) ps
   in filter good ps
@@ -108,37 +129,20 @@ nearbyFreePoints f start lid s =
 -- | Calculate loot's worth for a given faction.
 calculateTotal :: FactionId -> State -> (ItemBag, Int)
 calculateTotal fid s =
-  let bag = sharedAllOwned fid s
+  let bag = combinedItems fid s
       items = map (\(iid, (k, _)) -> (getItemBody iid s, k)) $ EM.assocs bag
-  in (bag, sum $ map itemPrice items)
+      price (item, k) = itemPrice k $ getItemKind item s
+  in (bag, sum $ map price items)
+
+-- | Price an item, taking count into consideration.
+itemPrice :: Int -> IK.ItemKind -> Int
+itemPrice jcount itemKind = case lookup "valuable" $ IK.ifreq itemKind of
+  Just k -> jcount * k
+  Nothing -> 0
 
 mergeItemQuant :: ItemQuant -> ItemQuant -> ItemQuant
 mergeItemQuant (k2, it2) (k1, it1) = (k1 + k2, it1 ++ it2)
 
-sharedInv :: FactionId -> State -> ItemBag
-sharedInv fid s =
-  let bs = inline fidActorNotProjAssocs fid s
-  in EM.unionsWith mergeItemQuant $ map (binv . snd) bs
-
-sharedEqp :: FactionId -> State -> ItemBag
-sharedEqp fid s =
-  let bs = inline fidActorNotProjAssocs fid s
-  in EM.unionsWith mergeItemQuant $ map (beqp . snd) bs
-
-sharedAllOwned :: FactionId -> State -> ItemBag
-sharedAllOwned fid s =
-  let shaBag = gsha $ sfactionD s EM.! fid
-  in EM.unionsWith mergeItemQuant [sharedEqp fid s, sharedInv fid s, shaBag]
-
-sharedAllOwnedFid :: Bool -> FactionId -> State -> ItemBag
-sharedAllOwnedFid onlyOrgans fid s =
-  let shaBag = gsha $ sfactionD s EM.! fid
-      bs = map snd $ inline fidActorNotProjAssocs fid s
-  in EM.unionsWith mergeItemQuant
-     $ if onlyOrgans
-       then map borgan bs
-       else map binv bs ++ map beqp bs ++ [shaBag]
-
 findIid :: ActorId -> FactionId -> ItemId -> State
         -> [(ActorId, (Actor, CStore))]
 findIid leader fid iid s =
@@ -152,31 +156,72 @@ findIid leader fid iid s =
       items = concatMap itemsOfActor actors
   in map snd $ filter ((== iid) . fst) items
 
+combinedInv :: FactionId -> State -> ItemBag
+combinedInv fid s =
+  let bs = inline fidActorNotProjAssocs fid s
+  in EM.unionsWith mergeItemQuant $ map (binv . snd) bs
+
+combinedEqp :: FactionId -> State -> ItemBag
+combinedEqp fid s =
+  let bs = inline fidActorNotProjAssocs fid s
+  in EM.unionsWith mergeItemQuant $ map (beqp . snd) bs
+
+-- Trunk not considered (if stolen).
+combinedOrgan :: FactionId -> State -> ItemBag
+combinedOrgan fid s =
+  let bs = inline fidActorNotProjAssocs fid s
+  in EM.unionsWith mergeItemQuant $ map (borgan . snd) bs
+
+-- Trunk not considered (if stolen).
+combinedItems :: FactionId -> State -> ItemBag
+combinedItems fid s =
+  let shaBag = gsha $ sfactionD s EM.! fid
+      bs = map snd $ inline fidActorNotProjAssocs fid s
+  in EM.unionsWith mergeItemQuant $ map binv bs ++ map beqp bs ++ [shaBag]
+
+combinedFromLore :: SLore -> FactionId -> State -> ItemBag
+combinedFromLore slore fid s = case slore of
+  SItem -> combinedItems fid s
+  SOrgan -> combinedOrgan fid s
+  STrunk -> combinedOrgan fid s
+  STmp -> combinedOrgan fid s
+  SBlast -> EM.empty
+  SEmbed -> EM.empty
+
 getActorBody :: ActorId -> State -> Actor
 {-# INLINE getActorBody #-}
 getActorBody aid s = sactorD s EM.! aid
 
-getActorAspect :: ActorId -> State -> AspectRecord
+getActorAspect :: ActorId -> State -> IA.AspectRecord
 {-# INLINE getActorAspect #-}
 getActorAspect aid s = sactorAspect s EM.! aid
 
-getCarriedAssocs :: Actor -> State -> [(ItemId, Item)]
-getCarriedAssocs b s =
+-- Check that the actor can move, also between levels and through doors.
+-- Otherwise, it's too awkward for human player to control, e.g.,
+-- being stuck in a room with revolving doors closing after one turn
+-- and the player needing to micromanage opening such doors with
+-- another actor all the time. Completely immovable actors
+-- e.g., an impregnable surveillance camera in a crowded corridor,
+-- are less of a problem due to micromanagment, but more due to
+-- the constant disturbing of other actor's running, etc..
+canTraverse :: ActorId -> State -> Bool
+canTraverse aid s =
+  let actorMaxSk = IA.aSkills $ getActorAspect aid s
+  in EM.findWithDefault 0 Ability.AbMove actorMaxSk > 0
+     && EM.findWithDefault 0 Ability.AbAlter actorMaxSk
+          >= fromEnum TK.talterForStairs
+
+getCarriedAssocsAndTrunk :: Actor -> State -> [(ItemId, Item)]
+getCarriedAssocsAndTrunk b s =
   -- The trunk is important for a case of spotting a caught projectile
-  -- that is one with a stolen trunk organ (the projectile item).
-  -- This actually does happen.
+  -- with a stolen projecting item. This actually does happen.
   let trunk = EM.singleton (btrunk b) (1, [])
   in bagAssocs s $ EM.unionsWith const [binv b, beqp b, borgan b, trunk]
 
 getCarriedIidCStore :: Actor -> [(ItemId, CStore)]
 getCarriedIidCStore b =
-  -- The trunk is important for a case of dominating an actor with stolen
-  -- trunk organ.
-  let trunk = EM.singleton (btrunk b) (1, [])
-      bagCarried (cstore, bag) = map (,cstore) $ EM.keys bag
-  in concatMap bagCarried [ (CInv, binv b)
-                          , (CEqp, beqp b)
-                          , (COrgan, EM.unionWith const (borgan b) trunk) ]
+  let bagCarried (cstore, bag) = map (,cstore) $ EM.keys bag
+  in concatMap bagCarried [(CInv, binv b), (CEqp, beqp b), (COrgan, borgan b)]
 
 getContainerBag :: Container -> State -> ItemBag
 getContainerBag c s = case c of
@@ -236,29 +281,35 @@ memActor aid lid s =
 getLocalTime :: LevelId -> State -> Time
 getLocalTime lid s = ltime $ sdungeon s EM.! lid
 
-regenCalmDelta :: Actor -> AspectRecord -> State -> Int64
-regenCalmDelta body AspectRecord{aMaxCalm} s =
+regenCalmDelta :: ActorId -> Actor -> State -> Int64
+regenCalmDelta aid body s =
   let calmIncr = oneM  -- normal rate of calm regen
+      IA.AspectRecord{aMaxCalm} = getActorAspect aid s
       maxDeltaCalm = xM aMaxCalm - bcalm body
       fact = (EM.! bfid body) . sfactionD $ s
       -- Worry actor by (even projectile) enemies felt (even if not seen)
       -- on the level within 3 steps. Even dying, but not hiding in wait.
-      isHeardFoe b = blid b == blid body
-                     && chessDist (bpos b) (bpos body) <= 3  -- a bit costly
-                     && not (waitedLastTurn b)  -- uncommon
-                     && inline isAtWar fact (bfid b)  -- costly
-  in if any isHeardFoe $ EM.elems $ sactorD s
+      isHeardFoe (!p, l) =
+        -- In case of multiple projectiles on the same position,
+        -- the following is not reliable, but that's OK (and fast).
+        let b = case l of
+              aid2 : _ -> getActorBody aid2 s
+              [] -> error $ "" `showFailure` (aid, body, p)
+        in inline chessDist p (bpos body) <= 3
+           && not (waitedLastTurn b)  -- uncommon
+           && inline isFoe (bfid body) fact (bfid b)  -- costly
+  in if any isHeardFoe $ EM.assocs $ lactor $ sdungeon s EM.! blid body
      then minusM  -- even if all calmness spent, keep informing the client
      else min calmIncr (max 0 maxDeltaCalm)  -- in case Calm is over max
 
 actorInAmbient :: Actor -> State -> Bool
 actorInAmbient b s =
   let lvl = (EM.! blid b) . sdungeon $ s
-  in Tile.isLit (Kind.coTileSpeedup $ scops s) (lvl `at` bpos b)
+  in Tile.isLit (coTileSpeedup $ scops s) (lvl `at` bpos b)
 
 canDeAmbientList :: Actor -> State -> [Point]
 canDeAmbientList b s =
-  let Kind.COps{coTileSpeedup} = scops s
+  let COps{coTileSpeedup} = scops s
       lvl = (EM.! blid b) . sdungeon $ s
       posDeAmbient p =
         let t = lvl `at` p
@@ -277,41 +328,76 @@ actorSkills mleader aid s =
       factionSkills
         | Just aid == mleader = Ability.zeroSkills
         | otherwise = fskillsOther player `Ability.addSkills` skillsFromTactic
-      itemSkills = aSkills ar
+      itemSkills = IA.aSkills ar
   in itemSkills `Ability.addSkills` factionSkills
 
 -- Check whether an actor can displace an enemy. We assume they are adjacent.
 -- If the actor is not, in fact, an enemy, we let it displace.
 dispEnemy :: ActorId -> ActorId -> Ability.Skills -> State -> Bool
 dispEnemy source target actorMaxSk s =
-  let hasSupport b =
+  let hasBackup b =
         let adjacentAssocs = actorAdjacentAssocs b s
-            fact = (EM.! bfid b) . sfactionD $ s
-            friendlyFid fid = fid == bfid b || isAllied fact fid
+            fact = sfactionD s EM.! bfid b
             friend (_, b2) =
-              not (bproj b2) && friendlyFid (bfid b2) && bhp b2 > 0
+              not (bproj b2) && isFriend (bfid b) fact (bfid b2) && bhp b2 > 0
         in any friend adjacentAssocs
       sb = getActorBody source s
       tb = getActorBody target s
+      tfact = sfactionD s EM.! bfid tb
   in bproj tb
-     || not (isAtWar ((EM.! bfid tb) . sfactionD $ s) (bfid sb))
+     || not (isFoe (bfid tb) tfact (bfid sb))
      || not (actorDying tb
              || braced tb
              || EM.findWithDefault 0 Ability.AbMove actorMaxSk <= 0
-             || hasSupport sb && hasSupport tb)  -- solo actors are flexible
+             || hasBackup sb && hasBackup tb)  -- solo actors are flexible
 
-itemToFull :: State -> ItemId -> ItemQuant -> ItemFull
-itemToFull s iid =
+itemToFull :: ItemId -> State -> ItemFull
+itemToFull iid s =
   itemToFull6 (scops s) (sdiscoKind s) (sdiscoAspect s) iid (getItemBody iid s)
 
 fullAssocs :: ActorId -> [CStore] -> State -> [(ItemId, ItemFull)]
 fullAssocs aid cstores s =
   let allAssocs = concatMap (\cstore -> getActorAssocsK aid cstore s) cstores
-      iToFull (iid, (item, kit)) =
-        (iid, itemToFull6 (scops s) (sdiscoKind s) (sdiscoAspect s)
-                          iid item kit)
+      iToFull (iid, (item, _kit)) =
+        (iid, itemToFull6 (scops s) (sdiscoKind s) (sdiscoAspect s) iid item)
   in map iToFull allAssocs
 
+kitAssocs :: ActorId -> [CStore] -> State -> [(ItemId, ItemFullKit)]
+kitAssocs aid cstores s =
+  let allAssocs = concatMap (\cstore -> getActorAssocsK aid cstore s) cstores
+      iToFull (iid, (item, kit)) =
+        (iid, ( itemToFull6 (scops s) (sdiscoKind s) (sdiscoAspect s) iid item
+              , kit ))
+  in map iToFull allAssocs
+
+getItemKindId :: Item -> State -> ContentId IK.ItemKind
+getItemKindId item s = case jkind item of
+  IdentityObvious ik -> ik
+  IdentityCovered ix ik -> fromMaybe ik $ EM.lookup ix $ sdiscoKind s
+
+getIidKindId :: ItemId -> State -> ContentId IK.ItemKind
+getIidKindId iid s = getItemKindId (getItemBody iid s) s
+
+getItemKind :: Item -> State -> IK.ItemKind
+getItemKind item s = okind (coitem $ scops s) $ getItemKindId item s
+
+getIidKind :: ItemId -> State -> IK.ItemKind
+getIidKind iid s = getItemKind (getItemBody iid s) s
+
+getItemKindIdServer :: Item -> State -> ContentId IK.ItemKind
+getItemKindIdServer item s = case jkind item of
+  IdentityObvious ik -> ik
+  IdentityCovered ix _ik -> fromJust $ EM.lookup ix $ sdiscoKind s
+
+getIidKindIdServer :: ItemId -> State -> ContentId IK.ItemKind
+getIidKindIdServer iid s = getItemKindIdServer (getItemBody iid s) s
+
+getItemKindServer :: Item -> State -> IK.ItemKind
+getItemKindServer item s = okind (coitem $ scops s) $ getItemKindIdServer item s
+
+getIidKindServer :: ItemId -> State -> IK.ItemKind
+getIidKindServer iid s = getItemKindServer (getItemBody iid s) s
+
 storeFromC :: Container -> CStore
 storeFromC c = case c of
   CFloor{} -> CGround
@@ -339,21 +425,11 @@ posFromC (CEmbed _ pos) _ = pos
 posFromC (CActor aid _) s = bpos $ getActorBody aid s
 posFromC c@CTrunk{} _ = error $ "" `showFailure` c
 
-isEscape :: LevelId -> Point -> State -> Bool
-isEscape lid p s =
-  let bag = getEmbedBag lid p s
-      is = map (`getItemBody` s) $ EM.keys bag
-      -- Contrived, for now.
-      isE Item{jname} = jname == "escape"
-  in any isE is
-
 isStair :: LevelId -> Point -> State -> Bool
 isStair lid p s =
   let bag = getEmbedBag lid p s
-      is = map (`getItemBody` s) $ EM.keys bag
-      -- Contrived, for now.
-      isE Item{jname} = jname == "staircase up" || jname == "staircase down"
-  in any isE is
+      ks = map (flip getIidKind s) $ EM.keys bag
+  in any (any IK.isEffAscend . IK.ieffects) ks
 
 -- | Require that any non-dying foe is adjacent. We include even
 -- projectiles that explode when stricken down, because they can be caught
@@ -370,7 +446,8 @@ anyFoeAdj aid s =
       f !mv = case posToAidsLvl (shift (bpos body) mv) lvl of
         [] -> False
         aid2 : _ -> g $ getActorBody aid2 s
-      g !b = isAtWar fact (bfid b) && bhp b > 0
+      g !b = inline isFoe (bfid body) fact (bfid b)
+             && bhp b > 0  -- uncommon
   in any f moves
 
 actorAdjacentAssocs :: Actor -> State -> [(ActorId, Actor)]
@@ -389,23 +466,39 @@ armorHurtBonus source target s =
       block200 b n = min 200 $ max (-200) $ n + if braced tb then b else 0
       sar = sactorAspect s EM.! source
       tar = sactorAspect s EM.! target
-      itemBonus = trim200 (aHurtMelee sar) - if bproj sb
-                                             then block200 25 (aArmorRanged tar)
-                                             else block200 50 (aArmorMelee tar)
+      itemBonus = trim200 (IA.aHurtMelee sar)
+                  - if bproj sb
+                    then block200 25 (IA.aArmorRanged tar)
+                    else block200 50 (IA.aArmorMelee tar)
   in 100 + min 99 (max (-99) itemBonus)  -- at least 1% of damage gets through
 
-inMelee :: Actor -> State -> Bool
-inMelee bodyOur s =
-  let fact = sfactionD s EM.! bfid bodyOur
-      f !b = blid b == blid bodyOur && isAtWar fact (bfid b) && bhp b > 0
-      -- We assume foes are less numerous, because usually they are heroes,
-      -- and so we compute them once and use many times.
-      -- For the same reason @anyFoeAdj@ would not speed up this computation
-      -- in normal gameplay (as opposed to AI vs AI benchmarks).
+-- | Check if any non-dying foe (projectile or not) is adjacent
+-- to any of our normal actors (whether they can melee or just need to flee,
+-- in which case alert is needed so that they are not slowed down by others).
+-- This is needed only by AI and computed as lazily as possible.
+inMelee :: FactionId -> LevelId -> State -> Bool
+inMelee !fid !lid s =
+  let fact = sfactionD s EM.! fid
+      f !b = blid b == lid
+             && inline isFoe fid fact (bfid b)  -- costly
+             && bhp b > 0  -- uncommon
       allFoes = filter f $ EM.elems $ sactorD s
-  in any (\body ->
-    bfid bodyOur == bfid body
-    && blid bodyOur == blid body
-    && not (bproj body)
-    && bhp body > 0
-    && any (\b -> adjacent (bpos b) (bpos body)) allFoes) $ sactorD s
+      g !b = bfid b == fid
+             && blid b == lid
+             && not (bproj b)
+             && bhp b > 0
+      allOurs = filter g $ EM.elems $ sactorD s
+      -- We assume foes are less numerous, even though they may come
+      -- from multiple factions and they contain projectiles,
+      -- because we see all our actors, while many foes may be hidden.
+      -- Consequently, we allocate the set of foe positions
+      -- and avoid allocating ours, by iterating over our actors.
+      -- This in O(mn) instead of O(m+n), but it allocates
+      -- less and multiplicative constants are lower.
+      -- We inspect adjacent locations of foe positions, not of ours,
+      -- thus increasing allocation a bit, but not by much, because
+      -- the set should be rather saturated.
+      -- If there are no foes in sight, we don't iterate at all.
+      setFoeVicinity = ES.fromList $ concatMap (vicinityUnsafe . bpos) allFoes
+  in not (ES.null setFoeVicinity)  -- shortcut
+     && any (\b -> bpos b `ES.member` setFoeVicinity) allOurs
