@@ -1,4 +1,5 @@
-{-# LANGUAGE DeriveGeneric, GeneralizedNewtypeDeriving, MagicHash #-}
+{-# LANGUAGE DeriveGeneric, GeneralizedNewtypeDeriving, MagicHash,
+             TypeFamilies #-}
 -- | Colours and text attributes.
 module Game.LambdaHack.Common.Color
   ( -- * Colours
@@ -18,6 +19,7 @@ import Prelude ()
 
 import Game.LambdaHack.Common.Prelude
 
+import           Control.DeepSeq
 import           Data.Binary
 import           Data.Bits (unsafeShiftL, unsafeShiftR, (.&.))
 import qualified Data.Char as Char
@@ -28,6 +30,8 @@ import           GHC.Generics (Generic)
 import           GHC.Prim (int2Word#)
 import           GHC.Word (Word32 (W32#))
 
+import qualified Game.LambdaHack.Common.PointArray as PointArray
+
 -- | Colours supported by the major frontends.
 data Color =
     Black
@@ -54,6 +58,8 @@ instance Binary Color where
 
 instance Hashable Color
 
+instance NFData Color
+
 -- | The default colours, to optimize attribute setting.
 defFG :: Color
 defFG = White
@@ -121,12 +127,6 @@ data Highlight =
   | HighlightMagenta
   deriving (Show, Eq, Ord, Enum, Bounded, Generic)
 
-instance Binary Highlight where
-  put = putWord8 . toEnum . fromEnum
-  get = fmap (toEnum . fromEnum) getWord8
-
-instance Hashable Highlight
-
 -- | Text attributes: foreground color and highlight.
 data Attr = Attr
   { fg :: Color      -- ^ foreground colour
@@ -154,7 +154,12 @@ data AttrChar = AttrChar
 -- can be done without going to and from @Int@.
 -- | Optimized representation of 'AttrChar'.
 newtype AttrCharW32 = AttrCharW32 {attrCharW32 :: Word32}
-  deriving (Show, Eq, Enum, Binary)
+  deriving (Show, Eq, Ord, Enum, Binary)
+
+instance PointArray.UnboxRepClass AttrCharW32 where
+  type UnboxRep AttrCharW32 = Word32
+  toUnboxRepUnsafe = attrCharW32
+  fromUnboxRep = AttrCharW32
 
 attrCharToW32 :: AttrChar -> AttrCharW32
 attrCharToW32 AttrChar{acAttr=Attr{..}, acChar} = AttrCharW32 $ toEnum $
