@@ -1,9 +1,10 @@
-{-# LANGUAGE DeriveGeneric, FlexibleInstances, TypeSynonymInstances #-}
+{-# LANGUAGE DeriveGeneric, FlexibleInstances, GeneralizedNewtypeDeriving,
+             TypeSynonymInstances #-}
 {-# OPTIONS_GHC -Wno-orphans #-}
 -- | Representation of dice scaled with current level depth.
 module Game.LambdaHack.Common.Dice
   ( -- * Frequency distribution for casting dice scaled with level depth
-    Dice, castDice, d, dl, z, zl, intToDice
+    Dice, AbsDepth(..), castDice, d, dL, z, zL, intToDice
   , minmaxDice, maxDice, minDice, meanDice, reduceDice
     -- * Dice for rolling a pair of integer parameters representing coordinates.
   , DiceXY(..), maxDiceXY, minDiceXY, meanDiceXY
@@ -16,7 +17,6 @@ import Game.LambdaHack.Common.Prelude
 import Control.DeepSeq
 import Data.Binary
 import Data.Hashable (Hashable)
-import Game.LambdaHack.Common.Misc
 import GHC.Generics (Generic)
 
 -- | Multiple dice rolls, some scaled with current level depth, in which case
@@ -40,28 +40,35 @@ data Dice =
   deriving (Eq, Ord, Generic)
 
 instance Show Dice where
-  show dice1 = case dice1 of
+  show = stripOuterParens . showDiceWithParens
+
+stripOuterParens :: String -> String
+stripOuterParens s@('(' : rest) = case uncons $ reverse rest of
+  Just (')', middle) -> reverse middle
+  _ -> s
+stripOuterParens s = s
+
+showDiceWithParens :: Dice -> String
+showDiceWithParens = sh
+ where
+  sh dice1 = case dice1 of
     DiceI k -> show k
     DiceD n k -> show n ++ "d" ++ show k
-    DiceDL n k -> show n ++ "dl" ++ show k
+    DiceDL n k -> show n ++ "dL" ++ show k
     DiceZ n k -> show n ++ "z" ++ show k
-    DiceZL n k -> show n ++ "zl" ++ show k
-    DicePlus d1 (DiceNegate d2) | simpleDice d2 -> show d1 ++ "-" ++ show d2
-    DicePlus d1 (DiceNegate d2) -> show d1 ++ "-" ++ "(" ++ show d2 ++ ")"
-    DicePlus d1 d2 -> show d1 ++ "+" ++ show d2
-    DiceTimes d1 d2 -> "(" ++ show d1 ++ ") * (" ++ show d2 ++ ")"  -- rare
-    DiceNegate (DiceI k) -> "-" ++ show k  -- "-2" parses as this, not as DiceI
-    DiceNegate d1 -> "- (" ++ show d1 ++ ")"
+    DiceZL n k -> show n ++ "zL" ++ show k
+    DicePlus d1 (DiceNegate d2) -> wrapInParens $ sh d1 ++ "-" ++ sh d2
+    DicePlus (DiceNegate d1) d2 -> wrapInParens $ "-" ++ sh d1 ++ "+" ++ sh d2
+    DicePlus d1 (DicePlus d2 d3) -> sh $ DicePlus (DicePlus d1 d2) d3
+    DicePlus (DicePlus d1 d2) d3 ->
+      wrapInParens $ stripOuterParens (sh $ DicePlus d1 d2) ++ "+" ++ sh d3
+    DicePlus d1 d2 -> wrapInParens $ sh d1 ++ "+" ++ sh d2
+    DiceTimes d1 d2 -> wrapInParens $ sh d1 ++ "*" ++ sh d2
+    DiceNegate d1 -> wrapInParens $ "-" ++ sh d1
 
-simpleDice :: Dice -> Bool
-simpleDice DiceI{} = True
-simpleDice DiceD{} = True
-simpleDice DiceDL{} = True
-simpleDice DiceZ{} = True
-simpleDice DiceZL{} = True
-simpleDice _ = False
-
-instance Hashable Dice
+wrapInParens :: String -> String
+wrapInParens "" = ""
+wrapInParens t = "(" <> t <> ")"
 
 instance Binary Dice
 
@@ -76,8 +83,15 @@ instance Num Dice where
   signum = undefined
   fromInteger n = DiceI (fromInteger n)
 
--- | Cast dice scaled with current level depth.
--- Note that at the first level, the scaled dice are always ignored.
+-- | Absolute depth in the dungeon. When used for the maximum depth
+-- of the whole dungeon, this can be different than dungeon size,
+-- e.g., when the dungeon is branched, and it can even be different
+-- than the length of the longest branch, if levels at some depths are missing.
+newtype AbsDepth = AbsDepth Int
+  deriving (Show, Eq, Ord, Hashable, Binary)
+
+-- | Cast dice scaled with current level depth. When scaling, we round up,
+-- so that the value of @1 `dL` 1@ is 1 even at the lowest level.
 --
 -- The implementation calls RNG as many times as there are dice rolls,
 -- which is costly, so content should prefer to case fewer dice
@@ -96,13 +110,13 @@ castDice randomR (AbsDepth lvlDepth) (AbsDepth maxDepth) dice = do
   let !_A = assert (lvlDepth >= 0 && lvlDepth <= maxDepth
                     `blame` "invalid depth for dice rolls"
                     `swith` (lvlDepth, maxDepth)) ()
-      castNK n start k = do
+      castNK n start k = if start == k then return $! n * k else do
           let f !acc 0 = return acc
               f acc count = do
                 r <- randomR (start, k)
                 f (acc + r) (count - 1)
           f 0 n
-      scaleL k = (k * max 0 (lvlDepth - 1)) `div` max 1 (maxDepth - 1)
+      scaleL k = (k * max 1 lvlDepth) `divUp` max 1 maxDepth
       castD :: Dice -> m Int
       castD dice1 = case dice1 of
         DiceI k -> return k
@@ -131,8 +145,8 @@ d n k = assert (n > 0 && k > 0 `blame` "die must be positive" `swith` (n, k))
 
 -- | A die rolled the given number of times, with the result scaled
 -- with dungeon level depth.
-dl :: Int -> Int -> Dice
-dl n k = assert (n > 0 && k > 0 `blame` "die must be positive" `swith` (n, k))
+dL :: Int -> Int -> Dice
+dL n k = assert (n > 0 && k > 0 `blame` "die must be positive" `swith` (n, k))
          $ DiceDL n k
 
 -- | A die, starting from zero, ending at one less than the bound,
@@ -144,8 +158,8 @@ z n k = assert (n > 0 && k > 0 `blame` "die must be positive" `swith` (n, k))
 -- | A die, starting from zero, ending at one less than the bound,
 -- rolled the given number of times,
 -- with the result scaled with dungeon level depth.
-zl :: Int -> Int -> Dice
-zl n k = assert (n > 0 && k > 0 `blame` "die must be positive" `swith` (n, k))
+zL :: Int -> Int -> Dice
+zL n k = assert (n > 0 && k > 0 `blame` "die must be positive" `swith` (n, k))
          $ DiceZL n k
 
 intToDice :: Int -> Dice
@@ -160,7 +174,7 @@ minmaxDice :: Dice -> (Int, Int)
 minmaxDice dice1 = case dice1 of
   DiceI k -> (k, k)
   DiceD n k -> (n, n * k)
-  DiceDL n k -> (0, n * k)  -- bottom and top level considered
+  DiceDL n k -> (1, n * k)  -- bottom and top level considered
   DiceZ n k -> (0, n * (k - 1))
   DiceZL n k -> (0, n * (k - 1))  -- bottom and top level considered
   DicePlus d1 d2 ->
@@ -213,10 +227,10 @@ reduceDice d1 =
 data DiceXY = DiceXY Dice Dice
   deriving (Show, Generic)
 
-instance Hashable DiceXY
-
 instance Binary DiceXY
 
+instance NFData DiceXY
+
 -- | Maximal value of DiceXY.
 maxDiceXY :: DiceXY -> (Int, Int)
 maxDiceXY (DiceXY x y) = (maxDice x, maxDice y)
