@@ -4,9 +4,9 @@
 module Game.LambdaHack.Common.Faction
   ( FactionId, FactionDict, Faction(..), Diplomacy(..), Status(..)
   , Target(..), TGoal(..), Challenge(..)
-  , gleader, tgtKindDescription, isHorrorFact, nameOfHorrorFact
+  , gleader, tgtKindDescription, isHorrorFact
   , noRunWithMulti, isAIFact, autoDungeonLevel, automatePlayer
-  , isAtWar, isAllied
+  , isFoe, isFriend
   , difficultyBound, difficultyDefault, difficultyCoeff, difficultyInverse
   , defaultChallenge
 #ifdef EXPOSE_INTERNAL
@@ -28,7 +28,7 @@ import qualified Game.LambdaHack.Common.Ability as Ability
 import           Game.LambdaHack.Common.Actor
 import qualified Game.LambdaHack.Common.Color as Color
 import           Game.LambdaHack.Common.Item
-import qualified Game.LambdaHack.Common.Kind as Kind
+import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.Point
 import           Game.LambdaHack.Common.Vector
@@ -49,9 +49,9 @@ data Faction = Faction
   , _gleader  :: Maybe ActorId   -- ^ the leader of the faction; don't use
                                  --   in place of sleader on clients
   , gsha      :: ItemBag         -- ^ faction's shared inventory
-  , gvictims  :: EM.EnumMap (Kind.Id ItemKind) Int  -- ^ members killed
-  , gvictimsD :: EM.EnumMap (Kind.Id ModeKind)
-                            (IM.IntMap (EM.EnumMap (Kind.Id ItemKind) Int))
+  , gvictims  :: EM.EnumMap (ContentId ItemKind) Int  -- ^ members killed
+  , gvictimsD :: EM.EnumMap (ContentId ModeKind)
+                            (IM.IntMap (EM.EnumMap (ContentId ItemKind) Int))
       -- ^ members killed in the past, by game mode and difficulty level
   }
   deriving (Show, Eq, Generic)
@@ -137,9 +137,6 @@ tgtKindDescription tgt = case tgt of
 isHorrorFact :: Faction -> Bool
 isHorrorFact fact = nameOfHorrorFact `elem` fgroups (gplayer fact)
 
-nameOfHorrorFact :: GroupName ItemKind
-nameOfHorrorFact = toGroupName "horror"
-
 -- A faction where other actors move at once or where some of leader change
 -- is automatic can't run with multiple actors at once. That would be
 -- overpowered or too complex to keep correct.
@@ -178,12 +175,19 @@ automatePlayer st pl =
   in pl {fleaderMode = autoLeader st pl}
 
 -- | Check if factions are at war. Assumes symmetry.
-isAtWar :: Faction -> FactionId -> Bool
-isAtWar fact fid = War == EM.findWithDefault Unknown fid (gdipl fact)
+isFoe :: FactionId -> Faction -> FactionId -> Bool
+isFoe fid1 fact1 fid2 =
+  fid1 /= fid2  -- shortcut
+  && War == EM.findWithDefault Unknown fid2 (gdipl fact1)
 
 -- | Check if factions are allied. Assumes symmetry.
-isAllied :: Faction -> FactionId -> Bool
-isAllied fact fid = Alliance == EM.findWithDefault Unknown fid (gdipl fact)
+isAlly :: Faction -> FactionId -> Bool
+{-# INLINE isAlly #-}
+isAlly fact1 fid2 = Alliance == EM.findWithDefault Unknown fid2 (gdipl fact1)
+
+-- | Check if factions are allied or are the same faction. Assumes symmetry.
+isFriend :: FactionId -> Faction -> FactionId -> Bool
+isFriend fid1 fact1 fid2 = fid1 == fid2 || isAlly fact1 fid2
 
 difficultyBound :: Int
 difficultyBound = 9
