@@ -2,9 +2,9 @@
 -- | The appearance of in-game items, as communicated to the player.
 module Game.LambdaHack.Common.Flavour
   ( -- * The @Flavour@ type
-    Flavour
+    Flavour(Flavour)
   , -- * Constructors
-    zipPlain, zipFancy, zipLiquid, stdFlav
+    zipPlain, zipFancy, zipLiquid
   , -- * Accessors
     flavourToColor, flavourToName
     -- * Assorted
@@ -19,18 +19,18 @@ import Prelude ()
 
 import Game.LambdaHack.Common.Prelude
 
+import Control.DeepSeq
 import Data.Binary
-import Data.Hashable (Hashable)
+import Data.Bits (unsafeShiftL, unsafeShiftR, (.&.))
+import Data.Hashable (Hashable (hashWithSalt), hashUsing)
 import GHC.Generics (Generic)
 
 import Game.LambdaHack.Common.Color
 
 data FancyName = Plain | Fancy | Liquid
-  deriving (Show, Eq, Ord, Generic)
+  deriving (Show, Eq, Ord, Enum, Bounded, Generic)
 
-instance Hashable FancyName
-
-instance Binary FancyName
+instance NFData FancyName
 
 -- | The type of item flavours.
 data Flavour = Flavour
@@ -39,9 +39,20 @@ data Flavour = Flavour
   }
   deriving (Show, Eq, Ord, Generic)
 
-instance Hashable Flavour
+instance Enum Flavour where
+  fromEnum Flavour{..} =
+    unsafeShiftL (fromEnum fancyName) 8 + fromEnum baseColor
+  toEnum n = Flavour (toEnum $ unsafeShiftR n 8)
+                     (toEnum $ n .&. (2 ^ (8 :: Int) - 1))
 
-instance Binary Flavour
+instance Hashable Flavour where
+  hashWithSalt = hashUsing fromEnum
+
+instance Binary Flavour where
+  put = put . (fromIntegral :: Int -> Word16) . fromEnum
+  get = fmap (toEnum . (fromIntegral :: Word16 -> Int)) get
+
+instance NFData Flavour
 
 -- | Turn a colour set into a flavour set.
 zipPlain, zipFancy, zipLiquid :: [Color] -> [Flavour]
@@ -49,10 +60,6 @@ zipPlain = map (Flavour Plain)
 zipFancy = map (Flavour Fancy)
 zipLiquid = map (Flavour Liquid)
 
--- | The standard full set of flavours.
-stdFlav :: [Flavour]
-stdFlav = zipPlain stdCol ++ zipFancy stdCol ++ zipLiquid stdCol
-
 -- | Get the underlying base colour of a flavour.
 flavourToColor :: Flavour -> Color
 flavourToColor Flavour{baseColor} = baseColor
@@ -114,7 +121,7 @@ colorToLiquidName White     = "cloudy"
 colorToLiquidName BrBlack   = "pitchy"
 colorToLiquidName BrRed     = "red-speckled"
 colorToLiquidName BrGreen   = "sappy"
-colorToLiquidName BrYellow  = "gold"
+colorToLiquidName BrYellow  = "golden"
 colorToLiquidName BrBlue    = "blue-speckled"
 colorToLiquidName BrMagenta = "hazy"
 colorToLiquidName BrCyan    = "misty"
