@@ -9,7 +9,7 @@ module Game.LambdaHack.Common.Frequency
   , scaleFreq, renameFreq, setFreq
     -- * Consumption
   , nullFreq, runFrequency, nameFrequency
-  , minFreq, maxFreq, mostFreq, meanFreq
+  , minFreq, maxFreq, mostFreq
   ) where
 
 import Prelude ()
@@ -17,9 +17,6 @@ import Prelude ()
 import Game.LambdaHack.Common.Prelude
 
 import Control.Applicative
-import Control.DeepSeq
-import Data.Binary
-import Data.Hashable (Hashable)
 import Data.Int (Int32)
 import Data.Ord (comparing)
 import GHC.Generics (Generic)
@@ -83,12 +80,6 @@ instance Alternative Frequency where
   (<|>) = mplus
   empty = mzero
 
-instance Hashable a => Hashable (Frequency a)
-
-instance Binary a => Binary (Frequency a)
-
-instance NFData a => NFData (Frequency a)
-
 -- | Uniform discrete frequency distribution.
 uniformFreq :: Text -> [a] -> Frequency a
 uniformFreq name l = Frequency (map (\x -> (1, x)) l) name
@@ -137,12 +128,3 @@ maxFreq fr = if nullFreq fr then Nothing else Just $ maximum fr
 mostFreq :: Frequency a -> Maybe a
 mostFreq fr = if nullFreq fr then Nothing
               else Just $ snd $ maximumBy (comparing fst) $ runFrequency fr
-
--- | Average value of an @Int@ distribution, rounded up to avoid truncating
--- it in the other code higher up, which would equate 1d0 with 1d1.
-meanFreq :: Frequency Int -> Int
-meanFreq fr@(Frequency xs _) = case xs of
-  [] -> error $ "empty frequency" `showFailure` fr
-  _ -> let sumX = sum [ p * x | (p, x) <- xs ]
-           sumP = sum $ map fst xs
-       in sumX `divUp` sumP
