@@ -21,14 +21,13 @@ import           Data.Time.LocalTime
 import           GHC.Generics (Generic)
 import qualified NLP.Miniutter.English as MU
 
-import           Game.LambdaHack.Common.Faction
-import qualified Game.LambdaHack.Common.Kind as Kind
-import           Game.LambdaHack.Common.Misc
-import           Game.LambdaHack.Common.Time
-import           Game.LambdaHack.Content.ItemKind (ItemKind)
-import           Game.LambdaHack.Content.ModeKind (HiCondPoly,
-                                                   HiIndeterminant (..),
-                                                   ModeKind, Outcome (..))
+import Game.LambdaHack.Common.Faction
+import Game.LambdaHack.Common.Kind
+import Game.LambdaHack.Common.Misc
+import Game.LambdaHack.Common.Time
+import Game.LambdaHack.Content.ItemKind (ItemKind)
+import Game.LambdaHack.Content.ModeKind (HiCondPoly, HiIndeterminant (..),
+                                         ModeKind, Outcome (..))
 
 -- | A single score record. Records are ordered in the highscore table,
 -- from the best to the worst, in lexicographic ordering wrt the fields below.
@@ -39,8 +38,8 @@ data ScoreRecord = ScoreRecord
   , status       :: Status     -- ^ reason of the game interruption
   , challenge    :: Challenge  -- ^ challenge setup of the game
   , gplayerName  :: Text       -- ^ name of the faction's gplayer
-  , ourVictims   :: EM.EnumMap (Kind.Id ItemKind) Int  -- ^ allies lost
-  , theirVictims :: EM.EnumMap (Kind.Id ItemKind) Int  -- ^ foes killed
+  , ourVictims   :: EM.EnumMap (ContentId ItemKind) Int  -- ^ allies lost
+  , theirVictims :: EM.EnumMap (ContentId ItemKind) Int  -- ^ foes killed
   }
   deriving (Show, Eq, Ord, Generic)
 
@@ -54,7 +53,7 @@ instance Show ScoreTable where
   show _ = "a score table"
 
 -- | A dictionary from game mode IDs to scores tables.
-type ScoreDict = EM.EnumMap (Kind.Id ModeKind) ScoreTable
+type ScoreDict = EM.EnumMap (ContentId ModeKind) ScoreTable
 
 -- | Empty score table
 empty :: ScoreDict
@@ -71,21 +70,23 @@ insertPos s (ScoreTable table) =
 -- | Register a new score in a score table.
 register :: ScoreTable  -- ^ old table
          -> Int         -- ^ the total value of faction items
+         -> Int         -- ^ the total value of dungeon items
          -> Time        -- ^ game time spent
          -> Status      -- ^ reason of the game interruption
          -> POSIXTime   -- ^ current date
          -> Challenge   -- ^ challenge setup
          -> Text        -- ^ name of the faction's gplayer
-         -> EM.EnumMap (Kind.Id ItemKind) Int  -- ^ allies lost
-         -> EM.EnumMap (Kind.Id ItemKind) Int  -- ^ foes killed
+         -> EM.EnumMap (ContentId ItemKind) Int  -- ^ allies lost
+         -> EM.EnumMap (ContentId ItemKind) Int  -- ^ foes killed
          -> HiCondPoly
          -> (Bool, (ScoreTable, Int))
-register table total time status@Status{stOutcome} date challenge gplayerName
-         ourVictims theirVictims hiCondPoly =
+register table total dungeonTotal time status@Status{stOutcome}
+         date challenge gplayerName ourVictims theirVictims hiCondPoly =
   let turnsSpent = fromIntegral $ timeFitUp time timeTurn
-      hiInValue (hi, c) = case hi of
+      hiInValue (hi, c) = assert (total <= dungeonTotal) $ case hi of
         HiConst -> c
-        HiLoot -> c * fromIntegral total
+        HiLoot | dungeonTotal == 0 -> c  -- a fluke; no gold generated
+        HiLoot -> c * fromIntegral total / fromIntegral dungeonTotal
         HiBlitz -> -- Up to 1000000/c turns matter.
                    sqrt $ max 0 (1000000 + c * turnsSpent)
         HiSurvival -> -- Up to 1000000/c turns matter.
@@ -117,7 +118,8 @@ showScore tz (pos, score) =
         Conquer  -> "slew all opposition"
         Escape   -> "emerged victorious"
         Restart  -> "resigned prematurely"
-      curDate = tshow . utcToLocalTime tz . posixSecondsToUTCTime . date $ score
+      curDate = T.take 19 . tshow . utcToLocalTime tz
+                . posixSecondsToUTCTime . date $ score
       turns = absoluteTimeNegate (negTime score) `timeFitUp` timeTurn
       tpos = T.justifyRight 3 ' ' $ tshow pos
       tscore = T.justifyRight 6 ' ' $ tshow $ points score
@@ -134,7 +136,7 @@ showScore tz (pos, score) =
        <> diffText <> "after" <+> tturns <+> "on" <+> curDate <> "."
      ]
 
-getTable :: Kind.Id ModeKind -> ScoreDict -> ScoreTable
+getTable :: ContentId ModeKind -> ScoreDict -> ScoreTable
 getTable = EM.findWithDefault (ScoreTable [])
 
 getRecord :: Int -> ScoreTable -> ScoreRecord
