@@ -1,8 +1,9 @@
+{-# LANGUAGE TypeFamilies #-}
 -- | Inhabited dungeon levels and the operations to query and change them
 -- as the game progresses.
 module Game.LambdaHack.Common.Level
   ( -- * Dungeon
-    LevelId, AbsDepth, Dungeon
+    LevelId, Dungeon
   , ascendInBranch, whereTo
     -- * The @Level@ type and its components
   , ItemFloor, ActorMap, TileMap, SmellMap, Level(..)
@@ -24,15 +25,15 @@ import           Data.Binary
 import qualified Data.EnumMap.Strict as EM
 
 import           Game.LambdaHack.Common.Actor
+import           Game.LambdaHack.Common.ContentData
+import qualified Game.LambdaHack.Common.Dice as Dice
 import           Game.LambdaHack.Common.Item
-import qualified Game.LambdaHack.Common.Kind as Kind
-import qualified Game.LambdaHack.Common.KindOps as KindOps
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.Point
 import qualified Game.LambdaHack.Common.PointArray as PointArray
 import           Game.LambdaHack.Common.Random
 import           Game.LambdaHack.Common.Time
-import           Game.LambdaHack.Content.ItemKind (ItemKind)
+import           Game.LambdaHack.Content.CaveKind (CaveKind)
 import           Game.LambdaHack.Content.TileKind (TileKind)
 
 -- | The complete dungeon is a map from level identifiers to levels.
@@ -88,7 +89,7 @@ type ItemFloor = EM.EnumMap Point ItemBag
 type ActorMap = EM.EnumMap Point [ActorId]
 
 -- | Tile kinds on the map.
-type TileMap = PointArray.GArray Word16 (Kind.Id TileKind)
+type TileMap = PointArray.Array (ContentId TileKind)
 
 -- | Current smell on map tiles.
 type SmellMap = EM.EnumMap Point Time
@@ -96,29 +97,25 @@ type SmellMap = EM.EnumMap Point Time
 -- | A view on single, inhabited dungeon level. "Remembered" fields
 -- carry a subset of the info in the client copies of levels.
 data Level = Level
-  { ldepth      :: AbsDepth   -- ^ absolute depth of the level
-  , lfloor      :: ItemFloor  -- ^ remembered items lying on the floor
-  , lembed      :: ItemFloor  -- ^ remembered items embedded in the tile
-  , lactor      :: ActorMap   -- ^ seen actors at positions on the level
-  , ltile       :: TileMap    -- ^ remembered level map
-  , lxsize      :: X          -- ^ width of the level
-  , lysize      :: Y          -- ^ height of the level
-  , lsmell      :: SmellMap   -- ^ remembered smells on the level
-  , lstair      :: ([Point], [Point])
-                              -- ^ positions of (up, down) stairs
-  , lseen       :: Int        -- ^ currently remembered clear tiles
-  , lexplorable :: Int        -- ^ total number of explorable tiles
-  , ltime       :: Time       -- ^ local time on the level (possibly frozen)
-  , lactorCoeff :: Int        -- ^ the lower, the more monsters spawn
-  , lactorFreq  :: Freqs ItemKind
-                              -- ^ frequency of spawned actors; [] for clients
-  , litemNum    :: Int        -- ^ number of initial items, 0 for clients
-  , litemFreq   :: Freqs ItemKind
-                              -- ^ frequency of initial items; [] for clients
-  , lescape     :: [Point]    -- ^ positions of IK.Escape tiles
-  , lnight      :: Bool       -- ^ whether the level is covered in darkness
-  , lname       :: Text       -- ^ level name
-  , ldesc       :: Text       -- ^ level description
+  { lkind   :: ContentId CaveKind
+                          -- ^ the kind of cave the level is an instance of
+  , ldepth  :: Dice.AbsDepth
+                          -- ^ absolute depth of the level
+  , lfloor  :: ItemFloor  -- ^ remembered items lying on the floor
+  , lembed  :: ItemFloor  -- ^ remembered items embedded in the tile
+  , lactor  :: ActorMap   -- ^ seen actors at positions on the level;
+                          --   could be recomputed at resume, but small enough
+  , ltile   :: TileMap    -- ^ remembered level map
+  , lxsize  :: X          -- ^ width of the level
+  , lysize  :: Y          -- ^ height of the level
+  , lsmell  :: SmellMap   -- ^ remembered smells on the level
+  , lstair  :: ([Point], [Point])
+                          -- ^ positions of (up, down) stairs
+  , lescape :: [Point]    -- ^ positions of IK.Escape tiles
+  , lseen   :: Int        -- ^ currently remembered clear tiles
+  , lexpl   :: Int        -- ^ total number of explorable tiles
+  , ltime   :: Time       -- ^ local time on the level (possibly frozen)
+  , lnight  :: Bool       -- ^ whether the level is covered in darkness
   }
   deriving (Show, Eq)
 
@@ -148,7 +145,7 @@ updateSmell :: (SmellMap -> SmellMap) -> Level -> Level
 updateSmell f lvl = lvl {lsmell = f (lsmell lvl)}
 
 -- | Query for tile kinds on the map.
-at :: Level -> Point -> Kind.Id TileKind
+at :: Level -> Point -> ContentId TileKind
 {-# INLINE at #-}
 at Level{ltile} p = ltile PointArray.! p
 
@@ -164,12 +161,12 @@ findPoint x y f =
   in search
 
 -- | Find a random position on the map satisfying a predicate.
-findPos :: TileMap -> (Point -> Kind.Id TileKind -> Bool) -> Rnd Point
+findPos :: TileMap -> (Point -> ContentId TileKind -> Bool) -> Rnd Point
 findPos ltile p =
   let (x, y) = PointArray.sizeA ltile
       search = do
         pxy <- randomR (0, (x - 1) * (y - 1))
-        let tile = KindOps.Id $ ltile `PointArray.accessI` pxy
+        let tile = ContentId $ ltile `PointArray.accessI` pxy
             pos = PointArray.punindex x pxy
         if p pos tile
         then return $! pos
@@ -184,18 +181,18 @@ findPos ltile p =
 -- predicate.
 findPosTry :: Int                                  -- ^ the number of tries
            -> TileMap                              -- ^ look up in this map
-           -> (Point -> Kind.Id TileKind -> Bool)  -- ^ mandatory predicate
-           -> [Point -> Kind.Id TileKind -> Bool]  -- ^ optional predicates
+           -> (Point -> ContentId TileKind -> Bool)  -- ^ mandatory predicate
+           -> [Point -> ContentId TileKind -> Bool]  -- ^ optional predicates
            -> Rnd Point
 {-# INLINE findPosTry #-}
 findPosTry numTries ltile m = findPosTry2 numTries ltile m [] undefined
 
 findPosTry2 :: Int                                  -- ^ the number of tries
             -> TileMap                              -- ^ look up in this map
-            -> (Point -> Kind.Id TileKind -> Bool)  -- ^ mandatory predicate
-            -> [Point -> Kind.Id TileKind -> Bool]  -- ^ optional predicates
-            -> (Point -> Kind.Id TileKind -> Bool)  -- ^ good to have predicate
-            -> [Point -> Kind.Id TileKind -> Bool]  -- ^ worst case predicates
+            -> (Point -> ContentId TileKind -> Bool)  -- ^ mandatory predicate
+            -> [Point -> ContentId TileKind -> Bool]  -- ^ optional predicates
+            -> (Point -> ContentId TileKind -> Bool)  -- ^ good to have pred.
+            -> [Point -> ContentId TileKind -> Bool]  -- ^ worst case predicates
             -> Rnd Point
 findPosTry2 numTries ltile m0 l g r = assert (numTries > 0) $
   let (x, y) = PointArray.sizeA ltile
@@ -204,7 +201,7 @@ findPosTry2 numTries ltile m0 l g r = assert (numTries > 0) $
         let search 0 = accomodate fallback m tl
             search !k = do
               pxy <- randomR (0, (x - 1) * (y - 1))
-              let tile = KindOps.Id $ ltile `PointArray.accessI` pxy
+              let tile = ContentId $ ltile `PointArray.accessI` pxy
                   pos = PointArray.punindex x pxy
               if m pos tile && hd pos tile
               then return $! pos
@@ -217,6 +214,7 @@ findPosTry2 numTries ltile m0 l g r = assert (numTries > 0) $
 
 instance Binary Level where
   put Level{..} = do
+    put lkind
     put ldepth
     put (assertSparseItems lfloor)
     put (assertSparseItems lembed)
@@ -226,18 +224,13 @@ instance Binary Level where
     put lysize
     put lsmell
     put lstair
-    put lseen
-    put lexplorable
-    put ltime
-    put lactorCoeff
-    put lactorFreq
-    put litemNum
-    put litemFreq
     put lescape
+    put lseen
+    put lexpl
+    put ltime
     put lnight
-    put lname
-    put ldesc
   get = do
+    lkind <- get
     ldepth <- get
     lfloor <- get
     lembed <- get
@@ -247,15 +240,9 @@ instance Binary Level where
     lysize <- get
     lsmell <- get
     lstair <- get
-    lseen <- get
-    lexplorable <- get
-    ltime <- get
-    lactorCoeff <- get
-    lactorFreq <- get
-    litemNum <- get
-    litemFreq <- get
     lescape <- get
+    lseen <- get
+    lexpl <- get
+    ltime <- get
     lnight <- get
-    lname <- get
-    ldesc <- get
     return $! Level{..}
