@@ -5,30 +5,30 @@ module Game.LambdaHack.Common.Misc
   ( -- * Game object identifiers
     FactionId, LevelId, ActorId
     -- * Item containers
-  , Container(..), CStore(..), ItemDialogMode(..)
+  , Container(..), CStore(..), SLore(..), ItemDialogMode(..)
     -- * Assorted
-  , GroupName, Freqs, Rarity, AbsDepth(..), Tactic(..)
-  , toGroupName, validateRarity, describeTactic
-  , makePhrase, makeSentence, normalLevelBound, breturn
-  , appDataDir, xM, xD, minusM, minusM1, oneM
+  , GroupName, Tactic(..)
+  , toGroupName, describeTactic
+  , makePhrase, makeSentence, squashedWWandW, normalLevelBound
+  , appDataDir, xM, xD, minusM, minusM1, oneM, tenthM
+  , workaroundOnMainThreadMVar
   ) where
 
 import Prelude ()
 
 import Game.LambdaHack.Common.Prelude
 
+import           Control.Concurrent
 import           Control.DeepSeq
 import           Data.Binary
 import qualified Data.Char as Char
 import qualified Data.EnumMap.Strict as EM
 import qualified Data.EnumSet as ES
 import qualified Data.Fixed as Fixed
-import           Data.Function
 import           Data.Hashable
 import qualified Data.HashMap.Strict as HM
 import           Data.Int (Int64)
 import           Data.Key
-import           Data.Ord
 import           Data.String (IsString (..))
 import qualified Data.Text as T
 import qualified Data.Time as Time
@@ -36,6 +36,7 @@ import           GHC.Generics (Generic)
 import qualified NLP.Miniutter.English as MU
 import           System.Directory (getAppUserDataDirectory)
 import           System.Environment (getProgName)
+import           System.IO.Unsafe (unsafePerformIO)
 
 import Game.LambdaHack.Common.Point
 
@@ -45,7 +46,11 @@ newtype FactionId = FactionId Int
 
 -- | Abstract level identifiers.
 newtype LevelId = LevelId Int
-  deriving (Show, Eq, Ord, Enum, Hashable, Binary)
+  deriving (Show, Eq, Ord, Hashable, Binary)
+
+instance Enum LevelId where
+  fromEnum (LevelId n) = n
+  toEnum = LevelId  -- picks the main branch of the dungeon
 
 -- | A unique identifier of an actor in the dungeon.
 newtype ActorId = ActorId Int
@@ -72,11 +77,23 @@ data CStore =
 
 instance Binary CStore
 
-instance Hashable CStore
-
 instance NFData CStore
 
-data ItemDialogMode = MStore CStore | MOwned | MStats | MLoreItem | MLoreOrgan
+-- | Item slot and lore categories.
+data SLore =
+    SItem
+  | SOrgan
+  | STrunk
+  | STmp
+  | SBlast
+  | SEmbed
+  deriving (Show, Read, Eq, Ord, Enum, Bounded, Generic)
+
+instance Binary SLore
+
+instance NFData SLore
+
+data ItemDialogMode = MStore CStore | MOrgans | MOwned | MStats | MLore SLore
   deriving (Show, Read, Eq, Ord, Generic)
 
 instance NFData ItemDialogMode
@@ -97,21 +114,6 @@ instance Show (GroupName a) where
 
 instance NFData (GroupName a)
 
--- | For each group that the kind belongs to, denoted by a @GroupName@
--- in the first component of a pair, the second component of a pair shows
--- how common the kind is within the group.
-type Freqs a = [(GroupName a, Int)]
-
--- | Rarity on given depths.
-type Rarity = [(Double, Int)]
-
--- | Absolute depth in the dungeon. When used for the maximum depth
--- of the whole dungeon, this can be different than dungeon size,
--- e.g., when the dungeon is branched, and it can even be different
--- than the length of the longest branch, if levels at some depths are missing.
-newtype AbsDepth = AbsDepth Int
-  deriving (Show, Eq, Ord, Hashable, Binary)
-
 -- | Tactic of non-leader actors. Apart of determining AI operation,
 -- each tactic implies a skill modifier, that is added to the non-leader skills
 -- defined in 'fskillsOther' field of 'Player'.
@@ -141,24 +143,12 @@ instance Show Tactic where
 
 instance Binary Tactic
 
-instance Hashable Tactic
+instance NFData Tactic
 
 toGroupName :: Text -> GroupName a
 {-# INLINE toGroupName #-}
 toGroupName = GroupName
 
-validateRarity :: Rarity -> [Text]
-validateRarity rarity =
-  let sortedRarity = sortBy (comparing fst) rarity
-  in [ "rarity not sorted" | sortedRarity /= rarity ]
-     ++ [ "rarity depth thresholds not unique"
-        | nubBy ((==) `on` fst) sortedRarity /= sortedRarity ]
-     ++ [ "rarity depth not between 0 and 10"
-        | case (sortedRarity, reverse sortedRarity) of
-            ((lowest, _) : _, (highest, _) : _) ->
-              lowest <= 0 || highest > 10
-            _ -> False ]
-
 describeTactic :: Tactic -> Text
 describeTactic TExplore = "investigate unknown positions, chase targets"
 describeTactic TFollow = "follow leader's target or position, grab items"
@@ -177,15 +167,28 @@ makePhrase, makeSentence :: [MU.Part] -> Text
 makePhrase = MU.makePhrase MU.defIrregular
 makeSentence = MU.makeSentence MU.defIrregular
 
+-- | Apply the @WWandW@ constructor, first representing repetitions
+-- as @CardinalWs@.
+-- The parts are not sorted, only grouped, to keep the order.
+-- The internal structure of speech parts is compared, not their string
+-- rendering, so some coincidental clashes are avoided (and code is simpler).
+squashedWWandW :: [MU.Part] -> (MU.Part, MU.Person)
+squashedWWandW parts =
+  let repetitions = group parts
+      f [] = error $ "empty group" `showFailure` parts
+      f [part] = (part, MU.Sg3rd)  -- avoid prefixing hero names with "a"
+      f l@(part : _) = (MU.CardinalWs (length l) part, MU.PlEtc)
+      cars = map f repetitions
+      person = case cars of
+        [] -> error $ "empty cars" `showFailure` parts
+        [(_, person1)] -> person1
+        _ -> MU.PlEtc
+  in (MU.WWandW $ map fst cars, person)
+
 -- | Level bounds.
 normalLevelBound :: (Int, Int)
 normalLevelBound = (79, 20)
 
--- | @breturn b a = [a | b]@
-breturn :: MonadPlus m => Bool -> a -> m a
-breturn True a  = return a
-breturn False _ = mzero
-
 -- | Personal data directory for the game. Depends on the OS and the game,
 -- e.g., for LambdaHack under Linux it's @~\/.LambdaHack\/@.
 appDataDir :: IO FilePath
@@ -200,10 +203,16 @@ xM k = fromIntegral k * 1000000
 xD :: Double -> Double
 xD k = k * 1000000
 
-minusM, minusM1, oneM :: Int64
+minusM, minusM1, oneM, tenthM :: Int64
 minusM = xM (-1)
 minusM1 = xM (-1) - 1
 oneM = xM 1
+tenthM = 100000
+
+-- Global variable for passing the action to run on main thread, if any.
+workaroundOnMainThreadMVar :: MVar (IO ())
+{-# NOINLINE workaroundOnMainThreadMVar #-}
+workaroundOnMainThreadMVar = unsafePerformIO newEmptyMVar
 
 -- Data.Binary orphan instances
 
