@@ -17,8 +17,7 @@ import           Game.LambdaHack.Common.Actor
 import           Game.LambdaHack.Common.ActorState
 import           Game.LambdaHack.Common.Faction
 import           Game.LambdaHack.Common.Item
-import           Game.LambdaHack.Common.ItemStrongest
-import qualified Game.LambdaHack.Common.Kind as Kind
+import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Level
 import           Game.LambdaHack.Common.ReqFailure
 import           Game.LambdaHack.Common.State
@@ -45,9 +44,9 @@ nUI = do
 
 getGameMode :: MonadStateRead m => m ModeKind
 getGameMode = do
-  Kind.COps{comode=Kind.Ops{okind}} <- getsState scops
+  COps{comode} <- getsState scops
   t <- getsState sgameModeId
-  return $! okind t
+  return $! okind comode t
 
 isNoConfirmsGame :: MonadStateRead m => m Bool
 isNoConfirmsGame = do
@@ -67,18 +66,18 @@ getEntryArena fact = do
 
 pickWeaponM :: MonadStateRead m
             => Maybe DiscoveryBenefit
-            -> [(ItemId, ItemFull)] -> Ability.Skills -> ActorId
-            -> m [(Double, (ItemId, ItemFull))]
-pickWeaponM mdiscoBenefit allAssocs actorSk source = do
+            -> [(ItemId, ItemFullKit)] -> Ability.Skills -> ActorId
+            -> m [(Double, (ItemId, ItemFullKit))]
+pickWeaponM mdiscoBenefit kitAss actorSk source = do
   sb <- getsState $ getActorBody source
   localTime <- getsState $ getLocalTime (blid sb)
   ar <- getsState $ getActorAspect source
   let calmE = calmEnough sb ar
       forced = bproj sb
-      permitted = permittedPrecious calmE forced
+      permitted = permittedPrecious forced calmE
       preferredPrecious = either (const False) id . permitted
-      permAssocs = filter (preferredPrecious . snd) allAssocs
+      permAssocs = filter (preferredPrecious . fst . snd) kitAss
       strongest = strongestMelee mdiscoBenefit localTime permAssocs
-  return $! if | forced -> map (1,) allAssocs
+  return $! if | forced -> map (1,) kitAss
                | EM.findWithDefault 0 Ability.AbMelee actorSk <= 0 -> []
                | otherwise -> strongest
