@@ -14,7 +14,6 @@ import Prelude ()
 
 import Game.LambdaHack.Common.Prelude
 
-import Control.DeepSeq
 import Data.Binary
 import Data.Bits (unsafeShiftL, unsafeShiftR, (.&.))
 import Data.Int (Int32)
@@ -42,8 +41,6 @@ instance Binary Point where
   put = put . (fromIntegral :: Int -> Int32) . fromEnum
   get = fmap (toEnum . (fromIntegral :: Int32 -> Int)) get
 
-instance NFData Point
-
 -- This conversion cannot be used for PointArray indexing,
 -- because it is not contiguous --- we don't know the horizontal
 -- width of the levels nor of the screen.
