@@ -1,6 +1,7 @@
+{-# LANGUAGE FlexibleContexts, StandaloneDeriving, TypeFamilies #-}
 -- | Arrays, based on Data.Vector.Unboxed, indexed by @Point@.
 module Game.LambdaHack.Common.PointArray
-  ( GArray(..), Array, pindex, punindex
+  ( UnboxRepClass(..), Array(..), pindex, punindex
   , (!), accessI, (//), unsafeUpdateA, unsafeWriteA, unsafeWriteManyA
   , replicateA, replicateMA, generateA, generateMA, unfoldrNA, sizeA
   , foldrA, foldrA', foldlA', ifoldrA, ifoldrA', ifoldlA', foldMA', ifoldMA'
@@ -9,7 +10,7 @@ module Game.LambdaHack.Common.PointArray
   , fromListA, toListA
 #ifdef EXPOSE_INTERNAL
     -- * Internal operations
-  , cnv
+  , toUnboxRep
 #endif
   ) where
 
@@ -27,18 +28,35 @@ import qualified Data.Vector.Unboxed.Mutable as VM
 
 import Game.LambdaHack.Common.Point
 
+class ( Ord c, Eq (UnboxRep c), Ord (UnboxRep c), Bounded (UnboxRep c)
+      , Binary (UnboxRep c), U.Unbox (UnboxRep c) )
+      => UnboxRepClass c where
+  type UnboxRep c
+  type instance UnboxRep c = c
+  toUnboxRepUnsafe :: c -> UnboxRep c  -- has to be total
+  fromUnboxRep :: UnboxRep c -> c  -- has to be total
+
+instance UnboxRepClass Bool where
+  toUnboxRepUnsafe c = c
+  fromUnboxRep c = c
+
+instance UnboxRepClass Word8 where
+  toUnboxRepUnsafe c = c
+  fromUnboxRep c = c
+
 -- | Arrays indexed by @Point@.
-data GArray w c = Array
+data Array c = Array
   { axsize  :: X
   , aysize  :: Y
-  , avector :: U.Vector w
+  , avector :: U.Vector (UnboxRep c)
   }
-  deriving Eq
 
-instance Show (GArray w c) where
-  show a = "PointArray.Array with size " ++ show (sizeA a)
+deriving instance UnboxRepClass c => Eq (Array c)
 
-instance (U.Unbox w, Binary w) => Binary (GArray w c) where
+instance Show (Array c) where
+  show a = "PointArray.Array with size " ++ show (axsize a, aysize a)
+
+instance UnboxRepClass c => Binary (Array c) where
   put Array{..} = do
     put axsize
     put aysize
@@ -49,12 +67,13 @@ instance (U.Unbox w, Binary w) => Binary (GArray w c) where
     avector <- get
     return $! Array{..}
 
--- | Arrays of, effectively, @Word8@, indexed by @Point@.
-type Array c = GArray Word8 c
-
-cnv :: (Enum a, Enum b) => a -> b
-{-# INLINE cnv #-}
-cnv = toEnum . fromEnum
+toUnboxRep :: UnboxRepClass c => c -> UnboxRep c
+{-# INLINE toUnboxRep #-}
+toUnboxRep c =
+#ifdef WITH_EXPENSIVE_ASSERTIONS
+  assert (c <= fromUnboxRep maxBound) $
+#endif
+    toUnboxRepUnsafe c
 
 -- Note that @Ord@ on @Int@ is not monotonic wrt @Ord@ on @Point@.
 -- We need to keep it that way, because we want close xs to have close indexes.
@@ -71,185 +90,182 @@ punindex xsize n = let (y, x) = n `quotRem` xsize
 -- since the extra few additions in @fromPoint@ may be less expensive than
 -- memory or register allocations needed for the extra @Int@ in @Point@.
 -- | Array lookup.
-(!) :: (U.Unbox w, Enum w, Enum c) => GArray w c -> Point -> c
+(!) :: UnboxRepClass c => Array c -> Point -> c
 {-# INLINE (!) #-}
-(!) Array{..} p = cnv $ avector U.! pindex axsize p
+(!) Array{..} p = fromUnboxRep $ avector U.! pindex axsize p
 
-accessI :: U.Unbox w => GArray w c -> Int -> w
+accessI :: UnboxRepClass c => Array c -> Int -> UnboxRep c
 {-# INLINE accessI #-}
 accessI Array{..} p = avector `U.unsafeIndex` p
 
 -- | Construct an array updated with the association list.
-(//) :: (U.Unbox w, Enum w, Enum c) => GArray w c -> [(Point, c)] -> GArray w c
+(//) :: UnboxRepClass c => Array c -> [(Point, c)] -> Array c
 {-# INLINE (//) #-}
-(//) Array{..} l = let v = avector U.// map (pindex axsize *** cnv) l
+(//) Array{..} l = let v = avector U.// map (pindex axsize *** toUnboxRep) l
                    in Array{avector = v, ..}
 
-unsafeUpdateA :: (U.Unbox w, Enum w, Enum c) => GArray w c -> [(Point, c)] -> ()
+unsafeUpdateA :: UnboxRepClass c => Array c -> [(Point, c)] -> ()
 {-# INLINE unsafeUpdateA #-}
 unsafeUpdateA Array{..} l = runST $ do
   vThawed <- U.unsafeThaw avector
-  mapM_ (\(p, c) -> VM.write vThawed (pindex axsize p) (cnv c)) l
+  mapM_ (\(p, c) -> VM.write vThawed (pindex axsize p) (toUnboxRep c)) l
   void $ U.unsafeFreeze vThawed
 
-unsafeWriteA :: (U.Unbox w, Enum w, Enum c) => GArray w c -> Point -> c -> ()
+unsafeWriteA :: UnboxRepClass c => Array c -> Point -> c -> ()
 {-# INLINE unsafeWriteA #-}
 unsafeWriteA Array{..} p c = runST $ do
   vThawed <- U.unsafeThaw avector
-  VM.write vThawed (pindex axsize p) (cnv c)
+  VM.write vThawed (pindex axsize p) (toUnboxRep c)
   void $ U.unsafeFreeze vThawed
 
-unsafeWriteManyA :: (U.Unbox w, Enum w, Enum c) => GArray w c -> [Point] -> c -> ()
+unsafeWriteManyA :: UnboxRepClass c => Array c -> [Point] -> c -> ()
 {-# INLINE unsafeWriteManyA #-}
 unsafeWriteManyA Array{..} l c = runST $ do
   vThawed <- U.unsafeThaw avector
-  let d = cnv c
+  let d = toUnboxRep c
   mapM_ (\p -> VM.write vThawed (pindex axsize p) d) l
   void $ U.unsafeFreeze vThawed
 
 -- | Create an array from a replicated element.
-replicateA :: (U.Unbox w, Enum w, Enum c) => X -> Y -> c -> GArray w c
+replicateA :: UnboxRepClass c => X -> Y -> c -> Array c
 {-# INLINE replicateA #-}
 replicateA axsize aysize c =
-  Array{avector = U.replicate (axsize * aysize) $ cnv c, ..}
+  Array{avector = U.replicate (axsize * aysize) $ toUnboxRep c, ..}
 
 -- | Create an array from a replicated monadic action.
-replicateMA :: (U.Unbox w, Enum w, Enum c, Monad m) => X -> Y -> m c -> m (GArray w c)
+replicateMA :: (Monad m, UnboxRepClass c) => X -> Y -> m c -> m (Array c)
 {-# INLINE replicateMA #-}
 replicateMA axsize aysize m = do
-  v <- U.replicateM (axsize * aysize) $ liftM cnv m
+  v <- U.replicateM (axsize * aysize) $ liftM toUnboxRep m
   return $! Array{avector = v, ..}
 
 -- | Create an array from a function.
-generateA :: (U.Unbox w, Enum w, Enum c) => X -> Y -> (Point -> c) -> GArray w c
+generateA :: UnboxRepClass c => X -> Y -> (Point -> c) -> Array c
 {-# INLINE generateA #-}
 generateA axsize aysize f =
-  let g n = cnv $ f $ punindex axsize n
+  let g n = toUnboxRep $ f $ punindex axsize n
   in Array{avector = U.generate (axsize * aysize) g, ..}
 
 -- | Create an array from a monadic function.
-generateMA :: (U.Unbox w, Enum w, Enum c, Monad m) => X -> Y -> (Point -> m c) -> m (GArray w c)
+generateMA :: (Monad m, UnboxRepClass c)
+           => X -> Y -> (Point -> m c) -> m (Array c)
 {-# INLINE generateMA #-}
 generateMA axsize aysize fm = do
-  let gm n = liftM cnv $ fm $ punindex axsize n
+  let gm n = liftM toUnboxRep $ fm $ punindex axsize n
   v <- U.generateM (axsize * aysize) gm
   return $! Array{avector = v, ..}
 
-unfoldrNA :: (U.Unbox w, Enum w, Enum c) => X -> Y -> (b -> (c, b)) -> b -> GArray w c
+unfoldrNA :: UnboxRepClass c => X -> Y -> (b -> (c, b)) -> b -> Array c
 {-# INLINE unfoldrNA #-}
 unfoldrNA axsize aysize fm b =
-  let gm = Just . first cnv . fm
+  let gm = Just . first toUnboxRep . fm
       v = U.unfoldrN (axsize * aysize) gm b
   in Array {avector = v, ..}
 
 -- | Content identifiers array size.
-sizeA :: GArray w c -> (X, Y)
+sizeA :: Array c -> (X, Y)
 {-# INLINE sizeA #-}
 sizeA Array{..} = (axsize, aysize)
 
 -- | Fold right over an array.
-foldrA :: (U.Unbox w, Enum w, Enum c) => (c -> a -> a) -> a -> GArray w c -> a
+foldrA :: UnboxRepClass c => (c -> a -> a) -> a -> Array c -> a
 {-# INLINE foldrA #-}
 foldrA f z0 Array{..} =
-  U.foldr (\c a-> f (cnv c) a) z0 avector
+  U.foldr (\c a-> f (fromUnboxRep c) a) z0 avector
 
 -- | Fold right strictly over an array.
-foldrA' :: (U.Unbox w, Enum w, Enum c) => (c -> a -> a) -> a -> GArray w c -> a
+foldrA' :: UnboxRepClass c => (c -> a -> a) -> a -> Array c -> a
 {-# INLINE foldrA' #-}
 foldrA' f z0 Array{..} =
-  U.foldr' (\c a-> f (cnv c) a) z0 avector
+  U.foldr' (\c a-> f (fromUnboxRep c) a) z0 avector
 
 -- | Fold left strictly over an array.
-foldlA' :: (U.Unbox w, Enum w, Enum c) => (a -> c -> a) -> a -> GArray w c -> a
+foldlA' :: UnboxRepClass c => (a -> c -> a) -> a -> Array c -> a
 {-# INLINE foldlA' #-}
 foldlA' f z0 Array{..} =
-  U.foldl' (\a c -> f a (cnv c)) z0 avector
+  U.foldl' (\a c -> f a (fromUnboxRep c)) z0 avector
 
 -- | Fold left strictly over an array
 -- (function applied to each element and its index).
-ifoldlA' :: (U.Unbox w, Enum w, Enum c)
-         => (a -> Point -> c -> a) -> a -> GArray w c -> a
+ifoldlA' :: UnboxRepClass c => (a -> Point -> c -> a) -> a -> Array c -> a
 {-# INLINE ifoldlA' #-}
 ifoldlA' f z0 Array{..} =
-  U.ifoldl' (\a n c -> f a (punindex axsize n) (cnv c)) z0 avector
+  U.ifoldl' (\a n c -> f a (punindex axsize n) (fromUnboxRep c)) z0 avector
 
 -- | Fold right over an array
 -- (function applied to each element and its index).
-ifoldrA :: (U.Unbox w, Enum w, Enum c)
-        => (Point -> c -> a -> a) -> a -> GArray w c -> a
+ifoldrA :: UnboxRepClass c => (Point -> c -> a -> a) -> a -> Array c -> a
 {-# INLINE ifoldrA #-}
 ifoldrA f z0 Array{..} =
-  U.ifoldr (\n c a -> f (punindex axsize n) (cnv c) a) z0 avector
+  U.ifoldr (\n c a -> f (punindex axsize n) (fromUnboxRep c) a) z0 avector
 
 -- | Fold right strictly over an array
 -- (function applied to each element and its index).
-ifoldrA' :: (U.Unbox w, Enum w, Enum c)
-         => (Point -> c -> a -> a) -> a -> GArray w c -> a
+ifoldrA' :: UnboxRepClass c => (Point -> c -> a -> a) -> a -> Array c -> a
 {-# INLINE ifoldrA' #-}
 ifoldrA' f z0 Array{..} =
-  U.ifoldr' (\n c a -> f (punindex axsize n) (cnv c) a) z0 avector
+  U.ifoldr' (\n c a -> f (punindex axsize n) (fromUnboxRep c) a) z0 avector
 
 -- | Fold monadically strictly over an array.
-foldMA' :: (Monad m, U.Unbox w, Enum w, Enum c)
-        => (a -> c -> m a) -> a -> GArray w c -> m a
+foldMA' :: (Monad m, UnboxRepClass c) => (a -> c -> m a) -> a -> Array c -> m a
 {-# INLINE foldMA' #-}
 foldMA' f z0 Array{..} =
-  U.foldM' (\a c -> f a (cnv c)) z0 avector
+  U.foldM' (\a c -> f a (fromUnboxRep c)) z0 avector
 
 -- | Fold monadically strictly over an array
 -- (function applied to each element and its index).
-ifoldMA' :: (Monad m, U.Unbox w, Enum w, Enum c)
-         => (a -> Point -> c -> m a) -> a -> GArray w c -> m a
+ifoldMA' :: (Monad m, UnboxRepClass c)
+         => (a -> Point -> c -> m a) -> a -> Array c -> m a
 {-# INLINE ifoldMA' #-}
 ifoldMA' f z0 Array{..} =
-  U.ifoldM' (\a n c -> f a (punindex axsize n) (cnv c)) z0 avector
+  U.ifoldM' (\a n c -> f a (punindex axsize n) (fromUnboxRep c)) z0 avector
 
 -- | Map over an array.
-mapA :: (U.Unbox w1, Enum w1, U.Unbox w2, Enum w2, Enum c, Enum d)
-     => (c -> d) -> GArray w1 c -> GArray w2 d
+mapA :: (UnboxRepClass c, UnboxRepClass d) => (c -> d) -> Array c -> Array d
 {-# INLINE mapA #-}
-mapA f Array{..} = Array{avector = U.map (cnv . f . cnv) avector, ..}
+mapA f Array{..} =
+  Array{avector = U.map (toUnboxRep . f . fromUnboxRep) avector, ..}
 
 -- | Map over an array (function applied to each element and its index).
-imapA :: (U.Unbox w1, Enum w1, U.Unbox w2, Enum w2, Enum c, Enum d)
-      => (Point -> c -> d) -> GArray w1 c -> GArray w2 d
+imapA :: (UnboxRepClass c, UnboxRepClass d)
+      =>  (Point -> c -> d) -> Array c -> Array d
 {-# INLINE imapA #-}
 imapA f Array{..} =
-  let v = U.imap (\n c -> cnv $ f (punindex axsize n) (cnv c)) avector
+  let v = U.imap (\n c ->
+                   toUnboxRep $ f (punindex axsize n) (fromUnboxRep c)) avector
   in Array{avector = v, ..}
 
 -- | Map monadically over an array (function applied to each element
 -- and its index) and ignore the results.
-imapMA_ :: (U.Unbox w, Enum w, Enum c, Monad m)
-             => (Point -> c -> m ()) -> GArray w c -> m ()
+imapMA_ :: (Monad m, UnboxRepClass c) => (Point -> c -> m ()) -> Array c -> m ()
 {-# INLINE imapMA_ #-}
 imapMA_ f Array{..} =
-  U.imapM_ (\n c -> f (punindex axsize n) (cnv c)) avector
+  U.imapM_ (\n c -> f (punindex axsize n) (fromUnboxRep c)) avector
 
 -- | Set all elements to the given value, in place.
-unsafeSetA :: (U.Unbox w, Enum w, Enum c) => c -> GArray w c -> GArray w c
+unsafeSetA :: UnboxRepClass c => c -> Array c -> Array c
 {-# INLINE unsafeSetA #-}
 unsafeSetA c Array{..} = runST $ do
   vThawed <- U.unsafeThaw avector
-  VM.set vThawed (cnv c)
+  VM.set vThawed (toUnboxRep c)
   vFrozen <- U.unsafeFreeze vThawed
   return $! Array{avector = vFrozen, ..}
 
 -- | Set all elements to the given value, in place, if possible.
-safeSetA :: (U.Unbox w, Enum w, Enum c) => c -> GArray w c -> GArray w c
+safeSetA :: UnboxRepClass c => c -> Array c -> Array c
 {-# INLINE safeSetA #-}
 safeSetA c Array{..} =
-  Array{avector = U.modify (\v -> VM.set v (cnv c)) avector, ..}
+  Array{avector = U.modify (\v -> VM.set v (toUnboxRep c)) avector, ..}
 
 -- | Yield the point coordinates of a minimum element of the array.
 -- The array may not be empty.
-minIndexA :: (U.Unbox w, Ord w) => GArray w c -> Point
+minIndexA :: UnboxRepClass c => Array c -> Point
 {-# INLINE minIndexA #-}
 minIndexA Array{..} = punindex axsize $ U.minIndex avector
 
 -- | Yield the point coordinates of the last minimum element of the array.
 -- The array may not be empty.
-minLastIndexA :: (U.Unbox w, Ord w) => GArray w c -> Point
+minLastIndexA :: UnboxRepClass c => Array c -> Point
 {-# INLINE minLastIndexA #-}
 minLastIndexA Array{..} =
   punindex axsize
@@ -260,25 +276,26 @@ minLastIndexA Array{..} =
 
 -- | Yield the point coordinates of all the minimum elements of the array.
 -- The array may not be empty.
-minIndexesA :: (U.Unbox w, Enum w, Ord w) => GArray w c -> [Point]
+minIndexesA :: UnboxRepClass c => Array c -> [Point]
 {-# INLINE minIndexesA #-}
 minIndexesA Array{..} =
-  map (punindex axsize)
-  $ Bundle.foldr imin [] . Bundle.indexed . G.stream
-  $ avector
+  Bundle.foldr imin [] . Bundle.indexed . G.stream $ avector
  where
-  imin (i, x) acc = i `seq` if x == minE then i : acc else acc
-  minE = cnv $ U.minimum avector
+  imin (i, x) acc = if x == minE
+                    then let !j = punindex axsize i
+                         in j : acc
+                    else acc
+  !minE = U.minimum avector
 
 -- | Yield the point coordinates of the first maximum element of the array.
 -- The array may not be empty.
-maxIndexA :: (U.Unbox w, Ord w) => GArray w c -> Point
+maxIndexA :: UnboxRepClass c => Array c -> Point
 {-# INLINE maxIndexA #-}
 maxIndexA Array{..} = punindex axsize $ U.maxIndex avector
 
 -- | Yield the point coordinates of the last maximum element of the array.
 -- The array may not be empty.
-maxLastIndexA :: (U.Unbox w, Ord w) => GArray w c -> Point
+maxLastIndexA :: UnboxRepClass c => Array c -> Point
 {-# INLINE maxLastIndexA #-}
 maxLastIndexA Array{..} =
   punindex axsize
@@ -288,15 +305,15 @@ maxLastIndexA Array{..} =
   imax (i, x) (j, y) = i `seq` j `seq` if x <= y then (j, y) else (i, x)
 
 -- | Force the array not to retain any extra memory.
-forceA :: U.Unbox w => GArray w c -> GArray w c
+forceA :: UnboxRepClass c => Array c -> Array c
 {-# INLINE forceA #-}
 forceA Array{..} = Array{avector = U.force avector, ..}
 
-fromListA :: (U.Unbox w, Enum w, Enum c) => X -> Y -> [c] -> GArray w c
+fromListA :: UnboxRepClass c => X -> Y -> [c] -> Array c
 {-# INLINE fromListA #-}
 fromListA axsize aysize l =
-  Array{avector = U.fromListN (axsize * aysize) $ map cnv l, ..}
+  Array{avector = U.fromListN (axsize * aysize) $ map toUnboxRep l, ..}
 
-toListA :: (U.Unbox w, Enum w, Enum c) => GArray w c -> [c]
+toListA :: UnboxRepClass c => Array c -> [c]
 {-# INLINE toListA #-}
-toListA Array{..} = map cnv $ U.toList avector
+toListA Array{..} = map fromUnboxRep $ U.toList avector
