@@ -4,12 +4,12 @@ module Game.LambdaHack.Common.State
     State
     -- * State components
   , sdungeon, stotalDepth, sactorD, sitemD, sitemIxMap, sfactionD, stime, scops
-  , shigh, sgameModeId, sdiscoKind, sdiscoAspect, sactorAspect
+  , sgold, shigh, sgameModeId, sdiscoKind, sdiscoAspect, sactorAspect
     -- * State construction
   , defStateGlobal, emptyState, localFromGlobal
     -- * State update
   , updateDungeon, updateDepth, updateActorD, updateItemD, updateItemIxMap
-  , updateFactionD, updateTime, updateCOps
+  , updateFactionD, updateTime, updateCOpsAndCachedData, updateGold
   , updateDiscoKind, updateDiscoAspect, updateActorAspect
     -- * State operations
   , getItemBody, aspectRecordFromItem, aspectRecordFromIid
@@ -28,15 +28,18 @@ import           Data.Binary
 import qualified Data.EnumMap.Strict as EM
 
 import           Game.LambdaHack.Common.Actor
+import qualified Game.LambdaHack.Common.Dice as Dice
 import           Game.LambdaHack.Common.Faction
 import qualified Game.LambdaHack.Common.HighScore as HighScore
 import           Game.LambdaHack.Common.Item
-import qualified Game.LambdaHack.Common.Kind as Kind
+import qualified Game.LambdaHack.Common.ItemAspect as IA
+import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Level
-import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.Point
 import qualified Game.LambdaHack.Common.PointArray as PointArray
 import           Game.LambdaHack.Common.Time
+import           Game.LambdaHack.Content.CaveKind (CaveKind)
+import qualified Game.LambdaHack.Content.ItemKind as IK
 import           Game.LambdaHack.Content.ModeKind
 import           Game.LambdaHack.Content.TileKind (TileKind, unknownId)
 
@@ -48,18 +51,21 @@ import           Game.LambdaHack.Content.TileKind (TileKind, unknownId)
 -- the actions to each client state in turn).
 data State = State
   { _sdungeon     :: Dungeon      -- ^ remembered dungeon
-  , _stotalDepth  :: AbsDepth     -- ^ absolute dungeon depth, for item creation
+  , _stotalDepth  :: Dice.AbsDepth
+                                  -- ^ absolute dungeon depth, for item creation
   , _sactorD      :: ActorDict    -- ^ remembered actors in the dungeon
   , _sitemD       :: ItemDict     -- ^ remembered items in the dungeon
   , _sitemIxMap   :: ItemIxMap    -- ^ spotted items with the same kind index
+                                  --   could be recomputed at resume, but small
   , _sfactionD    :: FactionDict  -- ^ remembered sides still in game
   , _stime        :: Time         -- ^ global game time, for UI display only
-  , _scops        :: ~Kind.COps   -- ^ remembered content
+  , _scops        :: COps         -- ^ remembered content
+  , _sgold        :: Int          -- ^ total value of human trinkets in dungeon
   , _shigh        :: HighScore.ScoreDict  -- ^ high score table
-  , _sgameModeId  :: Kind.Id ModeKind     -- ^ current game mode
-  , _sdiscoKind   :: DiscoveryKind     -- ^ item kind discoveries data
-  , _sdiscoAspect :: DiscoveryAspect   -- ^ item aspect data
-  , _sactorAspect :: ActorAspect       -- ^ actor aspect data
+  , _sgameModeId  :: ContentId ModeKind   -- ^ current game mode
+  , _sdiscoKind   :: DiscoveryKind    -- ^ item kind discoveries data
+  , _sdiscoAspect :: DiscoveryAspect  -- ^ item aspect data; could be recomputed
+  , _sactorAspect :: ActorAspect      -- ^ actor aspect records; is recomputed
   }
   deriving (Show, Eq)
 
@@ -72,6 +78,7 @@ instance Binary State where
     put _sitemIxMap
     put _sfactionD
     put _stime
+    put _sgold
     put _shigh
     put _sgameModeId
     put _sdiscoKind
@@ -84,19 +91,19 @@ instance Binary State where
     _sitemIxMap <- get
     _sfactionD <- get
     _stime <- get
+    _sgold <- get
     _shigh <- get
     _sgameModeId <- get
     _sdiscoKind <- get
     _sdiscoAspect <- get
-    let _scops = error $ "overwritten by recreated cops" `showFailure` ()
-        sNoActorAspect = State{_sactorAspect = EM.empty, ..}
-        _sactorAspect = actorAspectInDungeon sNoActorAspect
+    let _scops = emptyCOps
+        _sactorAspect = EM.empty
     return $! State{..}
 
 sdungeon :: State -> Dungeon
 sdungeon = _sdungeon
 
-stotalDepth :: State -> AbsDepth
+stotalDepth :: State -> Dice.AbsDepth
 stotalDepth = _stotalDepth
 
 sactorD :: State -> ActorDict
@@ -114,13 +121,16 @@ sfactionD = _sfactionD
 stime :: State -> Time
 stime = _stime
 
-scops :: State -> Kind.COps
+scops :: State -> COps
 scops = _scops
 
+sgold :: State -> Int
+sgold = _sgold
+
 shigh :: State -> HighScore.ScoreDict
 shigh = _shigh
 
-sgameModeId :: State -> Kind.Id ModeKind
+sgameModeId :: State -> ContentId ModeKind
 sgameModeId = _sgameModeId
 
 sdiscoKind :: State -> DiscoveryKind
@@ -132,14 +142,14 @@ sdiscoAspect = _sdiscoAspect
 sactorAspect :: State -> ActorAspect
 sactorAspect = _sactorAspect
 
-unknownLevel :: Kind.COps -> AbsDepth -> X -> Y
-             -> Text -> Text -> ([Point], [Point]) -> Int -> [Point] -> Bool
+unknownLevel :: COps -> ContentId CaveKind -> Dice.AbsDepth -> X -> Y
+             -> ([Point], [Point]) -> [Point] -> Int -> Bool
              -> Level
-unknownLevel Kind.COps{cotile=Kind.Ops{ouniqGroup}}
-             ldepth lxsize lysize lname ldesc
-             lstair lexplorable lescape lnight =
-  let outerId = ouniqGroup "basic outer fence"
-  in Level { ldepth
+unknownLevel COps{cotile}
+             lkind ldepth lxsize lysize lstair lescape lexpl lnight =
+  let outerId = ouniqGroup cotile "unknown outer fence"
+  in Level { lkind
+           , ldepth
            , lfloor = EM.empty
            , lembed = EM.empty
            , lactor = EM.empty
@@ -148,20 +158,14 @@ unknownLevel Kind.COps{cotile=Kind.Ops{ouniqGroup}}
            , lysize
            , lsmell = EM.empty
            , lstair
-           , lseen = 0
-           , lexplorable
-           , ltime = timeZero
-           , lactorCoeff = 0
-           , lactorFreq = []
-           , litemNum = 0
-           , litemFreq = []
            , lescape
+           , lseen = 0
+           , lexpl
+           , ltime = timeZero
            , lnight
-           , lname
-           , ldesc
            }
 
-unknownTileMap :: Kind.Id TileKind -> Int -> Int -> TileMap
+unknownTileMap :: ContentId TileKind -> Int -> Int -> TileMap
 unknownTileMap outerId lxsize lysize =
   let unknownMap = PointArray.replicateA lxsize lysize unknownId
       borders = [ Point x y
@@ -172,8 +176,8 @@ unknownTileMap outerId lxsize lysize =
   in unknownMap PointArray.// outerUpdate
 
 -- | Initial complete global game state.
-defStateGlobal :: Dungeon -> AbsDepth -> FactionDict -> Kind.COps
-               -> HighScore.ScoreDict -> Kind.Id ModeKind -> DiscoveryKind
+defStateGlobal :: Dungeon -> Dice.AbsDepth -> FactionDict -> COps
+               -> HighScore.ScoreDict -> ContentId ModeKind -> DiscoveryKind
                -> State
 defStateGlobal _sdungeon _stotalDepth _sfactionD _scops _shigh _sgameModeId
                _sdiscoKind =
@@ -182,25 +186,27 @@ defStateGlobal _sdungeon _stotalDepth _sfactionD _scops _shigh _sgameModeId
     , _sitemD = EM.empty
     , _sitemIxMap = EM.empty
     , _stime = timeZero
+    , _sgold = 0
     , _sdiscoAspect = EM.empty
     , _sactorAspect = EM.empty
     , ..
     }
 
 -- | Initial empty state.
-emptyState :: Kind.COps -> State
-emptyState _scops =
+emptyState :: State
+emptyState =
   State
     { _sdungeon = EM.empty
-    , _stotalDepth = AbsDepth 0
+    , _stotalDepth = Dice.AbsDepth 0
     , _sactorD = EM.empty
     , _sitemD = EM.empty
     , _sitemIxMap = EM.empty
     , _sfactionD = EM.empty
     , _stime = timeZero
-    , _scops
+    , _scops = emptyCOps
+    , _sgold = 0
     , _shigh = HighScore.empty
-    , _sgameModeId = minBound  -- the initial value is unused
+    , _sgameModeId = toEnum 0  -- the initial value is unused
     , _sdiscoKind = EM.empty
     , _sdiscoAspect = EM.empty
     , _sactorAspect = EM.empty
@@ -213,8 +219,8 @@ localFromGlobal State{..} =
   State
     { _sdungeon =
       EM.map (\Level{..} ->
-              unknownLevel _scops ldepth lxsize lysize lname ldesc
-                           lstair lexplorable lescape lnight)
+              unknownLevel _scops lkind ldepth lxsize lysize
+                           lstair lescape lexpl lnight)
              _sdungeon
     , ..
     }
@@ -224,7 +230,7 @@ updateDungeon :: (Dungeon -> Dungeon) -> State -> State
 updateDungeon f s = s {_sdungeon = f (_sdungeon s)}
 
 -- | Update dungeon depth.
-updateDepth :: (AbsDepth -> AbsDepth) -> State -> State
+updateDepth :: (Dice.AbsDepth -> Dice.AbsDepth) -> State -> State
 updateDepth f s = s {_stotalDepth = f (_stotalDepth s)}
 
 -- | Update the actor dictionary.
@@ -247,9 +253,15 @@ updateFactionD f s = s {_sfactionD = f (_sfactionD s)}
 updateTime :: (Time -> Time) -> State -> State
 updateTime f s = s {_stime = f (_stime s)}
 
--- | Update content data within state.
-updateCOps :: (Kind.COps -> Kind.COps) -> State -> State
-updateCOps f s = s {_scops = f (_scops s)}
+-- | Update content data within state and recompute the cached data.
+updateCOpsAndCachedData :: (COps -> COps) -> State -> State
+updateCOpsAndCachedData f s =
+  let s2 = s {_scops = f (_scops s)}
+  in s2 {_sactorAspect = actorAspectInDungeon s2}
+
+-- | Update total gold value in the dungeon.
+updateGold :: (Int -> Int) -> State -> State
+updateGold f s = s {_sgold = f (_sgold s)}
 
 updateDiscoKind :: (DiscoveryKind -> DiscoveryKind) -> State -> State
 updateDiscoKind f s = s {_sdiscoKind = f (_sdiscoKind s)}
@@ -263,21 +275,24 @@ updateActorAspect f s = s {_sactorAspect = f (_sactorAspect s)}
 getItemBody :: ItemId -> State -> Item
 getItemBody iid s = sitemD s EM.! iid
 
-aspectRecordFromItem :: ItemId -> Item -> State -> AspectRecord
+-- This is best guess, including mean aspect record, so we can take into
+-- consideration even the kind the item hides under.
+aspectRecordFromItem :: ItemId -> Item -> State -> IA.AspectRecord
 aspectRecordFromItem iid item s =
-  case EM.lookup iid (sdiscoAspect s) of
-    Just ar -> ar
-    Nothing -> case EM.lookup (jkindIx item) (sdiscoKind s) of
-        Just KindMean{kmMean} -> kmMean
-        Nothing -> emptyAspectRecord
+  let kindId = case jkind item of
+        IdentityObvious ik -> ik
+        IdentityCovered ix ik -> fromMaybe ik $ ix `EM.lookup` sdiscoKind s
+      COps{coItemSpeedup} = scops s
+      mean = IA.kmMean $ IK.getKindMean kindId coItemSpeedup
+  in fromMaybe mean $ EM.lookup iid $ sdiscoAspect s
 
-aspectRecordFromIid :: ItemId -> State -> AspectRecord
+aspectRecordFromIid :: ItemId -> State -> IA.AspectRecord
 aspectRecordFromIid iid s = aspectRecordFromItem iid (getItemBody iid s) s
 
-aspectRecordFromActor :: Actor -> State -> AspectRecord
+aspectRecordFromActor :: Actor -> State -> IA.AspectRecord
 aspectRecordFromActor b s =
   let processIid (iid, (k, _)) = (aspectRecordFromIid iid s, k)
-      processBag ass = sumAspectRecord $ map processIid ass
+      processBag ass = IA.sumAspectRecord $ map processIid ass
   in processBag $ EM.assocs (borgan b) ++ EM.assocs (beqp b)
 
 actorAspectInDungeon :: State -> ActorAspect
