@@ -3,10 +3,15 @@ module Game.LambdaHack.Common.Thread
   ( forkChild, waitForChildren
   ) where
 
+import Prelude ()
+
+import Game.LambdaHack.Common.Prelude
+
 import Control.Concurrent.Async
 import Control.Concurrent.MVar
 
--- Swiped from http://www.haskell.org/ghc/docs/latest/html/libraries/base/Control-Concurrent.html. Ported to Async to link exceptions, to let travis tests fail.
+-- Swiped from <http://www.haskell.org/ghc/docs/latest/html/libraries/base/Control-Concurrent.html>
+-- Ported to Async to link exceptions, to let travis tests fail.
 
 forkChild :: MVar [Async ()] -> IO () -> IO ()
 forkChild children io = do
