@@ -4,10 +4,10 @@ module Game.LambdaHack.Common.Time
   ( Time, timeZero, timeEpsilon, timeClip, timeTurn, timeSecond
   , absoluteTimeAdd, absoluteTimeSubtract, absoluteTimeNegate
   , timeFit, timeFitUp
-  , Delta(..), timeShift, timeDeltaToFrom
-  , timeDeltaSubtract, timeDeltaReverse, timeDeltaScale, timeDeltaPercent
-  , timeDeltaDiv, timeDeltaToDigit
-  , Speed, toSpeed, fromSpeed
+  , Delta(..), timeShift, timeDeltaToFrom, timeDeltaAdd, timeDeltaSubtract
+  , timeDeltaReverse, timeDeltaScale, timeDeltaPercent, timeDeltaDiv
+  , timeDeltaToDigit
+  , Speed, toSpeed, fromSpeed, minSpeed
   , speedZero, speedWalk, speedLimp, speedThrust, modifyDamageBySpeed
   , speedScale, speedAdd, speedNegate
   , ticksPerMeter, speedFromWeight, rangeFromSpeedAndLinger
@@ -110,7 +110,12 @@ timeDeltaToFrom :: Time -> Time -> Delta Time
 {-# INLINE timeDeltaToFrom #-}
 timeDeltaToFrom (Time t1) (Time t2) = Delta $ Time (t1 - t2)
 
--- | Time time vector between the second and the first absolute times.
+-- | Addition of time deltas.
+timeDeltaAdd :: Delta Time -> Delta Time -> Delta Time
+{-# INLINE timeDeltaAdd #-}
+timeDeltaAdd (Delta (Time t1)) (Delta (Time t2)) = Delta $ Time (t1 - t2)
+
+-- | Subtraction of time deltas.
 -- The arguments are in the same order as in the underlying scalar subtraction.
 timeDeltaSubtract :: Delta Time -> Delta Time -> Delta Time
 {-# INLINE timeDeltaSubtract #-}
@@ -171,10 +176,24 @@ fromSpeed :: Speed -> Int
 {-# INLINE fromSpeed #-}
 fromSpeed (Speed s) = fromEnum $ s * 10 `div` sInMs
 
--- Can't be lower or actors would slow down (via tmp organs and weight),
--- boost time with InsertMove, speed up and have lots of free moves.
+minSpeed :: Int
+minSpeed = 5
+
+-- | The minimal speed is half a meter (half a step across a tile)
+-- per second (two standard turns, which the time span during which
+-- projectile moves, unless it has modified linger value).
+-- This is four times slower than standard human movement speed.
+--
+-- It needen't be lower, because @rangeFromSpeed@ gives 0 steps
+-- with such speed, so the actor's trajectory is empty, so it drops down
+-- at once. Twice that speed already moves a normal projectile one step
+-- before it stops. It shouldn't be lower or a slow actor would incur
+-- such a time debt for performing a single action that he'd be paralyzed
+-- for many turns, e.g., leaving his dead body on the screen for very long.
 minimalSpeed :: Int64
-minimalSpeed = sInMs `div` 10
+minimalSpeed =
+  let Speed msp = toSpeed minSpeed
+  in assert (msp == sInMs `div` 2) msp
 
 -- | No movement possible at that speed.
 speedZero :: Speed
@@ -228,21 +247,20 @@ speedNegate (Speed n) = Speed (-n)
 ticksPerMeter :: Speed -> Delta Time
 {-# INLINE ticksPerMeter #-}
 ticksPerMeter (Speed v) =
+  -- Prevent division by zero or infinite time taken for any action.
   Delta $ Time $ timeTicks timeSecond * sInMs `divUp` max minimalSpeed v
 
 -- | Calculate projectile speed from item weight in grams
 -- and velocity percent modifier.
 -- See <https://github.com/LambdaHack/LambdaHack/wiki/Item-statistics>.
 speedFromWeight :: Int -> Int -> Speed
-speedFromWeight !weight !velocityPercent =
+speedFromWeight !weight !throwVelocity =
   let w = fromIntegral weight
-      vp = fromIntegral velocityPercent
       mpMs | w < 250 = sInMs * 20
            | w < 1500 = sInMs * 20 * 1250 `div` (w + 1000)
-           | w < 10500 = sInMs * (11500 - w) `div` 1000
-           | w < 200000 = sInMs  -- half a step per turn is the minimum
-           | otherwise = minimalSpeed  -- unless _very_ heavy
-      v = mpMs * vp `div` 100
+           | w < 10500 = sInMs * (11000 - w) `div` 1000
+           | otherwise = minimalSpeed * 2  -- move one step and drop
+      v = mpMs * fromIntegral throwVelocity `div` 100
       -- We round down to the nearest multiple of 2M (unless the speed
       -- is very low), to ensure both turns of flight cover the same distance
       -- and that the speed matches the distance traveled exactly.
@@ -254,13 +272,13 @@ speedFromWeight !weight !velocityPercent =
 -- | Calculate maximum range in meters of a projectile from its speed.
 -- See <https://github.com/LambdaHack/LambdaHack/wiki/Item-statistics>.
 -- With this formula, each projectile flies for at most 1 second,
--- that is 2 turns, and then drops to the ground.
+-- that is 2 standard turns, and then drops to the ground.
 rangeFromSpeed :: Speed -> Int
 {-# INLINE rangeFromSpeed #-}
 rangeFromSpeed (Speed v) = fromEnum $ v `div` sInMs
 
 -- | Calculate maximum range taking into account the linger percentage.
 rangeFromSpeedAndLinger :: Speed -> Int -> Int
-rangeFromSpeedAndLinger !speed !linger =
+rangeFromSpeedAndLinger !speed !throwLinger =
   let range = rangeFromSpeed speed
-  in linger * range `divUp` 100
+  in throwLinger * range `divUp` 100
