@@ -1,20 +1,28 @@
+{-# LANGUAGE DeriveGeneric #-}
 -- | The type of cave kinds.
 module Game.LambdaHack.Content.CaveKind
-  ( CaveKind(..), validateSingleCaveKind, validateAllCaveKind
+  ( CaveKind(..), makeData
+#ifdef EXPOSE_INTERNAL
+    -- * Internal operations
+  , validateSingle, validateAll
+#endif
   ) where
 
 import Prelude ()
 
 import Game.LambdaHack.Common.Prelude
 
+import           Control.DeepSeq
 import qualified Data.Text as T
+import           GHC.Generics (Generic)
 
+import           Game.LambdaHack.Common.ContentData
 import qualified Game.LambdaHack.Common.Dice as Dice
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.Point
 import           Game.LambdaHack.Common.Random
 import           Game.LambdaHack.Content.ItemKind (ItemKind)
-import           Game.LambdaHack.Content.PlaceKind
+import           Game.LambdaHack.Content.PlaceKind (PlaceKind)
 import           Game.LambdaHack.Content.TileKind (TileKind)
 
 -- | Parameters for the generation of dungeon levels.
@@ -41,7 +49,7 @@ data CaveKind = CaveKind
   , chidden         :: Int              -- ^ if not open, hidden one in n times
   , cactorCoeff     :: Int              -- ^ the lower, the more monsters spawn
   , cactorFreq      :: Freqs ItemKind   -- ^ actor groups to consider
-  , citemNum        :: Dice.Dice        -- ^ the number of items in the cave
+  , citemNum        :: Dice.Dice        -- ^ number of initial items in the cave
   , citemFreq       :: Freqs ItemKind   -- ^ item groups to consider
   , cplaceFreq      :: Freqs PlaceKind  -- ^ place groups to consider
   , cpassable       :: Bool
@@ -57,14 +65,16 @@ data CaveKind = CaveKind
   , cstairFreq      :: Freqs PlaceKind
       -- ^ place groups to consider for stairs; in this case the rarity
       --   of items in the group does not affect group choice
-  , cdesc           :: Text                -- ^ cave description
+  , cdesc           :: Text                -- ^ full cave description
   }
-  deriving Show  -- No Eq and Ord to make extending it logically sound
+  deriving (Show, Generic)  -- No Eq and Ord to make extending logically sound
+
+instance NFData CaveKind
 
 -- | Catch caves with not enough space for all the places. Check the size
 -- of the cave descriptions to make sure they fit on screen. Etc.
-validateSingleCaveKind :: CaveKind -> [Text]
-validateSingleCaveKind CaveKind{..} =
+validateSingle :: CaveKind -> [Text]
+validateSingle CaveKind{..} =
   let (minGridX, minGridY) = Dice.minDiceXY cgrid
       (maxGridX, maxGridY) = Dice.maxDiceXY cgrid
       (minMinSizeX, minMinSizeY) = Dice.minDiceXY cminPlaceSize
@@ -93,8 +103,49 @@ validateSingleCaveKind CaveKind{..} =
 -- | Validate all cave kinds.
 -- Note that names don't have to be unique: we can have several variants
 -- of a cave with a given name.
-validateAllCaveKind :: [CaveKind] -> [Text]
-validateAllCaveKind lk =
-  if any (maybe False (> 0) . lookup "default random" . cfreq) lk
-  then []
-  else ["no cave defined for \"default random\""]
+validateAll :: ContentData ItemKind
+            -> ContentData PlaceKind
+            -> ContentData TileKind
+            -> [CaveKind]
+            -> ContentData CaveKind
+            -> [Text]
+validateAll coitem coplace cotile content cocave =
+  let missingActorFreq = filter (not . omemberGroup coitem)
+                         $ concatMap (map fst . cactorFreq) content
+      missingItemFreq = filter (not . omemberGroup coitem)
+                        $ concatMap (map fst . citemFreq) content
+      missingPlaceFreq = filter (not . omemberGroup coplace)
+                         $ concatMap (map fst . cplaceFreq) content
+      missingEscapeGroup = filter (not . omemberGroup coplace)
+                           $ mapMaybe cescapeGroup content
+      missingStairFreq = filter (not . omemberGroup coplace)
+                         $ concatMap (map fst . cstairFreq) content
+      tileGroupFuns = [ cdefTile, cdarkCorTile, clitCorTile, cfillerTile
+                      , couterFenceTile, clegendDarkTile, clegendLitTile ]
+      g kind = map (\f -> f kind) tileGroupFuns
+      missingTileFreq = filter (not . omemberGroup cotile)
+                        $ concatMap g content
+  in [ "cactorFreq item groups not in content:" <+> tshow missingActorFreq
+     | not $ null missingActorFreq ]
+     ++ [ "citemFreq item groups not in content:" <+> tshow missingItemFreq
+        | not $ null missingItemFreq ]
+     ++ [ "cplaceFreq place groups not in content:" <+> tshow missingPlaceFreq
+        | not $ null missingPlaceFreq ]
+     ++ [ "cescapeGroup place groups not in content:"
+          <+> tshow missingEscapeGroup
+        | not $ null missingEscapeGroup ]
+     ++ [ "cstairFreq place groups not in content:" <+> tshow missingStairFreq
+        | not $ null missingStairFreq ]
+     ++ [ "tile groups not in content:" <+> tshow missingTileFreq
+        | not $ null missingTileFreq ]
+     ++ [ "no cave defined for \"default random\""
+        | not $ omemberGroup cocave "default random" ]
+
+makeData :: ContentData ItemKind
+         -> ContentData PlaceKind
+         -> ContentData TileKind
+         -> [CaveKind]
+         -> ContentData CaveKind
+makeData coitem coplace cotile =
+  makeContentData "CaveKind" cname cfreq validateSingle
+                  (validateAll coitem coplace cotile)
