@@ -1,12 +1,14 @@
 {-# LANGUAGE DeriveGeneric #-}
 -- | The type of kinds of game modes.
 module Game.LambdaHack.Content.ModeKind
-  ( ModeKind(..), Caves, Roster(..), Outcome(..)
+  ( ModeKind(..), makeData
+  , Caves, Roster(..), Outcome(..)
   , HiCondPoly, HiSummand, HiPolynomial, HiIndeterminant(..)
   , Player(..), LeaderMode(..), AutoLeader(..)
-  , validateSingleModeKind, validateAllModeKind
+  , nameOfHorrorFact
 #ifdef EXPOSE_INTERNAL
     -- * Internal operations
+  , validateSingle, validateAll
   , validateSingleRoster, validateSinglePlayer, hardwiredModeGroups
 #endif
   ) where
@@ -15,17 +17,18 @@ import Prelude ()
 
 import Game.LambdaHack.Common.Prelude
 
+import           Control.DeepSeq
 import           Data.Binary
 import qualified Data.EnumMap.Strict as EM
 import qualified Data.IntMap.Strict as IM
-import qualified Data.Set as S
 import qualified Data.Text as T
 import           GHC.Generics (Generic)
 
 import           Game.LambdaHack.Common.Ability
+import           Game.LambdaHack.Common.ContentData
 import qualified Game.LambdaHack.Common.Dice as Dice
 import           Game.LambdaHack.Common.Misc
-import           Game.LambdaHack.Content.CaveKind
+import           Game.LambdaHack.Content.CaveKind (CaveKind)
 import           Game.LambdaHack.Content.ItemKind (ItemKind)
 
 -- | Game mode specification.
@@ -37,7 +40,9 @@ data ModeKind = ModeKind
   , mcaves  :: Caves           -- ^ arena of the game
   , mdesc   :: Text            -- ^ description
   }
-  deriving Show
+  deriving (Show, Generic)
+
+instance NFData ModeKind
 
 -- | Requested cave groups for particular levels. The second component
 -- is the @Escape@ feature on the level. @True@ means it's represented
@@ -52,7 +57,9 @@ data Roster = Roster
   , rosterEnemy :: [(Text, Text)]  -- ^ the initial enmity matrix
   , rosterAlly  :: [(Text, Text)]  -- ^ the initial aliance matrix
   }
-  deriving Show
+  deriving (Show, Generic)
+
+instance NFData Roster
 
 -- | Outcome of a game.
 data Outcome =
@@ -66,6 +73,8 @@ data Outcome =
 
 instance Binary Outcome
 
+instance NFData Outcome
+
 -- | Conditional polynomial representing score calculation for this player.
 type HiCondPoly = [HiSummand]
 
@@ -78,6 +87,8 @@ data HiIndeterminant = HiConst | HiLoot | HiBlitz | HiSurvival | HiKill | HiLoss
 
 instance Binary HiIndeterminant
 
+instance NFData HiIndeterminant
+
 -- | Properties of a particular player.
 data Player = Player
   { fname        :: Text        -- ^ name of the player
@@ -102,6 +113,8 @@ data Player = Player
 
 instance Binary Player
 
+instance NFData Player
+
 -- | If a faction with @LeaderUI@ and @LeaderAI@ has any actor, it has a leader.
 data LeaderMode =
     LeaderNull  -- ^ faction can have no leader, is whole under AI control
@@ -111,6 +124,8 @@ data LeaderMode =
 
 instance Binary LeaderMode
 
+instance NFData LeaderMode
+
 data AutoLeader = AutoLeader
   { autoDungeon :: Bool
       -- ^ leader switching between levels is automatically done by the server
@@ -134,9 +149,14 @@ data AutoLeader = AutoLeader
 
 instance Binary AutoLeader
 
+instance NFData AutoLeader
+
+nameOfHorrorFact :: GroupName ItemKind
+nameOfHorrorFact = toGroupName "horror"
+
 -- | Catch invalid game mode kind definitions.
-validateSingleModeKind :: ModeKind -> [Text]
-validateSingleModeKind ModeKind{..} =
+validateSingle :: ModeKind -> [Text]
+validateSingle ModeKind{..} =
   [ "mname longer than 20" | T.length mname > 20 ]
   ++ validateSingleRoster mcaves mroster
 
@@ -164,7 +184,7 @@ validateSingleRoster caves Roster{..} =
      in concatMap f rosterList
 
 validateSinglePlayer :: Player -> [Text]
-validateSinglePlayer  Player{..} =
+validateSinglePlayer Player{..} =
   [ "fname empty:" <+> fname | T.null fname ]
   ++ [ "no UI client, but UI leader:" <+> fname
      | not fhasUI && case fleaderMode of
@@ -174,17 +194,35 @@ validateSinglePlayer  Player{..} =
      | any (>= 0) $ EM.elems fskillsOther ]
 
 -- | Validate game mode kinds together.
-validateAllModeKind :: [ModeKind] -> [Text]
-validateAllModeKind content =
-  let kindFreq :: S.Set (GroupName ModeKind)  -- cf. Kind.kindFreq
-      kindFreq = let tuples = [ cgroup
-                              | k <- content
-                              , (cgroup, n) <- mfreq k
-                              , n > 0 ]
-                 in S.fromList tuples
-      hardwiredAbsent = filter (`S.notMember` kindFreq) hardwiredModeGroups
-  in [ "Hardwired groups not in content:" <+> tshow hardwiredAbsent
-     | not $ null hardwiredAbsent ]
+validateAll :: ContentData CaveKind
+            -> ContentData ItemKind
+            -> [ModeKind]
+            -> ContentData ModeKind
+            -> [Text]
+validateAll cocave coitem content comode =
+  let missingCave = filter (not . omemberGroup cocave)
+                    $ concatMap (IM.elems . mcaves) content
+      f Roster{rosterList} =
+        concatMap (\(p, l) -> delete nameOfHorrorFact (fgroups p)
+                              ++ map (\(_, _, grp) -> grp) l)
+                  rosterList
+      missingRosterItems = filter (not . omemberGroup coitem)
+                           $ concatMap (f . mroster) content
+      hardwiredAbsent = filter (not . omemberGroup comode) hardwiredModeGroups
+  in [ "cave groups not in content:" <+> tshow missingCave
+     | not $ null missingCave ]
+     ++ [ "roster item groups not in content:" <+> tshow missingRosterItems
+        | not $ null missingRosterItems ]
+     ++ [ "Hardwired groups not in content:" <+> tshow hardwiredAbsent
+        | not $ null hardwiredAbsent ]
 
 hardwiredModeGroups :: [GroupName ModeKind]
 hardwiredModeGroups = [ "campaign scenario", "starting", "starting JS" ]
+
+makeData :: ContentData CaveKind
+         -> ContentData ItemKind
+         -> [ModeKind]
+         -> ContentData ModeKind
+makeData cocave coitem =
+  makeContentData "ModeKind" mname mfreq validateSingle
+                  (validateAll cocave coitem)
