@@ -1,7 +1,12 @@
+{-# LANGUAGE DeriveGeneric #-}
 -- | The type of kinds of rooms, halls and passages.
 module Game.LambdaHack.Content.PlaceKind
-  ( PlaceKind(..), Cover(..), Fence(..)
-  , validateSinglePlaceKind, validateAllPlaceKind
+  ( PlaceKind(..), makeData
+  , Cover(..), Fence(..)
+#ifdef EXPOSE_INTERNAL
+    -- * Internal operations
+  , validateSingle, validateAll
+#endif
   ) where
 
 import Prelude ()
@@ -10,8 +15,11 @@ import Game.LambdaHack.Common.Prelude
 
 import qualified Data.Text as T
 
+import Control.DeepSeq
+import Game.LambdaHack.Common.ContentData
 import Game.LambdaHack.Common.Misc
 import Game.LambdaHack.Content.TileKind (TileKind)
+import GHC.Generics (Generic)
 
 -- | Parameters for the generation of small areas within a dungeon level.
 data PlaceKind = PlaceKind
@@ -24,7 +32,9 @@ data PlaceKind = PlaceKind
   , ptopLeft  :: [Text]        -- ^ plan of the top-left corner of the place
   , poverride :: [(Char, GroupName TileKind)]  -- ^ legend override
   }
-  deriving Show  -- No Eq and Ord to make extending it logically sound
+  deriving (Show, Generic)  -- No Eq and Ord to make extending logically sound
+
+instance NFData PlaceKind
 
 -- | A method of filling the whole area (except for CVerbatim and CMirror,
 -- which are just placed in the middle of the area) by transforming
@@ -35,7 +45,9 @@ data Cover =
   | CReflect    -- ^ tile separately and symmetrically quarters of the place
   | CVerbatim   -- ^ just build the given interior, without filling the area
   | CMirror     -- ^ build the given interior in one of 4 mirrored variants
-  deriving (Show, Eq)
+  deriving (Show, Eq, Generic)
+
+instance NFData Cover
 
 -- | The choice of a fence type for the place.
 data Fence =
@@ -43,12 +55,14 @@ data Fence =
   | FFloor  -- ^ leave an empty space, like the rooms floor
   | FGround -- ^ leave an empty space, like the caves ground
   | FNone   -- ^ skip the fence and fill all with the place proper
-  deriving (Show, Eq)
+  deriving (Show, Eq, Generic)
+
+instance NFData Fence
 
 -- | Catch invalid place kind definitions. In particular, verify that
 -- the top-left corner map is rectangular and not empty.
-validateSinglePlaceKind :: PlaceKind -> [Text]
-validateSinglePlaceKind PlaceKind{..} =
+validateSingle :: PlaceKind -> [Text]
+validateSingle PlaceKind{..} =
   let dxcorner = case ptopLeft of
         [] -> 0
         l : _ -> T.length l
@@ -57,6 +71,15 @@ validateSinglePlaceKind PlaceKind{..} =
         | any (/= dxcorner) (map T.length ptopLeft) ]
      ++ validateRarity prarity
 
--- | Validate all place kinds. Currently always valid.
-validateAllPlaceKind :: [PlaceKind] -> [Text]
-validateAllPlaceKind _ = []
+-- | Validate all place kinds.
+validateAll :: ContentData TileKind -> [PlaceKind] -> ContentData PlaceKind
+            -> [Text]
+validateAll cotile content _ =
+  let missingOverride = filter (not . omemberGroup cotile)
+                        $ concatMap (map snd . poverride) content
+  in [ "poverride tile groups not in content:" <+> tshow missingOverride
+     | not $ null missingOverride ]
+
+makeData :: ContentData TileKind -> [PlaceKind] -> ContentData PlaceKind
+makeData cotile =
+  makeContentData "PlaceKind" pname pfreq validateSingle (validateAll cotile)
