@@ -1,12 +1,15 @@
 {-# LANGUAGE DeriveGeneric #-}
 -- | The type of kinds of terrain tiles.
 module Game.LambdaHack.Content.TileKind
-  ( TileKind(..), Feature(..), TileSpeedup(..), Tab(..)
-  , validateSingleTileKind, validateAllTileKind, actionFeatures
-  , isUknownSpace, unknownId, isSuspectKind, isOpenableKind, isClosableKind
+  ( TileKind(..), makeData
+  , Feature(..), TileSpeedup(..), Tab(..)
+  , emptyTileSpeedup, emptyTab
+  , actionFeatures, isUknownSpace, unknownId
+  , isSuspectKind, isOpenableKind, isClosableKind
   , talterForStairs, floorSymbol
 #ifdef EXPOSE_INTERNAL
     -- * Internal operations
+  , validateSingle, validateAll
   , validateDups, hardwiredTileGroups
 #endif
   ) where
@@ -20,16 +23,14 @@ import           Data.Binary
 import qualified Data.Char as Char
 import           Data.Hashable
 import qualified Data.IntSet as IS
-import qualified Data.Map.Strict as M
-import qualified Data.Set as S
 import qualified Data.Text as T
 import qualified Data.Vector.Unboxed as U
 import           GHC.Generics (Generic)
 
-import           Game.LambdaHack.Common.Color
-import qualified Game.LambdaHack.Common.KindOps as KindOps
-import           Game.LambdaHack.Common.Misc
-import           Game.LambdaHack.Content.ItemKind (ItemKind)
+import Game.LambdaHack.Common.Color
+import Game.LambdaHack.Common.ContentData
+import Game.LambdaHack.Common.Misc
+import Game.LambdaHack.Content.ItemKind (ItemKind)
 
 -- | The type of kinds of terrain tiles. See @Tile.hs@ for explanation
 -- of the absence of a corresponding type @Tile@ that would hold
@@ -38,7 +39,7 @@ import           Game.LambdaHack.Content.ItemKind (ItemKind)
 -- (that would lead to "a stairs"), so "road with cobblestones" is fine,
 -- but "granite cobblestones" is wrong.
 --
--- Tile kind for unknown space has the minimal @KindOps.Id@ index.
+-- Tile kind for unknown space has the minimal @ContentId@ index.
 -- The @talter@ for unknown space is @1@ and no other tile kind has that value.
 data TileKind = TileKind
   { tsymbol  :: Char         -- ^ map symbol
@@ -49,7 +50,9 @@ data TileKind = TileKind
   , talter   :: Word8        -- ^ minimal skill needed to alter the tile
   , tfeature :: [Feature]    -- ^ properties
   }
-  deriving Show  -- No Eq and Ord to make extending it logically sound
+  deriving (Show, Generic)  -- No Eq and Ord to make extending logically sound
+
+instance NFData TileKind
 
 -- | All possible terrain tile features.
 data Feature =
@@ -77,7 +80,6 @@ data Feature =
   | OftenActor           -- ^ initial actors often generated there
   | NoItem               -- ^ no items ever generated there
   | NoActor              -- ^ no actors ever generated there
-  | Indistinct           -- ^ is allowed to have the same look as another tile
   | ConsideredByAI       -- ^ even if otherwise uninteresting, taken into
                          --   account for triggering by AI
   | Trail                -- ^ used for visible trails throughout the level
@@ -94,6 +96,8 @@ instance Hashable Feature
 
 instance NFData Feature
 
+-- | A lot of tabulated maps from tile kind identifier to a property
+-- of the tile kind.
 data TileSpeedup = TileSpeedup
   { isClearTab        :: Tab Bool
   , isLitTab          :: Tab Bool
@@ -111,16 +115,31 @@ data TileSpeedup = TileSpeedup
   , alterMinSkillTab  :: Tab Word8
   , alterMinWalkTab   :: Tab Word8
   }
+  deriving Generic
+
+instance NFData TileSpeedup
 
 -- Vectors of booleans can be slower than arrays, because they are not packed,
 -- but with growing cache sizes they may as well turn out faster at some point.
 -- The advantage of vectors are exposed internals, in particular unsafe
--- indexing. Also, in JS bool arrays are obviously not packed.
-newtype Tab a = Tab (U.Vector a)  -- morally indexed by @Id a@
+-- indexing. Also, in JS, bool arrays are obviously not packed.
+-- | A map morally indexed by @ContentId TileKind@.
+newtype Tab a = Tab (U.Vector a)
+  deriving Generic
+
+instance NFData (Tab a)
+
+emptyTileSpeedup :: TileSpeedup
+emptyTileSpeedup = TileSpeedup emptyTab emptyTab emptyTab emptyTab emptyTab
+                               emptyTab emptyTab emptyTab emptyTab emptyTab
+                               emptyTab emptyTab emptyTab emptyTab emptyTab
+
+emptyTab :: U.Unbox a => Tab a
+emptyTab = Tab $! U.empty
 
 -- | Validate a single tile kind.
-validateSingleTileKind :: TileKind -> [Text]
-validateSingleTileKind t@TileKind{..} =
+validateSingle :: TileKind -> [Text]
+validateSingle t@TileKind{..} =
   [ "suspect tile is walkable" | Walkable `elem` tfeature
                                  && isSuspectKind t ]
   ++ [ "openable tile is open" | Walkable `elem` tfeature
@@ -156,7 +175,7 @@ validateSingleTileKind t@TileKind{..} =
       in ["more than one BuildAs specification" | length ts > 1])
   ++ concatMap (validateDups t)
        [ Walkable, Clear, Dark, OftenItem, OftenActor, NoItem, NoActor
-       , Indistinct, ConsideredByAI, Trail, Spice ]
+       , ConsideredByAI, Trail, Spice ]
 
 validateDups :: TileKind -> Feature -> [Text]
 validateDups TileKind{..} feat =
@@ -165,45 +184,58 @@ validateDups TileKind{..} feat =
 
 -- | Validate all tile kinds.
 --
--- If tiles look the same on the map (symbol and color), their substantial
--- features should be the same, too. Otherwise, the player has to inspect
--- manually all the tiles of that kind, or even experiment with them,
--- to see if any is special. This would be tedious. Note that tiles may freely
--- differ wrt text blurb, dungeon generation, AI preferences, etc.
-validateAllTileKind :: [TileKind] -> [Text]
-validateAllTileKind lt =
-  let kindFreq :: S.Set (GroupName TileKind)  -- cf. Kind.kindFreq
-      kindFreq = let tuples = [ cgroup
-                              | k <- lt
-                              , (cgroup, n) <- tfreq k
-                              , n > 0 ]
-                 in S.fromList tuples
-      listVis f = map (\kt -> ( (tsymbol kt, f kt)
-                              , [(kt, actionFeatures True kt)] )) lt
-      mapVis :: (TileKind -> Color)
-             -> M.Map (Char, Color) [(TileKind, IS.IntSet)]
-      mapVis f = M.fromListWith (++) $ listVis f
-      isConfused [] = error $ "isConfused" `showFailure` lt
-      isConfused [_] = False
-      isConfused (hd : tl) =
-        any ((Indistinct `notElem`) . tfeature . fst) (hd : tl)
-        && any ((/= snd hd) . snd) tl
-      confusions f = filter isConfused $ M.elems $ mapVis f
-      hardwiredAbsent = filter (`S.notMember` kindFreq) hardwiredTileGroups
+-- We don't check it any more, but if tiles look the same on the map
+-- (symbol and color), their substantial features should be the same, too,
+-- unless there is a good reason they shouldn't. Otherwise the player has
+-- to inspect manually all the tiles with this look to see if any is special.
+-- This tends to be tedious. Note that tiles may freely differ wrt text blurb,
+-- dungeon generation rules, AI preferences, etc., whithout causing the tedium.
+validateAll :: ContentData ItemKind -> [TileKind] -> ContentData TileKind
+            -> [Text]
+validateAll coitem content cotile =
+  let g :: Feature -> Maybe (GroupName TileKind)
+      g (OpenTo grp) = Just grp
+      g (CloseTo grp) = Just grp
+      g (ChangeTo grp) = Just grp
+      g (HideAs grp) = Just grp
+      g (BuildAs grp) = Just grp
+      g (RevealAs grp) = Just grp
+      g (ObscureAs grp) = Just grp
+      g _ = Nothing
+      missingTileGroups =
+        [ (tname k, absGroups)
+        | k <- content
+        , let grps = mapMaybe g $ tfeature k
+              absGroups = filter (not . omemberGroup cotile) grps
+        , not $ null absGroups
+        ]
+      h :: Feature -> Maybe (GroupName ItemKind)
+      h (Embed grp) = Just grp
+      h _ = Nothing
+      missingItemGroups =
+        [ (tname k, absGroups)
+        | k <- content
+        , let grps = mapMaybe h $ tfeature k
+              absGroups = filter (not . omemberGroup coitem) grps
+        , not $ null absGroups
+        ]
+      missingHardwiredGroups =
+        filter (not . omemberGroup cotile) hardwiredTileGroups
   in [ "first tile should be the unknown one"
-     | talter (head lt) /= 1 || tname (head lt) /= "unknown space" ]
+     | talter (head content) /= 1 || tname (head content) /= "unknown space" ]
      ++ [ "only unknown tile may have talter 1"
-        | any ((== 1) . talter) $ tail lt ]
-     ++ case confusions tcolor ++ confusions tcolor2 of
-       [] -> []
-       cfs -> ["tile confusions detected:" <+> tshow cfs]
-     ++ [ "Hardwired groups not in content:" <+> tshow hardwiredAbsent
-        | not $ null hardwiredAbsent ]
+        | any ((== 1) . talter) $ tail content ]
+     ++ [ "mentioned tile groups not in content:" <+> tshow missingTileGroups
+        | not $ null missingTileGroups ]
+     ++ [ "embedded item groups not in content:" <+> tshow missingItemGroups
+        | not $ null missingItemGroups ]
+     ++ [ "hardwired groups not in content:" <+> tshow missingHardwiredGroups
+        | not $ null missingHardwiredGroups ]
 
 hardwiredTileGroups :: [GroupName TileKind]
 hardwiredTileGroups =
-  [ "unknown space", "legendLit", "legendDark", "basic outer fence"
-  , "stair terminal" ]
+  [ "unknown space", "legendLit", "legendDark", "unknown outer fence"
+  , "basic outer fence", "stair terminal" ]
 
 -- | Features of tiles that differentiate them substantially from one another.
 -- The intention is the player can easily tell such tiles apart by their
@@ -233,19 +265,18 @@ actionFeatures markSuspect t =
         OftenActor -> Nothing
         NoItem -> Nothing
         NoActor -> Nothing
-        Indistinct -> Nothing
         ConsideredByAI -> Nothing
         Trail -> Just feat  -- doesn't affect tile behaviour, but important
         Spice -> Nothing
   in IS.fromList $ map hash $ mapMaybe f $ tfeature t
 
-isUknownSpace :: KindOps.Id TileKind -> Bool
+isUknownSpace :: ContentId TileKind -> Bool
 {-# INLINE isUknownSpace #-}
-isUknownSpace tt = KindOps.Id 0 == tt
+isUknownSpace tt = ContentId 0 == tt
 
-unknownId :: KindOps.Id TileKind
+unknownId :: ContentId TileKind
 {-# INLINE unknownId #-}
-unknownId = KindOps.Id 0
+unknownId = ContentId 0
 
 isSuspectKind :: TileKind -> Bool
 isSuspectKind t =
@@ -283,3 +314,7 @@ floorSymbol = Char.chr 183
 -- 50  considerable obstructions
 -- 100  walls
 -- maxBound  impenetrable walls, etc., can never be altered
+
+makeData :: ContentData ItemKind -> [TileKind] -> ContentData TileKind
+makeData coitem =
+  makeContentData "TileKind" tname tfreq validateSingle (validateAll coitem)
