@@ -24,7 +24,7 @@ import qualified System.Random as R
 -- remaining commandline should be passed and parsed by the client to extract
 -- client and ui options from and singnal an error if anything was left.
 
-import Game.LambdaHack.Client.ClientOptions
+import Game.LambdaHack.Client (ClientOptions (..))
 import Game.LambdaHack.Common.Faction
 import Game.LambdaHack.Common.Misc
 import Game.LambdaHack.Content.ModeKind
@@ -56,6 +56,7 @@ serverOptionsP = do
   skeepAutomated    <- keepAutomatedP
   sstopAfterSeconds <- stopAfterSecsP
   sstopAfterFrames  <- stopAfterFramesP
+  sprintEachScreen  <- printEachScreenP
   sbenchmark        <- benchmarkP
   sdungeonRng       <- setDungeonRngP
   smainRng          <- setMainRngP
@@ -163,6 +164,11 @@ stopAfterFramesP = optional $
               <> help "Exit game session after around N frames"
               <> metavar "N" )
 
+printEachScreenP :: Parser Bool
+printEachScreenP =
+  switch (  long "printEachScreen"
+         <> help "Take a screenshot of each rendered distinct frame (SDL only)" )
+
 benchmarkP :: Parser Bool
 benchmarkP =
   switch (  long "benchmark"
