@@ -1,15 +1,15 @@
 {-# LANGUAGE TupleSections #-}
 -- | Server operations common to many modules.
 module Game.LambdaHack.Server.CommonM
-  ( execFailure, revealItems, moveStores, generalMoveItem
+  ( revealItems, moveStores, generalMoveItem
   , deduceQuits, deduceKilled, electLeader, supplantLeader
   , updatePer, recomputeCachePer, projectFail
-  , addActor, registerActor, addActorIid, discoverIfNoEffects
+  , addActorFromGroup, registerActor, discoverIfMinorEffects
   , pickWeaponServer, currentSkillsServer
 #ifdef EXPOSE_INTERNAL
     -- * Internal operations
   , containerMoveItem, quitF, keepArenaFact, anyActorsAlive, projectBla
-  , addProjectile, getCacheLucid, getCacheTotal
+  , addProjectile, addActorIid, getCacheLucid, getCacheTotal
 #endif
   ) where
 
@@ -18,18 +18,15 @@ import Prelude ()
 import Game.LambdaHack.Common.Prelude
 
 import qualified Data.EnumMap.Strict as EM
-import qualified Data.Text as T
-import qualified Text.Show.Pretty as Show.Pretty
 
 import           Game.LambdaHack.Atomic
-import           Game.LambdaHack.Client
 import qualified Game.LambdaHack.Common.Ability as Ability
 import           Game.LambdaHack.Common.Actor
 import           Game.LambdaHack.Common.ActorState
 import           Game.LambdaHack.Common.Faction
 import           Game.LambdaHack.Common.Item
-import           Game.LambdaHack.Common.ItemStrongest
-import qualified Game.LambdaHack.Common.Kind as Kind
+import qualified Game.LambdaHack.Common.ItemAspect as IA
+import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Level
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.MonadStateRead
@@ -51,39 +48,16 @@ import           Game.LambdaHack.Server.MonadServer
 import           Game.LambdaHack.Server.ServerOptions
 import           Game.LambdaHack.Server.State
 
-execFailure :: MonadServerAtomic m
-            => ActorId -> RequestTimed a -> ReqFailure -> m ()
-execFailure aid req failureSer = do
-  -- Clients should rarely do that (only in case of invisible actors)
-  -- so we report it to the client, but do not crash
-  -- (server should work OK with stupid clients, too).
-  body <- getsState $ getActorBody aid
-  let fid = bfid body
-      msg = showReqFailure failureSer
-      impossible = impossibleReqFailure failureSer
-      debugShow :: Show a => a -> Text
-      debugShow = T.pack . Show.Pretty.ppShow
-      possiblyAlarm = if impossible
-                      then debugPossiblyPrintAndExit
-                      else debugPossiblyPrint
-  possiblyAlarm $
-    "execFailure:" <+> msg <> "\n"
-    <> debugShow body <> "\n" <> debugShow req <> "\n" <> debugShow failureSer
-  execSfxAtomic $ SfxMsgFid fid $ SfxUnexpected failureSer
-
 revealItems :: MonadServerAtomic m => Maybe FactionId -> m ()
 revealItems mfid = do
-  itemToF <- getsState itemToFull
-  let discover aid store iid k =
-        let itemFull = itemToF iid k
+  COps{coitem} <- getsState scops
+  let discover aid store iid _ = do
+        itemKindId <- getsState $ getIidKindIdServer iid
+        let itemKind = okind coitem itemKindId
             c = CActor aid store
-        in case itemDisco itemFull of
-          Just ItemDisco{itemKindId, itemKind} -> do
-            let isGem = maybe False (> 0) (lookup "gem" $ IK.ifreq itemKind)
-            unless isGem $ do  -- a hack
-              seed <- getsServer $ (EM.! iid) . sitemSeedD
-              execUpdAtomic $ UpdDiscover c iid itemKindId seed
-          _ -> error $ "" `showFailure` (mfid, c, iid, itemFull)
+        unless (IK.isHumanTrinket itemKind) $ do  -- a hack
+          discoAspect <- getsState sdiscoAspect
+          execUpdAtomic $ UpdDiscover c iid itemKindId $ discoAspect EM.! iid
       f aid = do
         b <- getsState $ getActorBody aid
         let ourSide = maybe True (== bfid b) mfid
@@ -154,7 +128,7 @@ quitF status fid = do
         -- so the score will get registered.
         registerScore status fid
       execUpdAtomic $ UpdQuitFaction fid oldSt $ Just status
-      modifyServer $ \ser -> ser {squit = True}  -- check game over ASAP
+      modifyServer $ \ser -> ser {sbreakLoop = True}  -- check game over
 
 -- Send any UpdQuitFaction actions that can be deduced from factions'
 -- current state.
@@ -185,7 +159,7 @@ deduceQuits fid0 status = do
       assocsUI = filter (factHasUI . snd) assocsInGame
       nonHorrorAIG = filter (not . isHorrorFact . snd) assocsInGame
       worldPeace =
-        all (\(fid1, _) -> all (\(_, fact2) -> not $ isAtWar fact2 fid1)
+        all (\(fid1, _) -> all (\(fid2, fact2) -> not $ isFoe fid2 fact2 fid1)
                            nonHorrorAIG)
         nonHorrorAIG
       othersInGame = filter ((/= fid0) . fst) assocsInGame
@@ -204,7 +178,8 @@ deduceQuits fid0 status = do
      | stOutcome status == Escape -> do
        -- Otherwise, in a game with many warring teams alive,
        -- only complete Victory matters, until enough of them die.
-       let (victors, losers) = partition (flip isAllied fid0 . snd) othersInGame
+       let (victors, losers) =
+             partition (\(fi, _) -> isFriend fid0 fact0 fi) othersInGame
        mapQuitF $ zip (repeat Escape) victors ++ zip (repeat Defeated) losers
      | otherwise -> quitF status fid0
 
@@ -222,9 +197,9 @@ keepArenaFact fact = fleaderMode (gplayer fact) /= LeaderNull
 -- @bfid@ of the actor body is still the old faction.
 deduceKilled :: MonadServerAtomic m => ActorId -> m ()
 deduceKilled aid = do
-  Kind.COps{corule} <- getsState scops
+  cops <- getsState scops
   body <- getsState $ getActorBody aid
-  let firstDeathEnds = rfirstDeathEnds $ Kind.stdRuleset corule
+  let firstDeathEnds = rfirstDeathEnds $ getStdRuleset cops
   fact <- getsState $ (EM.! bfid body) . sfactionD
   when (fneverEmpty $ gplayer fact) $ do
     actorsAlive <- anyActorsAlive (bfid body) aid
@@ -234,7 +209,8 @@ deduceKilled aid = do
 anyActorsAlive :: MonadServer m => FactionId -> ActorId -> m Bool
 anyActorsAlive fid aid = do
   as <- getsState $ fidActorNotProjAssocs fid
-  return $! map fst as /= [aid]
+  -- We test HP here, in case more than one actor goes to 0 HP in the same turn.
+  return $! any (\(aid2, b2) -> aid2 /= aid && bhp b2 > 0) as
 
 electLeader :: MonadServerAtomic m => FactionId -> LevelId -> ActorId -> m ()
 electLeader fid lid aidDead = do
@@ -259,6 +235,7 @@ supplantLeader fid aid = do
     -- First update and send Perception so that the new leader
     -- may report his environment.
     b <- getsState $ getActorBody aid
+    let !_A = assert (not $ bproj b) ()
     valid <- getsServer $ (EM.! blid b) . (EM.! fid) . sperValidFid
     unless valid $ updatePer fid (blid b)
     execUpdAtomic $ UpdLeadFaction fid (gleader fact) (Just aid)
@@ -270,14 +247,12 @@ updatePer fid lid = do
     ser {sperValidFid = EM.adjust (EM.insert lid True) fid $ sperValidFid ser}
   sperFidOld <- getsServer sperFid
   let perOld = sperFidOld EM.! fid EM.! lid
-  knowEvents <- getsServer $ sknowEvents . soptions
   -- Performed in the State after action, e.g., with a new actor.
   perNew <- recomputeCachePer fid lid
   let inPer = diffPer perNew perOld
       outPer = diffPer perOld perNew
   unless (nullPer outPer && nullPer inPer) $
-    unless knowEvents $  -- inconsistencies would quickly manifest
-      execSendPer fid lid outPer inPer perNew
+    execSendPer fid lid outPer inPer perNew
 
 recomputeCachePer :: MonadServer m => FactionId -> LevelId -> m Perception
 recomputeCachePer fid lid = do
@@ -294,12 +269,13 @@ projectFail :: MonadServerAtomic m
             => ActorId    -- ^ actor projecting the item (is on current lvl)
             -> Point      -- ^ target position of the projectile
             -> Int        -- ^ digital line parameter
+            -> Bool       -- ^ whether to start at the source position
             -> ItemId     -- ^ the item to be projected
             -> CStore     -- ^ whether the items comes from floor or inventory
             -> Bool       -- ^ whether the item is a blast
             -> m (Maybe ReqFailure)
-projectFail source tpxy eps iid cstore isBlast = do
-  Kind.COps{coTileSpeedup} <- getsState scops
+projectFail source tpxy eps center iid cstore blast = do
+  COps{coTileSpeedup} <- getsState scops
   sb <- getsState $ getActorBody source
   let lid = blid sb
       spos = bpos sb
@@ -311,41 +287,41 @@ projectFail source tpxy eps iid cstore isBlast = do
     Just (pos : restUnlimited) -> do
       bag <- getsState $ getBodyStoreBag sb cstore
       case EM.lookup iid bag of
-        Nothing ->  return $ Just ProjectOutOfReach
-        Just kit -> do
-          itemToF <- getsState itemToFull
+        Nothing -> return $ Just ProjectOutOfReach
+        Just _kit -> do
+          itemFull@ItemFull{itemKind} <- getsState $ itemToFull iid
           actorSk <- currentSkillsServer source
           ar <- getsState $ getActorAspect source
           let skill = EM.findWithDefault 0 Ability.AbProject actorSk
-              itemFull@ItemFull{itemBase} = itemToF iid kit
-              forced = isBlast || bproj sb
+              forced = blast || bproj sb
               calmE = calmEnough sb ar
-              legal = permittedProject forced skill calmE "" itemFull
+              legal = permittedProject forced skill calmE itemFull
           case legal of
             Left reqFail -> return $ Just reqFail
             Right _ -> do
-              let lobable = IK.Lobable `elem` jfeature itemBase
+              let lobable = IK.Lobable `elem` IK.ifeature itemKind
                   rest = if lobable
                          then take (chessDist spos tpxy - 1) restUnlimited
                          else restUnlimited
                   t = lvl `at` pos
               if not $ Tile.isWalkable coTileSpeedup t
-                then return $ Just ProjectBlockTerrain
+              then return $ Just ProjectBlockTerrain
+              else do
+                lab <- getsState $ posToAssocs pos lid
+                if not $ all (bproj . snd) lab
+                then if blast && bproj sb then do
+                       -- Hit the blocking actor.
+                       projectBla source spos (pos:rest) iid cstore blast
+                       return Nothing
+                     else return $ Just ProjectBlockActor
                 else do
-                  lab <- getsState $ posToAssocs pos lid
-                  if not $ all (bproj . snd) lab
-                    then if isBlast && bproj sb then do
-                           -- Hit the blocking actor.
-                           projectBla source spos (pos:rest) iid cstore isBlast
-                           return Nothing
-                         else return $ Just ProjectBlockActor
-                    else do
-                      if isBlast && bproj sb && eps `mod` 2 == 0 then
-                        -- Make the explosion a bit less regular.
-                        projectBla source spos (pos:rest) iid cstore isBlast
-                      else
-                        projectBla source pos rest iid cstore isBlast
-                      return Nothing
+                  -- Make the explosion less regular and weaker at edges.
+                  if blast && bproj sb && center then
+                    -- Start in the center, not around.
+                    projectBla source spos (pos:rest) iid cstore blast
+                  else
+                    projectBla source pos rest iid cstore blast
+                  return Nothing
 
 projectBla :: MonadServerAtomic m
            => ActorId    -- ^ actor projecting the item (is on current lvl)
@@ -355,81 +331,87 @@ projectBla :: MonadServerAtomic m
            -> CStore     -- ^ whether the items comes from floor or inventory
            -> Bool       -- ^ whether the item is a blast
            -> m ()
-projectBla source pos rest iid cstore isBlast = do
+projectBla source pos rest iid cstore blast = do
   sb <- getsState $ getActorBody source
-  item <- getsState $ getItemBody iid
   let lid = blid sb
   localTime <- getsState $ getLocalTime lid
-  unless isBlast $ execSfxAtomic $ SfxProject source iid cstore
+  unless blast $ execSfxAtomic $ SfxProject source iid cstore
   bag <- getsState $ getBodyStoreBag sb cstore
+  ItemFull{itemBase, itemKind} <- getsState $ itemToFull iid
   case iid `EM.lookup` bag of
     Nothing -> error $ "" `showFailure` (source, pos, rest, iid, cstore)
     Just kit@(_, it) -> do
-      let btime = absoluteTimeAdd timeEpsilon localTime
-      addProjectile pos rest iid kit lid (bfid sb) btime isBlast
+      let delay = if IK.iweight itemKind == 0 then timeTurn else timeClip
+          btime = absoluteTimeAdd delay localTime
+      addProjectile pos rest iid kit lid (bfid sb) btime
       let c = CActor source cstore
-      execUpdAtomic $ UpdLoseItem False iid item (1, take 1 it) c
+      execUpdAtomic $ UpdLoseItem False iid itemBase (1, take 1 it) c
 
-addProjectile :: MonadServerAtomic m
-              => Point -> [Point] -> ItemId -> ItemQuant -> LevelId
-              -> FactionId -> Time -> Bool
-              -> m ()
-addProjectile bpos rest iid (_, it) blid bfid btime _isBlast = do
-  itemToF <- getsState itemToFull
-  let itemFull@ItemFull{itemBase} = itemToF iid (1, take 1 it)
-      (trajectory, (speed, _)) = itemTrajectory itemBase (bpos : rest)
-      tweakBody b = b { bhp = oneM
-                      , bproj = True
-                      , btrajectory = Just (trajectory, speed)
-                      , beqp = EM.singleton iid (1, take 1 it)
-                      , borgan = EM.empty }  -- don't confer bonuses from trunk
-  void $ addActorIid iid itemFull True bfid bpos blid tweakBody btime
-
-addActor :: MonadServerAtomic m
-         => GroupName ItemKind -> FactionId -> Point -> LevelId
-         -> (Actor -> Actor) -> Time
-         -> m (Maybe ActorId)
-addActor actorGroup bfid pos lid tweakBody time = do
+addActorFromGroup :: MonadServerAtomic m
+                  => GroupName ItemKind -> FactionId -> Point -> LevelId -> Time
+                  -> m (Maybe ActorId)
+addActorFromGroup actorGroup bfid pos lid time = do
   -- We bootstrap the actor by first creating the trunk of the actor's body
   -- that contains the constant properties.
   let trunkFreq = [(actorGroup, 1)]
-  m5 <- rollItem 0 lid trunkFreq
-  case m5 of
+  m3 <- rollItem 0 lid trunkFreq
+  case m3 of
     Nothing -> return Nothing
-    Just (itemKnownRaw, itemFullRaw, _, seed, _) ->
-      registerActor itemKnownRaw itemFullRaw seed bfid pos lid tweakBody time
+    Just (itemKnown, itemFullKit, _) ->
+      Just <$> registerActor False itemKnown itemFullKit bfid pos lid time
 
 registerActor :: MonadServerAtomic m
-              => ItemKnown -> ItemFull -> ItemSeed
-              -> FactionId -> Point -> LevelId -> (Actor -> Actor) -> Time
-              -> m (Maybe ActorId)
-registerActor (kindIx, ar, damage, _) itemFullRaw seed
-              bfid pos lid tweakBody time = do
+              => Bool -> ItemKnown -> ItemFullKit
+              -> FactionId -> Point -> LevelId -> Time
+              -> m ActorId
+registerActor summoned (kindIx, ar, _) (itemFullRaw, kit) bfid pos lid time = do
   let container = CTrunk bfid lid pos
       jfid = Just bfid
-      itemKnown = (kindIx, ar, damage, jfid)
+      itemKnown = (kindIx, ar, jfid)
       itemFull = itemFullRaw {itemBase = (itemBase itemFullRaw) {jfid}}
-  trunkId <- registerItem itemFull itemKnown seed container False
-  addActorIid trunkId itemFull False bfid pos lid tweakBody time
+  trunkId <- registerItem (itemFull, kit) itemKnown container False
+  addNonProjectile summoned trunkId (itemFull, kit) bfid pos lid time
+
+addProjectile :: MonadServerAtomic m
+              => Point -> [Point] -> ItemId -> ItemQuant -> LevelId -> FactionId
+              -> Time
+              -> m ()
+addProjectile bpos rest iid (_, it) blid bfid btime = do
+  itemFull <- getsState $ itemToFull iid
+  let (trajectory, (speed, _)) =
+        IK.itemTrajectory (itemKind itemFull) (bpos : rest)
+      -- Trunk is added to equipment, not to organs, because it's the
+      -- projected item, so it's carried, not grown.
+      tweakBody b = b { bhp = oneM
+                      , btrajectory = Just (trajectory, speed)
+                      , beqp = EM.singleton iid (1, take 1 it) }
+  void $ addActorIid iid itemFull True bfid bpos blid tweakBody btime
+
+addNonProjectile :: MonadServerAtomic m
+                 => Bool -> ItemId -> ItemFullKit -> FactionId -> Point
+                 -> LevelId -> Time
+                 -> m ActorId
+addNonProjectile summoned trunkId (itemFull, kit) fid pos lid time = do
+  let tweakBody b = b { borgan = EM.singleton trunkId kit
+                      , bcalm = if summoned
+                                then bcalm b * 2 `div` 3 - xM 3
+                                  -- will summon in 3 turns, if calm regenerates
+                                else bcalm b }
+  addActorIid trunkId itemFull False fid pos lid tweakBody time
 
 addActorIid :: MonadServerAtomic m
             => ItemId -> ItemFull -> Bool -> FactionId -> Point -> LevelId
             -> (Actor -> Actor) -> Time
-            -> m (Maybe ActorId)
-addActorIid trunkId trunkFull@ItemFull{..} bproj
-            bfid pos lid tweakBody time = do
-  let trunkKind = case itemDisco of
-        Just ItemDisco{itemKind} -> itemKind
-        Nothing -> error $ "" `showFailure` trunkFull
-      aspects = fromJust $ itemAspect $ fromJust itemDisco
+            -> m ActorId
+addActorIid trunkId ItemFull{itemBase, itemKind, itemDisco}
+            bproj bfid pos lid tweakBody time = do
   -- Initial HP and Calm is based only on trunk and ignores organs.
-      hp = xM (max 2 $ aMaxHP aspects) `div` 2
+  let hp = xM (max 2 $ IA.aMaxHP $ itemAspect itemDisco) `div` 2
       -- Hard to auto-id items that refill Calm, but reduced sight at game
       -- start is more confusing and frustrating:
-      calm = xM (max 0 $ aMaxCalm aspects)
+      calm = xM (max 0 $ IA.aMaxCalm $ itemAspect itemDisco)
   -- Create actor.
   factionD <- getsState sfactionD
-  let fact = factionD EM.! bfid
   curChalSer <- getsServer $ scurChalSer . soptions
   nU <- nUI
   -- If difficulty is below standard, HP is added to the UI factions,
@@ -440,64 +422,66 @@ addActorIid trunkId trunkFull@ItemFull{..} bproj
         fhasUI gplayer || nU == 0 && fcanEscape gplayer
       boostFact = not bproj
                   && if diffBonusCoeff > 0
-                     then hasUIorEscapes fact
-                          || any hasUIorEscapes
-                                 (filter (`isAllied` bfid) $ EM.elems factionD)
-                     else any hasUIorEscapes
-                              (filter (`isAtWar` bfid) $ EM.elems factionD)
+                     then any (hasUIorEscapes . snd)
+                              (filter (\(fi, fa) -> isFriend fi fa bfid)
+                                      (EM.assocs factionD))
+                     else any (hasUIorEscapes . snd)
+                              (filter (\(fi, fa) -> isFoe fi fa bfid)
+                                      (EM.assocs factionD))
       diffHP | boostFact = if cdiff curChalSer `elem` [1, difficultyBound]
                            then xM 999 - hp -- as much as UI can stand
                            else hp * 2 ^ abs diffBonusCoeff
              | otherwise = hp
       bonusHP = fromEnum $ (diffHP - hp) `divUp` oneM
       healthOrgans = [(Just bonusHP, ("bonus HP", COrgan)) | bonusHP /= 0]
-      b = actorTemplate trunkId diffHP calm pos lid bfid
+      b = actorTemplate trunkId diffHP calm pos lid bfid bproj
       -- Insert the trunk as the actor's organ.
-      withTrunk = b { borgan = EM.singleton trunkId (itemK, itemTimer)
-                    , bweapon = if isMelee itemBase then 1 else 0 }
+      withTrunk = b {bweapon = if IK.isMelee itemKind then 1 else 0}
   aid <- getsServer sacounter
   modifyServer $ \ser -> ser {sacounter = succ aid}
-  execUpdAtomic $ UpdCreateActor aid (tweakBody withTrunk) [(trunkId, itemBase)]
   modifyServer $ \ser ->
     ser {sactorTime = updateActorTime bfid lid aid time $ sactorTime ser}
+  execUpdAtomic $ UpdCreateActor aid (tweakBody withTrunk) [(trunkId, itemBase)]
   -- Create, register and insert all initial actor items, including
   -- the bonus health organs from difficulty setting.
-  forM_ (healthOrgans ++ map (Nothing,) (IK.ikit trunkKind))
+  forM_ (healthOrgans ++ map (Nothing,) (IK.ikit itemKind))
         $ \(mk, (ikText, cstore)) -> do
     let container = CActor aid cstore
         itemFreq = [(ikText, 1)]
     mIidEtc <- rollAndRegisterItem lid itemFreq container False mk
     case mIidEtc of
       Nothing -> error $ "" `showFailure` (lid, itemFreq, container, mk)
-      Just (iid, (itemFull, _)) -> discoverIfNoEffects container iid itemFull
-  return $ Just aid
+      Just (iid, ((itemFull2, _), _)) ->
+        -- The items are create in inventory, so won't be picked up,
+        -- so we have to discover them now, if eligible.
+        discoverIfMinorEffects container iid (itemKindId itemFull2)
+  return aid
 
-discoverIfNoEffects :: MonadServerAtomic m
-                    => Container -> ItemId -> ItemFull -> m ()
-discoverIfNoEffects c iid itemFull = case itemFull of
-  ItemFull{itemDisco=Just ItemDisco{itemKind}}
-    | any IK.forIdEffect (IK.ieffects itemKind)
-      || maybe False (> 0) (lookup "gem" $ IK.ifreq itemKind) ->  -- a hack
-    return ()  -- discover by use, ignore gems
-  ItemFull{itemDisco=Just ItemDisco{itemKindId}} -> do
-    seed <- getsServer $ (EM.! iid) . sitemSeedD
-    execUpdAtomic $ UpdDiscover c iid itemKindId seed
-  _ -> error "server doesn't fully know item"
+discoverIfMinorEffects :: MonadServerAtomic m
+                       => Container -> ItemId -> ContentId ItemKind -> m ()
+discoverIfMinorEffects c iid itemKindId = do
+  COps{coitem} <- getsState scops
+  let itemKind = okind coitem itemKindId
+  if IK.onlyMinorEffects itemKind
+  then do
+    discoAspect <- getsState sdiscoAspect
+    execUpdAtomic $ UpdDiscover c iid itemKindId $ discoAspect EM.! iid
+  else return ()  -- discover by use when item's effects get activated later on
 
 pickWeaponServer :: MonadServer m => ActorId -> m (Maybe (ItemId, CStore))
 pickWeaponServer source = do
-  eqpAssocs <- getsState $ fullAssocs source [CEqp]
-  bodyAssocs <- getsState $ fullAssocs source [COrgan]
+  eqpAssocs <- getsState $ kitAssocs source [CEqp]
+  bodyAssocs <- getsState $ kitAssocs source [COrgan]
   actorSk <- currentSkillsServer source
   sb <- getsState $ getActorBody source
-  let allAssocsRaw = eqpAssocs ++ bodyAssocs
+  let kitAssRaw = eqpAssocs ++ bodyAssocs
       forced = bproj sb
-      allAssocs | forced = allAssocsRaw  -- for projectiles, anything is weapon
-                | otherwise = filter (isMelee . itemBase . snd) allAssocsRaw
+      kitAss | forced = kitAssRaw  -- for projectiles, anything is weapon
+             | otherwise = filter (IK.isMelee . itemKind . fst . snd) kitAssRaw
   -- Server ignores item effects or it would leak item discovery info.
   -- In particular, it even uses weapons that would heal opponent,
   -- and not only in case of projectiles.
-  strongest <- pickWeaponM Nothing allAssocs actorSk source
+  strongest <- pickWeaponM Nothing kitAss actorSk source
   case strongest of
     [] -> return Nothing
     iis@((maxS, _) : _) -> do
