@@ -19,8 +19,9 @@ import qualified Data.IntMap.Strict as IM
 import           Data.Tuple
 import qualified System.Random as R
 
+import qualified Game.LambdaHack.Common.Dice as Dice
 import           Game.LambdaHack.Common.Frequency
-import qualified Game.LambdaHack.Common.Kind as Kind
+import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Level
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.Point
@@ -36,12 +37,12 @@ import qualified Game.LambdaHack.Content.TileKind as TK
 import           Game.LambdaHack.Server.DungeonGen.Cave
 import           Game.LambdaHack.Server.DungeonGen.Place
 
-convertTileMaps :: Kind.COps -> Bool -> Rnd (Kind.Id TileKind)
-                -> Maybe (Rnd (Kind.Id TileKind)) -> Int -> Int -> TileMapEM
+convertTileMaps :: COps -> Bool -> Rnd (ContentId TileKind)
+                -> Maybe (Rnd (ContentId TileKind)) -> Int -> Int -> TileMapEM
                 -> Rnd TileMap
-convertTileMaps Kind.COps{coTileSpeedup} areAllWalkable
+convertTileMaps COps{coTileSpeedup} areAllWalkable
                 cdefTile mpickPassable cxsize cysize ltile = do
-  let runCdefTile :: R.StdGen -> (Kind.Id TileKind, R.StdGen)
+  let runCdefTile :: R.StdGen -> (ContentId TileKind, R.StdGen)
       runCdefTile = St.runState cdefTile
       runUnfold gen =
         let (gen1, gen2) = R.split gen
@@ -87,38 +88,35 @@ convertTileMaps Kind.COps{coTileSpeedup} areAllWalkable
             connect (not . yeven) blocksVertical walkable5 converted4
       return converted5
 
-buildTileMap :: Kind.COps -> Cave -> Rnd TileMap
-buildTileMap cops@Kind.COps{ cotile=Kind.Ops{opick}
-                           , cocave=Kind.Ops{okind=cokind} }
-             Cave{dkind, dmap, dnight} = do
-  let CaveKind{cxsize, cysize, cpassable, cdefTile} = cokind dkind
+buildTileMap :: COps -> Cave -> Rnd TileMap
+buildTileMap cops@COps{cotile, cocave} Cave{dkind, dmap, dnight} = do
+  let CaveKind{cxsize, cysize, cpassable, cdefTile} = okind cocave dkind
       nightCond kt = not (Tile.kindHasFeature TK.Walkable kt)
                      || (if dnight then id else not)
                            (Tile.kindHasFeature TK.Dark kt)
       pickDefTile = fromMaybe (error $ "" `showFailure` cdefTile)
-                    <$> opick cdefTile nightCond
+                    <$> opick cotile cdefTile nightCond
       wcond kt = Tile.isEasyOpenKind kt && nightCond kt
       mpickPassable =
         if cpassable
         then Just $ fromMaybe (error $ "" `showFailure` cdefTile)
-                    <$> opick cdefTile wcond
+                    <$> opick cotile cdefTile wcond
         else Nothing
       nwcond kt = not (Tile.kindHasFeature TK.Walkable kt) && nightCond kt
-  areAllWalkable <- isNothing <$> opick cdefTile nwcond
+  areAllWalkable <- isNothing <$> opick cotile cdefTile nwcond
   convertTileMaps cops areAllWalkable
                   pickDefTile mpickPassable cxsize cysize dmap
 
 -- Create a level from a cave.
-buildLevel :: Kind.COps -> Int -> GroupName CaveKind
-           -> Int -> AbsDepth -> [Point]
+buildLevel :: COps -> Int -> GroupName CaveKind
+           -> Int -> Dice.AbsDepth -> [Point]
            -> Rnd (Level, [Point])
-buildLevel cops@Kind.COps{cocave=Kind.Ops{okind=okind, opick}}
-           ln genName minD totalDepth lstairPrev = do
+buildLevel cops@COps{cocave} ln genName minD totalDepth lstairPrev = do
   dkind <- fromMaybe (error $ "" `showFailure` genName)
-           <$> opick genName (const True)
-  let kc = okind dkind
+           <$> opick cocave genName (const True)
+  let kc = okind cocave dkind
       -- Simple rule for now: level @ln@ has depth (difficulty) @abs ln@.
-      ldepth = AbsDepth $ abs ln
+      ldepth = Dice.AbsDepth $ abs ln
   -- Any stairs coming from above are considered extra stairs
   -- and if they don't exceed @extraStairs@,
   -- the amount is filled up with single downstairs.
@@ -165,9 +163,8 @@ buildLevel cops@Kind.COps{cocave=Kind.Ops{okind=okind, opick}}
   dsecret <- randomR (1, maxBound)
   cave <- buildCave cops ldepth totalDepth dsecret dkind fixedCenters
   cmap <- buildTileMap cops cave
-  litemNum <- castDice ldepth totalDepth $ citemNum kc
-  let lvl = levelFromCaveKind cops kc ldepth cmap lstair litemNum lescape
-                              (dnight cave)
+  let lvl =
+        levelFromCaveKind cops dkind ldepth cmap lstair lescape (dnight cave)
   return (lvl, lstairsDouble ++ lstairsSingleDown)
 
 -- Places yet another staircase (or escape), taking into account only
@@ -184,70 +181,61 @@ placeDownStairs kc@CaveKind{..} ps = do
                    $ ps ++ bootFixedCenters kc
       minDist = if length ps >= 3 then 0 else cminStairDist
       f p@Point{..} =
-        if p `inside` (9, 8, cxsize - 10, cysize - 9)
+        if p `inside` (9, 8, cxsize - 10, cysize - anchorDown - 5)
         then if dist minDist p && distProj p then Just p else Nothing
         else let nx = if | px < 9 -> 4
                          | px > cxsize - 10 -> cxsize - 5
                          | otherwise -> px
                  ny = if | py < 8 -> 3
-                         | py > cysize - 9 -> cysize - 4
+                         | py > cysize - anchorDown - 5 -> cysize - anchorDown
                          | otherwise -> py
                  np = Point nx ny
              in if dist 0 np && distProj np then Just np else Nothing
   findPoint cxsize cysize f
 
 -- Build rudimentary level from a cave kind.
-levelFromCaveKind :: Kind.COps -> CaveKind -> AbsDepth -> TileMap
-                  -> ([Point], [Point]) -> Int -> [Point] -> Bool
+levelFromCaveKind :: COps -> ContentId CaveKind -> Dice.AbsDepth
+                  -> TileMap -> ([Point], [Point]) -> [Point] -> Bool
                   -> Level
-levelFromCaveKind Kind.COps{coTileSpeedup}
-                  CaveKind{ cactorCoeff=lactorCoeff
-                          , cactorFreq=lactorFreq
-                          , citemFreq=litemFreq
-                          , ..
-                          }
-                  ldepth ltile lstair litemNum lescape lnight =
+levelFromCaveKind COps{cocave, coTileSpeedup}
+                  lkind ldepth ltile lstair lescape lnight =
   let f n t | Tile.isExplorable coTileSpeedup t = n + 1
             | otherwise = n
-      lexplorable = PointArray.foldlA' f 0 ltile
+      lexpl = PointArray.foldlA' f 0 ltile
+      CaveKind{cxsize, cysize} = okind cocave lkind
   in Level
-       { ldepth
+       { lkind
+       , ldepth
        , lfloor = EM.empty
-       , lembed = EM.empty  -- is populated inside $MonadServer$
+       , lembed = EM.empty
        , lactor = EM.empty
        , ltile
        , lxsize = cxsize
        , lysize = cysize
        , lsmell = EM.empty
        , lstair
-       , lseen = 0
-       , lexplorable
-       , ltime = timeZero
-       , lactorCoeff
-       , lactorFreq
-       , litemNum
-       , litemFreq
        , lescape
+       , lseen = 0
+       , lexpl
+       , ltime = timeZero
        , lnight
-       , lname = cname
-       , ldesc = cdesc
        }
 
 -- | Freshly generated and not yet populated dungeon.
 data FreshDungeon = FreshDungeon
-  { freshDungeon    :: Dungeon   -- ^ maps for all levels
-  , freshTotalDepth :: AbsDepth  -- ^ absolute dungeon depth
+  { freshDungeon    :: Dungeon        -- ^ maps for all levels
+  , freshTotalDepth :: Dice.AbsDepth  -- ^ absolute dungeon depth
   }
 
 -- | Generate the dungeon for a new game.
-dungeonGen :: Kind.COps -> Caves -> Rnd FreshDungeon
+dungeonGen :: COps -> Caves -> Rnd FreshDungeon
 dungeonGen cops caves = do
   let (minD, maxD) =
         case (IM.minViewWithKey caves, IM.maxViewWithKey caves) of
           (Just ((s, _), _), Just ((e, _), _)) -> (s, e)
           _ -> error $ "no caves" `showFailure` caves
       freshTotalDepth = assert (signum minD == signum maxD)
-                        $ AbsDepth
+                        $ Dice.AbsDepth
                         $ max 10 $ max (abs minD) (abs maxD)
       buildLvl :: ([(LevelId, Level)], [Point])
                -> (Int, GroupName CaveKind)
