@@ -1,6 +1,6 @@
 -- | Generation of caves (not yet inhabited dungeon levels) from cave kinds.
 module Game.LambdaHack.Server.DungeonGen.Cave
-  ( Cave(..), bootFixedCenters, buildCave
+  ( Cave(..), anchorDown, bootFixedCenters, buildCave
 #ifdef EXPOSE_INTERNAL
     -- * Internal operations
   , pickOpening, digCorridors
@@ -15,8 +15,8 @@ import qualified Data.EnumMap.Strict as EM
 import qualified Data.EnumSet as ES
 import           Data.Key (mapWithKeyM)
 
-import qualified Game.LambdaHack.Common.Kind as Kind
-import           Game.LambdaHack.Common.Level
+import qualified Game.LambdaHack.Common.Dice as Dice
+import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.Point
 import           Game.LambdaHack.Common.Random
@@ -31,7 +31,7 @@ import           Game.LambdaHack.Server.DungeonGen.Place
 
 -- | The type of caves (not yet inhabited dungeon levels).
 data Cave = Cave
-  { dkind   :: Kind.Id CaveKind  -- ^ the kind of the cave
+  { dkind   :: ContentId CaveKind  -- ^ the kind of the cave
   , dsecret :: Int               -- ^ secret tile seed
   , dmap    :: TileMapEM         -- ^ tile kinds in the cave
   , dplaces :: [Place]           -- ^ places generated in the cave
@@ -39,8 +39,12 @@ data Cave = Cave
   }
   deriving Show
 
+anchorDown :: Y
+anchorDown = 5  -- not 4, asymmetric vs up, for staircase variety
+
 bootFixedCenters :: CaveKind -> [Point]
-bootFixedCenters CaveKind{..} = [Point 4 3, Point (cxsize - 5) (cysize - 4)]
+bootFixedCenters CaveKind{..} =
+  [Point 4 3, Point (cxsize - 5) (cysize - anchorDown)]
 
 {- |
 Generate a cave using an algorithm inspired by the original Rogue,
@@ -64,19 +68,16 @@ as follows (in gross simplification):
   on the grid is connected, and a few more might be. It is not sufficient
   to always connect all adjacent rooms, because not each cell holds a room.
 -}
-buildCave :: Kind.COps         -- ^ content definitions
-          -> AbsDepth          -- ^ depth of the level to generate
-          -> AbsDepth          -- ^ absolute depth
+buildCave :: COps              -- ^ content definitions
+          -> Dice.AbsDepth     -- ^ depth of the level to generate
+          -> Dice.AbsDepth     -- ^ absolute depth
           -> Int               -- ^ secret tile seed
-          -> Kind.Id CaveKind  -- ^ cave kind to use for generation
+          -> ContentId CaveKind  -- ^ cave kind to use for generation
           -> EM.EnumMap Point (GroupName PlaceKind)  -- ^ pos of stairs, etc.
           -> Rnd Cave
-buildCave cops@Kind.COps{ cotile=cotile@Kind.Ops{opick}
-                        , cocave=Kind.Ops{okind}
-                        , coplace=Kind.Ops{okind=pokind}
-                        , coTileSpeedup }
+buildCave cops@COps{cotile, cocave, coplace, coTileSpeedup}
           ldepth totalDepth dsecret dkind fixedCenters = do
-  let kc@CaveKind{..} = okind dkind
+  let kc@CaveKind{..} = okind cocave dkind
   lgrid' <- castDiceXY ldepth totalDepth cgrid
   -- Make sure that in caves not filled with rock, there is a passage
   -- across the cave, even if a single room blocks most of the cave.
@@ -86,9 +87,9 @@ buildCave cops@Kind.COps{ cotile=cotile@Kind.Ops{opick}
       subFullArea = fromMaybe (error $ "" `showFailure` kc)
                     $ toArea (1, 1, cxsize - 2, cysize - 2)
   darkCorTile <- fromMaybe (error $ "" `showFailure` cdarkCorTile)
-                 <$> opick cdarkCorTile (const True)
+                 <$> opick cotile cdarkCorTile (const True)
   litCorTile <- fromMaybe (error $ "" `showFailure` clitCorTile)
-                <$> opick clitCorTile (const True)
+                <$> opick cotile clitCorTile (const True)
   dnight <- chanceDice ldepth totalDepth cnightChance
   let createPlaces lgr' = do
         let area | couterFenceTile /= "basic outer fence" = subFullArea
@@ -115,11 +116,14 @@ buildCave cops@Kind.COps{ cotile=cotile@Kind.Ops{opick}
                           isFixed p = case gs EM.! p of
                             SpecialFixed{} -> True
                             _ -> False
-                      in if | any isFixed
+                      in if -- Limit (the aggresive) merging of normal places
+                            -- and leave extra place for merging stairs.
+                            | any isFixed
                               $ vicinityCardinal gx gy (Point x y) -> Nothing
-                              -- Bias: prefer extending vertically.
-                            | y1 - y0 - 1 < snd minPlaceSize -> Just Vert
-                            | x1 - x0 - 1 < fst minPlaceSize -> Just Horiz
+                            -- Bias: prefer extending vertically.
+                            -- Not @-1@, but @-3@, to merge aggressively.
+                            | y1 - y0 - 3 < snd minPlaceSize -> Just Vert
+                            | x1 - x0 - 3 < fst minPlaceSize -> Just Horiz
                             | otherwise -> Nothing
                     _ -> Nothing
               in case special of
@@ -141,18 +145,23 @@ buildCave cops@Kind.COps{ cotile=cotile@Kind.Ops{opick}
                             mergeSpecial ar i{px = px i + 1} SpecialArea
                     _ -> gs0
                 SpecialFixed p placeGroup ar ->
+                  -- If single merge is sufficient to extend the fixed place
+                  -- to full size, and the merge is possible, we perform it.
+                  -- An empty inner list signifies some merge is needed,
+                  -- but not possible, and then we abort and don't waste space.
                   let (x0, y0, x1, y1) = fromArea ar
-                      d = 3
-                      vics = [ i {py = py i - 1}
-                             | py p - y0 < d && py i - 1 >= 0 ]
-                             ++ [ i {py = py i + 1}
-                                | y1 - py p < d && py i + 1 < gy ]
-                             ++ [ i {px = px i - 1}
-                                | px p - x0 < d + 1 && px i - 1 >= 0 ]
-                             ++ [ i {px = px i + 1}
-                                | x1 - px p < d + 1 && px i + 1 < gx ]
+                      d = 3  -- arbitrary, matches common content
+                      vics :: [[Point]]
+                      vics = [ [i {py = py i - 1} | py i - 1 >= 0]  -- possible
+                             | py p - y0 < d ]  -- needed
+                             ++ [ [i {py = py i + 1} | py i + 1 < gy]
+                                | y1 - py p < d ]
+                             ++ [ [i {px = px i - 1} | px i - 1 >= 0]
+                                | px p - x0 < d ]
+                             ++ [ [i {px = px i + 1} | px i + 1 < gx]
+                                | x1 - px p < d ]
                   in case vics of
-                    [p2] -> mergeSpecial ar p2 (SpecialFixed p placeGroup)
+                    [[p2]] -> mergeSpecial ar p2 (SpecialFixed p placeGroup)
                     _ -> gs0
                 SpecialMerged{} -> error $ "" `showFailure` (gs, gs0, i)
             gs2 = foldl' mergeFixed gs $ EM.assocs gs
@@ -189,7 +198,7 @@ buildCave cops@Kind.COps{ cotile=cotile@Kind.Ops{opick}
                     (tmap, place) <-
                       buildPlace cops kc dnight darkCorTile litCorTile
                                  ldepth totalDepth dsecret r Nothing
-                    let fence = pfence $ pokind $ qkind place
+                    let fence = pfence $ okind coplace $ qkind place
                     return ( EM.union tmap m
                            , place : pls
                            , EM.insert i (qarea place, fence, ar) qls )
@@ -208,7 +217,7 @@ buildCave cops@Kind.COps{ cotile=cotile@Kind.Ops{opick}
                   (tmap, place) <-
                     buildPlace cops kc dnight darkCorTile litCorTile
                                ldepth totalDepth dsecret r (Just placeGroup)
-                  let fence = pfence $ pokind $ qkind place
+                  let fence = pfence $ okind coplace $ qkind place
                   return ( EM.union tmap m
                          , place : pls
                          , EM.insert i (qarea place, fence, ar) qls )
@@ -270,10 +279,10 @@ buildCave cops@Kind.COps{ cotile=cotile@Kind.Ops{opick}
   dmap <- mapWithKeyM obscure umap
   return $! Cave {dkind, dsecret, dmap, dplaces, dnight}
 
-pickOpening :: Kind.COps -> CaveKind -> TileMapEM -> Kind.Id TileKind
-            -> Int -> Point -> (Kind.Id TileKind, Kind.Id TileKind)
-            -> Rnd (Kind.Id TileKind)
-pickOpening Kind.COps{cotile, coTileSpeedup}
+pickOpening :: COps -> CaveKind -> TileMapEM -> ContentId TileKind
+            -> Int -> Point -> (ContentId TileKind, ContentId TileKind)
+            -> Rnd (ContentId TileKind)
+pickOpening COps{cotile, coTileSpeedup}
             CaveKind{cxsize, cysize, cdoorChance, copenChance, chidden}
             lplaces litCorTile dsecret
             pos (hidden, cor) = do
@@ -305,7 +314,7 @@ pickOpening Kind.COps{cotile, coTileSpeedup}
     else return $! doorTrappedId  -- assume this is what content enforces
   else return $! nicerCorridor
 
-digCorridors :: Kind.Id TileKind -> Corridor -> TileMapEM
+digCorridors :: ContentId TileKind -> Corridor -> TileMapEM
 digCorridors tile (p1:p2:ps) =
   EM.union corPos (digCorridors tile (p2:ps))
  where
