@@ -18,9 +18,9 @@ import qualified Data.EnumMap.Strict as EM
 import qualified Data.EnumSet as ES
 import qualified Data.Text as T
 
+import qualified Game.LambdaHack.Common.Dice as Dice
 import           Game.LambdaHack.Common.Frequency
-import qualified Game.LambdaHack.Common.Kind as Kind
-import           Game.LambdaHack.Common.Level
+import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.Point
 import           Game.LambdaHack.Common.Random
@@ -34,20 +34,20 @@ import           Game.LambdaHack.Server.DungeonGen.Area
 -- | The parameters of a place. All are immutable and rolled and fixed
 -- at the time when a place is generated.
 data Place = Place
-  { qkind    :: Kind.Id PlaceKind
+  { qkind    :: ContentId PlaceKind
   , qarea    :: Area
   , qseen    :: Bool
   , qlegend  :: GroupName TileKind
-  , qFWall   :: Kind.Id TileKind
-  , qFFloor  :: Kind.Id TileKind
-  , qFGround :: Kind.Id TileKind
+  , qFWall   :: ContentId TileKind
+  , qFFloor  :: ContentId TileKind
+  , qFGround :: ContentId TileKind
   }
   deriving Show
 
 -- | The map of tile kinds in a place (and generally anywhere in a cave).
 -- The map is sparse. The default tile that eventually fills the empty spaces
 -- is specified in the cave kind specification with @cdefTile@.
-type TileMapEM = EM.EnumMap Point (Kind.Id TileKind)
+type TileMapEM = EM.EnumMap Point (ContentId TileKind)
 
 -- | For @CAlternate@ tiling, require the place be comprised
 -- of an even number of whole corners, with exactly one square
@@ -107,24 +107,23 @@ interiorArea kr r =
 
 -- | Given a few parameters, roll and construct a 'Place' datastructure
 -- and fill a cave section acccording to it.
-buildPlace :: Kind.COps         -- ^ the game content
+buildPlace :: COps         -- ^ the game content
            -> CaveKind          -- ^ current cave kind
            -> Bool              -- ^ whether the cave is dark
-           -> Kind.Id TileKind  -- ^ dark fence tile, if fence hollow
-           -> Kind.Id TileKind  -- ^ lit fence tile, if fence hollow
-           -> AbsDepth          -- ^ current level depth
-           -> AbsDepth          -- ^ absolute depth
+           -> ContentId TileKind  -- ^ dark fence tile, if fence hollow
+           -> ContentId TileKind  -- ^ lit fence tile, if fence hollow
+           -> Dice.AbsDepth       -- ^ current level depth
+           -> Dice.AbsDepth       -- ^ absolute depth
            -> Int               -- ^ secret tile seed
            -> Area              -- ^ whole area of the place, fence included
            -> Maybe (GroupName PlaceKind)  -- ^ optional fixed place group
            -> Rnd (TileMapEM, Place)
-buildPlace cops@Kind.COps{ cotile=Kind.Ops{opick}
-                         , coplace=Kind.Ops{ofoldlGroup'} }
+buildPlace cops@COps{cotile, coplace}
            CaveKind{..} dnight darkCorTile litCorTile
-           ldepth@(AbsDepth ld) totalDepth@(AbsDepth depth) dsecret
+           ldepth@(Dice.AbsDepth ld) totalDepth@(Dice.AbsDepth depth) dsecret
            r mplaceGroup = do
   qFWall <- fromMaybe (error $ "" `showFailure` cfillerTile)
-            <$> opick cfillerTile (const True)
+            <$> opick cotile cfillerTile (const True)
   let findInterval x1y1 [] = (x1y1, (11, 0))
       findInterval !x1y1 ((!x, !y) : rest) =
         if fromIntegral ld * 10 <= x * fromIntegral depth
@@ -141,7 +140,7 @@ buildPlace cops@Kind.COps{ cotile=Kind.Ops{opick}
       f !placeGroup !q !acc !p !pk !kind =
         let rarity = linearInterpolation (prarity kind)
         in (q * p * rarity, ((pk, kind), placeGroup)) : acc
-      g (placeGroup, q) = ofoldlGroup' placeGroup (f placeGroup q) []
+      g (placeGroup, q) = ofoldlGroup' coplace placeGroup (f placeGroup q) []
       pfreq = case mplaceGroup of
         Nothing -> cplaceFreq
         Just placeGroup -> [(placeGroup, 1)]
@@ -176,10 +175,10 @@ buildPlace cops@Kind.COps{ cotile=Kind.Ops{opick}
       isEdge (Point x y) = x `elem` [x0, x1] || y `elem` [y0, y1]
       digDay xy c | isEdge xy = lookupOneIn xlegendLit xy c
                   | otherwise = lookupOneIn xlegend xy c
-      lookupOneIn :: ( EM.EnumMap Char (Int, Kind.Id TileKind)
-                     , EM.EnumMap Char (Kind.Id TileKind) )
+      lookupOneIn :: ( EM.EnumMap Char (Int, ContentId TileKind)
+                     , EM.EnumMap Char (ContentId TileKind) )
                   -> Point -> Char
-                  -> Kind.Id TileKind
+                  -> ContentId TileKind
       lookupOneIn (mOneIn, m) xy c = case EM.lookup c mOneIn of
         Just (oneInChance, tk) ->
           if isChancePos oneInChance dsecret xy
@@ -197,50 +196,50 @@ isChancePos c dsecret (Point x y) =
   c > 0 && (dsecret `Bits.rotateR` x `Bits.xor` y + x) `mod` c == 0
 
 -- | Roll a legend of a place plan: a map from plan symbols to tile kinds.
-olegend :: Kind.COps -> GroupName TileKind
-        -> Rnd ( EM.EnumMap Char (Int, Kind.Id TileKind)
-               , EM.EnumMap Char (Kind.Id TileKind) )
-olegend Kind.COps{cotile=Kind.Ops{ofoldlWithKey', opick, okind}} cgroup =
+olegend :: COps -> GroupName TileKind
+        -> Rnd ( EM.EnumMap Char (Int, ContentId TileKind)
+               , EM.EnumMap Char (ContentId TileKind) )
+olegend COps{cotile} cgroup =
   let getSymbols !acc _ !tk =
         maybe acc (const $ ES.insert (TK.tsymbol tk) acc)
               (lookup cgroup $ TK.tfreq tk)
-      symbols = ofoldlWithKey' getSymbols ES.empty
+      symbols = ofoldlWithKey' cotile getSymbols ES.empty
       getLegend s !acc = do
         (mOneIn, m) <- acc
         let p f t = TK.tsymbol t == s && f (Tile.kindHasFeature TK.Spice t)
         tk <- fmap (fromMaybe $ error $ "" `showFailure` (cgroup, s))
-              $ opick cgroup (p not)
-        mtkSpice <- opick cgroup (p id)
+              $ opick cotile cgroup (p not)
+        mtkSpice <- opick cotile cgroup (p id)
         return $! case mtkSpice of
           Nothing -> (mOneIn, EM.insert s tk m)
           Just tkSpice ->
-            let n = fromJust (lookup cgroup (TK.tfreq (okind tk)))
-                k = fromJust (lookup cgroup (TK.tfreq (okind tkSpice)))
+            let n = fromJust (lookup cgroup (TK.tfreq (okind cotile tk)))
+                k = fromJust (lookup cgroup (TK.tfreq (okind cotile tkSpice)))
                 oneIn = (n + k) `divUp` k
             in (EM.insert s (oneIn, tkSpice) mOneIn, EM.insert s tk m)
       legend = ES.foldr' getLegend (return (EM.empty, EM.empty)) symbols
   in legend
 
-ooverride :: Kind.COps -> [(Char, GroupName TileKind)]
-          -> Rnd ( EM.EnumMap Char (Int, Kind.Id TileKind)
-                 , EM.EnumMap Char (Kind.Id TileKind) )
-ooverride Kind.COps{cotile=Kind.Ops{opick, okind}} poverride =
+ooverride :: COps -> [(Char, GroupName TileKind)]
+          -> Rnd ( EM.EnumMap Char (Int, ContentId TileKind)
+                 , EM.EnumMap Char (ContentId TileKind) )
+ooverride COps{cotile} poverride =
   let getLegend (s, cgroup) acc = do
         (mOneIn, m) <- acc
-        mtkSpice <- opick cgroup (Tile.kindHasFeature TK.Spice)
+        mtkSpice <- opick cotile cgroup (Tile.kindHasFeature TK.Spice)
         tk <- fromMaybe (error $ "" `showFailure` (s, cgroup, poverride))
-              <$> opick cgroup (not . Tile.kindHasFeature TK.Spice)
+              <$> opick cotile cgroup (not . Tile.kindHasFeature TK.Spice)
         return $! case mtkSpice of
           Nothing -> (mOneIn, EM.insert s tk m)
           Just tkSpice ->
-            let n = fromJust (lookup cgroup (TK.tfreq (okind tk)))
-                k = fromJust (lookup cgroup (TK.tfreq (okind tkSpice)))
+            let n = fromJust (lookup cgroup (TK.tfreq (okind cotile tk)))
+                k = fromJust (lookup cgroup (TK.tfreq (okind cotile tkSpice)))
                 oneIn = (n + k) `divUp` k
             in (EM.insert s (oneIn, tkSpice) mOneIn, EM.insert s tk m)
   in foldr getLegend (return (EM.empty, EM.empty)) poverride
 
 -- | Construct a fence around an area, with the given tile kind.
-buildFence :: Kind.Id TileKind -> Area -> TileMapEM
+buildFence :: ContentId TileKind -> Area -> TileMapEM
 buildFence fenceId area =
   let (x0, y0, x1, y1) = fromArea area
   in EM.fromList $ [ (Point x y, fenceId)
@@ -249,15 +248,15 @@ buildFence fenceId area =
                    | x <- [x0-1..x1+1], y <- [y0-1, y1+1] ]
 
 -- | Construct a fence around an area, with the given tile group.
-buildFenceRnd :: Kind.COps -> GroupName TileKind -> Area -> Rnd TileMapEM
-buildFenceRnd Kind.COps{cotile=Kind.Ops{opick}} couterFenceTile area = do
+buildFenceRnd :: COps -> GroupName TileKind -> Area -> Rnd TileMapEM
+buildFenceRnd COps{cotile} couterFenceTile area = do
   let (x0, y0, x1, y1) = fromArea area
       fenceIdRnd (xf, yf) = do
         let isCorner x y = x `elem` [x0-1, x1+1] && y `elem` [y0-1, y1+1]
             tileGroup | isCorner xf yf = "basic outer fence"
                       | otherwise = couterFenceTile
         fenceId <- fromMaybe (error $ "" `showFailure` tileGroup)
-                   <$> opick tileGroup (const True)
+                   <$> opick cotile tileGroup (const True)
         return (Point xf yf, fenceId)
       pointList = [ (x, y) | x <- [x0-1, x1+1], y <- [y0..y1] ]
                   ++ [ (x, y) | x <- [x0-1..x1+1], y <- [y0-1, y1+1] ]
