@@ -1,4 +1,3 @@
-{-# LANGUAGE GADTs #-}
 -- | Semantics of requests
 -- .
 -- A couple of them do not take time, the rest does.
@@ -10,12 +9,12 @@
 -- influence the outcome of the evaluation.
 module Game.LambdaHack.Server.HandleRequestM
   ( handleRequestAI, handleRequestUI, handleRequestTimed, switchLeader
-  , reqMove, reqDisplace, reqGameExit
+  , reqMove, reqDisplace, reqAlterFail, reqGameDropAndExit, reqGameSaveAndExit
 #ifdef EXPOSE_INTERNAL
     -- * Internal operations
-  , setBWait, managePerRequest, handleRequestTimedCases
-  , affectSmell, reqMelee, reqAlter, reqWait
-  , reqMoveItems, reqMoveItem, computeRndTimeout, reqProject, reqApply
+  , execFailure, setBWait, managePerRequest, handleRequestTimedCases
+  , affectSmell, reqMelee, reqMeleeChecked, reqAlter
+  , reqWait, reqMoveItems, reqMoveItem, computeRndTimeout, reqProject, reqApply
   , reqGameRestart, reqGameSave, reqTactic, reqAutomate
 #endif
   ) where
@@ -25,15 +24,19 @@ import Prelude ()
 import Game.LambdaHack.Common.Prelude
 
 import qualified Data.EnumMap.Strict as EM
+import qualified Data.Text as T
+import qualified Text.Show.Pretty as Show.Pretty
 
 import           Game.LambdaHack.Atomic
-import           Game.LambdaHack.Client
+import           Game.LambdaHack.Client (ReqAI (..), ReqUI (..),
+                                         RequestTimed (..))
 import qualified Game.LambdaHack.Common.Ability as Ability
 import           Game.LambdaHack.Common.Actor
 import           Game.LambdaHack.Common.ActorState
 import           Game.LambdaHack.Common.Faction
 import           Game.LambdaHack.Common.Item
-import qualified Game.LambdaHack.Common.Kind as Kind
+import qualified Game.LambdaHack.Common.ItemAspect as IA
+import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Level
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.MonadStateRead
@@ -55,11 +58,31 @@ import           Game.LambdaHack.Server.PeriodicM
 import           Game.LambdaHack.Server.ServerOptions
 import           Game.LambdaHack.Server.State
 
+execFailure :: MonadServerAtomic m
+            => ActorId -> RequestTimed -> ReqFailure -> m ()
+execFailure aid req failureSer = do
+  -- Clients should rarely do that (only in case of invisible actors)
+  -- so we report it to the client, but do not crash
+  -- (server should work OK with stupid clients, too).
+  body <- getsState $ getActorBody aid
+  let fid = bfid body
+      msg = showReqFailure failureSer
+      impossible = impossibleReqFailure failureSer
+      debugShow :: Show a => a -> Text
+      debugShow = T.pack . Show.Pretty.ppShow
+      possiblyAlarm = if impossible
+                      then debugPossiblyPrintAndExit
+                      else debugPossiblyPrint
+  possiblyAlarm $
+    "execFailure:" <+> msg <> "\n"
+    <> debugShow body <> "\n" <> debugShow req <> "\n" <> debugShow failureSer
+  execSfxAtomic $ SfxMsgFid fid $ SfxUnexpected failureSer
+
 -- | The semantics of server commands.
 -- AI always takes time and so doesn't loop.
 handleRequestAI :: MonadServerAtomic m
                 => ReqAI
-                -> m (Maybe RequestAnyAbility)
+                -> m (Maybe RequestTimed)
 handleRequestAI cmd = case cmd of
   ReqAITimed cmdT -> return $ Just cmdT
   ReqAINop -> return Nothing
@@ -67,11 +90,12 @@ handleRequestAI cmd = case cmd of
 -- | The semantics of server commands. Only the first two cases affect time.
 handleRequestUI :: MonadServerAtomic m
                 => FactionId -> ActorId -> ReqUI
-                -> m (Maybe RequestAnyAbility)
+                -> m (Maybe RequestTimed)
 handleRequestUI fid aid cmd = case cmd of
   ReqUITimed cmdT -> return $ Just cmdT
   ReqUIGameRestart t d -> reqGameRestart aid t d >> return Nothing
-  ReqUIGameExit -> reqGameExit aid >> return Nothing
+  ReqUIGameDropAndExit -> reqGameDropAndExit aid >> return Nothing
+  ReqUIGameSaveAndExit -> reqGameSaveAndExit aid >> return Nothing
   ReqUIGameSave -> reqGameSave >> return Nothing
   ReqUITactic toT -> reqTactic fid toT >> return Nothing
   ReqUIAutomate -> reqAutomate fid >> return Nothing
@@ -81,7 +105,7 @@ handleRequestUI fid aid cmd = case cmd of
 -- and unsetting in all other requests, we call this once before
 -- executing a request.
 setBWait :: MonadServerAtomic m
-         => RequestTimed a -> ActorId -> Actor -> m (Maybe Bool)
+         => RequestTimed -> ActorId -> Actor -> m (Maybe Bool)
 {-# INLINE setBWait #-}
 setBWait cmd aid b = do
   let mwait = case cmd of
@@ -93,7 +117,7 @@ setBWait cmd aid b = do
   return mwait
 
 handleRequestTimed :: MonadServerAtomic m
-                   => FactionId -> ActorId -> RequestTimed a -> m Bool
+                   => FactionId -> ActorId -> RequestTimed -> m Bool
 handleRequestTimed fid aid cmd = do
   b <- getsState $ getActorBody aid
   mwait <- setBWait cmd aid b
@@ -103,7 +127,7 @@ handleRequestTimed fid aid cmd = do
   -- benefit of seeing approaching danger up to almost a turn faster.
   -- It may be too late to block then, but not too late to sidestep or attack.
   unless (mwait == Just True) $ overheadActorTime fid (blid b)
-  advanceTime aid (if mwait == Just False then 10 else 100)
+  advanceTime aid (if mwait == Just False then 10 else 100) True
   handleRequestTimedCases aid cmd
   managePerRequest aid
   return $! isNothing mwait  -- for speed, we report if @cmd@ harmless
@@ -121,7 +145,7 @@ managePerRequest aid = do
     execUpdAtomic $ UpdRefillHP aid clearMark
 
 handleRequestTimedCases :: MonadServerAtomic m
-                        => ActorId -> RequestTimed a -> m ()
+                        => ActorId -> RequestTimed -> m ()
 handleRequestTimedCases aid cmd = case cmd of
   ReqMove target -> reqMove aid target
   ReqMelee target iid cstore -> reqMelee aid target iid cstore
@@ -183,7 +207,7 @@ affectSmell aid = do
   unless (bproj b) $ do
     fact <- getsState $ (EM.! bfid b) . sfactionD
     ar <- getsState $ getActorAspect aid
-    let smellRadius = aSmell ar
+    let smellRadius = IA.aSmell ar
     when (fhasGender (gplayer fact) || smellRadius > 0) $ do
       localTime <- getsState $ getLocalTime $ blid b
       lvl <- getLevel $ blid b
@@ -203,35 +227,73 @@ affectSmell aid = do
 -- to check if melee attack does not try to reach to a distant tile.
 reqMove :: MonadServerAtomic m => ActorId -> Vector -> m ()
 reqMove source dir = do
-  Kind.COps{coTileSpeedup} <- getsState scops
+  COps{coTileSpeedup} <- getsState scops
+  actorSk <- currentSkillsServer source
   sb <- getsState $ getActorBody source
-  let lid = blid sb
+  let abInSkill ab = isJust (btrajectory sb)
+                     || EM.findWithDefault 0 ab actorSk > 0
+      lid = blid sb
   lvl <- getLevel lid
   let spos = bpos sb           -- source position
       tpos = spos `shift` dir  -- target position
+  -- This predicate is symmetric wrt source and target, though the effect
+  -- of collision may not be (the source projectiles applies its effect
+  -- on the target particles, but loses 1 HP due to the collision).
+  -- The condision implies that it's impossible to shoot down a bullet
+  -- with a bullet, but a bullet can shoot down a burstable target,
+  -- as well as be swept away by it, and two burstable projectiles
+  -- burst when meeting mid-air. Projectiles that are not bursting
+  -- nor damaging never collide with any projectile.
+  collides <- getsState $ \s tb ->
+    let sitemKind = getIidKindServer (btrunk sb) s
+        titemKind = getIidKindServer (btrunk tb) s
+        -- Such projectiles are prone to bursitng or are themselves
+        -- particles of an explosion shockwave.
+        bursting itemKind = IK.Fragile `elem` IK.ifeature itemKind
+                            && IK.Lobable `elem` IK.ifeature itemKind
+        sbursting = bursting sitemKind
+        tbursting = bursting titemKind
+        -- Such projectiles, even if not bursting themselves, can cause
+        -- another projectile to burst.
+        damaging itemKind = IK.idamage itemKind /= 0
+        sdamaging = damaging sitemKind
+        tdamaging = damaging titemKind
+        -- Avoid explosion extinguishing itself via its own particles colliding.
+        sameBlast = IK.isBlast sitemKind
+                    && getIidKindIdServer (btrunk sb) s
+                       == getIidKindIdServer (btrunk tb) s
+    in not sameBlast
+       && (sbursting && (tdamaging || tbursting)
+           || (tbursting && (sdamaging || sbursting)))
   -- We start by checking actors at the target position.
   tgt <- getsState $ posToAssocs tpos lid
   case tgt of
-    (target, tb) : _ | not (bproj sb && bproj tb) -> do  -- visible or not
-      -- Projectiles are too small to hit each other.
-      -- Here the only weapon of projectiles is picked, too.
+    (target, tb) : _ | not (bproj sb) || not (bproj tb) || collides tb -> do
+      -- A projectile is too small and insubstantial to hit another projectile,
+      -- unless it's large enough or tends to explode (fragile and lobable).
+      -- The actor in the way is visible or not; server sees him always.
+      -- Below the only weapon (the only item) of projectiles is picked.
       mweapon <- pickWeaponServer source
       case mweapon of
-        Nothing -> reqWait source
-        Just (wp, cstore) -> reqMelee source target wp cstore
-    _
-      | Tile.isWalkable coTileSpeedup $ lvl `at` tpos -> do
-          -- Movement requires full access.
+        Just (wp, cstore) | abInSkill Ability.AbMelee ->
+          reqMeleeChecked source target wp cstore
+        _ -> return ()  -- waiting, even if no @AbWait@ ability
+    _ -> do
+      -- Either the position is empty, or all involved actors are proj.
+      -- Movement requires full access and skill.
+      if Tile.isWalkable coTileSpeedup $ lvl `at` tpos then
+        if abInSkill Ability.AbMove then do
           execUpdAtomic $ UpdMoveActor source spos tpos
           affectSmell source
-      | otherwise ->
-          -- Client foolishly tries to move into blocked, boring tile.
-          execFailure source (ReqMove dir) MoveNothing
+        else execFailure source (ReqMove dir) MoveUnskilled
+      else
+        -- Client foolishly tries to move into unwalkable tile.
+        execFailure source (ReqMove dir) MoveNothing
 
 -- * ReqMelee
 
 -- | Resolves the result of an actor moving into another.
--- Actors on blocked positions can be attacked without any restrictions.
+-- Actors on unwalkable positions can be attacked without any restrictions.
 -- For instance, an actor embedded in a wall can be attacked from
 -- an adjacent position. This function is analogous to projectGroupItem,
 -- but for melee and not using up the weapon.
@@ -240,20 +302,33 @@ reqMove source dir = do
 reqMelee :: MonadServerAtomic m
          => ActorId -> ActorId -> ItemId -> CStore -> m ()
 reqMelee source target iid cstore = do
+  actorSk <- currentSkillsServer source
+  if EM.findWithDefault 0 Ability.AbMelee actorSk > 0 then
+    reqMeleeChecked source target iid cstore
+  else execFailure source (ReqMelee target iid cstore) MeleeUnskilled
+
+reqMeleeChecked :: MonadServerAtomic m
+                => ActorId -> ActorId -> ItemId -> CStore -> m ()
+reqMeleeChecked source target iid cstore = do
   sb <- getsState $ getActorBody source
   tb <- getsState $ getActorBody target
-  let adj = checkAdjacent sb tb
-      req = ReqMelee target iid cstore
+  let req = ReqMelee target iid cstore
   if source == target then execFailure source req MeleeSelf
-  else if not adj then execFailure source req MeleeDistant
+  else if not (checkAdjacent sb tb) then execFailure source req MeleeDistant
   else do
     let sfid = bfid sb
         tfid = bfid tb
+        -- Let the missile drop down, but don't remove its trajectory
+        -- so that it doesn't pretend to have hit a wall.
+        haltProjectile aid b = case btrajectory b of
+          btra@(Just (l, speed)) | not $ null l ->
+            execUpdAtomic $ UpdTrajectory aid btra $ Just ([], speed)
+          _ -> return ()
     sfact <- getsState $ (EM.! sfid) . sfactionD
-    ttrunk <- getsState $ getItemBody $ btrunk tb
+    itemKind <- getsState $ getIidKindServer $ btrunk tb
     -- Only catch with appendages, never with weapons. Never steal trunk
     -- from an already caught projectile or one with many items inside.
-    if bproj tb && length (beqp tb) == 1 && jweight ttrunk > 1  -- not a blast
+    if bproj tb && EM.size (beqp tb) == 1 && not (IK.isBlast itemKind)
        && cstore == COrgan then do
       -- Catching the projectile, that is, stealing the item from its eqp.
       -- No effect from our weapon (organ) is applied to the projectile
@@ -273,42 +348,59 @@ reqMelee source target iid cstore = do
           upds <- generalMoveItem True iid2 k (CActor target CEqp)
                                               (CActor source CInv)
           mapM_ execUpdAtomic upds
+          itemFull <- getsState $ itemToFull iid2
+          discoverIfMinorEffects (CActor source CInv) iid2 (itemKindId itemFull)
         err -> error $ "" `showFailure` err
-      -- Let the caught missile vanish, but don't remove its trajectory
-      -- so that it doesn't pretend to be a non-projectile.
-      execUpdAtomic $ UpdTrajectory target (btrajectory tb)
-                                           (Just ([], toSpeed 0))
+      haltProjectile target tb
     else do
-      -- Normal hit, with effects. Msgs inside @SfxStrike@ describe
-      -- the source part of the strike.
-      execSfxAtomic $ SfxStrike source target iid cstore
-      let c = CActor source cstore
-      -- Msgs inside @itemEffect@ describe the target part of the strike.
-      -- If any effects and aspects, this is also where they are identified.
-      -- Here also the melee damage is applied, before any effects are.
-      meleeEffectAndDestroy source target iid c
+      if bproj sb && bproj tb then do
+        -- Special case for collision of projectiles, because they just
+        -- symmetrically ram into each other, so picking one to hit another,
+        -- based on random timing, would be wrong.
+        -- Instead of suffering melee attack, let the target projectile
+        -- get smashed and burst (if fragile and if not piercing).
+        -- The source projectile terminates flight (unless pierces) later on.
+        when (bhp tb > oneM) $
+          execUpdAtomic $ UpdRefillHP target minusM
+        when (bhp tb <= oneM) $
+          -- If projectile has too low HP to pierce, terminate its flight.
+          haltProjectile target tb
+      else do
+        -- Normal hit, with effects. Msgs inside @SfxStrike@ describe
+        -- the source part of the strike.
+        execSfxAtomic $ SfxStrike source target iid cstore
+        let c = CActor source cstore
+        -- Msgs inside @itemEffect@ describe the target part of the strike.
+        -- If any effects and aspects, this is also where they are identified.
+        -- Here also the melee damage is applied, before any effects are.
+        meleeEffectAndDestroy source target iid c
       sb2 <- getsState $ getActorBody source
       case btrajectory sb2 of
-        Just (tra, speed) | not $ null tra -> do
+        Just (tra, _speed) | not (null tra) -> do
           -- Deduct a hitpoint for a pierce of a projectile
           -- or due to a hurled actor colliding with another.
           -- Don't deduct if no pierce, to prevent spam.
-          when (not (bproj sb2) || bhp sb2 > oneM) $
-            refillHP False source sb2 minusM
+          -- Never kill in this way.
+          when (bhp sb2 > oneM) $ do
+            execUpdAtomic $ UpdRefillHP source minusM
+            unless (bproj sb2) $ do
+              execSfxAtomic $
+                SfxMsgFid (bfid sb2) $ SfxCollideActor (blid tb) source target
+              unless (bproj tb) $
+                execSfxAtomic $
+                  SfxMsgFid (bfid tb) $ SfxCollideActor (blid tb) source target
           when (not (bproj sb2) || bhp sb2 <= oneM) $
             -- Non-projectiles can't pierce, so terminate their flight.
             -- If projectile has too low HP to pierce, ditto.
-            execUpdAtomic
-            $ UpdTrajectory source (btrajectory sb2) (Just ([], speed))
+             haltProjectile source sb2
         _ -> return ()
       -- The only way to start a war is to slap an enemy. Being hit by
       -- and hitting projectiles count as unintentional friendly fire.
       let friendlyFire = bproj sb2 || bproj tb
           fromDipl = EM.findWithDefault Unknown tfid (gdipl sfact)
       unless (friendlyFire
-              || isAtWar sfact tfid  -- already at war
-              || isAllied sfact tfid  -- allies never at war
-              || sfid == tfid) $
+              || isFoe sfid sfact tfid  -- already at war
+              || isFriend sfid sfact tfid) $  -- allies never at war
         execUpdAtomic $ UpdDiplFaction sfid tfid fromDipl War
 
 -- * ReqDisplace
@@ -316,17 +408,21 @@ reqMelee source target iid cstore = do
 -- | Actor tries to swap positions with another.
 reqDisplace :: MonadServerAtomic m => ActorId -> ActorId -> m ()
 reqDisplace source target = do
-  Kind.COps{coTileSpeedup} <- getsState scops
+  COps{coTileSpeedup} <- getsState scops
+  actorSk <- currentSkillsServer source
   sb <- getsState $ getActorBody source
+  let abInSkill ab = isJust (btrajectory sb)
+                     || EM.findWithDefault 0 ab actorSk > 0
   tb <- getsState $ getActorBody target
   tfact <- getsState $ (EM.! bfid tb) . sfactionD
   let tpos = bpos tb
-      adj = checkAdjacent sb tb
-      atWar = isAtWar tfact (bfid sb)
+      atWar = isFoe (bfid tb) tfact (bfid sb)
       req = ReqDisplace target
   ar <- getsState $ getActorAspect target
-  dEnemy <- getsState $ dispEnemy source target $ aSkills ar
-  if | not adj -> execFailure source req DisplaceDistant
+  dEnemy <- getsState $ dispEnemy source target $ IA.aSkills ar
+  if | not (abInSkill Ability.AbDisplace) ->
+         execFailure source req DisplaceUnskilled
+     | not (checkAdjacent sb tb) -> execFailure source req DisplaceDistant
      | atWar && not dEnemy -> do  -- if not at war, can displace always
        -- We don't fail with DisplaceImmobile and DisplaceSupported.
        -- because it's quite common they can't be determined by the attacker,
@@ -336,8 +432,9 @@ reqDisplace source target = do
        -- verify here that they don't occur, for simplicity.
        mweapon <- pickWeaponServer source
        case mweapon of
-         Nothing -> reqWait source
-         Just (wp, cstore) -> reqMelee source target wp cstore
+         Just (wp, cstore) | abInSkill Ability.AbMelee ->
+           reqMeleeChecked source target wp cstore
+         _ -> return ()  -- waiting, even if no @AbWait@ ability
      | otherwise -> do
        let lid = blid sb
        lvl <- getLevel lid
@@ -362,29 +459,56 @@ reqDisplace source target = do
 -- | Search and/or alter the tile.
 reqAlter :: MonadServerAtomic m => ActorId -> Point -> m ()
 reqAlter source tpos = do
-  Kind.COps{ cotile=cotile@Kind.Ops{okind, opick}
-           , coTileSpeedup } <- getsState scops
+  mfail <- reqAlterFail source tpos
+  let req = ReqAlter tpos
+  maybe (return ()) (execFailure source req) mfail
+
+reqAlterFail :: MonadServerAtomic m => ActorId -> Point -> m (Maybe ReqFailure)
+reqAlterFail source tpos = do
+  COps{cotile, coTileSpeedup} <- getsState scops
   sb <- getsState $ getActorBody source
-  sClient <- getsServer $ (EM.! bfid sb) . sclientStates
-  actorSk <- currentSkillsServer source
-  let alterSkill = EM.findWithDefault 0 Ability.AbAlter actorSk
+  ar <- getsState $ getActorAspect source
+  let calmE = calmEnough sb ar
       lid = blid sb
-      req = ReqAlter tpos
+  sClient <- getsServer $ (EM.! bfid sb) . sclientStates
+  itemToF <- getsState $ flip itemToFull
+  actorSk <- currentSkillsServer source
+  localTime <- getsState $ getLocalTime lid
+  let alterSkill = EM.findWithDefault 0 Ability.AbAlter actorSk
+      applySkill = EM.findWithDefault 0 Ability.AbApply actorSk
   embeds <- getsState $ getEmbedBag lid tpos
   lvl <- getLevel lid
   let serverTile = lvl `at` tpos
       lvlClient = (EM.! lid) . sdungeon $ sClient
       clientTile = lvlClient `at` tpos
       hiddenTile = Tile.hideAs cotile serverTile
-      revealEmbed = unless (null embeds) $ do
+      revealEmbeds = unless (EM.null embeds) $ do
         s <- getState
         let ais = map (\iid -> (iid, getItemBody iid s)) (EM.keys embeds)
         execUpdAtomic $ UpdSpotItemBag (CEmbed lid tpos) embeds ais
-  if not $ adjacent (bpos sb) tpos then execFailure source req AlterDistant
+      tryApplyEmbeds = do
+        -- Can't send @SfxTrigger@ afterwards, because actor may be moved
+        -- by the embeds to another level, where @tpos@ is meaningless.
+        execSfxAtomic $ SfxTrigger source tpos
+        mapM_ tryApplyEmbed $ EM.assocs embeds
+      tryApplyEmbed (iid, kit) = do
+        let itemFull@ItemFull{itemKind} = itemToF iid
+            legal = permittedApply localTime applySkill calmE itemFull kit
+        -- Let even completely unskilled actors trigger basic embeds.
+        case legal of
+          Left ApplyNoEffects -> return ()  -- pure flavour embed
+          Left reqFail | reqFail `notElem` [ApplyUnskilled, NotCalmPrecious] ->
+            -- The failure is fully expected, because client may choose
+            -- to trigger some embeds, knowing that others won't fire.
+            execSfxAtomic $ SfxMsgFid (bfid sb)
+            $ SfxExpected ("embedded" <+> IK.iname itemKind) reqFail
+          _ -> itemEffectEmbedded source lid tpos iid
+  if chessDist tpos (bpos sb) > 1
+  then return $ Just AlterDistant
   else if Just clientTile == hiddenTile then  -- searches
     -- Only actors with AbAlter > 1 can search for hidden doors, etc.
     if alterSkill <= 1
-    then execFailure source req AlterUnskilled  -- don't leak about searching
+    then return $ Just AlterUnskilled  -- don't leak about searching
     else do
       -- Blocking by items nor actors does not prevent searching.
       -- Searching broadcasted, in case actors from other factions are present
@@ -396,18 +520,19 @@ reqAlter source tpos = do
       -- If the items are already seen by the client
       -- (e.g., due to item detection, despite tile being still hidden),
       -- the command is ignored on the client.
-      revealEmbed
+      revealEmbeds
       -- Seaching triggers the embeds as well, after they are revealed.
       -- The rationale is that the items were all the time present
       -- (just invisible to the client), so they need to be triggered.
       -- The exception is changable tiles, because they are not so easy
       -- to trigger; they need subsequent altering.
       unless (Tile.isDoor coTileSpeedup serverTile
-              || Tile.isChangable coTileSpeedup serverTile) $
-        itemEffectEmbedded source tpos embeds
+              || Tile.isChangable coTileSpeedup serverTile)
+        tryApplyEmbeds
+      return Nothing  -- success
   else if clientTile == serverTile then  -- alters
     if alterSkill < Tile.alterMinSkill coTileSpeedup serverTile
-    then execFailure source req AlterUnskilled  -- don't leak about altering
+    then return $ Just AlterUnskilled  -- don't leak about altering
     else do
       let changeTo tgroup = do
             lvl2 <- getLevel lid
@@ -418,13 +543,13 @@ reqAlter source tpos = do
                                     (Tile.kindHasFeature TK.Dark kt)
             -- Sometimes the tile is determined precisely by the ambient light
             -- of the source tiles. If not, default to cave day/night condition.
-            mtoTile <- rndToAction $ opick tgroup nightCond
+            mtoTile <- rndToAction $ opick cotile tgroup nightCond
             toTile <- maybe (rndToAction
                              $ fromMaybe (error $ "" `showFailure` tgroup)
-                               <$> opick tgroup (const True))
+                               <$> opick cotile tgroup (const True))
                             return
                             mtoTile
-            unless (toTile == serverTile) $ do
+            unless (toTile == serverTile) $ do  -- don't regenerate same tile
               -- At most one of these two will be accepted on any given client.
               execUpdAtomic $ UpdAlterTile lid tpos serverTile toTile
               -- This case happens when a client does not see a searching
@@ -438,22 +563,22 @@ reqAlter source tpos = do
                 (False, True) -> execUpdAtomic $ UpdAlterExplorable lid 1
                 (True, False) -> execUpdAtomic $ UpdAlterExplorable lid (-1)
                 _ -> return ()
-            -- At the end we replace old embeds (even if partially used up)
-            -- with new ones.
-            -- If the source tile was hidden, the items could not be visible
-            -- on a client, in which case the command would be ignored
-            -- on the client, without causing any problems. Otherwise,
-            -- if the position is in view, client has accurate info.
-            case EM.lookup tpos (lembed lvl2) of
-              Just bag -> do
-                s <- getState
-                let ais = map (\iid -> (iid, getItemBody iid s)) (EM.keys bag)
-                execUpdAtomic $ UpdLoseItemBag (CEmbed lid tpos) bag ais
-              Nothing -> return ()
-            -- Altering always reveals the outcome tile, so it's not hidden
-            -- and so its embedded items are always visible.
-            embedItem lid tpos toTile
-          feats = TK.tfeature $ okind serverTile
+              -- At the end we replace old embeds (even if partially used up)
+              -- with new ones.
+              -- If the source tile was hidden, the items could not be visible
+              -- on a client, in which case the command would be ignored
+              -- on the client, without causing any problems. Otherwise,
+              -- if the position is in view, client has accurate info.
+              case EM.lookup tpos (lembed lvl2) of
+                Just bag -> do
+                  s <- getState
+                  let ais = map (\iid -> (iid, getItemBody iid s)) (EM.keys bag)
+                  execUpdAtomic $ UpdLoseItemBag (CEmbed lid tpos) bag ais
+                Nothing -> return ()
+              -- Altering always reveals the outcome tile, so it's not hidden
+              -- and so its embedded items are always visible.
+              embedItem lid tpos toTile
+          feats = TK.tfeature $ okind cotile serverTile
           toAlter feat =
             case feat of
               TK.OpenTo tgroup -> Just tgroup
@@ -461,9 +586,8 @@ reqAlter source tpos = do
               TK.ChangeTo tgroup -> Just tgroup
               _ -> Nothing
           groupsToAlterTo = mapMaybe toAlter feats
-      if null groupsToAlterTo && null embeds then
-        -- No altering possible; silly client.
-        execFailure source req AlterNothing
+      if null groupsToAlterTo && EM.null embeds then
+        return $ Just AlterNothing  -- no altering possible; silly client
       else
         if EM.notMember tpos $ lfloor lvl then
           if null (posToAidsLvl tpos lvl) then do
@@ -474,16 +598,17 @@ reqAlter source tpos = do
             -- The items are first revealed for the sake of clients that
             -- may see the tile as hidden. Note that the tile is not revealed
             -- (unless it's altered later on, in which case the new one is).
-            revealEmbed
-            itemEffectEmbedded source tpos embeds
+            revealEmbeds
+            tryApplyEmbeds
             case groupsToAlterTo of
               [] -> return ()
               [groupToAlterTo] -> changeTo groupToAlterTo
               l -> error $ "tile changeable in many ways" `showFailure` l
-          else execFailure source req AlterBlockActor
-        else execFailure source req AlterBlockItem
+            return Nothing  -- success
+          else return $ Just AlterBlockActor
+        else return $ Just AlterBlockItem
   else  -- client is misguided re tile at that position, so bail out
-    execFailure source req AlterNothing
+    return $ Just AlterNothing
 
 -- * ReqWait
 
@@ -492,19 +617,25 @@ reqAlter source tpos = do
 -- Something is sometimes done in 'setBWait'.
 reqWait :: MonadServerAtomic m => ActorId -> m ()
 {-# INLINE reqWait #-}
-reqWait _ = return ()
+reqWait source = do
+  actorSk <- currentSkillsServer source
+  unless (EM.findWithDefault 0 Ability.AbWait actorSk > 0) $
+    execFailure source ReqWait WaitUnskilled
 
 -- * ReqMoveItems
 
 reqMoveItems :: MonadServerAtomic m
              => ActorId -> [(ItemId, Int, CStore, CStore)] -> m ()
-reqMoveItems aid l = do
-  b <- getsState $ getActorBody aid
-  ar <- getsState $ getActorAspect aid
-  -- Server accepts item movement based on calm at the start, not end
-  -- or in the middle, to avoid interrupted or partially ignored commands.
-  let calmE = calmEnough b ar
-  mapM_ (reqMoveItem aid calmE) l
+reqMoveItems source l = do
+  actorSk <- currentSkillsServer source
+  if EM.findWithDefault 0 Ability.AbMoveItem actorSk > 0 then do
+    b <- getsState $ getActorBody source
+    ar <- getsState $ getActorAspect source
+    -- Server accepts item movement based on calm at the start, not end
+    -- or in the middle, to avoid interrupted or partially ignored commands.
+    let calmE = calmEnough b ar
+    mapM_ (reqMoveItem source calmE) l
+  else execFailure source (ReqMoveItems l) MoveItemUnskilled
 
 reqMoveItem :: MonadServerAtomic m
             => ActorId -> Bool -> (ItemId, Int, CStore, CStore) -> m ()
@@ -523,11 +654,11 @@ reqMoveItem aid calmE (iid, k, fromCStore, toCStore) = do
    | (fromCStore == CSha || toCStore == CSha) && not calmE ->
      execFailure aid req ItemNotCalm
    | otherwise -> do
-    itemToF <- getsState itemToFull
-    let itemFull = itemToF iid (k, [])
-    when (fromCStore == CGround) $ discoverIfNoEffects fromC iid itemFull
     upds <- generalMoveItem True iid k fromC toC
     mapM_ execUpdAtomic upds
+    itemFull <- getsState $ itemToFull iid
+    when (fromCStore == CGround) $  -- pick up
+      discoverIfMinorEffects toC iid (itemKindId itemFull)
     -- Reset timeout for equipped periodic items and also for items
     -- moved out of the shared stash, in which timeouts are not consistently
     -- wrt some local time, because actors from many levels put items there
@@ -543,7 +674,7 @@ reqMoveItem aid calmE (iid, k, fromCStore, toCStore) = do
       localTime <- getsState $ getLocalTime (blid b)
       -- The first recharging period after pick up is random,
       -- between 1 and 2 standard timeouts of the item.
-      mrndTimeout <- rndToAction $ computeRndTimeout localTime iid itemFull
+      mrndTimeout <- rndToAction $ computeRndTimeout localTime itemFull
       let beforeIt = case iid `EM.lookup` bagBefore of
             Nothing -> []  -- no such items before move
             Just (_, it2) -> it2
@@ -562,17 +693,14 @@ reqMoveItem aid calmE (iid, k, fromCStore, toCStore) = do
             execUpdAtomic $ UpdTimeItem iid toC afterIt resetIt
         Nothing -> return ()  -- no Periodic or Timeout aspect; don't touch
 
-computeRndTimeout :: Time -> ItemId -> ItemFull -> Rnd (Maybe Time)
-computeRndTimeout localTime iid ItemFull{..}=
-  case itemDisco of
-    Just ItemDisco{itemKind, itemAspect=Just ar} ->
-      case aTimeout ar of
-        t | t /= 0 && IK.Periodic `elem` IK.ieffects itemKind -> do
-          rndT <- randomR (0, t)
-          let rndTurns = timeDeltaScale (Delta timeTurn) rndT
-          return $ Just $ timeShift localTime rndTurns
-        _ -> return Nothing
-    _ -> error $ "" `showFailure` iid
+computeRndTimeout :: Time -> ItemFull -> Rnd (Maybe Time)
+computeRndTimeout localTime ItemFull{itemKind, itemDisco} =
+  case IA.aTimeout $ itemAspect itemDisco of
+    t | t /= 0 && IK.Periodic `elem` IK.ifeature itemKind -> do
+      rndT <- randomR (0, t)
+      let rndTurns = timeDeltaScale (Delta timeTurn) (t + rndT)
+      return $ Just $ timeShift localTime rndTurns
+    _ -> return Nothing
 
 -- * ReqProject
 
@@ -590,7 +718,7 @@ reqProject source tpxy eps iid cstore = do
   let calmE = calmEnough b ar
   if cstore == CSha && not calmE then execFailure source req ItemNotCalm
   else do
-    mfail <- projectFail source tpxy eps iid cstore False
+    mfail <- projectFail source tpxy eps False iid cstore False
     maybe (return ()) (execFailure source req) mfail
 
 -- * ReqApply
@@ -611,12 +739,11 @@ reqApply aid iid cstore = do
     case EM.lookup iid bag of
       Nothing -> execFailure aid req ApplyOutOfReach
       Just kit -> do
-        itemToF <- getsState itemToFull
+        itemFull <- getsState $ itemToFull iid
         actorSk <- currentSkillsServer aid
         localTime <- getsState $ getLocalTime (blid b)
         let skill = EM.findWithDefault 0 Ability.AbApply actorSk
-            itemFull = itemToF iid kit
-            legal = permittedApply localTime skill calmE " " itemFull
+            legal = permittedApply localTime skill calmE itemFull kit
         case legal of
           Left reqFail -> execFailure aid req reqFail
           Right _ -> applyItem aid iid cstore
@@ -630,9 +757,8 @@ reqGameRestart aid groupName scurChalSer = do
   modifyServer $ \ser -> ser {soptionsNxt = (soptionsNxt ser) {scurChalSer}}
   b <- getsState $ getActorBody aid
   oldSt <- getsState $ gquit . (EM.! bfid b) . sfactionD
-  modifyServer $ \ser ->
-    ser { swriteSave = True  -- fake saving to abort turn
-        , squit = True }  -- do this at once
+  -- We don't save game and don't wait for clips end. ASAP.
+  modifyServer $ \ser -> ser {sbreakASAP = True}
   isNoConfirms <- isNoConfirmsGame
   -- This call to `revealItems` is really needed, because the other
   -- happens only at game conclusion, not at quitting.
@@ -640,23 +766,39 @@ reqGameRestart aid groupName scurChalSer = do
   execUpdAtomic $ UpdQuitFaction (bfid b) oldSt
                 $ Just $ Status Restart (fromEnum $ blid b) (Just groupName)
 
--- * ReqGameExit
+-- * ReqGameDropAndExit
 
-reqGameExit :: MonadServerAtomic m => ActorId -> m ()
-reqGameExit aid = do
+-- After we break out of the game loop, we will notice from @Camping@
+-- we shouldn exit the game.
+reqGameDropAndExit :: MonadServerAtomic m => ActorId -> m ()
+reqGameDropAndExit aid = do
   b <- getsState $ getActorBody aid
   oldSt <- getsState $ gquit . (EM.! bfid b) . sfactionD
-  modifyServer $ \ser -> ser { swriteSave = True
-                             , squit = True }  -- do this at once
+  modifyServer $ \ser -> ser {sbreakLoop = True}
+  execUpdAtomic $ UpdQuitFaction (bfid b) oldSt
+                $ Just $ Status Camping (fromEnum $ blid b) Nothing
+
+-- * ReqGameSaveAndExit
+
+-- After we break out of the game loop, we will notice from @Camping@
+-- we shouldn exit the game.
+reqGameSaveAndExit :: MonadServerAtomic m => ActorId -> m ()
+reqGameSaveAndExit aid = do
+  b <- getsState $ getActorBody aid
+  oldSt <- getsState $ gquit . (EM.! bfid b) . sfactionD
+  modifyServer $ \ser -> ser { sbreakASAP = True
+                             , swriteSave = True }
   execUpdAtomic $ UpdQuitFaction (bfid b) oldSt
                 $ Just $ Status Camping (fromEnum $ blid b) Nothing
 
 -- * ReqGameSave
 
+-- After we break out of the game loop, we will notice we shouldn't quit
+-- the game and we will enter the game loop again.
 reqGameSave :: MonadServer m => m ()
 reqGameSave =
-  modifyServer $ \ser -> ser { swriteSave = True
-                             , squit = True }  -- do this at once
+  modifyServer $ \ser -> ser { sbreakASAP = True
+                             , swriteSave = True }
 
 -- * ReqTactic
 
