@@ -33,12 +33,12 @@ import           System.IO (hFlush, stdout)
 import qualified System.Random as R
 
 import           Game.LambdaHack.Atomic
-import           Game.LambdaHack.Client
+import           Game.LambdaHack.Client (sbenchmark)
 import           Game.LambdaHack.Common.ActorState
 import           Game.LambdaHack.Common.Faction
 import           Game.LambdaHack.Common.File
 import qualified Game.LambdaHack.Common.HighScore as HighScore
-import qualified Game.LambdaHack.Common.Kind as Kind
+import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.MonadStateRead
 import           Game.LambdaHack.Common.Perception
@@ -118,11 +118,11 @@ dumpRngs rngs = liftIO $ do
   hFlush stdout
 
 -- | Read the high scores dictionary. Return the empty table if no file.
-restoreScore :: forall m. MonadServer m => Kind.COps -> m HighScore.ScoreDict
-restoreScore Kind.COps{corule} = do
+restoreScore :: forall m. MonadServer m => COps -> m HighScore.ScoreDict
+restoreScore cops = do
   bench <- getsServer $ sbenchmark . sclientOptions . soptions
   mscore <- if bench then return Nothing else do
-    let stdRuleset = Kind.stdRuleset corule
+    let stdRuleset = getStdRuleset cops
         scoresFile = rscoresFile stdRuleset
     dataDir <- liftIO appDataDir
     let path bkp = dataDir </> bkp <> scoresFile
@@ -149,23 +149,24 @@ restoreScore Kind.COps{corule} = do
 -- | Generate a new score, register it and save.
 registerScore :: MonadServer m => Status -> FactionId -> m ()
 registerScore status fid = do
-  cops@Kind.COps{corule} <- getsState scops
-  fact <- getsState $ (EM.! fid) . sfactionD
+  cops <- getsState scops
   total <- getsState $ snd . calculateTotal fid
-  let stdRuleset = Kind.stdRuleset corule
+  let stdRuleset = getStdRuleset cops
       scoresFile = rscoresFile stdRuleset
   dataDir <- liftIO appDataDir
   -- Re-read the table in case it's changed by a concurrent game.
   scoreDict <- restoreScore cops
   gameModeId <- getsState sgameModeId
   time <- getsState stime
+  dungeonTotal <- getsState sgold
   date <- liftIO getPOSIXTime
   tz <- liftIO $ getTimeZone $ posixSecondsToUTCTime date
   curChalSer <- getsServer $ scurChalSer . soptions
   factionD <- getsState sfactionD
   bench <- getsServer $ sbenchmark . sclientOptions . soptions
   noConfirmsGame <- isNoConfirmsGame
-  let path = dataDir </> scoresFile
+  let fact = factionD EM.! fid
+      path = dataDir </> scoresFile
       outputScore (worthMentioning, (ntable, pos)) =
         -- If testing or fooling around, dump instead of registering.
         -- In particular don't register score for the auto-* scenarios.
@@ -179,16 +180,16 @@ registerScore status fid = do
       chal | fhasUI $ gplayer fact = curChalSer
            | otherwise = curChalSer
                            {cdiff = difficultyInverse (cdiff curChalSer)}
-      theirVic (fi, fa) | isAtWar fact fi
+      theirVic (fi, fa) | isFoe fid fact fi
                           && not (isHorrorFact fa) = Just $ gvictims fa
                         | otherwise = Nothing
       theirVictims = EM.unionsWith (+) $ mapMaybe theirVic $ EM.assocs factionD
-      ourVic (fi, fa) | isAllied fact fi || fi == fid = Just $ gvictims fa
+      ourVic (fi, fa) | isFriend fid fact fi = Just $ gvictims fa
                       | otherwise = Nothing
       ourVictims = EM.unionsWith (+) $ mapMaybe ourVic $ EM.assocs factionD
       table = HighScore.getTable gameModeId scoreDict
       registeredScore =
-        HighScore.register table total time status date chal
+        HighScore.register table total dungeonTotal time status date chal
                            (T.unwords $ tail $ T.words $ gname fact)
                            ourVictims theirVictims
                            (fhiCondPoly $ gplayer fact)
@@ -197,10 +198,10 @@ registerScore status fid = do
 -- | Invoke pseudo-random computation with the generator kept in the state.
 rndToAction :: MonadServer m => Rnd a -> m a
 rndToAction r = do
-  gen <- getsServer srandom
-  let (gen1, gen2) = R.split gen
-  modifyServer $ \ser -> ser {srandom = gen1}
-  return $! St.evalState r gen2
+  gen1 <- getsServer srandom
+  let (a, gen2) = St.runState r gen1
+  modifyServer $ \ser -> ser {srandom = gen2}
+  return a
 
 -- | Gets a random generator from the user-submitted options or, if not present,
 -- generates one.
