@@ -30,10 +30,12 @@ import Game.LambdaHack.Server.ServerOptions
 -- basic game state, but not the server's basic state.
 data StateServer = StateServer
   { sactorTime    :: ActorTime      -- ^ absolute times of next actions
+  , sactorStasis  :: ES.EnumSet ActorId
+                                    -- ^ actors currently in time stasis,
+                                    --   invulnerable to time warps until move
   , sdiscoKindRev :: DiscoveryKindRev
                                     -- ^ reverse map, used for item creation
   , suniqueSet    :: UniqueSet      -- ^ already generated unique items
-  , sitemSeedD    :: ItemSeedDict   -- ^ map from item ids to item seeds
   , sitemRev      :: ItemRev        -- ^ reverse id map, used for item creation
   , sflavour      :: FlavourMap     -- ^ association of flavour to items
   , sacounter     :: ActorId        -- ^ stores next actor index
@@ -52,12 +54,14 @@ data StateServer = StateServer
   , svalidArenas  :: Bool           -- ^ whether active arenas valid
   , srandom       :: R.StdGen       -- ^ current random generator
   , srngs         :: RNGs           -- ^ initial random generators
-  , squit         :: Bool           -- ^ exit the game loop
-  , swriteSave    :: Bool           -- ^ write savegame to a file now
+  , sbreakLoop    :: Bool           -- ^ exit game loop after clip's end;
+                                    --   usually no game save follows
+  , sbreakASAP    :: Bool           -- ^ exit game loop ASAP; usually with save
+  , swriteSave    :: Bool           -- ^ write savegame to file after loop exit
   , soptions      :: ServerOptions  -- ^ current commandline options
   , soptionsNxt   :: ServerOptions  -- ^ options for the next game
   }
-  deriving (Show)
+  deriving Show
 
 -- | Position in time for each actor, grouped by level and by faction.
 type ActorTime =
@@ -68,9 +72,9 @@ emptyStateServer :: StateServer
 emptyStateServer =
   StateServer
     { sactorTime = EM.empty
-    , sdiscoKindRev = EM.empty
+    , sactorStasis = ES.empty
+    , sdiscoKindRev = emptyDiscoveryKindRev
     , suniqueSet = ES.empty
-    , sitemSeedD = EM.empty
     , sitemRev = HM.empty
     , sflavour = emptyFlavourMap
     , sacounter = toEnum 0
@@ -89,7 +93,8 @@ emptyStateServer =
     , srandom = R.mkStdGen 42
     , srngs = RNGs { dungeonRandomGenerator = Nothing
                    , startingRandomGenerator = Nothing }
-    , squit = False
+    , sbreakLoop = False
+    , sbreakASAP = False
     , swriteSave = False
     , soptions = defServerOptions
     , soptionsNxt = defServerOptions
@@ -108,9 +113,9 @@ ageActor !fid !lid !aid !delta =
 instance Binary StateServer where
   put StateServer{..} = do
     put sactorTime
+    put sactorStasis
     put sdiscoKindRev
     put suniqueSet
-    put sitemSeedD
     put sitemRev
     put sflavour
     put sacounter
@@ -122,9 +127,9 @@ instance Binary StateServer where
     put soptions
   get = do
     sactorTime <- get
+    sactorStasis <- get
     sdiscoKindRev <- get
     suniqueSet <- get
-    sitemSeedD <- get
     sitemRev <- get
     sflavour <- get
     sacounter <- get
@@ -144,7 +149,8 @@ instance Binary StateServer where
         sfovLitLid = EM.empty
         sarenas = []
         svalidArenas = False
-        squit = False
+        sbreakLoop = False
+        sbreakASAP = False
         swriteSave = False
         soptionsNxt = defServerOptions
     return $! StateServer{..}
