@@ -9,6 +9,7 @@ import Prelude ()
 import Game.LambdaHack.Common.Prelude
 
 import           Game.LambdaHack.Client.UI.Content.KeyKind
+import           Game.LambdaHack.Client.UI.HandleHelperM (ppSLore)
 import           Game.LambdaHack.Client.UI.HumanCmd
 import           Game.LambdaHack.Common.Misc
 import qualified Game.LambdaHack.Content.TileKind as TK
@@ -26,34 +27,71 @@ standardKeys = KeyKind $ map evalKeyDef $
   -- Remember to put commands that show information (e.g., enter aiming
   -- mode) first.
 
-  -- Main Menu
+  -- Main menu
   [ ("e", ([CmdMainMenu], "enter challenges menu>", ChallengesMenu))
   , ("s", ([CmdMainMenu], "start new game", GameRestart))
   , ("x", ([CmdMainMenu], "exit to desktop", GameExit))
   , ("v", ([CmdMainMenu], "visit settings menu>", SettingsMenu))
   , ("a", ([CmdMainMenu], "automate faction", Automate))
-  , ("?", ([CmdMainMenu], "see command Help", Help))
+  , ("?", ([CmdMainMenu], "see command help", Help))
+  , ("F12", ([CmdMainMenu], "go to dashboard", Dashboard))
   , ("Escape", ([CmdMainMenu], "back to playing", Cancel))
 
-  -- Item use, 1st part
+  -- Minimal command set, in the desired presentation order
   , ("g", addCmdCategory CmdMinimal $ grabItems "grab item(s)")
+      -- actually it's not necessary, ground items menu suffices
+  , ("Escape", ( [CmdMinimal, CmdAim]
+               , "cancel aiming/open main menu"
+               , ByAimMode { exploration = ExecuteIfClear MainMenu
+                           , aiming = Cancel } ))
+  , ("Return", ( [CmdMinimal, CmdAim]
+               , "accept target/open dashboard"
+               , ByAimMode { exploration = ExecuteIfClear Dashboard
+                           , aiming = Accept } ))
+  , ("space", ( [CmdMinimal, CmdMeta]
+              , "clear messages/display history"
+              , ExecuteIfClear History ))
+      -- not necessary, because messages available from dashboard
+  , ("BackTab", ( [CmdMinimal, CmdMove]
+              , "cycle among all party members"
+              , MemberBack ))
+  , ("KP_Multiply", ( [CmdMinimal, CmdAim]
+                    , "cycle x-hair among enemies"
+                    , AimEnemy ))
+      -- not necessary, because flinging from item menu enters aiming mode
+  , ("C-c", ([CmdMinimal, CmdMove], "open or close or alter", AlterDir []))
+  , ("+", ([CmdMinimal, CmdAim], "swerve the aiming line", EpsIncr True))
+
+  -- Item menu, first part of item use commands
   , ("comma", grabItems "")
   , ("d", dropItems "drop item(s)")
   , ("period", dropItems "")
   , ("f", addCmdCategory CmdItemMenu $ projectA flingTs)
   , ("C-f", addCmdCategory CmdItemMenu
             $ replaceDesc "fling without aiming" $ projectI flingTs)
-  , ("a", addCmdCategory CmdItemMenu $ applyI [ApplyItem
-            { verb = "apply"
-            , object = "consumable"
-            , symbol = ' ' }])
+  , ("a", addCmdCategory CmdItemMenu $ applyI [TriggerItem
+            { tiverb = "apply"
+            , tiobject = "consumable"
+            , tisymbols = "!?/" }])
   , ("C-a", addCmdCategory CmdItemMenu
-            $ replaceDesc "apply and keep choice" $ applyIK [ApplyItem
-              { verb = "apply"
-              , object = "consumable"
-              , symbol = ' ' }])
+            $ replaceDesc "apply and keep choice" $ applyIK [TriggerItem
+              { tiverb = "apply"
+              , tiobject = "consumable"
+              , tisymbols = "!?/" }])
+  , ("p", moveItemTriple [CGround, CEqp, CSha] CInv
+                         "item" False)
+  , ("e", moveItemTriple [CGround, CInv, CSha] CEqp
+                         "item" False)
+  , ("s", moveItemTriple [CGround, CInv, CEqp] CSha
+                         "and share item" False)
 
   -- Terrain exploration and alteration
+  , ("Tab", ( [CmdMove]
+            , "cycle among party members on the level"
+            , MemberCycle ))
+  , ("c", ([CmdMove], descTs closeDoorTriggers, AlterDir closeDoorTriggers))
+  , ("=", ( [CmdMove], "select (or deselect) party member", SelectActor) )
+  , ("_", ([CmdMove], "deselect (or select) all on the level", SelectNone))
   , ("semicolon", ( [CmdMove]
                   , "go to x-hair for 25 steps"
                   , Macro ["C-semicolon", "C-/", "C-V"] ))
@@ -69,70 +107,68 @@ standardKeys = KeyKind $ map evalKeyDef $
   , ("R", ([CmdMove], "rest (wait 25 times)", Macro ["KP_5", "C-V"]))
   , ("C-R", ( [CmdMove], "lurk (wait 0.1 turns 100 times)"
             , Macro ["C-KP_5", "V"] ))
-  , ("c", ( [CmdMove, CmdMinimal]
-          , descTs closeDoorTriggers
-          , AlterDir closeDoorTriggers ))
 
   -- Item use, continued
-  , ("^", ( [CmdItem], "sort items by kind and stats", SortSlots))
-  , ("p", moveItemTriple [CGround, CEqp, CSha] CInv
-                         "item" False)
-  , ("e", moveItemTriple [CGround, CInv, CSha] CEqp
-                         "item" False)
-  , ("s", moveItemTriple [CGround, CInv, CEqp] CSha
-                         "and share item" False)
-  , ("P", ( [CmdMinimal, CmdItem]
+  , ("^", ( [CmdItem], "sort items by ownership, kind and stats", SortSlots))
+  , ("P", ( [CmdItem, CmdDashboard]
           , "manage item pack of the leader"
           , ChooseItemMenu (MStore CInv) ))
-  , ("G", ( [CmdItem]
+  , ("G", ( [CmdItem, CmdDashboard]
           , "manage items on the ground"
           , ChooseItemMenu (MStore CGround) ))
-  , ("E", ( [CmdItem]
+  , ("E", ( [CmdItem, CmdDashboard]
           , "manage equipment of the leader"
           , ChooseItemMenu (MStore CEqp) ))
-  , ("S", ( [CmdItem]
+  , ("S", ( [CmdItem, CmdDashboard]
           , "manage the shared party stash"
           , ChooseItemMenu (MStore CSha) ))
-  , ("A", ( [CmdItem]
+  , ("A", ( [CmdItem, CmdDashboard]
           , "manage all owned items"
           , ChooseItemMenu MOwned ))
-  , ("@", ( [CmdItem]
+  , ("@", ( [CmdItem, CmdDashboard]
           , "describe organs of the leader"
-          , ChooseItemMenu (MStore COrgan) ))
-  , ("#", ( [CmdItem]
+          , ChooseItemMenu MOrgans ))
+  , ("#", ( [CmdItem, CmdDashboard]
           , "show stat summary of the leader"
           , ChooseItemMenu MStats ))
   , ("~", ( [CmdItem]
           , "display known lore"
-          , ChooseItemMenu MLoreItem ))
-  , ("q", addCmdCategory CmdItem $ applyI [ApplyItem
-            { verb = "quaff"
-            , object = "potion"
-            , symbol = '!' }])
-  , ("r", addCmdCategory CmdItem $ applyI [ApplyItem
-            { verb = "read"
-            , object = "scroll"
-            , symbol = '?' }])
+          , ChooseItemMenu (MLore SItem) ))
+  , ("q", addCmdCategory CmdItem $ applyI [TriggerItem
+            { tiverb = "quaff"
+            , tiobject = "potion"
+            , tisymbols = "!" }])
+  , ("r", addCmdCategory CmdItem $ applyI [TriggerItem
+            { tiverb = "read"
+            , tiobject = "scroll"
+            , tisymbols = "?" }])
 
   , ("t", addCmdCategory CmdItem $ projectA
-            [ ApplyItem { verb = "throw"
-                        , object = "missile"
-                        , symbol = '|' } ])
---  , ("z", projectA [ApplyItem { verb = "zap"
---                              , object = "wand"
---                              , symbol = '/' }])
+            [ TriggerItem { tiverb = "throw"
+                          , tiobject = "missile"
+                          , tisymbols = "|" } ])
+--  , ("z", projectA [TriggerItem { tiverb = "zap"
+--                                , tiobject = "instrument"
+--                                , tisymbol = "/" }])
+
+  -- Dashboard, in addition to commands marked above
+  , ("safeD0", ([CmdInternal, CmdDashboard], "", Cancel))  -- blank line
+  ]
+  ++
+  map (\(k,  slore) -> ("safeD" ++ show (k :: Int)
+                       , ( [CmdInternal, CmdDashboard]
+                         , "display" <+> ppSLore slore <+> "lore"
+                         , ChooseItemMenu (MLore slore) )))
+      (zip [1..] [minBound..maxBound])
+  ++
+  [ ("safeD99", ([CmdInternal, CmdDashboard], "", Cancel))  -- blank line
 
   -- Aiming
-  , ("KP_Multiply", ( [CmdAim, CmdMinimal]
-                    , "cycle x-hair among enemies", AimEnemy ))
-      -- not really minimal, because flinging from Item Menu enters aiming
-      -- mode, first screen mentions aiming mode not in fling context
   , ("!", ([CmdAim], "", AimEnemy))
   , ("KP_Divide", ([CmdAim], "cycle x-hair among items", AimItem))
   , ("/", ([CmdAim], "", AimItem))
-  , ("\\", ([CmdAim], "cycle aiming modes", AimFloor))
-  , ("+", ([CmdAim, CmdMinimal], "swerve the aiming line", EpsIncr True))
   , ("-", ([CmdAim], "unswerve the aiming line", EpsIncr False))
+  , ("\\", ([CmdAim], "cycle aiming modes", AimFloor))
   , ("C-?", ( [CmdAim]
             , "set x-hair to nearest unknown spot"
             , XhairUnknown ))
@@ -143,52 +179,40 @@ standardKeys = KeyKind $ map evalKeyDef $
             , "set x-hair to nearest upstairs"
             , XhairStair True ))
   , ("C-}", ( [CmdAim]
-            , "set x-hair to nearest downstairs"
+            , "set x-hair to nearest dnstairs"
             , XhairStair False ))
-  , ("<", ([CmdAim], "move aiming one level higher" , AimAscend 1))
-  , ("C-<", ( [CmdNoHelp], "move aiming 10 levels higher"
+  , ("<", ([CmdAim], "move aiming one level up" , AimAscend 1))
+  , ("C-<", ( [CmdNoHelp], "move aiming 10 levels up"
             , AimAscend 10) )
-  , (">", ([CmdAim], "move aiming one level lower", AimAscend (-1)))
-  , ("C->", ( [CmdNoHelp], "move aiming 10 levels lower"
+  , (">", ([CmdAim], "move aiming one level down", AimAscend (-1)))
+      -- 'lower' would be misleading in some games, just as 'deeper'
+  , ("C->", ( [CmdNoHelp], "move aiming 10 levels down"
             , AimAscend (-10)) )
   , ("BackSpace" , ( [CmdAim]
-                 , "clear chosen item and target"
-                 , ComposeUnlessError ItemClear TgtClear ))
-  , ("Escape", ( [CmdAim, CmdMinimal]
-               , "cancel aiming/open Main Menu"
-               , ByAimMode {exploration = MainMenu, aiming = Cancel} ))
-  , ("Return", ( [CmdAim, CmdMinimal]
-               , "accept target/open Help"
-               , ByAimMode {exploration = Help, aiming = Accept} ))
+                   , "clear chosen item and target"
+                   , ComposeUnlessError ItemClear TgtClear ))
 
   -- Assorted
-  , ("space", ( [CmdMinimal, CmdMeta]
-              , "clear messages/display history", Clear ))
-  , ("?", ([CmdMeta], "display Help", Help))
-  , ("F1", ([CmdMeta], "", Help))
-  , ("Tab", ( [CmdMeta]
-            , "cycle among party members on the level"
-            , MemberCycle ))
-  , ("BackTab", ( [CmdMeta, CmdMinimal]
-              , "cycle among all party members"
-              , MemberBack ))
-  , ("=", ( [CmdMinimal, CmdMeta]
-          , "select (or deselect) party member", SelectActor) )
-  , ("_", ([CmdMeta], "deselect (or select) all on the level", SelectNone))
+  , ("F12", ([CmdMeta], "open dashboard", Dashboard))
+  , ("?", ([CmdMeta], "display help", Hint))
+  , ("F1", ([CmdMeta], "", Hint))
   , ("v", ([CmdMeta], "voice again the recorded commands", Repeat 1))
   , ("V", repeatTriple 100)
   , ("C-v", repeatTriple 1000)
   , ("C-V", repeatTriple 25)
   , ("'", ([CmdMeta], "start recording commands", Record))
+  , ("C-P", ([CmdMeta], "print screen", PrintScreen))
+
+  -- Dashboard, in addition to commands marked above
+  , ("safeD100", ([CmdInternal, CmdDashboard], "display help", Help))
+  , ("safeD101", ([CmdInternal, CmdDashboard], "display history", History))
 
   -- Mouse
   , ("LeftButtonRelease", mouseLMB)
   , ("RightButtonRelease", mouseRMB)
   , ("C-LeftButtonRelease", replaceDesc "" mouseRMB)  -- Mac convention
   , ( "C-RightButtonRelease"
-    , ( [CmdMouse]
-      , "open or close door"
-      , AlterWithPointer $ closeDoorTriggers ++ openDoorTriggers ) )
+    , ([CmdMouse], "open or close or alter at pointer", AlterWithPointer []) )
   , ("MiddleButtonRelease", mouseMMB)
   , ("WheelNorth", ([CmdMouse], "swerve the aiming line", Macro ["+"]))
   , ("WheelSouth", ([CmdMouse], "unswerve the aiming line", Macro ["-"]))
@@ -223,8 +247,8 @@ standardKeys = KeyKind $ map evalKeyDef $
   , ("safe6", ( [CmdInternal]
               , "fling at enemy under pointer"
               , aimFlingCmd ))
-  , ("safe7", ( [CmdInternal]
-              , "open Main Menu"
+  , ("safe7", ( [CmdInternal, CmdDashboard]
+              , "open main menu"
               , MainMenu ))
   , ("safe8", ( [CmdInternal]
               , "cancel aiming"
@@ -241,34 +265,18 @@ standardKeys = KeyKind $ map evalKeyDef $
   ]
   ++ map defaultHeroSelect [0..6]
 
-closeDoorTriggers :: [Trigger]
+closeDoorTriggers :: [TriggerTile]
 closeDoorTriggers =
-  [ AlterFeature { verb = "close"
-                 , object = "door"
-                 , feature = TK.CloseTo "closed vertical door Lit" }
-  , AlterFeature { verb = "close"
-                 , object = "door"
-                 , feature = TK.CloseTo "closed horizontal door Lit" }
-  , AlterFeature { verb = "close"
-                 , object = "door"
-                 , feature = TK.CloseTo "closed vertical door Dark" }
-  , AlterFeature { verb = "close"
-                 , object = "door"
-                 , feature = TK.CloseTo "closed horizontal door Dark" }
-  ]
-
-openDoorTriggers :: [Trigger]
-openDoorTriggers =
-  [ AlterFeature { verb = "open"
-                 , object = "door"
-                 , feature = TK.OpenTo "open vertical door Lit" }
-  , AlterFeature { verb = "open"
-                 , object = "door"
-                 , feature = TK.OpenTo "open horizontal door Lit" }
-  , AlterFeature { verb = "open"
-                 , object = "door"
-                 , feature = TK.OpenTo "open vertical door Dark" }
-  , AlterFeature { verb = "open"
-                 , object = "door"
-                 , feature = TK.OpenTo "open horizontal door Dark" }
+  [ TriggerTile { ttverb = "close"
+                , ttobject = "door"
+                , ttfeature = TK.CloseTo "closed vertical door Lit" }
+  , TriggerTile { ttverb = "close"
+                , ttobject = "door"
+                , ttfeature = TK.CloseTo "closed horizontal door Lit" }
+  , TriggerTile { ttverb = "close"
+                , ttobject = "door"
+                , ttfeature = TK.CloseTo "closed vertical door Dark" }
+  , TriggerTile { ttverb = "close"
+                , ttobject = "door"
+                , ttfeature = TK.CloseTo "closed horizontal door Dark" }
   ]
