@@ -1,6 +1,6 @@
 -- | Cave properties.
 module Content.CaveKind
-  ( cdefs
+  ( content
   ) where
 
 import Prelude ()
@@ -9,22 +9,15 @@ import Game.LambdaHack.Common.Prelude
 
 import Data.Ratio
 
-import Game.LambdaHack.Common.ContentDef
 import Game.LambdaHack.Common.Dice
 import Game.LambdaHack.Common.Misc
 import Game.LambdaHack.Content.CaveKind
 
-cdefs :: ContentDef CaveKind
-cdefs = ContentDef
-  { getSymbol = csymbol
-  , getName = cname
-  , getFreq = cfreq
-  , validateSingle = validateSingleCaveKind
-  , validateAll = validateAllCaveKind
-  , content = contentFromList
-      [rogue, arena, arena2, laboratory, empty, noise, noise2, shallow2rogue, shallow1rogue, raid, brawl, shootout, escape, zoo, ambush, battle, safari1, safari2, safari3]
-  }
-rogue,        arena, arena2, laboratory, empty, noise, noise2, shallow2rogue, shallow1rogue, raid, brawl, shootout, escape, zoo, ambush, battle, safari1, safari2, safari3 :: CaveKind
+content :: [CaveKind]
+content =
+  [rogue, arena, arena2, laboratory, empty, noise, noise2, shallow2rogue, shallow1rogue, raid, brawl, shootout, escape, zoo, ambush, battle, safari1, safari2, safari3]
+
+rogue,    arena, arena2, laboratory, empty, noise, noise2, shallow2rogue, shallow1rogue, raid, brawl, shootout, escape, zoo, ambush, battle, safari1, safari2, safari3 :: CaveKind
 
 rogue = CaveKind
   { csymbol       = 'R'
@@ -35,8 +28,8 @@ rogue = CaveKind
   , cysize        = snd normalLevelBound + 1
   , cgrid         = DiceXY (3 `d` 2) 4
   , cminPlaceSize = DiceXY (2 `d` 2 + 4) 5
-  , cmaxPlaceSize = DiceXY 15 10
-  , cdarkChance   = 1 `d` 54 + 1 `dl` 20
+  , cmaxPlaceSize = DiceXY 16 20
+  , cdarkChance   = 1 `d` 54 + 1 `dL` 20
   , cnightChance  = 51  -- always night
   , cauxConnects  = 1%2
   , cmaxVoid      = 1%6
@@ -45,10 +38,10 @@ rogue = CaveKind
   , cdoorChance   = 3%4
   , copenChance   = 1%5
   , chidden       = 7
-  , cactorCoeff   = 130  -- the maze requires time to explore
+  , cactorCoeff   = 65  -- the maze requires time to explore
   , cactorFreq    = [("monster", 60), ("animal", 40)]
-  , citemNum      = 6 `d` 5
-  , citemFreq     = [("useful", 40), ("treasure", 60)]
+  , citemNum      = 6 `d` 5 - 4 `dL` 1  -- deeper down quality over quantity
+  , citemFreq     = [("common item", 40), ("treasure", 60)]
   , cplaceFreq    = [("rogue", 100)]
   , cpassable     = False
   , cdefTile        = "fillerWall"
@@ -66,7 +59,7 @@ arena = rogue
   { csymbol       = 'A'
   , cname         = "Dusty underground library"
   , cfreq         = [("default random", 40), ("caveArena", 1)]
-  , cgrid         = DiceXY (2 + 1 `d` 2) (1 `d` 3)
+  , cgrid         = DiceXY (2 + 1 `d` 2) (1 + 1 `d` 2)
   , cminPlaceSize = DiceXY (2 `d` 2 + 4) 6
   , cmaxPlaceSize = DiceXY 16 12
   , cdarkChance   = 49 + 1 `d` 10  -- almost all rooms dark (1 in 10 lit)
@@ -76,12 +69,12 @@ arena = rogue
   , cnightChance  = 0  -- always day
   , cauxConnects  = 1
   , cmaxVoid      = 1%8
-  , cextraStairs  = 1 `d` 3
+  , cextraStairs  = 1
   , chidden       = 0
-  , cactorCoeff   = 100
+  , cactorCoeff   = 50
   , cactorFreq    = [("monster", 30), ("animal", 70)]
-  , citemNum      = 5 `d` 5  -- few rooms
-  , citemFreq     = [("useful", 20), ("treasure", 40), ("any scroll", 40)]
+  , citemNum      = 4 `d` 5  -- few rooms
+  , citemFreq     = [("common item", 20), ("treasure", 40), ("any scroll", 40)]
   , cplaceFreq    = [("arena", 100)]
   , cpassable     = True
   , cdefTile      = "arenaSetLit"
@@ -95,8 +88,8 @@ arena2 = arena
   , cdarkChance   = 41 + 1 `d` 10  -- almost all rooms lit (1 in 10 dark)
   -- Trails provide enough light for fun stealth.
   , cnightChance  = 51  -- always night
-  , citemNum      = 7 `d` 5  -- rare, so make it exciting
-  , citemFreq     = [("useful", 20), ("treasure", 40), ("any vial", 40)]
+  , citemNum      = 6 `d` 5  -- rare, so make it exciting
+  , citemFreq     = [("common item", 20), ("treasure", 40), ("any vial", 40)]
   , cdefTile      = "arenaSetDark"
   , cdesc         = "Velvet couches exude the strong smell of tobacco."
   }
@@ -106,16 +99,17 @@ laboratory = arena2
   , cfreq         = [("deep random", 20), ("caveLaboratory", 1)]
   , cgrid         = DiceXY (2 `d` 2 + 7) 3
   , cminPlaceSize = DiceXY (3 `d` 2 + 4) 5
-  , cdarkChance   = 1 `d` 54 + 1 `dl` 20  -- most rooms lit, to compensate for corridors
+  , cdarkChance   = 1 `d` 54 + 1 `dL` 20
+      -- most rooms lit, to compensate for corridors
   , cnightChance  = 0  -- always day
   , cauxConnects  = 1%10
   , cmaxVoid      = 1%10
-  , cextraStairs  = 1 `d` 2
+  , cextraStairs  = 1
   , cdoorChance   = 1
   , copenChance   = 1%2
   , chidden       = 7
-  , citemNum      = 7 `d` 5  -- reward difficulty
-  , citemFreq     = [("useful", 20), ("treasure", 40), ("any vial", 40)]
+  , citemNum      = 6 `d` 5  -- reward difficulty
+  , citemFreq     = [("common item", 20), ("treasure", 40), ("explosive", 40)]
   , cplaceFreq    = [("laboratory", 100)]
   , cpassable     = False
   , cdefTile      = "fillerWall"
@@ -127,15 +121,15 @@ empty = rogue
   { csymbol       = 'E'
   , cname         = "Tall cavern"
   , cfreq         = [("caveEmpty", 1)]
-  , cgrid         = DiceXY 1 1
-  , cminPlaceSize = DiceXY 12 12
+  , cgrid         = DiceXY 2 2
+  , cminPlaceSize = DiceXY 12 9
   , cmaxPlaceSize = DiceXY 48 32  -- favour large rooms
-  , cdarkChance   = 1 `d` 100 + 1 `dl` 100
+  , cdarkChance   = 1 `d` 100 + 1 `dL` 100
   , cnightChance  = 0  -- always day
   , cauxConnects  = 3%2
   , cminStairDist = 30
   , cmaxVoid      = 0  -- too few rooms to have void and fog common anyway
-  , cextraStairs  = 1 `d` 2
+  , cextraStairs  = 1
   , cdoorChance   = 0
   , copenChance   = 0
   , chidden       = 0
@@ -160,7 +154,7 @@ noise = rogue
   , cname         = "Leaky burrowed sediment"
   , cfreq         = [("default random", 10), ("caveNoise", 1)]
   , cgrid         = DiceXY (2 + 1 `d` 3) 3
-  , cminPlaceSize = DiceXY 8 5
+  , cminPlaceSize = DiceXY 8 6
   , cmaxPlaceSize = DiceXY 20 10
   , cdarkChance   = 51
   -- Light is deadly, because nowhere to hide and pillars enable spawning
@@ -168,12 +162,12 @@ noise = rogue
   , cnightChance  = 0  -- harder variant, but looks cheerful
   , cauxConnects  = 1%10
   , cmaxVoid      = 1%100
-  , cextraStairs  = 1 `d` 4
+  , cextraStairs  = 1 + 1 `d` 2
   , cdoorChance   = 1  -- to avoid lit quasi-door tiles
   , chidden       = 0
-  , cactorCoeff   = 160  -- the maze requires time to explore
+  , cactorCoeff   = 80  -- the maze requires time to explore
   , cactorFreq    = [("monster", 80), ("animal", 20)]
-  , citemNum      = 7 `d` 5  -- an incentive to explore the labyrinth
+  , citemNum      = 6 `d` 5  -- an incentive to explore the labyrinth
   , cpassable     = True
   , cplaceFreq    = [("noise", 100)]
   , cdefTile      = "noiseSet"
@@ -186,7 +180,9 @@ noise2 = noise
   { cname         = "Frozen derelict mine"
   , cfreq         = [("caveNoise2", 1)]
   , cnightChance  = 51  -- easier variant, but looks sinister
-  , citemNum      = 13 `d` 5  -- an incentive to explore the final labyrinth
+  , citemNum      = 11 `d` 5  -- an incentive to explore the final labyrinth
+  , citemFreq     = [ ("common item", 20), ("treasure", 60)
+                    , ("explosive", 20) ]
   , cplaceFreq    = [("noise", 1), ("mine", 99)]
   , cstairFreq    = [("gated staircase", 100)]
   , cdesc         = "Pillars of shining ice create a frozen labyrinth."
@@ -214,10 +210,10 @@ raid = rogue
   , cfreq         = [("caveRaid", 1)]
   , cdarkChance   = 0  -- all rooms lit, for a gentle start
   , cmaxVoid      = 1%10
-  , cactorCoeff   = 500  -- deep level with no kit, so slow spawning
+  , cactorCoeff   = 250  -- deep level with no kit, so slow spawning
   , cactorFreq    = [("animal", 100)]
   , citemNum      = 6 `d` 8  -- just one level, hard enemies, treasure
-  , citemFreq     = [("useful", 33), ("gem", 33), ("currency", 33)]
+  , citemFreq     = [("common item", 100), ("currency", 500)]
   , cescapeGroup  = Just "escape up"
   , cdesc         = ""
   }
@@ -238,12 +234,13 @@ brawl = rogue  -- many random solid tiles, to break LOS, since it's a day
   , chidden       = 0
   , cactorFreq    = []
   , citemNum      = 5 `d` 8
-  , citemFreq     = [("useful", 100)]
+  , citemFreq     = [("common item", 100)]
   , cplaceFreq    = [("brawl", 60), ("rogue", 40)]
   , cpassable     = True
   , cdefTile      = "brawlSetLit"
   , cdarkCorTile  = "floorArenaLit"
   , clitCorTile   = "floorArenaLit"
+  , couterFenceTile = "outdoor outer fence"
   , cdesc         = "Sunlight falls through the trees and dapples on the ground."
   }
 shootout = rogue  -- a scenario with strong missiles;
@@ -267,9 +264,8 @@ shootout = rogue  -- a scenario with strong missiles;
   , citemNum      = 5 `d` 16
                       -- less items in inventory, more to be picked up,
                       -- to reward explorer and aggressor and punish camper
-  , citemFreq     = [ ("useful", 30)
-                    , ("any arrow", 400), ("harpoon", 300)
-                    , ("any vial", 60) ]
+  , citemFreq     = [ ("common item", 30)
+                    , ("any arrow", 400), ("harpoon", 300), ("explosive", 50) ]
                       -- Many consumable buffs are needed in symmetric maps
                       -- so that aggressor prepares them in advance and camper
                       -- needs to waste initial turns to buff for the defence.
@@ -278,6 +274,7 @@ shootout = rogue  -- a scenario with strong missiles;
   , cdefTile      = "shootoutSetLit"
   , cdarkCorTile  = "floorArenaLit"
   , clitCorTile   = "floorArenaLit"
+  , couterFenceTile = "outdoor outer fence"
   , cdesc         = ""
   }
 escape = rogue  -- a scenario with weak missiles, because heroes don't depend
@@ -294,18 +291,20 @@ escape = rogue  -- a scenario with weak missiles, because heroes don't depend
   , cdarkChance   = 51  -- colonnade rooms should always be dark
   , cnightChance  = 51  -- always night
   , cauxConnects  = 3%2
-  , cmaxVoid      = 1%20
+  , cmaxVoid      = 1%10
   , cextraStairs  = 1
   , chidden       = 0
   , cactorFreq    = []
   , citemNum      = 6 `d` 8
-  , citemFreq     = [ ("useful", 30), ("treasure", 30), ("gem", 100)
-                    , ("weak arrow", 500), ("harpoon", 400) ]
+  , citemFreq     = [ ("common item", 30), ("gem", 150)
+                    , ("weak arrow", 500), ("harpoon", 400)
+                    , ("explosive", 100) ]
   , cplaceFreq    = [("park", 100)]  -- the same rooms as in ambush
   , cpassable     = True
   , cdefTile      = "escapeSetDark"  -- different tiles, not burning yet
-  , cdarkCorTile  = "trailLit"  -- let trails give off light
-  , clitCorTile   = "trailLit"
+  , cdarkCorTile  = "alarmingTrailLit"  -- let trails give off light
+  , clitCorTile   = "alarmingTrailLit"
+  , couterFenceTile = "outdoor outer fence"
   , cescapeGroup  = Just "escape outdoor down"
   , cdesc         = ""
   }
@@ -320,18 +319,19 @@ zoo = rogue  -- few lights and many solids, to help the less numerous heroes
   , cnightChance  = 51  -- always night
   , cauxConnects  = 1%4
   , cmaxVoid      = 1%20
-  , cdoorChance   = 2%10
+  , cdoorChance   = 7%10
   , copenChance   = 9%10
   , cextraStairs  = 1
   , chidden       = 0
   , cactorFreq    = []
   , citemNum      = 7 `d` 8
-  , citemFreq     = [("useful", 100), ("light source", 1000)]
+  , citemFreq     = [("common item", 100), ("light source", 1000)]
   , cplaceFreq    = [("zoo", 50)]
   , cpassable     = True
   , cdefTile      = "zooSet"
   , cdarkCorTile  = "trailLit"  -- let trails give off light
   , clitCorTile   = "trailLit"
+  , couterFenceTile = "outdoor outer fence"
   , cdesc         = ""
   }
 ambush = rogue  -- a scenario with strong missiles;
@@ -358,12 +358,14 @@ ambush = rogue  -- a scenario with strong missiles;
   , chidden       = 0
   , cactorFreq    = []
   , citemNum      = 5 `d` 8
-  , citemFreq     = [("useful", 30), ("any arrow", 400), ("harpoon", 300)]
+  , citemFreq     = [ ("common item", 30)
+                    , ("any arrow", 400), ("harpoon", 300), ("explosive", 50) ]
   , cplaceFreq    = [("park", 100)]
   , cpassable     = True
   , cdefTile      = "ambushSet"
   , cdarkCorTile  = "trailLit"  -- let trails give off light
   , clitCorTile   = "trailLit"
+  , couterFenceTile = "outdoor outer fence"
   , cdesc         = ""
   }
 battle = rogue  -- few lights and many solids, to help the less numerous heroes
@@ -383,19 +385,18 @@ battle = rogue  -- few lights and many solids, to help the less numerous heroes
   , chidden       = 0
   , cactorFreq    = []
   , citemNum      = 5 `d` 8
-  , citemFreq     = [("useful", 100), ("light source", 200)]
+  , citemFreq     = [("common item", 100), ("light source", 200)]
   , cplaceFreq    = [("battle", 50), ("rogue", 50)]
   , cpassable     = True
   , cdefTile      = "battleSet"
   , cdarkCorTile  = "trailLit"  -- let trails give off light
   , clitCorTile   = "trailLit"
-  , couterFenceTile = "noise fence"  -- ensures no cut-off parts from collapsed
+  , couterFenceTile = "outdoor outer fence"
   , cdesc         = ""
   }
 safari1 = brawl
   { cname         = "Hunam habitat"
   , cfreq         = [("caveSafari1", 1)]
-  , cescapeGroup  = Nothing
   , cstairFreq    = [("staircase outdoor", 1)]
   , cdesc         = "\"Act 1. Hunams scavenge in a forest in their usual disgusting way.\""
   }
