@@ -10,18 +10,22 @@ import Game.LambdaHack.Common.Prelude
 import Game.LambdaHack.Common.Ability
 import Game.LambdaHack.Common.Color
 import Game.LambdaHack.Common.Flavour
+import Game.LambdaHack.Common.ItemAspect (Aspect (..))
 import Game.LambdaHack.Common.Misc
 import Game.LambdaHack.Content.ItemKind
 
 actors :: [ItemKind]
 actors =
-  [warrior, warrior2, warrior3, warrior4, warrior5, scout, ranger, escapist, ambusher, soldier, civilian, civilian2, civilian3, civilian4, civilian5, eye, fastEye, nose, elbow, torsor, goldenJackal, griffonVulture, skunk, armadillo, gilaMonster, rattlesnake, komodoDragon, hyena, alligator, rhinoceros, beeSwarm, hornetSwarm, thornbush,
-   geyserBoiling, geyserArsenic, geyserSulfur]
+  [warrior, warrior2, warrior3, warrior4, warrior5, scout, ranger, escapist, ambusher, soldier, civilian, civilian2, civilian3, civilian4, civilian5, eye, fastEye, nose, elbow, torsor, goldenJackal, griffonVulture, skunk, armadillo, gilaMonster, rattlesnake, komodoDragon, hyena, alligator, rhinoceros, beeSwarm, hornetSwarm, thornbush]
+  -- LH-specific
+  ++ [geyserBoiling, geyserArsenic, geyserSulfur]
 
-warrior,    warrior2, warrior3, warrior4, warrior5, scout, ranger, escapist, ambusher, soldier, civilian, civilian2, civilian3, civilian4, civilian5, eye, fastEye, nose, elbow, torsor, goldenJackal, griffonVulture, skunk, armadillo, gilaMonster, rattlesnake, komodoDragon, hyena, alligator, rhinoceros, beeSwarm, hornetSwarm, thornbush,
-   geyserBoiling, geyserArsenic, geyserSulfur :: ItemKind
+warrior,    warrior2, warrior3, warrior4, warrior5, scout, ranger, escapist, ambusher, soldier, civilian, civilian2, civilian3, civilian4, civilian5, eye, fastEye, nose, elbow, torsor, goldenJackal, griffonVulture, skunk, armadillo, gilaMonster, rattlesnake, komodoDragon, hyena, alligator, rhinoceros, beeSwarm, hornetSwarm, thornbush :: ItemKind
+-- LH-specific
+geyserBoiling, geyserArsenic, geyserSulfur :: ItemKind
 
 -- * Hunams
+
 warrior = ItemKind
   { isymbol  = '@'
   , iname    = "warrior"  -- modified if initial actors in hero faction
@@ -31,14 +35,15 @@ warrior = ItemKind
   , irarity  = [(1, 5)]
   , iverbHit = "thud"
   , iweight  = 80000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [ AddMaxHP 80  -- partially from clothes and assumed first aid
                , AddMaxCalm 70, AddSpeed 20, AddNocto 2
                , AddAbility AbProject 2, AddAbility AbApply 1
                , AddAbility AbAlter 2 ]
   , ieffects = []
-  , ifeature = [Durable, Identified]
-  , idesc    = "A hardened veteran of combat."
+  , ifeature = [Durable]
+  , idesc    = ""
+  -- , idesc    = "A hardened veteran of combat."
   , ikit     = [ ("fist", COrgan), ("foot", COrgan), ("eye 6", COrgan)
                , ("sapient brain", COrgan) ]
   }
@@ -71,7 +76,9 @@ scout = warrior
 ranger = warrior
   { iname    = "ranger"
   , ifreq    = [("ranger hero", 100), ("mobile", 1)]
-  , ikit     = ikit warrior ++ [("weak arrow", CInv), ("armor ranged", CEqp)]
+  , ikit     = ikit warrior
+               ++ [ ("armor ranged", CEqp)
+                  , ("weak arrow", CInv) ]
   -- , idesc    = ""
   }
 escapist = warrior
@@ -79,10 +86,10 @@ escapist = warrior
   , ifreq    = [("escapist hero", 100), ("mobile", 1)]
   , ikit     = ikit warrior
                ++ [ ("add sight", CEqp)
-                  , ("weak arrow", CInv)  -- mostly for probing
                   , ("armor ranged", CEqp)
-                  , ("flask", CInv)
+                  , ("weak arrow", CInv)  -- mostly for probing
                   , ("light source", CInv)
+                  , ("wooden torch", CInv)
                   , ("blanket", CInv) ]
   -- , idesc    = ""
   }
@@ -91,14 +98,19 @@ ambusher = warrior
   , ifreq    = [("ambusher hero", 100), ("mobile", 1)]
   , ikit     = ikit warrior  -- dark and numerous, so more kit without exploring
                ++ [ ("ring of opportunity sniper", CEqp)
-                  , ("light source", CEqp), ("wooden torch", CInv)
-                  , ("weak arrow", CInv), ("any arrow", CSha), ("flask", CSha) ]
+                  , ("any arrow", CSha)
+                  , ("weak arrow", CInv)
+                  , ("explosive", CSha)
+                  , ("light source", CEqp)
+                  , ("wooden torch", CInv) ]
   -- , idesc    = ""
   }
 soldier = warrior
   { iname    = "soldier"
   , ifreq    = [("soldier hero", 100), ("mobile", 1)]
-  , ikit     = ikit warrior ++ [("starting weapon", CEqp)]
+  , ikit     = ikit warrior
+               ++ [ ("starting weapon", CEqp)
+                  , ("explosive", CSha) ]
   -- , idesc    = ""
   }
 
@@ -139,13 +151,13 @@ eye = ItemKind
   , irarity  = [(1, 10), (10, 6)]
   , iverbHit = "thud"
   , iweight  = 80000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [ AddMaxHP 16, AddMaxCalm 70, AddSpeed 20, AddNocto 2
                , AddAggression 1
                , AddAbility AbProject 2, AddAbility AbApply 1
                , AddAbility AbAlter 2 ]
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "Under your stare, it reduces to the bits that define its essence. Under introspection, the bits slow down and solidify into an arbitrary form again. It must be huge inside, for holographic principle to manifest so overtly."  -- holographic principle is an anachronism for XIX or most of XX century, but "the cosmological scale effects" is too weak
   , ikit     = [ ("lash", COrgan), ("pupil", COrgan)
                , ("sapient brain", COrgan) ]
@@ -160,12 +172,12 @@ fastEye = ItemKind
   , irarity  = [(5, 5), (10, 5)]
   , iverbHit = "thud"
   , iweight  = 80000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [ AddMaxHP 5, AddMaxCalm 70, AddSpeed 30, AddNocto 2
                , AddAggression 1
                , AddAbility AbAlter 2 ]
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "Hungers but never eats. Bites but never swallows. Burrows its own image through, but never carries anything back."  -- rather weak: not about injective objects, but puny, concrete, injective functions  --- where's the madness in that?
   , ikit     = [ ("tooth", COrgan), ("speed gland 10", COrgan)
                , ("lip", COrgan), ("vision 6", COrgan)
@@ -180,12 +192,12 @@ nose = ItemKind  -- depends solely on smell
   , irarity  = [(1, 5), (4, 2), (10, 5)]
   , iverbHit = "thud"
   , iweight  = 80000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [ AddMaxHP 30, AddMaxCalm 30, AddSpeed 18, AddNocto 2
                , AddAggression 1
                , AddAbility AbProject (-1), AddAbility AbAlter 2 ]
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "No mouth, yet it devours everything around, constantly sniffing itself inward; pure movement structure, no constant point to focus one's maddened gaze on."
   , ikit     = [ ("nose tip", COrgan), ("lip", COrgan), ("nostril", COrgan)
                , ("sapient brain", COrgan) ]
@@ -200,12 +212,12 @@ elbow = ItemKind
   , irarity  = [(7, 1), (10, 5)]
   , iverbHit = "thud"
   , iweight  = 80000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [ AddMaxHP 8, AddMaxCalm 80, AddSpeed 21, AddNocto 2
                , AddAbility AbProject 2, AddAbility AbApply 1
                , AddAbility AbAlter 2, AddAbility AbMelee (-1) ]
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "An arm strung like a bow. A few edges, but none keen enough. A few points, but none piercing. Deadly objects zip out of the void."
   , ikit     = [ ("speed gland 4", COrgan), ("armored skin", COrgan)
                , ("vision 16", COrgan)
@@ -222,13 +234,12 @@ torsor = ItemKind
   , irarity  = [(9, 0), (10, 1000)]  -- unique
   , iverbHit = "thud"
   , iweight  = 80000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [ AddMaxHP 300, AddMaxCalm 100, AddSpeed 10, AddNocto 2
-               , AddAggression 3
-               , AddAbility AbProject 2, AddAbility AbApply 1 ]
-                 -- can't exit the gated level, the boss
-  , ieffects = [Unique]
-  , ifeature = [Durable, Identified]
+               , AddAggression 3, AddAbility AbProject 2, AddAbility AbApply 1
+               , AddAbility AbAlter 1]  -- can't exit the gated level, the boss
+  , ieffects = []
+  , ifeature = [Unique, Durable]
   , idesc    = "A principal homogeneous manifold, that acts freely and with enormous force, but whose stabilizers are trivial, making it rather helpless without a support group."
   , ikit     = [ ("right torsion", COrgan), ("left torsion", COrgan)
                , ("pupil", COrgan)
@@ -256,10 +267,10 @@ goldenJackal = ItemKind  -- basically a much smaller and slower hyena
   , irarity  = [(1, 3)]
   , iverbHit = "thud"
   , iweight  = 13000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [ AddMaxHP 12, AddMaxCalm 70, AddSpeed 24, AddNocto 2 ]
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "An opportunistic predator, feeding on carrion and the weak."
   , ikit     = [ ("small jaw", COrgan), ("eye 6", COrgan), ("nostril", COrgan)
                , ("animal brain", COrgan) ]
@@ -274,7 +285,7 @@ griffonVulture = ItemKind
   , irarity  = [(1, 5)]
   , iverbHit = "thud"
   , iweight  = 13000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [ AddMaxHP 12, AddMaxCalm 80, AddSpeed 22, AddNocto 2
                , AddAbility AbAlter (-2) ]  -- can't use stairs nor doors
       -- Animals don't have leader, usually, so even if only one of level,
@@ -282,7 +293,7 @@ griffonVulture = ItemKind
       -- them on par with human leaders moving solo. Random double moves,
       -- on either side, are just too frustrating.
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "It soars high above, searching for vulnerable prey."
   , ikit     = [ ("screeching beak", COrgan)  -- in reality it grunts and hisses
                , ("small claw", COrgan), ("eye 7", COrgan)
@@ -297,11 +308,11 @@ skunk = ItemKind
   , irarity  = [(1, 5), (10, 3)]
   , iverbHit = "thud"
   , iweight  = 4000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [ AddMaxHP 10, AddMaxCalm 30, AddSpeed 22, AddNocto 2
                , AddAbility AbAlter (-2) ]  -- can't use stairs nor doors
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "Its only defence is the terrible stench."
   , ikit     = [ ("scent gland", COrgan)
                , ("small claw", COrgan), ("snout", COrgan)
@@ -317,11 +328,11 @@ armadillo = ItemKind
   , irarity  = [(1, 5)]
   , iverbHit = "thud"
   , iweight  = 80000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [ AddMaxHP 10, AddMaxCalm 30, AddSpeed 20, AddNocto 2
                , AddAbility AbAlter (-2) ]  -- can't use stairs nor doors
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "When threatened, it rolls into a ball."
   , ikit     = [ ("hooked claw", COrgan), ("snout", COrgan)
                , ("armored skin", COrgan), ("armored skin", COrgan)
@@ -337,11 +348,11 @@ gilaMonster = ItemKind
   , irarity  = [(2, 5), (10, 3)]
   , iverbHit = "thud"
   , iweight  = 80000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [ AddMaxHP 12, AddMaxCalm 50, AddSpeed 18, AddNocto 2
                , AddAbility AbAlter (-2) ]  -- can't use stairs nor doors
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "Numbing venom ensures that even the fastest prey has no escape."
   , ikit     = [ ("venom tooth", COrgan), ("small claw", COrgan)
                , ("eye 3", COrgan), ("nostril", COrgan)
@@ -356,11 +367,11 @@ rattlesnake = ItemKind
   , irarity  = [(5, 1), (10, 12)]
   , iverbHit = "thud"
   , iweight  = 80000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [ AddMaxHP 25, AddMaxCalm 60, AddSpeed 16, AddNocto 2
                , AddAbility AbAlter (-2) ]  -- can't use stairs nor doors
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "Beware its rattle - it serves as a warning of an agonising death."
   , ikit     = [ ("venom fang", COrgan)
                , ("eye 4", COrgan), ("nostril", COrgan)
@@ -375,10 +386,10 @@ komodoDragon = ItemKind  -- bad hearing; regeneration makes it very powerful
   , irarity  = [(9, 0), (10, 10)]
   , iverbHit = "thud"
   , iweight  = 80000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [ AddMaxHP 41, AddMaxCalm 60, AddSpeed 18, AddNocto 2 ]
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "Larger and more aggressive than any other lizard."
   , ikit     = [ ("large tail", COrgan), ("jaw", COrgan)
                , ("hooked claw", COrgan), ("speed gland 4", COrgan)
@@ -395,10 +406,10 @@ hyena = ItemKind
   , irarity  = [(4, 1), (10, 8)]
   , iverbHit = "thud"
   , iweight  = 60000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [ AddMaxHP 20, AddMaxCalm 70, AddSpeed 32, AddNocto 2 ]
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "Skulking in the shadows, waiting for easy prey."
   , ikit     = [ ("jaw", COrgan), ("eye 6", COrgan), ("nostril", COrgan)
                , ("animal brain", COrgan) ]
@@ -412,10 +423,10 @@ alligator = ItemKind
   , irarity  = [(8, 1), (10, 9)]
   , iverbHit = "thud"
   , iweight  = 80000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [ AddMaxHP 41, AddMaxCalm 70, AddSpeed 18, AddNocto 2 ]
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "An armored predator from the dawn of time."
   , ikit     = [ ("large jaw", COrgan), ("large tail", COrgan)
                , ("small claw", COrgan)
@@ -431,15 +442,15 @@ rhinoceros = ItemKind
   , irarity  = [(2, 0), (3, 1000000), (4, 0)]  -- unique
   , iverbHit = "thud"
   , iweight  = 80000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [ AddMaxHP 90, AddMaxCalm 60, AddSpeed 27, AddNocto 2
                , AddAggression 2
                , AddAbility AbAlter (-1) ]  -- can't switch levels, a miniboss
-  , ieffects = [Unique]
-  , ifeature = [Durable, Identified]
+  , ieffects = []
+  , ifeature = [Unique, Durable]
   , idesc    = "The last of its kind. Blind with rage. Charges at deadly speed."
   , ikit     = [ ("armored skin", COrgan), ("eye 2", COrgan)
-               , ("horn", COrgan), ("snout", COrgan)
+               , ("rhino horn", COrgan), ("snout", COrgan)
                , ("animal brain", COrgan) ]
   }
 
@@ -454,12 +465,12 @@ beeSwarm = ItemKind
   , irarity  = [(1, 2), (10, 4)]
   , iverbHit = "thud"
   , iweight  = 1000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [ AddMaxHP 8, AddMaxCalm 60
                , AddSpeed 30, AddNocto 2  -- armor in sting
                , AddAbility AbAlter (-2) ]  -- can't use stairs nor doors
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "Every bee would die for the queen."
   , ikit     = [ ("bee sting", COrgan), ("vision 6", COrgan)
                , ("insect mortality", COrgan), ("animal brain", COrgan) ]
@@ -473,12 +484,12 @@ hornetSwarm = ItemKind
   , irarity  = [(5, 1), (10, 8)]
   , iverbHit = "thud"
   , iweight  = 1000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [ AddMaxHP 8, AddMaxCalm 70, AddSpeed 30, AddNocto 2
                , AddAbility AbAlter (-2)  -- can't use stairs nor doors
                , AddArmorMelee 80, AddArmorRanged 40 ]
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "A vicious cloud of stings and hate."
   , ikit     = [ ("sting", COrgan), ("vision 8", COrgan)
                , ("insect mortality", COrgan), ("animal brain", COrgan) ]
@@ -486,71 +497,70 @@ hornetSwarm = ItemKind
 thornbush = ItemKind
   { isymbol  = 't'
   , iname    = "thornbush"
-  , ifreq    = [("animal", 20), ("immobile animal", 30)]
+  , ifreq    = [("animal", 18), ("immobile animal", 30)]
   , iflavour = zipPlain [Brown]
   , icount   = 1
   , irarity  = [(1, 10)]
   , iverbHit = "thud"
   , iweight  = 80000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [ AddMaxHP 20, AddMaxCalm 999, AddSpeed 22, AddNocto 2
                , AddAbility AbWait 1, AddAbility AbMelee 1 ]
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "Each branch bears long, curved thorns."
   , ikit     = [("thorn", COrgan), ("armored skin", COrgan)]
   }
 geyserBoiling = ItemKind
   { isymbol  = 'g'
   , iname    = "geyser"
-  , ifreq    = [("animal", 30), ("immobile animal", 60)]
+  , ifreq    = [("animal", 25), ("immobile animal", 60)]
   , iflavour = zipPlain [Blue]
   , icount   = 1
   , irarity  = [(1, 3), (5, 3)]
   , iverbHit = "thud"
   , iweight  = 80000
-  , idamage  = toDmg 0
-  , iaspects = [ AddMaxHP 20, AddMaxCalm 999, AddSpeed 10, AddNocto 2
-               , AddAbility AbWait 1, AddAbility AbMelee 1
-               , AddArmorMelee 40, AddArmorRanged 20 ]  -- hard material
+  , idamage  = 0
+  , iaspects = [ AddMaxHP 10, AddMaxCalm 999, AddSpeed 11, AddNocto 2
+               , AddAbility AbWait 1, AddAbility AbMelee 1 ]
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "A jet of acidic water, hot enough to melt flesh."
   , ikit     = [("boiling vent", COrgan), ("boiling fissure", COrgan)]
   }
 geyserArsenic = ItemKind
   { isymbol  = 'g'
   , iname    = "arsenic geyser"
-  , ifreq    = [("animal", 10), ("immobile animal", 40)]
+  , ifreq    = [("animal", 8), ("immobile animal", 40)]
   , iflavour = zipPlain [Cyan]
   , icount   = 1
   , irarity  = [(1, 10), (5, 10)]
   , iverbHit = "thud"
   , iweight  = 80000
-  , idamage  = toDmg 0
-  , iaspects = [ AddMaxHP 30, AddMaxCalm 999, AddSpeed 22
+  , idamage  = 0
+  , iaspects = [ AddMaxHP 20, AddMaxCalm 999, AddSpeed 22
                , AddNocto 2, AddShine 3
                , AddAbility AbWait 1, AddAbility AbMelee 1 ]
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "The sharp scent betrays the poison within the spray."
   , ikit     = [("arsenic vent", COrgan), ("arsenic fissure", COrgan)]
   }
 geyserSulfur = ItemKind
   { isymbol  = 'g'
   , iname    = "sulfur geyser"
-  , ifreq    = [("animal", 10), ("immobile animal", 120)]
+  , ifreq    = [("animal", 8), ("immobile animal", 120)]
   , iflavour = zipPlain [BrYellow]  -- exception, animal with bright color
   , icount   = 1
   , irarity  = [(1, 10), (5, 10)]
   , iverbHit = "thud"
   , iweight  = 80000
-  , idamage  = toDmg 0
-  , iaspects = [ AddMaxHP 30, AddMaxCalm 999, AddSpeed 22
+  , idamage  = 0
+  , iaspects = [ AddMaxHP 20, AddMaxCalm 999, AddSpeed 22
                , AddNocto 2, AddShine 3
                , AddAbility AbWait 1, AddAbility AbMelee 1 ]
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "The pool boils and bubbles, stinking of rotten eggs. Despite the smell, these waters purify and strengthen."
   , ikit     = [("sulfur vent", COrgan), ("sulfur fissure", COrgan)]
   }
