@@ -10,92 +10,236 @@ import Game.LambdaHack.Common.Prelude
 import Game.LambdaHack.Common.Color
 import Game.LambdaHack.Common.Dice
 import Game.LambdaHack.Common.Flavour
+import Game.LambdaHack.Common.ItemAspect (Aspect (..))
 import Game.LambdaHack.Common.Misc
 import Game.LambdaHack.Content.ItemKind
 
 blasts :: [ItemKind]
 blasts =
-  [burningOil2, burningOil3, burningOil4, explosionBlast2, explosionBlast10, explosionBlast20, firecracker2, firecracker3, firecracker4, firecracker5, firecracker6, firecracker7, fragrance, pheromone, mistCalming, odorDistressing, mistHealing, mistHealing2, mistWounding, distortion, glassPiece, smoke, boilingWater, glue, singleSpark, spark, denseShower, sparseShower, protectingBalmMelee, protectingBalmRanged, vulnerabilityBalm, resolutionDust, hasteSpray, slownessMist, eyeDrop, ironFiling, smellyDroplet, eyeShine, whiskeySpray, waste, youthSprinkle, poisonCloud, mistAntiSlow, mistAntidote]
+  [burningOil2, burningOil3, burningOil4, firecracker1, firecracker2, firecracker3, firecracker4, firecracker5, spreadFragmentation, spreadFragmentation8, focusedFragmentation, spreadConcussion, spreadConcussion8, focusedConcussion, spreadFlash, spreadFlash8, focusedFlash, singleSpark, glassPiece, focusedGlass,fragrance, pheromone, mistCalming, odorDistressing, mistHealing, mistHealing2, mistWounding, distortion, smoke, boilingWater, glue, waste, denseShower, sparseShower, protectingBalmMelee, protectingBalmRanged, vulnerabilityBalm, resolutionDust, hasteSpray, slownessMist, eyeDrop, ironFiling, smellyDroplet, eyeShine, whiskeySpray, youthSprinkle, poisonCloud, mistAntiSlow, mistAntidote]
 
-burningOil2,    burningOil3, burningOil4, explosionBlast2, explosionBlast10, explosionBlast20, firecracker2, firecracker3, firecracker4, firecracker5, firecracker6, firecracker7, fragrance, pheromone, mistCalming, odorDistressing, mistHealing, mistHealing2, mistWounding, distortion, glassPiece, smoke, boilingWater, glue, singleSpark, spark, denseShower, sparseShower, protectingBalmMelee, protectingBalmRanged, vulnerabilityBalm, resolutionDust, hasteSpray, slownessMist, eyeDrop, ironFiling, smellyDroplet, eyeShine, whiskeySpray, waste, youthSprinkle, poisonCloud, mistAntiSlow, mistAntidote :: ItemKind
+burningOil2,    burningOil3, burningOil4, firecracker1, firecracker2, firecracker3, firecracker4, firecracker5, spreadFragmentation, spreadFragmentation8, focusedFragmentation, spreadConcussion, spreadConcussion8, focusedConcussion, spreadFlash, spreadFlash8, focusedFlash, singleSpark, glassPiece, focusedGlass,fragrance, pheromone, mistCalming, odorDistressing, mistHealing, mistHealing2, mistWounding, distortion, smoke, boilingWater, glue, waste, denseShower, sparseShower, protectingBalmMelee, protectingBalmRanged, vulnerabilityBalm, resolutionDust, hasteSpray, slownessMist, eyeDrop, ironFiling, smellyDroplet, eyeShine, whiskeySpray, youthSprinkle, poisonCloud, mistAntiSlow, mistAntidote :: ItemKind
 
 -- We take care (e.g., in burningOil below) that blasts are not faster
 -- than 100% fastest natural speed, or some frames would be skipped,
 -- which is a waste of prefectly good frames.
 
--- * Parameterized immediate effect blasts
+-- * Parameterized blasts
 
 burningOil :: Int -> ItemKind
 burningOil n = ItemKind
   { isymbol  = '*'
   , iname    = "burning oil"
   , ifreq    = [(toGroupName $ "burning oil" <+> tshow n, 1)]
-  , iflavour = zipFancy [BrYellow]
+  , iflavour = zipPlain [BrYellow]
   , icount   = intToDice (n * 8)
   , irarity  = [(1, 1)]
   , iverbHit = "sear"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [AddShine 2]
-  , ieffects = [Burn 1, Paralyze 2]  -- tripping on oil
+  , ieffects = [ Burn 1
+               , toOrganBad "slowed" (2 + 1 `d` 2) ]  -- tripping on oil
   , ifeature = [ toVelocity (min 100 $ n `div` 2 * 10)
-               , Fragile, Identified ]
+               , Fragile, Blast ]
   , idesc    = "Sticky oil, burning brightly."
   , ikit     = []
   }
 burningOil2 = burningOil 2  -- 2 steps, 2 turns
 burningOil3 = burningOil 3  -- 3 steps, 2 turns
 burningOil4 = burningOil 4  -- 4 steps, 2 turns
-explosionBlast :: Int -> ItemKind
-explosionBlast n = ItemKind
-  { isymbol  = '*'
-  , iname    = "blast"
-  , ifreq    = [(toGroupName $ "blast" <+> tshow n, 1)]
-  , iflavour = zipPlain [BrRed]
-  , icount   = 16  -- strong and wide, but few, so not always hits target
-  , irarity  = [(1, 1)]
-  , iverbHit = "tear apart"
-  , iweight  = 1
-  , idamage  = toDmg 0
-  , iaspects = [AddShine $ intToDice $ min 10 n]
-  , ieffects = [RefillHP (- n `div` 2)]
-               ++ [PushActor (ThrowMod (100 * (n `div` 5)) 50)| n >= 10]
-               ++ [DropItem 1 maxBound COrgan "temporary condition" | n >= 10]
-               ++ [DropItem 1 maxBound COrgan "impressed" | n >= 10]  -- shock
-  , ifeature = [toLinger 20, Fragile, Identified]  -- 4 steps, 1 turn
-  , idesc    = ""
-  , ikit     = []
-  }
-explosionBlast2 = explosionBlast 2
-explosionBlast10 = explosionBlast 10
-explosionBlast20 = explosionBlast 20
 firecracker :: Int -> ItemKind
 firecracker n = ItemKind
   { isymbol  = '*'
   , iname    = "firecracker"
-  , ifreq    = [(toGroupName $ "firecracker" <+> tshow n, 1)]
-  , iflavour = zipPlain [brightCol !! (n `mod` length brightCol)]
-  , icount   = intToDice (n `div` 6) + 1 `d` (n `div` 2)
+  , ifreq    = [(toGroupName $ if n == 5
+                               then "firecracker"
+                               else "firecracker" <+> tshow n, 1)]
+  , iflavour = zipPlain [brightCol !! ((n + 2) `mod` length brightCol)]
+  , icount   = if n <= 3 then 1 `d` min 2 n else 2 + 1 `d` 2
   , irarity  = [(1, 1)]
-  , iverbHit = "crack"
+  , iverbHit = if n >= 4 then "singe" else "crack"
   , iweight  = 1
-  , idamage  = toDmg 0
-  , iaspects = [AddShine $ intToDice $ n `div` 2]
-  , ieffects = [ RefillCalm (3 - n) | n >= 5 ]
-               ++ [ DropBestWeapon | n >= 5]
+  , idamage  = 0
+  , iaspects = [AddShine $ intToDice $ 1 + n `div` 2]
+  , ieffects = [if n >= 4 then Burn 1 else RefillCalm (-2)]
+               ++ [DropBestWeapon | n >= 4]
                ++ [ OnSmash $ Explode
                     $ toGroupName $ "firecracker" <+> tshow (n - 1)
-                  | n > 2 ]
-  , ifeature = [toVelocity 5, Fragile, Identified]
+                  | n >= 2 ]
+  , ifeature = [toVelocity 5, Fragile, Blast]
   , idesc    = "Scraps of burnt paper, covering little pockets of black powder, buffeted by colorful explosions."
   , ikit     = []
   }
-firecracker7 = firecracker 7
-firecracker6 = firecracker 6
 firecracker5 = firecracker 5
 firecracker4 = firecracker 4
 firecracker3 = firecracker 3
 firecracker2 = firecracker 2
+firecracker1 = firecracker 1
+
+-- * Focused blasts
+
+spreadFragmentation = ItemKind
+  { isymbol  = '*'
+  , iname    = "fragmentation burst"
+  , ifreq    = [("violent fragmentation", 1)]
+  , iflavour = zipPlain [Red]  -- flying shards; some fire and smoke
+  , icount   = 16  -- strong but few, so not always hits target
+  , irarity  = [(1, 1)]
+  , iverbHit = "tear apart"
+  , iweight  = 1
+  , idamage  = 3 `d` 1  -- deadly and adjacent actor hit by 2 on average;
+                        -- however, moderate armour blocks completely
+  , iaspects = [AddShine 3, AddHurtMelee $ -12 * 5]
+  , ieffects = [DropItem 1 maxBound COrgan "condition"]
+  , ifeature = [toLinger 20, Lobable, Fragile, Blast]  -- 4 steps, 1 turn
+  , idesc    = ""
+  , ikit     = []
+  }
+spreadFragmentation8 = spreadFragmentation
+  { iname    = "fragmentation burst"
+  , ifreq    = [("fragmentation", 1)]
+  , icount   = 8
+  , ifeature = [toLinger 10, Lobable, Fragile, Blast]  -- 2 steps, 1 turn
+      -- smaller radius, so worse for area effect, but twice the direct damage
+  }
+focusedFragmentation = ItemKind
+  { isymbol  = '`'
+  , iname    = "deflagration ignition"  -- black powder
+  , ifreq    = [("focused fragmentation", 1)]
+  , iflavour = zipPlain [BrYellow]
+  , icount   = 4  -- 32 in total vs 16; on average 4 hits
+  , irarity  = [(1, 1)]
+  , iverbHit = "ignite"
+  , iweight  = 1
+  , idamage  = 0
+  , iaspects = []
+  , ieffects = [OnSmash $ Explode "fragmentation"]
+  , ifeature = [toLinger 0, Fragile, Blast]  -- 0 steps, 1 turn
+      -- when the target position is occupied, the explosion starts one step
+      -- away, hence we set range to 0 steps, to limit dispersal
+  , idesc    = ""
+  , ikit     = []
+  }
+spreadConcussion = ItemKind
+  { isymbol  = '*'
+  , iname    = "concussion blast"
+  , ifreq    = [("violent concussion", 1)]
+  , iflavour = zipPlain [Magenta]  -- mosty shock wave; some fire and smoke
+  , icount   = 16
+  , irarity  = [(1, 1)]
+  , iverbHit = "shock"
+  , iweight  = 1
+  , idamage  = 1 `d` 1  -- only air pressure, so not as deadly as fragmentation,
+                        -- but armour can't block completely that easily
+  , iaspects = [AddShine 3, AddHurtMelee $ -8 * 5]
+  , ieffects = [ DropItem maxBound 1 CEqp "misc armor"
+               , PushActor (ThrowMod 400 25)  -- 1 step, fast; after DropItem
+                   -- this produces spam for braced actors; too bad
+               , DropItem 1 maxBound COrgan "condition" ]
+  , ifeature = [toLinger 20, Lobable, Fragile, Blast]  -- 4 steps, 1 turn
+      -- outdoors it has short range, but we only model indoors in the game;
+      -- it's much faster than black powder shock wave, but we are beyond
+      -- human-noticeable speed differences on short distances anyway
+  , idesc    = ""
+  , ikit     = []
+  }
+spreadConcussion8 = spreadConcussion
+  { iname    = "concussion blast"
+  , ifreq    = [("concussion", 1)]
+  , icount   = 8
+  , ifeature = [toLinger 10, Lobable, Fragile, Blast]  -- 2 steps, 1 turn
+  }
+focusedConcussion = ItemKind
+  { isymbol  = '`'
+  , iname    = "detonation ignition"  -- nitroglycerine
+  , ifreq    = [("focused concussion", 1)]
+  , iflavour = zipPlain [BrYellow]
+  , icount   = 4
+  , irarity  = [(1, 1)]
+  , iverbHit = "ignite"
+  , iweight  = 1
+  , idamage  = 0
+  , iaspects = []
+  , ieffects = [OnSmash $ Explode "concussion"]
+  , ifeature = [toLinger 0, Fragile, Blast]  -- 0 steps, 1 turn
+  , idesc    = ""
+  , ikit     = []
+  }
+spreadFlash = ItemKind
+  { isymbol  = '`'
+  , iname    = "magnesium flash"
+  , ifreq    = [("violent flash", 1)]
+  , iflavour = zipPlain [BrWhite]  -- very brigh flash
+  , icount   = 16
+  , irarity  = [(1, 1)]
+  , iverbHit = "dazzle"
+  , iweight  = 1
+  , idamage  = 0
+  , iaspects = [AddShine 5]
+  , ieffects = [toOrganBad "blind" 10, toOrganBad "weakened" 30]
+                 -- Wikipedia says: blind for five seconds and afterimage
+                 -- for much longer, harming aim
+  , ifeature = [toLinger 20, Fragile, Blast]  -- 4 steps, 1 turn
+  , idesc    = "A flash of fire."
+  , ikit     = []
+  }
+spreadFlash8 = spreadFlash
+  { iname    = "spark"
+  , ifreq    = [("spark", 1)]
+  , icount   = 8
+  , iverbHit = "blind"
+  , ifeature = [toLinger 10, Fragile, Blast]  -- 2 steps, 1 turn
+  }
+focusedFlash = ItemKind
+  { isymbol  = '`'
+  , iname    = "magnesium ignition"
+  , ifreq    = [("focused flash", 1)]
+  , iflavour = zipPlain [BrYellow]
+  , icount   = 4
+  , irarity  = [(1, 1)]
+  , iverbHit = "ignite"
+  , iweight  = 1
+  , idamage  = 0
+  , iaspects = []
+  , ieffects = [OnSmash $ Explode "spark"]
+  , ifeature = [toLinger 0, Fragile, Blast]  -- 0 steps, 1 turn
+  , idesc    = ""
+  , ikit     = []
+  }
+singleSpark = spreadFlash
+  { iname    = "single spark"
+  , ifreq    = [("single spark", 1)]
+  , icount   = 1
+  , iverbHit = "spark"
+  , iaspects = [AddShine 3]
+  , ieffects = []
+  , ifeature = [toLinger 5, Fragile, Blast]  -- 1 step, 1 turn
+  , idesc    = "A glowing ember."
+  , ikit     = []
+  }
+glassPiece = ItemKind
+  { isymbol  = '*'
+  , iname    = "glass piece"
+  , ifreq    = [("glass hail", 1)]
+  , iflavour = zipPlain [Blue]
+  , icount   = 8
+  , irarity  = [(1, 1)]
+  , iverbHit = "cut"
+  , iweight  = 1
+  , idamage  = 1 `d` 1
+  , iaspects = [AddHurtMelee $ -15 * 5]  -- brittle, not too dense; armor blocks
+  , ieffects = [RefillHP (-1)]
+  , ifeature = [toLinger 20, Fragile, Blast]  -- 4 steps, 1 turn
+  , idesc    = "Swift, sharp edges."
+  , ikit     = []
+  }
+focusedGlass = glassPiece  -- when blowing up windows
+  { ifreq    = [("focused glass hail", 1)]
+  , icount   = 4
+  , ieffects = [RefillHP (-1), OnSmash $ Explode "glass hail"]
+  , ifeature = [toLinger 0, Fragile, Blast]  -- 0 steps, 1 turn
+  }
 
 -- * Assorted immediate effect blasts
 
@@ -103,17 +247,17 @@ fragrance = ItemKind
   { isymbol  = '`'
   , iname    = "fragrance"  -- instant, fast fragrance
   , ifreq    = [("fragrance", 1)]
-  , iflavour = zipFancy [Magenta]
-  , icount   = 16
+  , iflavour = zipPlain [Magenta]
+  , icount   = 12
   , irarity  = [(1, 1)]
   , iverbHit = "engulf"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
   , ieffects = [Impress]
   -- Linger 10, because sometimes it takes 2 turns due to starting just
   -- before actor turn's end (e.g., via a necklace).
-  , ifeature = [toLinger 10, Fragile, Identified]  -- 2 steps, 1 turn
+  , ifeature = [toLinger 10, Fragile, Blast]  -- 2 steps, 1 turn
   , idesc    = "A pleasant scent."
   , ikit     = []
   }
@@ -121,15 +265,15 @@ pheromone = ItemKind
   { isymbol  = '`'
   , iname    = "musky whiff"  -- a kind of mist rather than fragrance
   , ifreq    = [("pheromone", 1)]
-  , iflavour = zipFancy [BrMagenta]
+  , iflavour = zipPlain [BrMagenta]
   , icount   = 16
   , irarity  = [(1, 1)]
   , iverbHit = "tempt"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
   , ieffects = [Impress, RefillCalm (-10)]
-  , ifeature = [toVelocity 10, Fragile, Identified]  -- 2 steps, 2 turns
+  , ifeature = [toVelocity 10, Fragile, Blast]  -- 2 steps, 2 turns
   , idesc    = "A sharp, strong scent."
   , ikit     = []
   }
@@ -137,15 +281,15 @@ mistCalming = ItemKind  -- unused
   { isymbol  = '`'
   , iname    = "mist"
   , ifreq    = [("calming mist", 1)]
-  , iflavour = zipFancy [BrGreen]
+  , iflavour = zipPlain [BrGreen]
   , icount   = 8
   , irarity  = [(1, 1)]
   , iverbHit = "sooth"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
   , ieffects = [RefillCalm 2]
-  , ifeature = [toVelocity 5, Fragile, Identified]  -- 1 step, 1 turn
+  , ifeature = [toVelocity 5, Fragile, Blast]  -- 1 step, 1 turn
   , idesc    = "A soothing, gentle cloud."
   , ikit     = []
   }
@@ -153,15 +297,15 @@ odorDistressing = ItemKind
   { isymbol  = '`'
   , iname    = "distressing whiff"
   , ifreq    = [("distressing odor", 1)]
-  , iflavour = zipFancy [BrRed]
-  , icount   = 16
+  , iflavour = zipFancy [BrRed]  -- salmon
+  , icount   = 8
   , irarity  = [(1, 1)]
   , iverbHit = "distress"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
   , ieffects = [RefillCalm (-20)]
-  , ifeature = [toLinger 10, Fragile, Identified]  -- 2 steps, 1 turn
+  , ifeature = [toLinger 10, Fragile, Blast]  -- 2 steps, 1 turn
   , idesc    = "It turns the stomach."
   , ikit     = []
   }
@@ -174,10 +318,10 @@ mistHealing = ItemKind
   , irarity  = [(1, 1)]
   , iverbHit = "revitalize"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [AddShine 1]
   , ieffects = [RefillHP 2]
-  , ifeature = [toVelocity 5, Fragile, Identified]  -- 1 step, 1 turn
+  , ifeature = [toVelocity 5, Fragile, Blast]  -- 1 step, 1 turn
   , idesc    = "It fills the air with light and life."
   , ikit     = []
   }
@@ -185,15 +329,15 @@ mistHealing2 = ItemKind
   { isymbol  = '`'
   , iname    = "mist"
   , ifreq    = [("healing mist 2", 1)]
-  , iflavour = zipFancy [BrGreen]
+  , iflavour = zipPlain [Green]
   , icount   = 8
   , irarity  = [(1, 1)]
   , iverbHit = "revitalize"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [AddShine 2]
   , ieffects = [RefillHP 4]
-  , ifeature = [toVelocity 5, Fragile, Identified]  -- 1 step, 1 turn
+  , ifeature = [toVelocity 5, Fragile, Blast]  -- 1 step, 1 turn
   , idesc    = "At its touch, wounds close and bruises fade."
   , ikit     = []
   }
@@ -201,15 +345,15 @@ mistWounding = ItemKind
   { isymbol  = '`'
   , iname    = "mist"
   , ifreq    = [("wounding mist", 1)]
-  , iflavour = zipFancy [BrRed]
+  , iflavour = zipPlain [BrRed]
   , icount   = 8
   , irarity  = [(1, 1)]
   , iverbHit = "devitalize"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
   , ieffects = [RefillHP (-2)]
-  , ifeature = [toVelocity 5, Fragile, Identified]  -- 1 step, 1 turn
+  , ifeature = [toVelocity 5, Fragile, Blast]  -- 1 step, 1 turn
   , idesc    = "The air itself stings and itches."
   , ikit     = []
   }
@@ -217,34 +361,18 @@ distortion = ItemKind
   { isymbol  = 'v'
   , iname    = "vortex"
   , ifreq    = [("distortion", 1)]
-  , iflavour = zipFancy [White]
-  , icount   = 16
+  , iflavour = zipPlain [White]
+  , icount   = 8  -- braced are immune to Teleport; avoid failure messages
   , irarity  = [(1, 1)]
   , iverbHit = "engulf"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
   , ieffects = [Teleport $ 15 + 1 `d` 10]
-  , ifeature = [toLinger 10, Fragile, Identified]  -- 2 steps, 1 turn
+  , ifeature = [toLinger 10, Lobable, Fragile, Blast]  -- 2 steps, 1 turn
   , idesc    = "The air shifts oddly, as though light is being warped."
   , ikit     = []
   }
-glassPiece = ItemKind  -- when blowing up windows
-  { isymbol  = '*'
-  , iname    = "glass piece"
-  , ifreq    = [("glass piece", 1)]
-  , iflavour = zipPlain [BrBlue]
-  , icount   = 16
-  , irarity  = [(1, 1)]
-  , iverbHit = "cut"
-  , iweight  = 1
-  , idamage  = toDmg 0
-  , iaspects = []
-  , ieffects = [RefillHP (-1)]  -- high velocity, so can't do idamage
-  , ifeature = [toLinger 20, Fragile, Identified]  -- 4 steps, 1 turn
-  , idesc    = "Swift, sharp edges."
-  , ikit     = []
-  }
 smoke = ItemKind  -- when stuff burns out  -- unused
   { isymbol  = '`'
   , iname    = "smoke"
@@ -252,12 +380,12 @@ smoke = ItemKind  -- when stuff burns out  -- unused
   , iflavour = zipPlain [BrBlack]
   , icount   = 16
   , irarity  = [(1, 1)]
-  , iverbHit = "choke"  -- or obscure
+  , iverbHit = "choke"  -- or "obscure"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
   , ieffects = []
-  , ifeature = [toVelocity 20, Fragile, Identified]  -- 4 steps, 2 turns
+  , ifeature = [toVelocity 20, Fragile, Blast]  -- 4 steps, 2 turns
   , idesc    = "Twirling clouds of grey smoke."
   , ikit     = []
   }
@@ -265,15 +393,15 @@ boilingWater = ItemKind
   { isymbol  = '*'
   , iname    = "boiling water"
   , ifreq    = [("boiling water", 1)]
-  , iflavour = zipPlain [BrWhite]
-  , icount   = 16
+  , iflavour = zipPlain [White]
+  , icount   = 18
   , irarity  = [(1, 1)]
   , iverbHit = "boil"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
   , ieffects = [Burn 1]
-  , ifeature = [toVelocity 30, Fragile, Identified]  -- 6 steps, 2 turns
+  , ifeature = [toVelocity 30, Fragile, Blast]  -- 6 steps, 2 turns
   , idesc    = "It bubbles and hisses."
   , ikit     = []
   }
@@ -282,69 +410,54 @@ glue = ItemKind
   , iname    = "hoof glue"
   , ifreq    = [("glue", 1)]
   , iflavour = zipPlain [Cyan]
-  , icount   = 16
+  , icount   = 8  -- Paralyze doesn't stack; avoid failure messages
   , irarity  = [(1, 1)]
   , iverbHit = "glue"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
   , ieffects = [Paralyze 10]
-  , ifeature = [toVelocity 20, Fragile, Identified]  -- 4 steps, 2 turns
+  , ifeature = [toVelocity 20, Fragile, Blast]  -- 4 steps, 2 turns
   , idesc    = "Thick and clinging."
   , ikit     = []
   }
-singleSpark = ItemKind
-  { isymbol  = '`'
-  , iname    = "single spark"
-  , ifreq    = [("single spark", 1)]
-  , iflavour = zipPlain [BrWhite]
-  , icount   = 1
-  , irarity  = [(1, 1)]
-  , iverbHit = "spark"
-  , iweight  = 1
-  , idamage  = toDmg 0
-  , iaspects = [AddShine 4]
-  , ieffects = []
-  , ifeature = [toLinger 5, Fragile, Identified]  -- 1 step, 1 turn
-  , idesc    = "A glowing ember."
-  , ikit     = []
-  }
-spark = ItemKind
-  { isymbol  = '`'
-  , iname    = "spark"
-  , ifreq    = [("spark", 1)]
-  , iflavour = zipPlain [BrWhite]
+waste = ItemKind
+  { isymbol  = '*'
+  , iname    = "waste"
+  , ifreq    = [("waste", 1)]
+  , iflavour = zipPlain [Brown]
   , icount   = 16
   , irarity  = [(1, 1)]
-  , iverbHit = "scorch"
+  , iverbHit = "splosh"
   , iweight  = 1
-  , idamage  = toDmg 0
-  , iaspects = [AddShine 4]
+  , idamage  = 0
+  , iaspects = []
   , ieffects = [Burn 1]
-  , ifeature = [toLinger 10, Fragile, Identified]  -- 2 steps, 1 turn
-  , idesc    = "A flash of fire."
+  , ifeature = [toLinger 10, Fragile, Blast]
+  , idesc    = "Sodden and foul-smelling."
   , ikit     = []
   }
 
--- * Temporary condition blasts strictly matching the aspects
+-- * Temporary condition blasts
 
 -- Almost all have @toLinger 10@, that travels 2 steps in 1 turn.
 -- These are very fast projectiles, not getting into the way of big
 -- actors and not burdening the engine for long.
+-- A few are slower 'mists'.
 
 denseShower = ItemKind
   { isymbol  = '`'
   , iname    = "dense shower"
   , ifreq    = [("dense shower", 1)]
   , iflavour = zipFancy [Green]
-  , icount   = 16
+  , icount   = 12
   , irarity  = [(1, 1)]
   , iverbHit = "strengthen"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
-  , ieffects = [toOrganActorTurn "strengthened" (3 + 1 `d` 3)]
-  , ifeature = [toLinger 10, Fragile, Identified]
+  , ieffects = [toOrganGood "strengthened" (3 + 1 `d` 3)]
+  , ifeature = [toLinger 10, Fragile, Blast]
   , idesc    = "A thick rain of droplets."
   , ikit     = []
   }
@@ -353,14 +466,14 @@ sparseShower = ItemKind
   , iname    = "sparse shower"
   , ifreq    = [("sparse shower", 1)]
   , iflavour = zipFancy [Red]
-  , icount   = 16
+  , icount   = 8
   , irarity  = [(1, 1)]
   , iverbHit = "weaken"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
-  , ieffects = [toOrganGameTurn "weakened" (3 + 1 `d` 3)]
-  , ifeature = [toLinger 10, Fragile, Identified]
+  , ieffects = [toOrganBad "weakened" (3 + 1 `d` 3)]
+  , ifeature = [toLinger 10, Fragile, Blast]
   , idesc    = "Light droplets that cling to clothing."
   , ikit     = []
   }
@@ -368,15 +481,15 @@ protectingBalmMelee = ItemKind
   { isymbol  = '`'
   , iname    = "balm droplet"
   , ifreq    = [("melee protective balm", 1)]
-  , iflavour = zipPlain [Brown]
+  , iflavour = zipFancy [Brown]
   , icount   = 16
   , irarity  = [(1, 1)]
   , iverbHit = "balm"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
-  , ieffects = [toOrganActorTurn "protected from melee" (3 + 1 `d` 3)]
-  , ifeature = [toLinger 10, Fragile, Identified]
+  , ieffects = [toOrganGood "protected from melee" (3 + 1 `d` 3)]
+  , ifeature = [toLinger 10, Fragile, Blast]
   , idesc    = "A thick ointment that hardens the skin."
   , ikit     = []
   }
@@ -389,10 +502,10 @@ protectingBalmRanged = ItemKind
   , irarity  = [(1, 1)]
   , iverbHit = "balm"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
-  , ieffects = [toOrganActorTurn "protected from ranged" (3 + 1 `d` 3)]
-  , ifeature = [toLinger 10, Fragile, Identified]
+  , ieffects = [toOrganGood "protected from ranged" (3 + 1 `d` 3)]
+  , ifeature = [toLinger 10, Fragile, Blast]
   , idesc    = "Grease that protects from flying death."
   , ikit     = []
   }
@@ -400,15 +513,15 @@ vulnerabilityBalm = ItemKind
   { isymbol  = '?'
   , iname    = "PhD defense question"
   , ifreq    = [("PhD defense question", 1)]
-  , iflavour = zipPlain [BrRed]
+  , iflavour = zipFancy [BrRed]
   , icount   = 16
   , irarity  = [(1, 1)]
   , iverbHit = "nag"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
-  , ieffects = [toOrganGameTurn "defenseless" (3 + 1 `d` 3)]
-  , ifeature = [toLinger 10, Fragile, Identified]
+  , ieffects = [toOrganBad "defenseless" (3 + 1 `d` 3)]
+  , ifeature = [toLinger 10, Fragile, Blast]
   , idesc    = "Only the most learned make use of this."
   , ikit     = []
   }
@@ -421,11 +534,11 @@ resolutionDust = ItemKind
   , irarity  = [(1, 1)]
   , iverbHit = "calm"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
-  , ieffects = [toOrganActorTurn "resolute" (3 + 1 `d` 3)]
+  , ieffects = [toOrganGood "resolute" (3 + 1 `d` 3)]
                  -- short enough duration that @calmEnough@ not a big problem
-  , ifeature = [toLinger 10, Fragile, Identified]
+  , ifeature = [toLinger 10, Fragile, Blast]
   , idesc    = "A handful of honest earth, to strengthen the soul."
   , ikit     = []
   }
@@ -433,15 +546,15 @@ hasteSpray = ItemKind
   { isymbol  = '`'
   , iname    = "haste spray"
   , ifreq    = [("haste spray", 1)]
-  , iflavour = zipPlain [BrYellow]
+  , iflavour = zipFancy [BrYellow]
   , icount   = 16
   , irarity  = [(1, 1)]
   , iverbHit = "haste"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
-  , ieffects = [toOrganActorTurn "hasted" (3 + 1 `d` 3)]
-  , ifeature = [toLinger 10, Fragile, Identified]
+  , ieffects = [toOrganGood "hasted" (3 + 1 `d` 3)]
+  , ifeature = [toLinger 10, Fragile, Blast]
   , idesc    = "A quick spurt."
   , ikit     = []
   }
@@ -453,11 +566,12 @@ slownessMist = ItemKind
   , icount   = 8
   , irarity  = [(1, 1)]
   , iverbHit = "slow"
-  , iweight  = 1
-  , idamage  = toDmg 0
+  , iweight  = 0
+  , idamage  = 0
   , iaspects = []
-  , ieffects = [toOrganGameTurn "slowed" (3 + 1 `d` 3)]
-  , ifeature = [toVelocity 5, Fragile, Identified]  -- 1 step, 1 turn, mist
+  , ieffects = [toOrganBad "slowed" (3 + 1 `d` 3)]
+  , ifeature = [toVelocity 5, Fragile, Blast]
+                 -- 1 step, 1 turn, mist, slow
   , idesc    = "Clammy fog, making each movement an effort."
   , ikit     = []
   }
@@ -465,15 +579,15 @@ eyeDrop = ItemKind
   { isymbol  = '`'
   , iname    = "eye drop"
   , ifreq    = [("eye drop", 1)]
-  , iflavour = zipPlain [BrCyan]
+  , iflavour = zipFancy [BrCyan]
   , icount   = 16
   , irarity  = [(1, 1)]
   , iverbHit = "cleanse"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
-  , ieffects = [toOrganActorTurn "far-sighted" (3 + 1 `d` 3)]
-  , ifeature = [toLinger 10, Fragile, Identified]
+  , ieffects = [toOrganGood "far-sighted" (3 + 1 `d` 3)]
+  , ifeature = [toLinger 10, Fragile, Blast]
   , idesc    = "Not to be taken orally."
   , ikit     = []
   }
@@ -481,15 +595,15 @@ ironFiling = ItemKind
   { isymbol  = '`'
   , iname    = "iron filing"
   , ifreq    = [("iron filing", 1)]
-  , iflavour = zipPlain [Brown]
+  , iflavour = zipPlain [Red]
   , icount   = 16
   , irarity  = [(1, 1)]
   , iverbHit = "blind"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
-  , ieffects = [toOrganActorTurn "blind" (10 + 1 `d` 10)]
-  , ifeature = [toLinger 10, Fragile, Identified]
+  , ieffects = [toOrganBad "blind" (10 + 1 `d` 10)]
+  , ifeature = [toLinger 10, Fragile, Blast]
   , idesc    = "A shaving of bright metal."
   , ikit     = []
   }
@@ -497,15 +611,15 @@ smellyDroplet = ItemKind
   { isymbol  = '`'
   , iname    = "smelly droplet"
   , ifreq    = [("smelly droplet", 1)]
-  , iflavour = zipPlain [Blue]
+  , iflavour = zipFancy [Blue]
   , icount   = 16
   , irarity  = [(1, 1)]
   , iverbHit = "sensitize"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
-  , ieffects = [toOrganActorTurn "keen-smelling" (3 + 1 `d` 3)]
-  , ifeature = [toLinger 10, Fragile, Identified]
+  , ieffects = [toOrganGood "keen-smelling" (3 + 1 `d` 3)]
+  , ifeature = [toLinger 10, Fragile, Blast]
   , idesc    = "A viscous lump that stains the skin."
   , ikit     = []
   }
@@ -513,66 +627,47 @@ eyeShine = ItemKind
   { isymbol  = '`'
   , iname    = "eye shine"
   , ifreq    = [("eye shine", 1)]
-  , iflavour = zipPlain [Cyan]
+  , iflavour = zipFancy [Cyan]
   , icount   = 16
   , irarity  = [(1, 1)]
   , iverbHit = "smear"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
-  , ieffects = [toOrganActorTurn "shiny-eyed" (3 + 1 `d` 3)]
-  , ifeature = [toLinger 10, Fragile, Identified]
+  , ieffects = [toOrganGood "shiny-eyed" (3 + 1 `d` 3)]
+  , ifeature = [toLinger 10, Fragile, Blast]
   , idesc    = "They almost glow in the dark."
   , ikit     = []
   }
-
--- * Assorted temporary condition blasts or related (also, matching flasks)
-
 whiskeySpray = ItemKind
   { isymbol  = '`'
   , iname    = "whiskey spray"
   , ifreq    = [("whiskey spray", 1)]
-  , iflavour = zipPlain [Brown]
+  , iflavour = zipFancy [Brown]
   , icount   = 16
   , irarity  = [(1, 1)]
   , iverbHit = "inebriate"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
-  , ieffects = [toOrganActorTurn "drunk" (3 + 1 `d` 3)]
-  , ifeature = [toLinger 10, Fragile, Identified]
+  , ieffects = [toOrganGood "drunk" (3 + 1 `d` 3)]
+  , ifeature = [toLinger 10, Fragile, Blast]
   , idesc    = "It burns in the best way."
   , ikit     = []
   }
-waste = ItemKind
-  { isymbol  = '*'
-  , iname    = "waste"
-  , ifreq    = [("waste", 1)]
-  , iflavour = zipPlain [Brown]
-  , icount   = 16
-  , irarity  = [(1, 1)]
-  , iverbHit = "splosh"
-  , iweight  = 1
-  , idamage  = toDmg 0
-  , iaspects = []
-  , ieffects = [Burn 1]
-  , ifeature = [toLinger 10, Fragile, Identified]
-  , idesc    = "Sodden and foul-smelling."
-  , ikit     = []
-  }
 youthSprinkle = ItemKind
   { isymbol  = '`'
   , iname    = "youth sprinkle"
   , ifreq    = [("youth sprinkle", 1)]
-  , iflavour = zipPlain [BrGreen]
+  , iflavour = zipFancy [BrGreen]
   , icount   = 16
   , irarity  = [(1, 1)]
   , iverbHit = "sprinkle"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
-  , ieffects = [toOrganNone "regenerating"]
-  , ifeature = [toLinger 10, Fragile, Identified]
+  , ieffects = [toOrganNoTimer "regenerating"]
+  , ifeature = [toLinger 10, Fragile, Blast]
   , idesc    = "Bright and smelling of the Spring."
   , ikit     = []
   }
@@ -580,15 +675,15 @@ poisonCloud = ItemKind
   { isymbol  = '`'
   , iname    = "poison cloud"
   , ifreq    = [("poison cloud", 1)]
-  , iflavour = zipPlain [BrMagenta]
+  , iflavour = zipFancy [BrMagenta]
   , icount   = 16
   , irarity  = [(1, 1)]
   , iverbHit = "poison"
-  , iweight  = 1
-  , idamage  = toDmg 0
+  , iweight  = 0
+  , idamage  = 0
   , iaspects = []
-  , ieffects = [toOrganNone "poisoned"]
-  , ifeature = [toVelocity 10, Fragile, Identified]  -- 2 steps, 2 turns
+  , ieffects = [toOrganNoTimer "poisoned"]
+  , ifeature = [toVelocity 10, Fragile, Blast]  -- 2 steps, 2 turns
   , idesc    = "Choking gas that stings the eyes."
   , ikit     = []
   }
@@ -596,15 +691,15 @@ mistAntiSlow = ItemKind
   { isymbol  = '`'
   , iname    = "mist"
   , ifreq    = [("anti-slow mist", 1)]
-  , iflavour = zipPlain [BrYellow]
+  , iflavour = zipFancy [BrYellow]
   , icount   = 8
   , irarity  = [(1, 1)]
   , iverbHit = "propel"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
   , ieffects = [DropItem 1 1 COrgan "slowed"]
-  , ifeature = [toVelocity 5, Fragile, Identified]  -- 1 step, 1 turn
+  , ifeature = [toVelocity 5, Fragile, Blast]  -- 1 step, 1 turn
   , idesc    = "A cleansing rain."
   , ikit     = []
   }
@@ -612,15 +707,15 @@ mistAntidote = ItemKind
   { isymbol  = '`'
   , iname    = "mist"
   , ifreq    = [("antidote mist", 1)]
-  , iflavour = zipPlain [BrBlue]
+  , iflavour = zipFancy [BrBlue]
   , icount   = 8
   , irarity  = [(1, 1)]
   , iverbHit = "cure"
   , iweight  = 1
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = []
   , ieffects = [DropItem 1 maxBound COrgan "poisoned"]
-  , ifeature = [toVelocity 5, Fragile, Identified]  -- 1 step, 1 turn
+  , ifeature = [toVelocity 5, Fragile, Blast]  -- 1 step, 1 turn
   , idesc    = "Washes away death's dew."
   , ikit     = []
   }
