@@ -11,16 +11,17 @@ import Game.LambdaHack.Common.Ability
 import Game.LambdaHack.Common.Color
 import Game.LambdaHack.Common.Dice
 import Game.LambdaHack.Common.Flavour
+import Game.LambdaHack.Common.ItemAspect (Aspect (..))
 import Game.LambdaHack.Common.Misc
 import Game.LambdaHack.Content.ItemKind
 
 organs :: [ItemKind]
 organs =
-  [fist, foot, hookedClaw, smallClaw, snout, smallJaw, jaw, largeJaw, horn, tentacle, thorn, boilingFissure, arsenicFissure, sulfurFissure, beeSting, sting, venomTooth, venomFang, screechingBeak, largeTail, armoredSkin, eye2, eye3, eye4, eye5, eye6, eye7, eye8, vision4, vision5, vision6, vision7, vision8, vision10, vision12, vision14, vision16, nostril, insectMortality, sapientBrain, animalBrain, speedGland2, speedGland4, speedGland6, speedGland8, speedGland10, scentGland, boilingVent, arsenicVent, sulfurVent, bonusHP]
+  [fist, foot, hookedClaw, smallClaw, snout, smallJaw, jaw, largeJaw, antler, horn, rhinoHorn, tentacle, thorn, boilingFissure, arsenicFissure, sulfurFissure, beeSting, sting, venomTooth, venomFang, screechingBeak, largeTail, armoredSkin, eye2, eye3, eye4, eye5, eye6, eye7, eye8, vision4, vision5, vision6, vision7, vision8, vision10, vision12, vision14, vision16, nostril, insectMortality, sapientBrain, animalBrain, speedGland2, speedGland4, speedGland6, speedGland8, speedGland10, scentGland, boilingVent, arsenicVent, sulfurVent, bonusHP, impressed]
   -- LH-specific
   ++ [tooth, lash, noseTip, lip, torsionRight, torsionLeft, pupil]
 
-fist,    foot, hookedClaw, smallClaw, snout, smallJaw, jaw, largeJaw, horn, tentacle, thorn, boilingFissure, arsenicFissure, sulfurFissure, beeSting, sting, venomTooth, venomFang, screechingBeak, largeTail, armoredSkin, eye2, eye3, eye4, eye5, eye6, eye7, eye8, vision4, vision5, vision6, vision7, vision8, vision10, vision12, vision14, vision16, nostril, insectMortality, sapientBrain, animalBrain, speedGland2, speedGland4, speedGland6, speedGland8, speedGland10, scentGland, boilingVent, arsenicVent, sulfurVent, bonusHP :: ItemKind
+fist,    foot, hookedClaw, smallClaw, snout, smallJaw, jaw, largeJaw, antler, horn, rhinoHorn, tentacle, thorn, boilingFissure, arsenicFissure, sulfurFissure, beeSting, sting, venomTooth, venomFang, screechingBeak, largeTail, armoredSkin, eye2, eye3, eye4, eye5, eye6, eye7, eye8, vision4, vision5, vision6, vision7, vision8, vision10, vision12, vision14, vision16, nostril, insectMortality, sapientBrain, animalBrain, speedGland2, speedGland4, speedGland6, speedGland8, speedGland10, scentGland, boilingVent, arsenicVent, sulfurVent, bonusHP, impressed :: ItemKind
 -- LH-specific
 tooth, lash, noseTip, lip, torsionRight, torsionLeft, pupil :: ItemKind
 
@@ -37,10 +38,10 @@ fist = ItemKind
   , irarity  = [(1, 1)]
   , iverbHit = "punch"
   , iweight  = 2000
-  , idamage  = toDmg $ 4 `d` 1
+  , idamage  = 4 `d` 1
   , iaspects = []
   , ieffects = []
-  , ifeature = [Durable, Identified, Meleeable]
+  , ifeature = [Durable, Meleeable]
   , idesc    = "Simple but effective."
   , ikit     = []
   }
@@ -48,27 +49,27 @@ foot = fist
   { iname    = "foot"
   , ifreq    = [("foot", 50)]
   , iverbHit = "kick"
-  , idamage  = toDmg $ 4 `d` 1
+  , idamage  = 4 `d` 1
   , idesc    = "A weapon you can still use if disarmed."
   }
 
--- * Universal weapon organs
+-- * Other weapon organs
 
 hookedClaw = fist
   { iname    = "hooked claw"
   , ifreq    = [("hooked claw", 50)]
   , icount   = 2  -- even if more, only the fore claws used for fighting
   , iverbHit = "hook"
-  , idamage  = toDmg $ 2 `d` 1
-  , iaspects = [Timeout $ 4 + 1 `d` 4]
-  , ieffects = [Recharging (toOrganGameTurn "slowed" 2)]
+  , idamage  = 2 `d` 1
+  , iaspects = [Timeout $ 12 - 1 `dL` 3]
+  , ieffects = [Recharging (toOrganBad "slowed" 2)]
   , idesc    = "A curved talon."
   }
 smallClaw = fist
   { iname    = "small claw"
   , ifreq    = [("small claw", 50)]
   , iverbHit = "slash"
-  , idamage  = toDmg $ 2 `d` 1
+  , idamage  = 2 `d` 1
   , idesc    = "A pearly spike."
   }
 snout = fist
@@ -76,7 +77,7 @@ snout = fist
   , ifreq    = [("snout", 10)]
   , icount   = 1
   , iverbHit = "bite"
-  , idamage  = toDmg $ 2 `d` 1
+  , idamage  = 2 `d` 1
   , idesc    = "Sensitive and wide-nostrilled."
   }
 smallJaw = fist
@@ -84,7 +85,7 @@ smallJaw = fist
   , ifreq    = [("small jaw", 20)]
   , icount   = 1
   , iverbHit = "rip"
-  , idamage  = toDmg $ 3 `d` 1
+  , idamage  = 3 `d` 1
   , idesc    = "Filled with small, even teeth."
   }
 jaw = fist
@@ -92,7 +93,7 @@ jaw = fist
   , ifreq    = [("jaw", 20)]
   , icount   = 1
   , iverbHit = "rip"
-  , idamage  = toDmg $ 5 `d` 1
+  , idamage  = 5 `d` 1
   , idesc    = "Delivers a powerful bite."
   }
 largeJaw = fist
@@ -100,27 +101,44 @@ largeJaw = fist
   , ifreq    = [("large jaw", 100)]
   , icount   = 1
   , iverbHit = "crush"
-  , idamage  = toDmg $ 10 `d` 1
+  , idamage  = 10 `d` 1
   , idesc    = "Enough to swallow anything in a single gulp."
   }
+antler = fist
+  { iname    = "antler"
+  , ifreq    = [("antler", 100)]
+  , icount   = 2
+  , iverbHit = "ram"
+  , idamage  = 4 `d` 1
+  , iaspects = [Timeout $ 3 + (1 `d` 3) * 3, AddArmorMelee 10]  -- bonus doubled
+  , ieffects = [Recharging (PushActor (ThrowMod 100 50))]  -- 1 step, slow
+  , idesc    = ""
+  }
 horn = fist
   { iname    = "horn"
-  , ifreq    = [("horn", 20)]
+  , ifreq    = [("horn", 100)]
   , icount   = 2
   , iverbHit = "impale"
-  , idamage  = toDmg $ 6 `d` 1
-  , iaspects = [AddHurtMelee 20]
-  , idesc    = "Sharp and solid, for defence or attack."
+  , idamage  = 5 `d` 1
+  , iaspects = [AddHurtMelee 10, AddArmorMelee 10]  -- bonus doubled
+  , idesc    = "Sharp and long, for defence or attack."
+  }
+rhinoHorn = fist
+  { iname    = "ugly horn"  -- made of keratin, unlike real horns
+  , ifreq    = [("rhino horn", 20)]
+  , icount   = 1  -- single, unlike real horns
+  , iverbHit = "impale"
+  , idamage  = 5 `d` 1
+  , iaspects = [Timeout 7, AddHurtMelee 20]
+  , ieffects = [Recharging Impress]  -- the owner is a mid-boss, after all
+  , idesc    = "Very solid, considering it has the same composition as fingernails."
   }
-
--- * Special weapon organs
-
 tentacle = fist
   { iname    = "tentacle"
   , ifreq    = [("tentacle", 50)]
   , icount   = 4
   , iverbHit = "slap"
-  , idamage  = toDmg $ 4 `d` 1
+  , idamage  = 4 `d` 1
   , idesc    = "Damp and dextrous."
   }
 thorn = fist
@@ -128,8 +146,8 @@ thorn = fist
   , ifreq    = [("thorn", 100)]
   , icount   = 2 + 1 `d` 3
   , iverbHit = "impale"
-  , idamage  = toDmg $ 1 `d` 3
-  , ifeature = [Identified, Meleeable]  -- not Durable
+  , idamage  = 2 `d` 1
+  , ifeature = [Meleeable]  -- not Durable
   , idesc    = "Sharp yet brittle."
   }
 boilingFissure = fist
@@ -137,25 +155,25 @@ boilingFissure = fist
   , ifreq    = [("boiling fissure", 100)]
   , icount   = 5 + 1 `d` 5
   , iverbHit = "hiss at"
-  , idamage  = toDmg $ 1 `d` 1
+  , idamage  = 1 `d` 1
   , iaspects = [AddHurtMelee 20]  -- decreasing as count decreases
-  , ieffects = [InsertMove $ 1 `d` 3]
-  , ifeature = [Identified, Meleeable]  -- not Durable
+  , ieffects = [DropItem 1 1 COrgan "condition"]  -- useful; limited
+  , ifeature = [Meleeable]  -- not Durable
   , idesc    = "A deep crack to the underworld."
   }
 arsenicFissure = boilingFissure
   { iname    = "fissure"
   , ifreq    = [("arsenic fissure", 100)]
   , icount   = 3 + 1 `d` 3
-  , idamage  = toDmg $ 2 `d` 1
-  , ieffects = [toOrganGameTurn "weakened" (2 + 1 `d` 2)]
+  , idamage  = 2 `d` 1
+  , ieffects = []  -- nothing interesting fits the weaken/poison biological data
   , idesc    = ""
   }
 sulfurFissure = boilingFissure
   { iname    = "fissure"
   , ifreq    = [("sulfur fissure", 100)]
   , icount   = 2 + 1 `d` 2
-  , idamage  = toDmg 0
+  , idamage  = 0  -- heal not via (negative) idamage, for armour would block it
   , ieffects = [RefillHP 5]
   , idesc    = ""
   }
@@ -164,10 +182,10 @@ beeSting = fist
   , ifreq    = [("bee sting", 100)]
   , icount   = 1
   , iverbHit = "sting"
-  , idamage  = toDmg 0
-  , iaspects = [AddArmorMelee 90, AddArmorRanged 45]
+  , idamage  = 0
+  , iaspects = [AddArmorMelee 200, AddArmorRanged 45]
   , ieffects = [Paralyze 6, RefillHP 4]
-  , ifeature = [Identified, Meleeable]  -- not Durable
+  , ifeature = [Meleeable]  -- not Durable
   , idesc    = "Painful, but beneficial."
   }
 sting = fist
@@ -175,8 +193,8 @@ sting = fist
   , ifreq    = [("sting", 100)]
   , icount   = 1
   , iverbHit = "sting"
-  , idamage  = toDmg $ 1 `d` 1
-  , iaspects = [Timeout $ 1 + 1 `d` 5, AddHurtMelee 40]
+  , idamage  = 1 `d` 1
+  , iaspects = [Timeout $ 10 - 1 `dL` 4, AddHurtMelee 40]
   , ieffects = [Recharging (Paralyze 4)]
   , idesc    = "Painful, debilitating and harmful."
   }
@@ -185,9 +203,9 @@ venomTooth = fist
   , ifreq    = [("venom tooth", 100)]
   , icount   = 2
   , iverbHit = "bite"
-  , idamage  = toDmg $ 2 `d` 1
-  , iaspects = [Timeout $ 5 + 1 `d` 3]
-  , ieffects = [Recharging (toOrganGameTurn "slowed" (3 + 1 `d` 3))]
+  , idamage  = 2 `d` 1
+  , iaspects = [Timeout $ 7 - 1 `dL` 3]
+  , ieffects = [Recharging (toOrganBad "slowed" (3 + 1 `d` 3))]
   , idesc    = "A chilling numbness spreads from its bite."
   }
 venomFang = fist
@@ -195,9 +213,9 @@ venomFang = fist
   , ifreq    = [("venom fang", 100)]
   , icount   = 2
   , iverbHit = "bite"
-  , idamage  = toDmg $ 2 `d` 1
-  , iaspects = [Timeout $ 7 + 1 `d` 5]
-  , ieffects = [Recharging (toOrganNone "poisoned")]
+  , idamage  = 2 `d` 1
+  , iaspects = [Timeout $ 10 - 1 `dL` 4]
+  , ieffects = [Recharging (toOrganNoTimer "poisoned")]
   , idesc    = "Dripping with deadly venom."
   }
 screechingBeak = fist
@@ -205,9 +223,9 @@ screechingBeak = fist
   , ifreq    = [("screeching beak", 100)]
   , icount   = 1
   , iverbHit = "peck"
-  , idamage  = toDmg $ 2 `d` 1
-  , iaspects = [Timeout $ 5 + 1 `d` 5]
-  , ieffects = [Recharging $ Summon "scavenger" $ 1 + 1 `dl` 2]
+  , idamage  = 2 `d` 1
+  , iaspects = [Timeout $ 7 - 1 `dL` 3]
+  , ieffects = [Recharging $ Summon "scavenger" $ 1 `dL` 3]
   , idesc    = "Both a weapon and a beacon, calling more scavengers to the meal."
   }
 largeTail = fist
@@ -215,9 +233,9 @@ largeTail = fist
   , ifreq    = [("large tail", 50)]
   , icount   = 1
   , iverbHit = "knock"
-  , idamage  = toDmg $ 6 `d` 1
+  , idamage  = 7 `d` 1
   , iaspects = [Timeout $ 1 + 1 `d` 3, AddHurtMelee 20]
-  , ieffects = [Recharging (PushActor (ThrowMod 400 25))]
+  , ieffects = [Recharging (PushActor (ThrowMod 400 50))]  -- 2 steps
   , idesc    = "Slow but heavy."
   }
 
@@ -234,10 +252,10 @@ armoredSkin = ItemKind
   , irarity  = [(1, 1)]
   , iverbHit = "bash"
   , iweight  = 2000
-  , idamage  = toDmg 0
+  , idamage  = 0
   , iaspects = [AddArmorMelee 30, AddArmorRanged 15]
   , ieffects = []
-  , ifeature = [Durable, Identified]
+  , ifeature = [Durable]
   , idesc    = "Homemade armour is just as good."
   , ikit     = []
   }
@@ -292,8 +310,9 @@ insectMortality = armoredSkin
   { iname    = "insect mortality"
   , ifreq    = [("insect mortality", 100)]
   , iverbHit = "age"
-  , iaspects = [Timeout $ 40 + 1 `d` 10]
-  , ieffects = [Periodic, Recharging (RefillHP (-1))]
+  , iaspects = [Timeout $ 30 + (1 `d` 2) * 10]
+  , ieffects = [Recharging (RefillHP (-1))]
+  , ifeature = [Periodic] ++ ifeature armoredSkin
   , idesc    = ""
   }
 sapientBrain = armoredSkin
@@ -321,7 +340,8 @@ speedGland n = armoredSkin
   , iverbHit = "spit at"
   , iaspects = [ AddSpeed $ intToDice n
                , Timeout $ intToDice $ 100 `div` n ]
-  , ieffects = [Periodic, Recharging (RefillHP 1)]
+  , ieffects = [Recharging (RefillHP 1)]
+  , ifeature = [Periodic] ++ ifeature armoredSkin
   , idesc    = ""
   }
 speedGland2 = speedGland 2
@@ -332,10 +352,13 @@ speedGland10 = speedGland 10
 scentGland = armoredSkin
   { iname    = "scent gland"
   , ifreq    = [("scent gland", 100)]
+  , icount   = 2 + 1 `d` 3  -- runs out
   , iverbHit = "spray at"
-  , iaspects = [Timeout $ (10 + 1 `d` 2) * 5 ]
-  , ieffects = [ Periodic, Recharging (Explode "distressing odor")
+  , iaspects = [Timeout $ (1 `d` 3) * 10]
+  , ieffects = [ Recharging (Temporary "look spent")
+               , Recharging (Explode "distressing odor")
                , Recharging ApplyPerfume ]
+  , ifeature = [Periodic]  -- not Durable
   , idesc    = ""
   }
 boilingVent = armoredSkin
@@ -344,9 +367,9 @@ boilingVent = armoredSkin
   , iflavour = zipPlain [Blue]
   , iverbHit = "menace"
   , iaspects = [Timeout $ (2 + 1 `d` 2) * 5]
-  , ieffects = [Periodic
-               , Recharging (Explode "boiling water")
+  , ieffects = [ Recharging (Explode "boiling water")
                , Recharging (RefillHP 2) ]
+  , ifeature = [Periodic] ++ ifeature armoredSkin
   , idesc    = ""
   }
 arsenicVent = armoredSkin
@@ -355,9 +378,9 @@ arsenicVent = armoredSkin
   , iflavour = zipPlain [Cyan]
   , iverbHit = "menace"
   , iaspects = [Timeout $ (2 + 1 `d` 2) * 5]
-  , ieffects = [ Periodic
-               , Recharging (Explode "sparse shower")
+  , ieffects = [ Recharging (Explode "sparse shower")
                , Recharging (RefillHP 2) ]
+  , ifeature = [Periodic] ++ ifeature armoredSkin
   , idesc    = ""
   }
 sulfurVent = armoredSkin
@@ -366,21 +389,37 @@ sulfurVent = armoredSkin
   , iflavour = zipPlain [BrYellow]
   , iverbHit = "menace"
   , iaspects = [Timeout $ (2 + 1 `d` 2) * 5]
-  , ieffects = [ Periodic
-               , Recharging (Explode "dense shower")
+  , ieffects = [ Recharging (Explode "dense shower")
                , Recharging (RefillHP 2) ]
+  , ifeature = [Periodic] ++ ifeature armoredSkin
   , idesc    = ""
   }
+
+-- * Special
+
 bonusHP = armoredSkin
-  { isymbol  = '+'
+  { isymbol  = 'H'  -- '+' reserved for conditions
   , iname    = "bonus HP"
   , iflavour = zipPlain [BrBlue]
   , ifreq    = [("bonus HP", 1)]
   , iverbHit = "intimidate"
-  , iweight  = 1  -- weight 0 reserved for tmp organs
+  , iweight  = 0
   , iaspects = [AddMaxHP 1]
   , idesc    = ""
   }
+impressed = armoredSkin
+  { isymbol  = '!'
+  , iname    = "impressed"
+  , iflavour = zipPlain [BrRed]
+  , ifreq    = [("impressed", 1), ("condition", 1)]
+  , iverbHit = "confuse"
+  , iweight  = 0
+  , iaspects = [AddMaxCalm (-1)]  -- to help player notice on main screen
+                                  -- and to count as bad condition
+  , ieffects = [OnSmash $ tmpNoLonger "impressed"]  -- not @Periodic@
+  , ifeature = [Fragile, Durable]  -- hack: destroy on drop
+  , idesc    = ""
+  }
 
 -- * LH-specific
 
@@ -389,7 +428,7 @@ tooth = fist
   , ifreq    = [("tooth", 20)]
   , icount   = 3
   , iverbHit = "nail"
-  , idamage  = toDmg $ 2 `d` 1
+  , idamage  = 2 `d` 1
   , idesc    = ""
   }
 lash = fist
@@ -397,7 +436,7 @@ lash = fist
   , ifreq    = [("lash", 100)]
   , icount   = 1
   , iverbHit = "lash"
-  , idamage  = toDmg $ 3 `d` 1
+  , idamage  = 3 `d` 1
   , idesc    = ""
   }
 noseTip = fist
@@ -405,7 +444,7 @@ noseTip = fist
   , ifreq    = [("nose tip", 50)]
   , icount   = 1
   , iverbHit = "poke"
-  , idamage  = toDmg $ 2 `d` 1
+  , idamage  = 2 `d` 1
   , idesc    = ""
   }
 lip = fist
@@ -413,9 +452,9 @@ lip = fist
   , ifreq    = [("lip", 10)]
   , icount   = 1
   , iverbHit = "lap"
-  , idamage  = toDmg $ 1 `d` 1
-  , iaspects = [Timeout $ 3 + 1 `d` 3]
-  , ieffects = [Recharging (toOrganGameTurn "weakened" (2 + 1 `d` 2))]
+  , idamage  = 1 `d` 1
+  , iaspects = [Timeout $ 3 + 1 `d` 2]
+  , ieffects = [Recharging (toOrganBad "weakened" (2 + 1 `dL` 3))]
   , idesc    = ""
   }
 torsionRight = fist
@@ -423,9 +462,9 @@ torsionRight = fist
   , ifreq    = [("right torsion", 100)]
   , icount   = 1
   , iverbHit = "twist"
-  , idamage  = toDmg $ 13 `d` 1
+  , idamage  = 13 `d` 1
   , iaspects = [Timeout $ 5 + 1 `d` 5, AddHurtMelee 20]
-  , ieffects = [Recharging (toOrganGameTurn "slowed" (3 + 1 `d` 3))]
+  , ieffects = [Recharging (toOrganBad "slowed" (3 + 1 `d` 3))]
   , idesc    = ""
   }
 torsionLeft = fist
@@ -433,9 +472,9 @@ torsionLeft = fist
   , ifreq    = [("left torsion", 100)]
   , icount   = 1
   , iverbHit = "twist"
-  , idamage  = toDmg $ 13 `d` 1
+  , idamage  = 13 `d` 1
   , iaspects = [Timeout $ 5 + 1 `d` 5, AddHurtMelee 20]
-  , ieffects = [Recharging (toOrganGameTurn "weakened" (3 + 1 `d` 3))]
+  , ieffects = [Recharging (toOrganBad "weakened" (3 + 1 `d` 3))]
   , idesc    = ""
   }
 pupil = fist
@@ -443,9 +482,9 @@ pupil = fist
   , ifreq    = [("pupil", 100)]
   , icount   = 1
   , iverbHit = "gaze at"
-  , idamage  = toDmg $ 1 `d` 1
-  , iaspects = [AddSight 12, Timeout $ 5 + 1 `d` 5]
-  , ieffects = [ Recharging (DropItem 1 maxBound COrgan "temporary condition")
+  , idamage  = 1 `d` 1
+  , iaspects = [AddSight 12, Timeout 12]
+  , ieffects = [ Recharging (DropItem 1 maxBound COrgan "condition")
                , Recharging $ RefillCalm (-10)
                ]
   , idesc    = ""
