@@ -1,6 +1,6 @@
 -- | Game mode definitions.
 module Content.ModeKind
-  ( cdefs
+  ( content
   ) where
 
 import Prelude ()
@@ -10,21 +10,14 @@ import Game.LambdaHack.Common.Prelude
 import qualified Data.IntMap.Strict as IM
 
 import Content.ModeKindPlayer
-import Game.LambdaHack.Common.ContentDef
 import Game.LambdaHack.Common.Dice
 import Game.LambdaHack.Content.ModeKind
 
-cdefs :: ContentDef ModeKind
-cdefs = ContentDef
-  { getSymbol = msymbol
-  , getName = mname
-  , getFreq = mfreq
-  , validateSingle = validateSingleModeKind
-  , validateAll = validateAllModeKind
-  , content = contentFromList
-      [raid, brawl, shootout, escape, zoo, ambush, crawl, crawlSurvival, safari, safariSurvival, battle, battleSurvival, defense, screensaverRaid, screensaverBrawl, screensaverShootout, screensaverEscape, screensaverZoo, screensaverAmbush, screensaverCrawl, screensaverSafari]
-  }
-raid,        brawl, shootout, escape, zoo, ambush, crawl, crawlSurvival, safari, safariSurvival, battle, battleSurvival, defense, screensaverRaid, screensaverBrawl, screensaverShootout, screensaverEscape, screensaverZoo, screensaverAmbush, screensaverCrawl, screensaverSafari :: ModeKind
+content :: [ModeKind]
+content =
+  [raid, brawl, shootout, escape, zoo, ambush, crawl, crawlEmpty, crawlSurvival, safari, safariSurvival, battle, battleSurvival, defense, defenseEmpty, screensaverRaid, screensaverBrawl, screensaverShootout, screensaverEscape, screensaverZoo, screensaverAmbush, screensaverCrawl, screensaverSafari]
+
+raid,    brawl, shootout, escape, zoo, ambush, crawl, crawlEmpty, crawlSurvival, safari, safariSurvival, battle, battleSurvival, defense, defenseEmpty, screensaverRaid, screensaverBrawl, screensaverShootout, screensaverEscape, screensaverZoo, screensaverAmbush, screensaverCrawl, screensaverSafari :: ModeKind
 
 -- What other symmetric (two only-one-moves factions) and asymmetric vs crowd
 -- scenarios make sense (e.g., are good for a tutorial or for standalone
@@ -115,7 +108,7 @@ crawl = ModeKind
   , mfreq   = [("crawl", 1), ("campaign scenario", 1)]
   , mroster = rosterCrawl
   , mcaves  = cavesCrawl
-  , mdesc   = "Enjoy the peaceful seclusion of these cold austere tunnels, but don't let wanton curiosity, greed and the ever-creeping abstraction madness keep you down there for too long. If find survivors (whole or perturbed or segmented) of the past scientific missions, exercise extreme caution and engage or ignore at your discretion."
+  , mdesc   = "Enjoy the peaceful seclusion of these cold austere tunnels, but don't let wanton curiosity, greed and the ever-creeping abstraction madness keep you down there for too long. If you find survivors (whole or perturbed or segmented) of the past scientific missions, exercise extreme caution and engage or ignore at your discretion."
   }
 
 safari = ModeKind  -- easter egg available only via screensaver
@@ -129,6 +122,15 @@ safari = ModeKind  -- easter egg available only via screensaver
 
 -- * Testing modes
 
+crawlEmpty = ModeKind
+  { msymbol = 'c'
+  , mname   = "crawl empty"
+  , mfreq   = [("crawl empty", 1)]
+  , mroster = rosterCrawlEmpty
+  , mcaves  = cavesCrawl
+  , mdesc   = "Enjoy the free space."
+  }
+
 crawlSurvival = ModeKind
   { msymbol = 'd'
   , mname   = "crawl survival"
@@ -174,6 +176,15 @@ defense = ModeKind  -- perhaps a real scenario in the future
   , mdesc   = "Don't let human interlopers defile your abstract secrets and flee unpunished!"
   }
 
+defenseEmpty = ModeKind
+  { msymbol = 'e'
+  , mname   = "defense empty"
+  , mfreq   = [("defense empty", 1)]
+  , mroster = rosterDefenseEmpty
+  , mcaves  = cavesCrawl
+  , mdesc   = "Lord over."
+  }
+
 -- * Screensaver modes
 
 screensave :: AutoLeader -> Roster -> Roster
@@ -232,7 +243,7 @@ screensaverSafari = safari
               screensave (AutoLeader False True) rosterSafari
   }
 
-rosterRaid, rosterBrawl, rosterShootout, rosterEscape, rosterZoo, rosterAmbush, rosterCrawl, rosterCrawlSurvival, rosterSafari, rosterSafariSurvival, rosterBattle, rosterBattleSurvival, rosterDefense :: Roster
+rosterRaid, rosterBrawl, rosterShootout, rosterEscape, rosterZoo, rosterAmbush, rosterCrawl, rosterCrawlEmpty, rosterCrawlSurvival, rosterSafari, rosterSafariSurvival, rosterBattle, rosterBattleSurvival, rosterDefense, rosterDefenseEmpty :: Roster
 
 rosterRaid = Roster
   { rosterList = [ ( playerHero {fhiCondPoly = hiRaid}
@@ -286,6 +297,7 @@ rosterEscape = Roster
                    , [(-7, 1, "scout hero"), (-7, 2, "escapist hero")] )
                  , ( playerAntiHero { fname = "Indigo Researcher"
                                     , fcanEscape = False  -- start on escape
+                                    , fneverEmpty = False  -- loot after killing
                                     , fhiCondPoly = hiDweller }
                    , [(-7, 1, "scout hero"), (-7, 7, "ambusher hero")] )
                  , (playerHorror, []) ]
@@ -333,6 +345,13 @@ rosterCrawl = Roster
                   , ("Explorer", "Animal Kingdom") ]
   , rosterAlly = [("Monster Hive", "Animal Kingdom")] }
 
+rosterCrawlEmpty = Roster
+  { rosterList = [ ( playerHero
+                   , [(-1, 1, "hero")] )
+                 , (playerHorror, []) ]  -- for summoned monsters
+  , rosterEnemy = []
+  , rosterAlly = [] }
+
 rosterCrawlSurvival = rosterCrawl
   { rosterList = [ ( playerHero { fleaderMode =
                                     LeaderAI $ AutoLeader True False
@@ -416,6 +435,13 @@ rosterDefense = rosterCrawl
                    , [ (-1, 1 + 1 `d` 2, "animal")
                      , (-10, 100, "mobile animal") ] ) ] }
 
+rosterDefenseEmpty = rosterCrawl
+  { rosterList = [ ( playerAntiMonster {fneverEmpty = True}
+                   , [(-4, 1, "scout monster")] )
+                 , (playerHorror, []) ]  -- for summoned animals
+  , rosterEnemy = []
+  , rosterAlly = [] }
+
 cavesRaid, cavesBrawl, cavesShootout, cavesEscape, cavesZoo, cavesAmbush, cavesCrawl, cavesSafari, cavesBattle :: Caves
 
 cavesRaid = IM.fromList [(-2, "caveRaid")]
