@@ -12,7 +12,6 @@ import Prelude ()
 import Game.LambdaHack.Common.Prelude
 
 import Game.LambdaHack.Common.Ability
-import Game.LambdaHack.Common.Faction
 import Game.LambdaHack.Common.Misc
 import Game.LambdaHack.Content.ModeKind
 
@@ -51,7 +50,7 @@ playerCivilian = Player
 
 playerMonster = Player
   { fname = "Monster Hive"
-  , fgroups = ["monster", "mobile monster", "immobile monster"]
+  , fgroups = ["monster", "mobile monster"]
   , fskillsOther = zeroSkills
   , fcanEscape = False
   , fneverEmpty = False
@@ -129,13 +128,13 @@ victoryOutcomes = [Conquer, Escape]
 hiHero, hiRaid, hiDweller, hiEscapist :: HiCondPoly
 
 -- Heroes rejoice in loot.
-hiHero = [ ( [(HiLoot, 1)]
+hiHero = [ ( [(HiLoot, 1000)]  -- multiplied by fraction of collected
            , [minBound..maxBound] )
          , ( [(HiConst, 1000), (HiLoss, -1)]
            , victoryOutcomes )
          ]
 
-hiRaid = [ ( [(HiLoot, 1)]
+hiRaid = [ ( [(HiLoot, 100)]
            , [minBound..maxBound] )
          , ( [(HiConst, 100)]
            , victoryOutcomes )
@@ -153,6 +152,6 @@ hiDweller = [ ( [(HiConst, 1000)]  -- no loot, so big win reward
               , [minBound..maxBound] \\ victoryOutcomes )
             ]
 
-hiEscapist = ( [(HiLoot, 1)]  -- loot matters a little bit
+hiEscapist = ( [(HiLoot, 200)]  -- loot matters a little bit
              , [minBound..maxBound] )
              : hiDweller
