@@ -1,7 +1,7 @@
 {-# LANGUAGE TemplateHaskell #-}
 -- | Game rules and assorted game setup data.
 module Content.RuleKind
-  ( cdefs
+  ( content
   ) where
 
 import Prelude ()
@@ -15,19 +15,10 @@ import System.IO (readFile)
 -- Cabal
 import qualified Paths_LambdaHack as Self (getDataFileName, version)
 
-import Game.LambdaHack.Common.ContentDef
 import Game.LambdaHack.Content.RuleKind
 
-cdefs :: ContentDef RuleKind
-cdefs = ContentDef
-  { getSymbol = rsymbol
-  , getName = rname
-  , getFreq = rfreq
-  , validateSingle = validateSingleRuleKind
-  , validateAll = validateAllRuleKind
-  , content = contentFromList
-      [standard]
-  }
+content :: [RuleKind]
+content = [standard]
 
 standard :: RuleKind
 standard = RuleKind
@@ -47,19 +38,35 @@ standard = RuleKind
       qAddDependentFile path
       x <- qRunIO (readFile path)
       lift x)
-  -- ASCII art for the Main Menu. Only pure 7-bit ASCII characters are allowed
-  -- for the art part. The picture should be exactly 24 rows by 80 columns.
-  -- For a different screen size, the picture is centered and padded.
-  -- with spaces. When displayed in the Main Menu screen, the picture
-  -- is overwritten with game and engine version strings and keybindings.
+  -- ASCII art for the main menu. Only pure 7-bit ASCII characters are allowed.
+  -- When displayed in the main menu screen, the picture is overwritten
+  -- with game and engine version strings and keybindings.
   -- The keybindings overwrite places marked with left curly brace signs.
-  -- The sign is forbidden anywhere else. The Main Menu is displayed dull
-  -- white on black.
+  -- This sign is forbidden anywhere else in the picture.
+  -- The picture and the whole main menu is displayed dull white on black.
+  -- The glyphs, or at least the character cells, are perfect squares.
+  -- The picture should be exactly 45 rows by 80 columns.
+  -- For larger screen sizes, the picture is centered and padded with spaces,
+  -- so it makes sense for some or all of the picture borders to be spaces.
   , rmainMenuArt = $(do
       let path = "GameDefinition/MainMenu.ascii"
       qAddDependentFile path
       x <- qRunIO (readFile path)
       lift x)
+  , rintroScreen = $(do
+      let path = "GameDefinition/PLAYING.md"
+      qAddDependentFile path
+      x <- qRunIO (readFile path)
+      let paragraphs :: [String] -> [String] -> [[String]]
+          paragraphs [] rows = [reverse rows]
+          paragraphs (l : ls) rows = if null l
+                                     then reverse rows : paragraphs ls []
+                                     else paragraphs ls (l : rows)
+          intro = case paragraphs (lines x) [] of
+            _title : _blurb : par1 : par2 : _rest ->
+              ["", ""] ++ par1 ++ [""] ++ par2 ++ ["", ""]
+            _ -> error "not enough paragraphs in intro screen text"
+      lift intro)
   , rfirstDeathEnds = False
   , rwriteSaveClips = 1000
   , rleadLevelClips = 50
