@@ -1,6 +1,6 @@
 -- | Terrain tile definitions.
 module Content.TileKind
-  ( cdefs
+  ( content
   ) where
 
 import Prelude ()
@@ -10,23 +10,16 @@ import Game.LambdaHack.Common.Prelude
 import qualified Data.Text as T
 
 import Game.LambdaHack.Common.Color
-import Game.LambdaHack.Common.ContentDef
 import Game.LambdaHack.Common.Misc
 import Game.LambdaHack.Content.TileKind
 
-cdefs :: ContentDef TileKind
-cdefs = ContentDef
-  { getSymbol = tsymbol
-  , getName = tname
-  , getFreq = tfreq
-  , validateSingle = validateSingleTileKind
-  , validateAll = validateAllTileKind
-  , content = contentFromList $
-      [unknown, hardRock, bedrock, wall, wallSuspect, wallObscured, wallH, wallSuspectH, wallObscuredDefacedH, wallObscuredFrescoedH, pillar, pillarCache, lampPost, signboardUnread, signboardRead, tree, treeBurnt, treeBurning, rubble, rubbleSpice, doorTrapped, doorClosed, doorTrappedH, doorClosedH, stairsUp, stairsTaintedUp, stairsOutdoorUp, stairsGatedUp, stairsDown, stairsTaintedDown, stairsOutdoorDown, stairsGatedDown, escapeUp, escapeDown, escapeOutdoorDown, wallGlass, wallGlassSpice, wallGlassH, wallGlassHSpice, pillarIce, pulpit, bush, bushBurnt, bushBurning, floorFog, floorFogDark, floorSmoke, floorSmokeDark, doorOpen, doorOpenH, floorCorridor, floorArena, floorNoise, floorDirt, floorDirtSpice, floorActor, floorActorItem, floorRed, floorBlue, floorGreen, floorBrown, floorArenaShade ]
-      ++ map makeDark ldarkable
-      ++ map makeDarkColor ldarkColorable
-  }
-unknown,        hardRock, bedrock, wall, wallSuspect, wallObscured, wallH, wallSuspectH, wallObscuredDefacedH, wallObscuredFrescoedH, pillar, pillarCache, lampPost, signboardUnread, signboardRead, tree, treeBurnt, treeBurning, rubble, rubbleSpice, doorTrapped, doorClosed, doorTrappedH, doorClosedH, stairsUp, stairsTaintedUp, stairsOutdoorUp, stairsGatedUp, stairsDown, stairsTaintedDown, stairsOutdoorDown, stairsGatedDown, escapeUp, escapeDown, escapeOutdoorDown, wallGlass, wallGlassSpice, wallGlassH, wallGlassHSpice, pillarIce, pulpit, bush, bushBurnt, bushBurning, floorFog, floorFogDark, floorSmoke, floorSmokeDark, doorOpen, doorOpenH, floorCorridor, floorArena, floorNoise, floorDirt, floorDirtSpice, floorActor, floorActorItem, floorRed, floorBlue, floorGreen, floorBrown, floorArenaShade :: TileKind
+content :: [TileKind]
+content =
+  [unknown, unknownOuterFence, basicOuterFence, bedrock, wall, wallSuspect, wallObscured, wallH, wallSuspectH, wallObscuredDefacedH, wallObscuredFrescoedH, pillar, pillarCache, lampPost, signboardUnread, signboardRead, tree, treeBurnt, treeBurning, rubble, rubbleSpice, doorTrapped, doorClosed, doorTrappedH, doorClosedH, stairsUp, stairsTrappedUp, stairsOutdoorUp, stairsGatedUp, stairsDown, stairsTrappedDown, stairsOutdoorDown, stairsGatedDown, escapeUp, escapeDown, escapeOutdoorDown, wallGlass, wallGlassSpice, wallGlassH, wallGlassHSpice, pillarIce, pulpit, bush, bushBurnt, bushBurning, floorFog, floorFogDark, floorSmoke, floorSmokeDark, doorOpen, doorOpenH, floorCorridor, floorArena, floorNoise, floorDirt, floorDirtSpice, floorActor, floorActorItem, floorRed, floorBlue, floorGreen, floorBrown, floorArenaShade, outdoorFence ]
+  ++ map makeDark ldarkable
+  ++ map makeDarkColor ldarkColorable
+
+unknown,    unknownOuterFence, basicOuterFence, bedrock, wall, wallSuspect, wallObscured, wallH, wallSuspectH, wallObscuredDefacedH, wallObscuredFrescoedH, pillar, pillarCache, lampPost, signboardUnread, signboardRead, tree, treeBurnt, treeBurning, rubble, rubbleSpice, doorTrapped, doorClosed, doorTrappedH, doorClosedH, stairsUp, stairsTrappedUp, stairsOutdoorUp, stairsGatedUp, stairsDown, stairsTrappedDown, stairsOutdoorDown, stairsGatedDown, escapeUp, escapeDown, escapeOutdoorDown, wallGlass, wallGlassSpice, wallGlassH, wallGlassHSpice, pillarIce, pulpit, bush, bushBurnt, bushBurning, floorFog, floorFogDark, floorSmoke, floorSmokeDark, doorOpen, doorOpenH, floorCorridor, floorArena, floorNoise, floorDirt, floorDirtSpice, floorActor, floorActorItem, floorRed, floorBlue, floorGreen, floorBrown, floorArenaShade, outdoorFence :: TileKind
 
 ldarkable :: [TileKind]
 ldarkable = [wall, wallSuspect, wallObscured, wallH, wallSuspectH, wallObscuredDefacedH, wallObscuredFrescoedH, doorTrapped, doorClosed, doorTrappedH, doorClosedH, wallGlass, wallGlassH, doorOpen, doorOpenH, floorCorridor]
@@ -66,16 +59,25 @@ unknown = TileKind  -- needs to have index 0 and alter 1
   , tcolor   = defFG
   , tcolor2  = defFG
   , talter   = 1
-  , tfeature = [Dark, Indistinct]
+  , tfeature = [Dark]
   }
-hardRock = TileKind
+unknownOuterFence = TileKind
+  { tsymbol  = ' '
+  , tname    = "unknown space"
+  , tfreq    = [("unknown outer fence", 1)]
+  , tcolor   = defFG
+  , tcolor2  = defFG
+  , talter   = maxBound  -- impenetrable
+  , tfeature = [Dark]
+  }
+basicOuterFence = TileKind
   { tsymbol  = ' '
   , tname    = "impenetrable bedrock"
   , tfreq    = [("basic outer fence", 1), ("noise fence", 1)]
   , tcolor   = defFG
   , tcolor2  = defFG
   , talter   = maxBound  -- impenetrable
-  , tfeature = [Dark, Indistinct]
+  , tfeature = [Dark]
   }
 bedrock = TileKind
   { tsymbol  = ' '
@@ -84,7 +86,7 @@ bedrock = TileKind
   , tcolor   = defFG
   , tcolor2  = defFG
   , talter   = 100
-  , tfeature = [Dark, Indistinct]
+  , tfeature = [Dark]
       -- Bedrock being dark is bad for AI (forces it to backtrack to explore
       -- bedrock at corridor turns) and induces human micromanagement
       -- if there can be corridors joined diagonally (humans have to check
@@ -104,7 +106,7 @@ wall = TileKind
   , tcolor   = BrWhite
   , tcolor2  = defFG
   , talter   = 100
-  , tfeature = [BuildAs "suspect vertical wall Lit", Indistinct]
+  , tfeature = [BuildAs "suspect vertical wall Lit"]
   }
 wallSuspect = TileKind  -- only on client
   { tsymbol  = '|'
@@ -115,7 +117,6 @@ wallSuspect = TileKind  -- only on client
   , talter   = 2
   , tfeature = [ RevealAs "trapped vertical door Lit"
                , ObscureAs "obscured vertical wall Lit"
-               , Indistinct
                ]
   }
 wallObscured = TileKind
@@ -127,7 +128,6 @@ wallObscured = TileKind
   , talter   = 5
   , tfeature = [ Embed "scratch on wall"
                , HideAs "suspect vertical wall Lit"
-               , Indistinct
                ]
   }
 wallH = TileKind
@@ -137,9 +137,7 @@ wallH = TileKind
   , tcolor   = BrWhite
   , tcolor2  = defFG
   , talter   = 100
-  , tfeature = [ BuildAs "suspect horizontal wall Lit"
-               , Indistinct
-               ]
+  , tfeature = [BuildAs "suspect horizontal wall Lit"]
   }
 wallSuspectH = TileKind  -- only on client
   { tsymbol  = '-'
@@ -150,7 +148,6 @@ wallSuspectH = TileKind  -- only on client
   , talter   = 2
   , tfeature = [ RevealAs "trapped horizontal door Lit"
                , ObscureAs "obscured horizontal wall Lit"
-               , Indistinct
                ]
   }
 wallObscuredDefacedH = TileKind
@@ -160,9 +157,8 @@ wallObscuredDefacedH = TileKind
   , tcolor   = BrWhite
   , tcolor2  = defFG
   , talter   = 5
-  , tfeature = [ Embed "obscene pictograms"
+  , tfeature = [ Embed "obscene pictogram"
                , HideAs "suspect horizontal wall Lit"
-               , Indistinct
                ]
   }
 wallObscuredFrescoedH = TileKind
@@ -174,8 +170,8 @@ wallObscuredFrescoedH = TileKind
   , talter   = 5
   , tfeature = [ Embed "subtle fresco"
                , HideAs "suspect horizontal wall Lit"
-               , Indistinct
                ]  -- a bit beneficial, but AI would loop if allowed to trigger
+                  -- so no @ConsideredByAI@
   }
 pillar = TileKind
   { tsymbol  = 'O'
@@ -187,19 +183,19 @@ pillar = TileKind
   , tcolor   = BrCyan  -- not BrWhite, to tell from heroes
   , tcolor2  = Cyan
   , talter   = 100
-  , tfeature = [Indistinct]
+  , tfeature = []
   }
 pillarCache = TileKind
   { tsymbol  = 'O'
-  , tname    = "cache"
-  , tfreq    = [("cachable", 30), ("stair terminal", 1)]
+  , tname    = "smoothed rock"
+  , tfreq    = [("cachable", 30), ("cache", 1), ("stair terminal", 1)]
   , tcolor   = BrBlue
   , tcolor2  = Blue
   , talter   = 5
-  , tfeature = [ Embed "terrain cache", Embed "terrain cache trap"
-               , ChangeTo "cachable", ConsideredByAI, Indistinct ]
+  , tfeature = [ Embed "treasure cache", Embed "treasure cache trap"
+               , ChangeTo "cachable", ConsideredByAI ]
       -- Not explorable, but prominently placed, so hard to miss.
-      -- Very beneficial, so AI eager to trigger.
+      -- Very beneficial, so AI eager to trigger, unless wary of traps.
   }
 lampPost = TileKind
   { tsymbol  = 'O'
@@ -219,23 +215,22 @@ signboardUnread = TileKind  -- client only, indicates never used by this faction
   , talter   = 5
   , tfeature = [ ConsideredByAI  -- changes after use, so safe for AI
                , RevealAs "signboard"  -- to display as hidden
-               , Indistinct
                ]
   }
 signboardRead = TileKind
   { tsymbol  = 'O'
   , tname    = "signboard"
-  , tfreq    = [("signboard", 1), ("zooSet", 2)]
+  , tfreq    = [("signboard", 1), ("escapeSetDark", 1)]
   , tcolor   = BrCyan
   , tcolor2  = Cyan
   , talter   = 5
-  , tfeature = [Embed "signboard", HideAs "signboard unread", Indistinct]
+  , tfeature = [Embed "signboard", HideAs "signboard unread"]
   }
 tree = TileKind
   { tsymbol  = 'O'
   , tname    = "tree"
   , tfreq    = [ ("brawlSetLit", 140), ("shootoutSetLit", 10)
-               , ("escapeSetLit", 30), ("treeShadeOver_O_Lit", 1) ]
+               , ("escapeSetLit", 35), ("treeShadeOver_O_Lit", 1) ]
   , tcolor   = BrGreen
   , tcolor2  = Green
   , talter   = 50
@@ -243,14 +238,14 @@ tree = TileKind
   }
 treeBurnt = tree
   { tname    = "burnt tree"
-  , tfreq    = [("ambushSet", 3), ("zooSet", 3), ("tree with fire", 30)]
+  , tfreq    = [("ambushSet", 3), ("zooSet", 7), ("tree with fire", 30)]
   , tcolor   = BrBlack
   , tcolor2  = BrBlack
   , tfeature = Dark : tfeature tree
   }
 treeBurning = tree
   { tname    = "burning tree"
-  , tfreq    = [("ambushSet", 30), ("zooSet", 30), ("tree with fire", 70)]
+  , tfreq    = [("ambushSet", 30), ("zooSet", 70), ("tree with fire", 70)]
   , tcolor   = BrRed
   , tcolor2  = Red
   , talter   = 5
@@ -260,7 +255,7 @@ treeBurning = tree
   }
 rubble = TileKind
   { tsymbol  = '&'
-  , tname    = "rubble"
+  , tname    = "rubble pile"
   , tfreq    = []  -- [("floorCorridorLit", 1)]
                    -- disabled while it's all or nothing per cave and per room;
                    -- we need a new mechanism, Spice is not enough, because
@@ -269,18 +264,18 @@ rubble = TileKind
       -- until we can sync change of tile and activation, it always takes 1 turn
   , tcolor   = BrYellow
   , tcolor2  = Brown
-  , talter   = 5
-  , tfeature = [OpenTo "rubbleOrNot", Embed "rubble", Indistinct]
+  , talter   = 4  -- boss can dig through
+  , tfeature = [OpenTo "rubbleOrNot", Embed "rubble"]
   }
 rubbleSpice = TileKind
   { tsymbol  = '&'
-  , tname    = "rubble"
+  , tname    = "rubble pile"
   , tfreq    = [ ("smokeClumpOver_f_Lit", 1), ("emptySet", 1), ("noiseSet", 5)
                , ("zooSet", 100), ("ambushSet", 20) ]
   , tcolor   = BrYellow
   , tcolor2  = Brown
-  , talter   = 5
-  , tfeature = [Spice, OpenTo "rubbleSpiceOrNot", Embed "rubble", Indistinct]
+  , talter   = 4  -- boss can dig through
+  , tfeature = [Spice, OpenTo "rubbleSpiceOrNot", Embed "rubble"]
       -- It's not explorable, due to not being walkable nor clear and due
       -- to being a door (@OpenTo@), which is kind of OK, because getting
       -- the item is risky and, e.g., AI doesn't attempt it.
@@ -296,7 +291,6 @@ doorTrapped = TileKind
   , tfeature = [ Embed "doorway trap"
                , OpenTo "open vertical door Lit"
                , HideAs "suspect vertical wall Lit"
-               , Indistinct
                ]
   }
 doorClosed = TileKind
@@ -306,7 +300,7 @@ doorClosed = TileKind
   , tcolor   = Brown
   , tcolor2  = BrBlack
   , talter   = 2
-  , tfeature = [OpenTo "open vertical door Lit", Indistinct]  -- never hidden
+  , tfeature = [OpenTo "open vertical door Lit"]  -- never hidden
   }
 doorTrappedH = TileKind
   { tsymbol  = '+'
@@ -318,7 +312,6 @@ doorTrappedH = TileKind
   , tfeature = [ Embed "doorway trap"
                , OpenTo "open horizontal door Lit"
                , HideAs "suspect horizontal wall Lit"
-               , Indistinct
                ]
   }
 doorClosedH = TileKind
@@ -328,7 +321,7 @@ doorClosedH = TileKind
   , tcolor   = Brown
   , tcolor2  = BrBlack
   , talter   = 2
-  , tfeature = [OpenTo "open horizontal door Lit", Indistinct]  -- never hidden
+  , tfeature = [OpenTo "open horizontal door Lit"]  -- never hidden
   }
 stairsUp = TileKind
   { tsymbol  = '<'
@@ -339,9 +332,9 @@ stairsUp = TileKind
   , talter   = talterForStairs
   , tfeature = [Embed "staircase up", ConsideredByAI]
   }
-stairsTaintedUp = TileKind
+stairsTrappedUp = TileKind
   { tsymbol  = '<'
-  , tname    = "tainted staircase up"
+  , tname    = "windy staircase up"
   , tfreq    = [("staircase up", 1)]
   , tcolor   = BrRed
   , tcolor2  = Red
@@ -357,7 +350,7 @@ stairsOutdoorUp = stairsUp
 stairsGatedUp = stairsUp
   { tname    = "gated staircase up"
   , tfreq    = [("gated staircase up", 1)]
-  , talter   = talterForStairs + 1  -- animals and bosses can't use
+  , talter   = talterForStairs + 2  -- animals and bosses can't use
   }
 stairsDown = TileKind
   { tsymbol  = '>'
@@ -368,9 +361,9 @@ stairsDown = TileKind
   , talter   = talterForStairs
   , tfeature = [Embed "staircase down", ConsideredByAI]
   }
-stairsTaintedDown = TileKind
+stairsTrappedDown = TileKind
   { tsymbol  = '>'
-  , tname    = "tainted staircase down"
+  , tname    = "crooked staircase down"
   , tfreq    = [("staircase down", 1)]
   , tcolor   = BrRed
   , tcolor2  = Red
@@ -385,7 +378,7 @@ stairsOutdoorDown = stairsDown
 stairsGatedDown = stairsDown
   { tname    = "gated staircase down"
   , tfreq    = [("gated staircase down", 1)]
-  , talter   = talterForStairs + 1  -- animals and bosses can't use
+  , talter   = talterForStairs + 2  -- animals and bosses can't use
   }
 escapeUp = TileKind
   { tsymbol  = '<'
@@ -444,7 +437,7 @@ pillarIce = TileKind
   , tfreq    = [("noiseSet", 30)]
   , tcolor   = BrBlue
   , tcolor2  = Blue
-  , talter   = 5
+  , talter   = 4  -- boss can dig through
   , tfeature = [Clear, Embed "frost", OpenTo "damp stone floor"]
       -- Is door, due to @OpenTo@, so is not explorable, but it's OK, because
       -- it doesn't generate items nor clues. This saves on the need to
@@ -453,17 +446,17 @@ pillarIce = TileKind
 pulpit = TileKind
   { tsymbol  = '%'
   , tname    = "pulpit"
-  , tfreq    = [("pulpit", 1), ("zooSet", 2)]
+  , tfreq    = [("pulpit", 1)]
   , tcolor   = BrYellow
   , tcolor2  = Brown
   , talter   = 5
-  , tfeature = [Clear, Embed "pulpit", Indistinct]
+  , tfeature = [Clear, Embed "pulpit"]
                  -- mixed blessing, so AI ignores, saved for player fun
   }
 bush = TileKind
   { tsymbol  = '%'
   , tname    = "bush"
-  , tfreq    = [ ("bush Lit", 1), ("shootoutSetLit", 30), ("escapeSetLit", 30)
+  , tfreq    = [ ("bush Lit", 1), ("shootoutSetLit", 30), ("escapeSetLit", 40)
                , ("bushClumpOver_f_Lit", 1) ]
   , tcolor   = BrGreen
   , tcolor2  = Green
@@ -495,35 +488,33 @@ floorFog = TileKind
   { tsymbol  = ';'
   , tname    = "faint fog"
   , tfreq    = [ ("lit fog", 1), ("emptySet", 5), ("shootoutSetLit", 20)
-               , ("fogClumpOver_f_Lit", 60) ]
+               , ("noiseSet", 10), ("fogClumpOver_f_Lit", 60) ]
       -- lit fog is OK for shootout, because LOS is mutual, as opposed
       -- to dark fog, and so camper has little advantage, especially
       -- on big maps, where he doesn't know on which side of fog patch to hide
   , tcolor   = BrCyan
   , tcolor2  = Cyan
   , talter   = 0
-  , tfeature = [Walkable, NoItem, Indistinct, OftenActor]
+  , tfeature = [Walkable, NoItem, OftenActor]
   }
 floorFogDark = floorFog
   { tname    = "thick fog"
-  , tfreq    = [("noiseSet", 10), ("escapeSetDark", 60)]
+  , tfreq    = [("noiseSet", 10), ("escapeSetDark", 50)]
   , tfeature = Dark : tfeature floorFog
   }
 floorSmoke = TileKind
   { tsymbol  = ';'
   , tname    = "billowing smoke"
-  , tfreq    = [ ("lit smoke", 1)
-               , ("ambushSet", 30), ("zooSet", 30), ("battleSet", 5)
-               , ("labTrailLit", 1), ("stair terminal", 2)
+  , tfreq    = [ ("lit smoke", 1), ("labTrailLit", 1), ("stair terminal", 2)
                , ("smokeClumpOver_f_Lit", 1) ]
   , tcolor   = Brown
   , tcolor2  = BrBlack
   , talter   = 0
-  , tfeature = [Walkable, NoItem, Indistinct]  -- not dark, embers
+  , tfeature = [Walkable, NoItem]  -- not dark, embers
   }
 floorSmokeDark = floorSmoke
   { tname    = "lingering smoke"
-  , tfreq    = [("ambushSet", 30)]
+  , tfreq    = [("ambushSet", 60), ("zooSet", 20), ("battleSet", 5)]
   , tfeature = Dark : tfeature floorSmoke
   }
 
@@ -558,13 +549,13 @@ floorCorridor = TileKind
   , tcolor   = BrWhite
   , tcolor2  = defFG
   , talter   = 0
-  , tfeature = [Walkable, Clear, Indistinct]
+  , tfeature = [Walkable, Clear]
   }
 floorArena = floorCorridor
   { tsymbol  = floorSymbol
   , tname    = "stone floor"
   , tfreq    = [ ("floorArenaLit", 1), ("rubbleSpiceOrNot", 30)
-               , ("arenaSetLit", 1), ("emptySet", 97), ("zooSet", 1000) ]
+               , ("arenaSetLit", 1), ("emptySet", 97), ("zooSet", 600) ]
   }
 floorNoise = floorArena
   { tname    = "damp stone floor"
@@ -592,10 +583,10 @@ floorActorItem = floorActor
 floorRed = floorCorridor
   { tsymbol  = floorSymbol
   , tname    = "brick pavement"
-  , tfreq    = [("trailLit", 30)]
+  , tfreq    = [("trailLit", 30), ("alarmingTrailLit", 70)]
   , tcolor   = BrRed
   , tcolor2  = Red
-  , tfeature = Trail : tfeature floorCorridor  -- no Indistinct
+  , tfeature = [Trail, Walkable, Clear]
   }
 floorBlue = floorRed
   { tname    = "cobblestone path"
@@ -611,7 +602,7 @@ floorGreen = floorRed
   }
 floorBrown = floorRed
   { tname    = "rotting mahogany deck"
-  , tfreq    = [("trailLit", 10)]
+  , tfreq    = [("trailLit", 10), ("alarmingTrailLit", 30)]
   , tcolor   = BrMagenta
   , tcolor2  = Magenta
   }
@@ -622,6 +613,16 @@ floorArenaShade = floorActor
   , tfeature = Dark : NoItem : tfeature floorActor
   }
 
+outdoorFence = TileKind
+  { tsymbol  = ' '
+  , tname    = "event horizon"
+  , tfreq    = [("outdoor outer fence", 1)]
+  , tcolor   = defFG
+  , tcolor2  = defFG
+  , talter   = maxBound  -- impenetrable
+  , tfeature = [Dark]
+  }
+
 makeDark :: TileKind -> TileKind
 makeDark k = let darkText :: GroupName TileKind -> GroupName TileKind
                  darkText t = maybe t (toGroupName . (<> "Dark"))
